@@ -2,11 +2,11 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2013, 2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2021-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013, 2015 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2007 John Brooks <special@inspircd.org>
+ *   Copyright (C) 2007 John Brooks <john@jbrooks.io>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2006, 2008 Craig Edwards <brain@inspircd.org>
  *
@@ -26,16 +26,12 @@
 
 #pragma once
 
-#include "base.h"
-#include "event.h"
+#include "timeutils.h"
 
-#include <string>
-#include <sstream>
-#include <map>
-
-class HTTPQueryParameters : public insp::flat_multimap<std::string, std::string>
+class HTTPQueryParameters final
+	: public insp::flat_multimap<std::string, std::string>
 {
- public:
+public:
 	bool get(const std::string& key, std::string& value) const
 	{
 		const_iterator it = find(key);
@@ -68,7 +64,7 @@ class HTTPQueryParameters : public insp::flat_multimap<std::string, std::string>
 	unsigned long getDuration(const std::string& key, unsigned long def = 0) const
 	{
 		unsigned long value;
-		if (!InspIRCd::Duration(getString(key, "0"), value))
+		if (!Duration::TryFrom(getString(key, "0"), value))
 			return def;
 
 		return value;
@@ -80,7 +76,7 @@ class HTTPQueryParameters : public insp::flat_multimap<std::string, std::string>
 	}
 };
 
-struct HTTPRequestURI
+struct HTTPRequestURI final
 {
 	std::string path;
 	HTTPQueryParameters query_params;
@@ -89,16 +85,16 @@ struct HTTPRequestURI
 
 /** A modifiable list of HTTP header fields
  */
-class HTTPHeaders
+class HTTPHeaders final
 {
- protected:
-	std::map<std::string,std::string> headers;
- public:
+protected:
+	std::map<std::string, std::string> headers;
+public:
 
 	/** Set the value of a header
 	 * Sets the value of the named header. If the header is already present, it will be replaced
 	 */
-	void SetHeader(const std::string &name, const std::string &data)
+	void SetHeader(const std::string& name, const std::string& data)
 	{
 		headers[name] = data;
 	}
@@ -106,7 +102,7 @@ class HTTPHeaders
 	/** Set the value of a header, only if it doesn't exist already
 	 * Sets the value of the named header. If the header is already present, it will NOT be updated
 	 */
-	void CreateHeader(const std::string &name, const std::string &data)
+	void CreateHeader(const std::string& name, const std::string& data)
 	{
 		if (!IsSet(name))
 			SetHeader(name, data);
@@ -114,7 +110,7 @@ class HTTPHeaders
 
 	/** Remove the named header
 	 */
-	void RemoveHeader(const std::string &name)
+	void RemoveHeader(const std::string& name)
 	{
 		headers.erase(name);
 	}
@@ -129,9 +125,9 @@ class HTTPHeaders
 	/** Get the value of a header
 	 * @return The value of the header, or an empty string
 	 */
-	std::string GetHeader(const std::string &name)
+	std::string GetHeader(const std::string& name)
 	{
-		std::map<std::string,std::string>::iterator it = headers.find(name);
+		std::map<std::string, std::string>::iterator it = headers.find(name);
 		if (it == headers.end())
 			return std::string();
 
@@ -141,9 +137,9 @@ class HTTPHeaders
 	/** Check if the given header is specified
 	 * @return true if the header is specified
 	 */
-	bool IsSet(const std::string &name)
+	bool IsSet(const std::string& name)
 	{
-		std::map<std::string,std::string>::iterator it = headers.find(name);
+		std::map<std::string, std::string>::iterator it = headers.find(name);
 		return (it != headers.end());
 	}
 
@@ -152,12 +148,10 @@ class HTTPHeaders
 	 */
 	std::string GetFormattedHeaders()
 	{
-		std::string re;
-
-		for (std::map<std::string,std::string>::iterator i = headers.begin(); i != headers.end(); i++)
-			re += i->first + ": " + i->second + "\r\n";
-
-		return re;
+		std::stringstream buf;
+		for (const auto& [key, value] : headers)
+			buf << key << ": " << value << "\r\n";
+		return buf.str();
 	}
 };
 
@@ -165,17 +159,17 @@ class HttpServerSocket;
 
 /** This class represents a HTTP request.
  */
-class HTTPRequest
+class HTTPRequest final
 {
- protected:
+protected:
 	std::string type;
 	std::string ipaddr;
 	std::string postdata;
 	HTTPRequestURI parseduri;
 
- public:
+public:
 
-	HTTPHeaders *headers;
+	HTTPHeaders* headers;
 	int errorcode;
 
 	/** A socket pointer, which you must return in your HTTPDocument class
@@ -247,9 +241,9 @@ class HTTPRequest
  * form a valid HTTP response: the document data and a response code.
  * You can add additional HTTP headers, if you want.
  */
-class HTTPDocumentResponse
+class HTTPDocumentResponse final
 {
- public:
+public:
 	/** Module that generated this reply
 	 */
 	Module* const module;
@@ -271,14 +265,18 @@ class HTTPDocumentResponse
 	 * based upon the response code.
 	 */
 	HTTPDocumentResponse(Module* mod, HTTPRequest& req, std::stringstream* doc, unsigned int response)
-		: module(mod), document(doc), responsecode(response), src(req)
+		: module(mod)
+		, document(doc)
+		, responsecode(response)
+		, src(req)
 	{
 	}
 };
 
-class HTTPdAPIBase : public DataProvider
+class HTTPdAPIBase
+	: public DataProvider
 {
- public:
+public:
 	HTTPdAPIBase(Module* parent)
 		: DataProvider(parent, "m_httpd_api")
 	{
@@ -293,31 +291,34 @@ class HTTPdAPIBase : public DataProvider
 /** The API provided by the httpd module that allows other modules to respond to incoming
  * HTTP requests
  */
-class HTTPdAPI : public dynamic_reference<HTTPdAPIBase>
+class HTTPdAPI final
+	: public dynamic_reference<HTTPdAPIBase>
 {
- public:
+public:
 	HTTPdAPI(Module* parent)
 		: dynamic_reference<HTTPdAPIBase>(parent, "m_httpd_api")
 	{
 	}
 };
 
-class HTTPACLEventListener : public Events::ModuleEventListener
+class HTTPACLEventListener
+	: public Events::ModuleEventListener
 {
- public:
-	HTTPACLEventListener(Module* mod)
-		: ModuleEventListener(mod, "event/http-acl")
+public:
+	HTTPACLEventListener(Module* mod, unsigned int eventprio = DefaultPriority)
+		: ModuleEventListener(mod, "event/http-acl", eventprio)
 	{
 	}
 
 	virtual ModResult OnHTTPACLCheck(HTTPRequest& req) = 0;
 };
 
-class HTTPRequestEventListener : public Events::ModuleEventListener
+class HTTPRequestEventListener
+	: public Events::ModuleEventListener
 {
- public:
-	HTTPRequestEventListener(Module* mod)
-		: ModuleEventListener(mod, "event/http-request")
+public:
+	HTTPRequestEventListener(Module* mod, unsigned int eventprio = DefaultPriority)
+		: ModuleEventListener(mod, "event/http-request", eventprio)
 	{
 	}
 
