@@ -1,8 +1,9 @@
 #
 # InspIRCd -- Internet Relay Chat Daemon
 #
+#   Copyright (C) 2024 satmd <satmd@satmd.de>
 #   Copyright (C) 2020 Nicole Kleinhoff <ilbelkyr@shalture.org>
-#   Copyright (C) 2013-2022 Sadie Powell <sadie@witchery.services>
+#   Copyright (C) 2013-2022, 2024 Sadie Powell <sadie@witchery.services>
 #   Copyright (C) 2012 Robby <robby@chatbelgie.be>
 #   Copyright (C) 2007-2008 Craig Edwards <brain@inspircd.org>
 #   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
@@ -23,7 +24,7 @@
 
 package make::configure;
 
-use v5.10.0;
+use v5.26.0;
 use strict;
 use warnings FATAL => qw(all);
 
@@ -49,8 +50,6 @@ our @EXPORT = qw(CONFIGURE_CACHE_FILE
                  run_test
                  test_file
                  test_header
-                 module_expand
-                 module_shrink
                  write_configure_cache
                  get_compiler_info
                  find_compiler
@@ -58,8 +57,8 @@ our @EXPORT = qw(CONFIGURE_CACHE_FILE
 
 sub __get_socketengines {
 	my @socketengines;
-	for (<${\CONFIGURE_ROOT}/src/socketengines/socketengine_*.cpp>) {
-		s/src\/socketengines\/socketengine_(\w+)\.cpp/$1/;
+	for (<${\CONFIGURE_ROOT}/src/socketengines/*.cpp>) {
+		s/src\/socketengines\/(\w+)\.cpp/$1/;
 		push @socketengines, $1;
 	}
 	return @socketengines;
@@ -94,7 +93,7 @@ sub __get_template_settings($$$) {
 
 sub __test_compiler($) {
 	my $compiler = shift;
-	return 0 unless run_test("`$compiler`", !system "$compiler -v ${\CONFIGURE_ERROR_PIPE}");
+	return 0 unless run_test("`$compiler`", !system "$compiler --version ${\CONFIGURE_ERROR_PIPE}");
 	return 0 unless run_test("`$compiler`", test_file($compiler, 'compiler.cpp', '-fno-rtti'), 'compatible');
 	return 1;
 }
@@ -125,7 +124,7 @@ non-interactive configuration is started and any omitted values are defaulted.
                                 stored.
                                 [${\CONFIGURE_ROOT}/run/bin]
   <|BOLD --config-dir <DIR>|>            The location where the configuration files and
-                                SSL certificates are stored.
+                                TLS certificates are stored.
                                 [${\CONFIGURE_ROOT}/run/conf]
   <|BOLD --data-dir <DIR>|>              The location where the data files, such as the
                                 xline database, are stored.
@@ -176,7 +175,7 @@ non-interactive configuration is started and any omitted values are defaulted.
 
   <|BOLD CXX=<NAME>|>                    Sets the C++ compiler to use when building the
                                 server. If not specified then the build system
-                                will search for c++, g++, clang++, icpx, or icpc.
+                                will search for c++, g++, clang++, or icpx.
   <|BOLD INSPIRCD_VERBOSE=<0|1>|>        Shows additional information for debugging.
 
 If you have any problems with configuring InspIRCd then visit our IRC channel
@@ -213,7 +212,7 @@ sub test_file($$;$) {
 	my ($compiler, $file, $args) = @_;
 	my $status = 0;
 	$args //= '';
-	$status ||= system "$compiler -o ${\CONFIGURE_ROOT}/__test_$file ${\CONFIGURE_ROOT}/make/test/$file $args ${\CONFIGURE_ERROR_PIPE}";
+	$status ||= system "$compiler -std=c++17 -o ${\CONFIGURE_ROOT}/__test_$file ${\CONFIGURE_ROOT}/make/test/$file $args ${\CONFIGURE_ERROR_PIPE}";
 	$status ||= system "${\CONFIGURE_ROOT}/__test_$file ${\CONFIGURE_ERROR_PIPE}";
 	unlink "${\CONFIGURE_ROOT}/__test_$file";
 	return !$status;
@@ -228,19 +227,6 @@ sub test_header($$;$) {
 	return !$?;
 }
 
-sub module_expand($) {
-	my $module = shift;
-	$module = "m_$module" unless $module =~ /^(?:m|core)_/;
-	$module = "$module.cpp" unless $module =~ /\.cpp$/;
-	return $module;
-}
-
-sub module_shrink($) {
-	my $module = basename shift;
-	$module =~ s/(?:^m_|\.cpp$)//g;
-	return $module;
-}
-
 sub write_configure_cache(%) {
 	unless (-e CONFIGURE_DIRECTORY) {
 		say console_format "Creating <|GREEN ${\abs2rel CONFIGURE_DIRECTORY, CONFIGURE_ROOT}|> ...";
@@ -255,10 +241,10 @@ sub write_configure_cache(%) {
 sub get_compiler_info($) {
 	my $binary = shift;
 	my %info = (NAME => 'Unknown', VERSION => '0.0');
-	return %info if system "$binary -o __compiler_info ${\CONFIGURE_ROOT}/make/test/compiler_info.cpp ${\CONFIGURE_ERROR_PIPE}";
+	return %info if system "$binary -o __compiler_info ${\CONFIGURE_ROOT}/make/test/compiler.cpp ${\CONFIGURE_ERROR_PIPE}";
 	open(my $fh, '-|', './__compiler_info 2>/dev/null');
 	while (my $line = <$fh>) {
-		$info{$1} = $2 if $line =~ /^([A-Z]+)\s(.+)$/;
+		$info{$1} = $2 if $line =~ /^([A-Z_]+)\s(.*)$/;
 	}
 	close $fh;
 	unlink './__compiler_info';
@@ -266,7 +252,7 @@ sub get_compiler_info($) {
 }
 
 sub find_compiler {
-	my @compilers = qw(c++ g++ clang++ icpx icpc);
+	my @compilers = qw(c++ g++ clang++ icpx);
 	for my $compiler (shift // @compilers) {
 		return $compiler if __test_compiler $compiler;
 		return "xcrun $compiler" if $^O eq 'darwin' && __test_compiler "xcrun $compiler";
