@@ -2,8 +2,7 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2013-2014, 2016-2023 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2013 ChrisTX <xpipe@hotmail.de>
+ *   Copyright (C) 2013-2014, 2016-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2014 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
@@ -22,15 +21,24 @@
  */
 
 
-#include "inspircd.h"
+#include <cinttypes>
+#include <filesystem>
 #include <fstream>
+
+#include "inspircd.h"
 #include "configparser.h"
+#include "timeutils.h"
+#include "utility/string.h"
+
+#ifdef _WIN32
+# define pclose _pclose
+# define popen _popen
+#else
+# include <unistd.h>
+#endif
 
 enum ParseFlags
 {
-	// Legacy config parsing should be used.
-	FLAG_USE_COMPAT = 1,
-
 	// Executable includes are disabled.
 	FLAG_NO_EXEC = 2,
 
@@ -44,88 +52,26 @@ enum ParseFlags
 	FLAG_MISSING_OKAY = 16
 };
 
-// Represents the position within a config file.
-struct FilePosition
-{
-	// The name of the file which is being read.
-	std::string name;
-
-	// The line of the file that this position points to.
-	unsigned int line;
-
-	// The column of the file that this position points to.
-	unsigned int column;
-
-	FilePosition(const std::string& Name)
-		: name(Name)
-		, line(1)
-		, column(0)
-	{
-	}
-
-	FilePosition()
-		: line(0)
-		, column(0)
-	{
-	}
-
-	/** Returns a string that represents this file position. */
-	std::string str()
-	{
-		return name + ":" + ConvToStr(line) + ":" + ConvToStr(column);
-	}
-};
-
-// RAII wrapper for FILE* which closes the file when it goes out of scope.
-class FileWrapper
-{
- private:
-	// Whether this file handle should be closed with pclose.
-	bool close_with_pclose;
-
-	// The file handle which is being wrapped.
-	FILE* const file;
-
- public:
-	FileWrapper(FILE* File, bool CloseWithPClose = false)
-		: close_with_pclose(CloseWithPClose)
-		, file(File)
-	{
-	}
-
-	// Operator which determines whether the file is open.
-	operator bool() { return (file != NULL); }
-
-	// Operator which retrieves the underlying FILE pointer.
-	operator FILE*() { return file; }
-
-	~FileWrapper()
-	{
-		if (!file)
-			return;
-
-		if (close_with_pclose)
-			pclose(file);
-		else
-			fclose(file);
-	}
-};
-
-
-struct Parser
+struct Parser final
 {
 	ParseStack& stack;
 	int flags;
-	FILE* const file;
+	FilePtr file;
 	FilePosition current;
 	FilePosition last_tag;
-	reference<ConfigTag> tag;
-	int ungot;
+	std::shared_ptr<ConfigTag> tag;
+	int ungot = -1;
 	std::string mandatory_tag;
 
-	Parser(ParseStack& me, int myflags, FILE* conf, const std::string& name, const std::string& mandatorytag)
-		: stack(me), flags(myflags), file(conf), current(name), ungot(-1), mandatory_tag(mandatorytag)
-	{ }
+	Parser(ParseStack& me, int myflags, FilePtr conf, const std::string& name, const std::string& mandatorytag)
+		: stack(me)
+		, flags(myflags)
+		, file(std::move(conf))
+		, current(name, 1, 0)
+		, last_tag(name, 0, 0)
+		, mandatory_tag(mandatorytag)
+	{
+	}
 
 	int next(bool eof_ok = false)
 	{
@@ -135,7 +81,7 @@ struct Parser
 			ungot = -1;
 			return ch;
 		}
-		int ch = fgetc(file);
+		int ch = fgetc(file.get());
 		if (ch == EOF && !eof_ok)
 		{
 			throw CoreException("Unexpected end-of-file");
@@ -143,7 +89,7 @@ struct Parser
 		else if (ch == '\n')
 		{
 			current.line++;
-			current.column = 0;
+			current.column = 1;
 		}
 		else
 		{
@@ -161,7 +107,7 @@ struct Parser
 
 	void comment()
 	{
-		while (1)
+		while (true)
 		{
 			int ch = next(true);
 			if (ch == '\n')
@@ -175,7 +121,7 @@ struct Parser
 		}
 	}
 
-	bool wordchar(char ch)
+	static bool wordchar(int ch)
 	{
 		return isalnum(ch)
 			|| ch == '-'
@@ -196,7 +142,7 @@ struct Parser
 		unget(ch);
 	}
 
-	bool kv(ConfigItems* items)
+	bool kv()
 	{
 		std::string key;
 		nextword(key);
@@ -219,15 +165,15 @@ struct Parser
 		ch = next();
 		if (ch != '"')
 		{
-			throw CoreException("Invalid character in value of <" + tag->tag + ":" + key + ">");
+			throw CoreException("Invalid character in value of <" + tag->name + ":" + key + ">");
 		}
-		while (1)
+		while (true)
 		{
 			ch = next();
-			if (ch == '&' && !(flags & FLAG_USE_COMPAT))
+			if (ch == '&')
 			{
 				std::string varname;
-				while (1)
+				while (true)
 				{
 					ch = next();
 					if (wordchar(ch) || (varname.empty() && ch == '#'))
@@ -236,7 +182,7 @@ struct Parser
 						break;
 					else
 					{
-						stack.errstr << "Invalid XML entity name in value of <" + tag->tag + ":" + key + ">\n"
+						stack.errstr << "Invalid XML entity name in value of <" + tag->name + ":" + key + ">\n"
 							<< "To include an ampersand or quote, use &amp; or &quot;\n";
 						throw CoreException("Parse error");
 					}
@@ -277,26 +223,14 @@ struct Parser
 					value.append(var->second);
 				}
 			}
-			else if (ch == '\\' && (flags & FLAG_USE_COMPAT))
-			{
-				int esc = next();
-				if (esc == 'n')
-					value.push_back('\n');
-				else if (isalpha(esc))
-					throw CoreException("Unknown escape character \\" + std::string(1, esc));
-				else
-					value.push_back(esc);
-			}
 			else if (ch == '"')
 				break;
 			else if (ch != '\r')
 				value.push_back(ch);
 		}
 
-		if (items->find(key) != items->end())
+		if (!tag->GetItems().insert({key, value}).second)
 			throw CoreException("Duplicate key '" + key + "' found");
-
-		(*items)[key] = value;
 		return true;
 	}
 
@@ -315,10 +249,8 @@ struct Parser
 		if (name.empty())
 			throw CoreException("Empty tag name");
 
-		ConfigItems* items;
-		tag = ConfigTag::create(name, current.name, current.line, items);
-
-		while (kv(items))
+		tag = std::make_shared<ConfigTag>(name, last_tag);
+		while (kv())
 		{
 			// Do nothing here (silences a GCC warning).
 		}
@@ -329,57 +261,42 @@ struct Parser
 			mandatory_tag.clear();
 		}
 
-		if (stdalgo::string::equalsci(name, "include"))
+		if (insp::equalsci(name, "include"))
 		{
 			stack.DoInclude(tag, flags);
 		}
-		else if (stdalgo::string::equalsci(name, "files"))
+		else if (insp::equalsci(name, "files"))
 		{
-			for(ConfigItems::iterator i = items->begin(); i != items->end(); i++)
-			{
-				stack.DoReadFile(i->first, i->second, flags, false);
-			}
+			for (const auto& [key, value] : tag->GetItems())
+				stack.DoReadFile(key, value, flags, false);
 		}
-		else if (stdalgo::string::equalsci(name, "execfiles"))
+		else if (insp::equalsci(name, "execfiles"))
 		{
-			for(ConfigItems::iterator i = items->begin(); i != items->end(); i++)
-			{
-				stack.DoReadFile(i->first, i->second, flags, true);
-			}
+			for (const auto& [key, value] : tag->GetItems())
+				stack.DoReadFile(key, value, flags, true);
 		}
-		else if (stdalgo::string::equalsci(name, "define"))
+		else if (insp::equalsci(name, "define"))
 		{
-			if (flags & FLAG_USE_COMPAT)
-				throw CoreException("<define> tags may only be used in XML-style config (add <config format=\"xml\">)");
-			std::string varname = tag->getString("name");
-			std::string value = tag->getString("value");
+			const std::string varname = tag->getString("name");
 			if (varname.empty())
-				throw CoreException("Variable definition must include variable name");
-			stack.vars[varname] = value;
-		}
-		else if (stdalgo::string::equalsci(name, "config"))
-		{
-			std::string format = tag->getString("format");
-			if (stdalgo::string::equalsci(format, "xml"))
-				flags &= ~FLAG_USE_COMPAT;
-			else if (stdalgo::string::equalsci(format, "compat"))
-				flags |= FLAG_USE_COMPAT;
-			else if (!format.empty())
-				throw CoreException("Unknown configuration format " + format);
+				throw CoreException("Variable definition must include a variable name, at " + tag->source.str());
+
+			if (stack.vars.find(varname) == stack.vars.end() || tag->getBool("replace", true))
+				stack.vars[varname] = tag->getString("value");
 		}
 		else
 		{
-			stack.output.insert(std::make_pair(name, tag));
+			stack.output.emplace(name, tag);
 		}
-		// this is not a leak; reference<> takes care of the delete
-		tag = NULL;
+		// this is not a leak; shared_ptr takes care of the delete
+		tag = nullptr;
 	}
 
 	bool outer_parse()
 	{
 		try
 		{
-			while (1)
+			while (true)
 			{
 				int ch = next(true);
 				switch (ch)
@@ -403,26 +320,38 @@ struct Parser
 					case 0xFE:
 					case 0xFF:
 						stack.errstr << "Do not save your files as UTF-16 or UTF-32, use UTF-8!\n";
-						/*@fallthrough@*/
+						[[fallthrough]];
 					default:
 						throw CoreException("Syntax error - start of tag expected");
 				}
 			}
 		}
-		catch (CoreException& err)
+		catch (const CoreException& err)
 		{
 			stack.errstr << err.GetReason() << " at " << current.str();
 			if (tag)
-				stack.errstr << " (inside <" << tag->tag << "> tag on line " << tag->src_line << ")";
-			else if (!last_tag.name.empty())
+				stack.errstr << " (inside <" << tag->name << "> tag on line " << tag->source.line << ")";
+			else if (last_tag.line)
 				stack.errstr << " (last tag was on line " << last_tag.line << ")";
-			stack.errstr << " \n";
+			stack.errstr << '\n';
 		}
 		return false;
 	}
 };
 
-void ParseStack::DoInclude(ConfigTag* tag, int flags)
+FilePosition::FilePosition(const std::string& Name, unsigned long Line, unsigned long Column)
+	: name(Name)
+	, line(Line)
+	, column(Column)
+{
+}
+
+std::string FilePosition::str() const
+{
+	return name + ":" + ConvToStr(line) + ":" + ConvToStr(column);
+}
+
+void ParseStack::DoInclude(const std::shared_ptr<ConfigTag>& tag, int flags)
 {
 	if (flags & FLAG_NO_INC)
 		throw CoreException("Invalid <include> tag in file included with noinclude=\"yes\"");
@@ -447,7 +376,7 @@ void ParseStack::DoInclude(ConfigTag* tag, int flags)
 		else
 			flags &= ~FLAG_MISSING_OKAY;
 
-		if (!ParseFile(ServerInstance->Config->Paths.PrependConfig(name), flags, mandatorytag))
+		if (!ParseFile(name, flags, mandatorytag))
 			throw CoreException("Included");
 	}
 	else if (tag->readString("directory", name))
@@ -460,18 +389,31 @@ void ParseStack::DoInclude(ConfigTag* tag, int flags)
 			flags |= FLAG_NO_ENV;
 
 		const std::string includedir = ServerInstance->Config->Paths.PrependConfig(name);
-		std::vector<std::string> files;
-		if (!FileSystem::GetFileList(includedir, files, "*.conf"))
-			throw CoreException("Unable to read directory for include: " + includedir);
-
-		std::sort(files.begin(), files.end());
-		for (std::vector<std::string>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
+		std::set<std::string> configs;
+		try
 		{
-			const std::string path = includedir + '/' + *iter;
-			if (!ParseFile(path, flags, mandatorytag))
+			for (const auto& entry : std::filesystem::directory_iterator(includedir))
+			{
+				if (!entry.is_regular_file())
+					continue;
+
+				const std::string path = entry.path().string();
+				if (InspIRCd::Match(path, "*.conf"))
+					configs.insert(path);
+			}
+		}
+		catch (const std::filesystem::filesystem_error& err)
+		{
+			throw CoreException("Unable to read directory for include " + includedir + ": " + err.what());
+		}
+
+		for (const auto& config : configs)
+		{
+			if (!ParseFile(config, flags, mandatorytag))
 				throw CoreException("Included");
 		}
 	}
+
 	else if (tag->readString("executable", name))
 	{
 		if (flags & FLAG_NO_EXEC)
@@ -488,28 +430,33 @@ void ParseStack::DoInclude(ConfigTag* tag, int flags)
 	}
 }
 
-namespace
+FilePtr ParseStack::DoOpenFile(const std::string& name, bool isexec)
 {
-	void CheckOwnership(const std::string& path)
+	if (isexec)
 	{
-#ifndef _WIN32
-		struct stat pathinfo;
-		if (stat(path.c_str(), &pathinfo))
-			return; // Will be handled when fopen fails.
+		ServerInstance->Logs.Debug("CONFIG", "Opening executable: {}", name);
+		return FilePtr(popen(name.c_str(), "r"), pclose);
+	}
 
+	const std::string path = ServerInstance->Config->Paths.PrependConfig(name);
+	ServerInstance->Logs.Debug("CONFIG", "Opening file: {}", path);
+#ifndef _WIN32
+	struct stat pathinfo;
+	if (stat(path.c_str(), &pathinfo) == 0)
+	{
 		if (getegid() != pathinfo.st_gid)
 		{
-			ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, "Possible configuration error: %s is owned by group %u but the server is running as group %u.",
-				path.c_str(), pathinfo.st_gid, getegid());
+			ServerInstance->Logs.Warning("CONFIG", "Possible configuration error: {} is owned by group {} but the server is running as group {}.",
+				path, pathinfo.st_gid, getegid());
 		}
-
 		if (geteuid() != pathinfo.st_uid)
 		{
-			ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, "Possible configuration error: %s is owned by user %u but the server is running as user %u.",
-				path.c_str(), pathinfo.st_uid, geteuid());
+			ServerInstance->Logs.Warning("CONFIG", "Possible configuration error: {} is owned by user {} but the server is running as user {}.",
+				path, pathinfo.st_uid, geteuid());
 		}
-#endif
 	}
+#endif
+	return FilePtr(fopen(path.c_str(), "r"), fclose);
 }
 
 void ParseStack::DoReadFile(const std::string& key, const std::string& name, int flags, bool exec)
@@ -519,114 +466,107 @@ void ParseStack::DoReadFile(const std::string& key, const std::string& name, int
 	if (exec && (flags & FLAG_NO_EXEC))
 		throw CoreException("Invalid <execfiles> tag in file included with noexec=\"yes\"");
 
-	const std::string path = ServerInstance->Config->Paths.PrependConfig(name);
-	if (!exec)
-		CheckOwnership(path);
-
-	FileWrapper file(exec ? popen(name.c_str(), "r") : fopen(path.c_str(), "r"), exec);
-	if (!file)
-		throw CoreException("Could not read \"" + path + "\" for \"" + key + "\" file");
-
-	file_cache& cache = FilesOutput[key];
-	cache.clear();
-
-	char linebuf[5120];
-	while (fgets(linebuf, sizeof(linebuf), file))
+	if (FilesOutput.emplace(key, std::make_pair(name, exec)).second)
 	{
-		size_t len = strlen(linebuf);
-		if (len)
-		{
-			if (linebuf[len-1] == '\n')
-				len--;
-			cache.push_back(std::string(linebuf, len));
-		}
+		ServerInstance->Logs.Debug("CONFIG", "Stored config key: {} => {} (executable: {}).",
+			key, name, exec ? "yes" : "no");
 	}
 }
 
 ParseStack::ParseStack(ServerConfig* conf)
 	: output(conf->config_data)
-	, FilesOutput(conf->Files)
+	, FilesOutput(conf->filesources)
 	, errstr(conf->errstr)
 {
-	// Special character escapes.
-	vars["newline"] = "\n";
-	vars["nl"]      = "\n";
+	vars = {
+		// Special character escapes.
+		{ "newline", "\n" },
+		{ "nl",      "\n" },
 
-	// XML escapes.
-	vars["amp"]  = "&";
-	vars["apos"] = "'";
-	vars["gt"]   = ">";
-	vars["lt"]   = "<";
-	vars["quot"] = "\"";
+		// XML escapes.
+		{ "amp",  "&"  },
+		{ "apos", "'"  },
+		{ "gt",   ">"  },
+		{ "lt",   "<"  },
+		{ "quot", "\"" },
 
-	// Directories that were set at build time.
-	vars["dir.config"]  = INSPIRCD_CONFIG_PATH;
-	vars["dir.data"]    = INSPIRCD_DATA_PATH;
-	vars["dir.log"]     = INSPIRCD_LOG_PATH;
-	vars["dir.module"]  = INSPIRCD_MODULE_PATH;
-	vars["dir.runtime"] = INSPIRCD_RUNTIME_PATH;
+		// Directories that were set at build time.
+		{ "dir.config",  INSPIRCD_CONFIG_PATH  },
+		{ "dir.example", INSPIRCD_EXAMPLE_PATH },
+		{ "dir.data",    INSPIRCD_DATA_PATH    },
+		{ "dir.log",     INSPIRCD_LOG_PATH     },
+		{ "dir.module",  INSPIRCD_MODULE_PATH  },
+		{ "dir.runtime", INSPIRCD_RUNTIME_PATH },
 
-	// IRC formatting codes.
-	vars["irc.bold"]          = "\x02";
-	vars["irc.color"]         = "\x03";
-	vars["irc.colour"]        = "\x03";
-	vars["irc.hexcolor"]      = "\x04";
-	vars["irc.hexcolour"]     = "\x04";
-	vars["irc.italic"]        = "\x1D";
-	vars["irc.monospace"]     = "\x11";
-	vars["irc.reset"]         = "\x0F";
-	vars["irc.reverse"]       = "\x16";
-	vars["irc.strikethrough"] = "\x1E";
-	vars["irc.underline"]     = "\x1F";
+		// IRC formatting codes.
+		{ "irc.bold",          "\x02" },
+		{ "irc.color",         "\x03" },
+		{ "irc.colour",        "\x03" },
+		{ "irc.hexcolor",      "\x04" },
+		{ "irc.hexcolour",     "\x04" },
+		{ "irc.italic",        "\x1D" },
+		{ "irc.monospace",     "\x11" },
+		{ "irc.reset",         "\x0F" },
+		{ "irc.reverse",       "\x16" },
+		{ "irc.strikethrough", "\x1E" },
+		{ "irc.underline",     "\x1F" },
+	};
 }
 
 bool ParseStack::ParseFile(const std::string& path, int flags, const std::string& mandatory_tag, bool isexec)
 {
-	ServerInstance->Logs->Log("CONFIG", LOG_DEBUG, "Reading (isexec=%d) %s", isexec, path.c_str());
 	if (stdalgo::isin(reading, path))
 		throw CoreException((isexec ? "Executable " : "File ") + path + " is included recursively (looped inclusion)");
 
-	if (!isexec)
-		CheckOwnership(path);
-
 	/* It's not already included, add it to the list of files we've loaded */
-	FileWrapper file((isexec ? popen(path.c_str(), "r") : fopen(path.c_str(), "r")), isexec);
+
+	FilePtr file = DoOpenFile(path, isexec);
 	if (!file)
 	{
 		if (flags & FLAG_MISSING_OKAY)
 			return true;
 
-		throw CoreException("Could not read \"" + path + "\" for include");
+		throw CoreException(INSP_FORMAT("Could not read \"{}\" for include: {}", path, strerror(errno)));
 	}
 
 	reading.push_back(path);
-	Parser p(*this, flags, file, path, mandatory_tag);
+	Parser p(*this, flags, std::move(file), path, mandatory_tag);
 	bool ok = p.outer_parse();
 	reading.pop_back();
 	return ok;
 }
 
-bool ConfigTag::readString(const std::string& key, std::string& value, bool allow_lf)
+void ConfigTag::LogMalformed(const std::string& key, const std::string& val, const std::string& def, const std::string& reason) const
 {
-	for(ConfigItems::iterator j = items.begin(); j != items.end(); ++j)
+	ServerInstance->Logs.Warning("CONFIG", "The value of <{}:{}> at {} ({}) is {}; using the default ({}) instead.",
+		name, key, source.str(), val, reason, def);
+}
+
+bool ConfigTag::readString(const std::string& key, std::string& value, bool allow_lf) const
+{
+	for (const auto& [ikey, ivalue] : items)
 	{
-		if(j->first != key)
+		if (!insp::equalsci(ikey, key))
 			continue;
-		value = j->second;
+
+		value = ivalue;
 		if (!allow_lf && (value.find('\n') != std::string::npos))
 		{
-			ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, "Value of <" + tag + ":" + key + "> at " + getTagLocation() +
+			ServerInstance->Logs.Warning("CONFIG", "Value of <" + name + ":" + key + "> at " + source.str() +
 				" contains a linefeed, and linefeeds in this value are not permitted -- stripped to spaces.");
-			for (std::string::iterator n = value.begin(); n != value.end(); n++)
-				if (*n == '\n')
-					*n = ' ';
+
+			for (auto& chr : value)
+			{
+				if (chr == '\n')
+					chr = ' ';
+			}
 		}
 		return true;
 	}
 	return false;
 }
 
-std::string ConfigTag::getString(const std::string& key, const std::string& def, const TR1NS::function<bool(const std::string&)>& validator)
+std::string ConfigTag::getString(const std::string& key, const std::string& def, const std::function<bool(const std::string&)>& validator) const
 {
 	std::string res;
 	if (!readString(key, res))
@@ -634,14 +574,13 @@ std::string ConfigTag::getString(const std::string& key, const std::string& def,
 
 	if (!validator(res))
 	{
-		ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, "WARNING: The value of <%s:%s> is not valid; value set to %s.",
-			tag.c_str(), key.c_str(), def.c_str());
+		LogMalformed(key, res, def, "not valid");
 		return def;
 	}
 	return res;
 }
 
-std::string ConfigTag::getString(const std::string& key, const std::string& def, size_t minlen, size_t maxlen)
+std::string ConfigTag::getString(const std::string& key, const std::string& def, size_t minlen, size_t maxlen) const
 {
 	std::string res;
 	if (!readString(key, res))
@@ -649,8 +588,7 @@ std::string ConfigTag::getString(const std::string& key, const std::string& def,
 
 	if (res.length() < minlen || res.length() > maxlen)
 	{
-		ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, "WARNING: The length of <%s:%s> is not between %ld and %ld; value set to %s.",
-			tag.c_str(), key.c_str(), (unsigned long)minlen, (unsigned long)maxlen, def.c_str());
+		LogMalformed(key, res, def, "not between " + ConvToStr(minlen) + " and " + ConvToStr(maxlen)  + " characters in length");
 		return def;
 	}
 	return res;
@@ -669,7 +607,7 @@ namespace
 	 * @param tail The location in the config value at which the magnifier is located.
 	 */
 	template <typename Numeric>
-	void CheckMagnitude(const std::string& tag, const std::string& key, const std::string& val, Numeric& num, Numeric def, const char* tail)
+	void CheckMagnitude(const ConfigTag* tag, const std::string& key, const std::string& val, Numeric& num, Numeric def, const char* tail)
 	{
 		// If this is NULL then no magnitude specifier was given.
 		if (!*tail)
@@ -688,12 +626,12 @@ namespace
 			case 'G':
 				num *= 1024 * 1024 * 1024;
 				return;
-		}
 
-		const std::string message = "WARNING: <" + tag + ":" + key + "> value of " + val + " contains an invalid magnitude specifier '"
-			+ tail + "'; value set to " + ConvToStr(def) + ".";
-		ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, message);
-		num = def;
+			default:
+				num = def;
+				tag->LogMalformed(key, val, ConvToStr(def), "contains an invalid magnitude specifier (" + ConvToStr(*tail) +")");
+				return;
+		}
 	}
 
 	/** Check for an out of range value. If the value falls outside the boundaries a warning is
@@ -706,136 +644,108 @@ namespace
 	 * @param max Maximum accepted value for \p res.
 	 */
 	template <typename Numeric>
-	void CheckRange(const std::string& tag, const std::string& key, Numeric& num, Numeric def, Numeric min, Numeric max)
+	void CheckRange(const ConfigTag* tag, const std::string& key, Numeric& num, Numeric def, Numeric min, Numeric max)
 	{
 		if (num >= min && num <= max)
 			return;
 
-		const std::string message = "WARNING: <" + tag + ":" + key + "> value of " + ConvToStr(num) + " is not between "
-			+ ConvToStr(min) + " and " + ConvToStr(max) + "; value set to " + ConvToStr(def) + ".";
-		ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, message);
+		tag->LogMalformed(key, ConvToStr(num), ConvToStr(def), "not between " + ConvToStr(min) + " and " + ConvToStr(max));
 		num = def;
 	}
 }
 
-long ConfigTag::getInt(const std::string &key, long def, long min, long max)
+intmax_t ConfigTag::getSInt(const std::string& key, intmax_t def, intmax_t min, intmax_t max) const
 {
 	std::string result;
 	if(!readString(key, result) || result.empty())
 		return def;
 
 	const char* res_cstr = result.c_str();
-	char* res_tail = NULL;
-	long res = strtol(res_cstr, &res_tail, 0);
+	char* res_tail = nullptr;
+	intmax_t res = strtoimax(res_cstr, &res_tail, 0);
 	if (res_tail == res_cstr)
 		return def;
 
-	CheckMagnitude(tag, key, result, res, def, res_tail);
-	CheckRange(tag, key, res, def, min, max);
+	CheckMagnitude(this, key, result, res, def, res_tail);
+	CheckRange(this, key, res, def, min, max);
 	return res;
 }
 
-unsigned long ConfigTag::getUInt(const std::string& key, unsigned long def, unsigned long min, unsigned long max)
+uintmax_t ConfigTag::getUInt(const std::string& key, uintmax_t def, uintmax_t min, uintmax_t max) const
 {
 	std::string result;
 	if (!readString(key, result) || result.empty())
 		return def;
 
 	const char* res_cstr = result.c_str();
-	char* res_tail = NULL;
-	unsigned long res = strtoul(res_cstr, &res_tail, 0);
+	char* res_tail = nullptr;
+	uintmax_t res = strtoumax(res_cstr, &res_tail, 0);
 	if (res_tail == res_cstr)
 		return def;
 
-	CheckMagnitude(tag, key, result, res, def, res_tail);
-	CheckRange(tag, key, res, def, min, max);
+	CheckMagnitude(this, key, result, res, def, res_tail);
+	CheckRange(this, key, res, def, min, max);
 	return res;
 }
 
-unsigned long ConfigTag::getDuration(const std::string& key, unsigned long def, unsigned long min, unsigned long max)
+unsigned long ConfigTag::getDuration(const std::string& key, unsigned long def, unsigned long min, unsigned long max) const
 {
 	std::string duration;
 	if (!readString(key, duration) || duration.empty())
 		return def;
 
 	unsigned long ret;
-	if (!InspIRCd::Duration(duration, ret))
+	if (!Duration::TryFrom(duration, ret))
 	{
-		ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, "Value of <" + tag + ":" + key + "> at " + getTagLocation() +
-			" is not a duration; value set to " + ConvToStr(def) + ".");
+		LogMalformed(key, duration, ConvToStr(def), "is not a duration");
 		return def;
 	}
 
-	CheckRange(tag, key, ret, def, min, max);
+	CheckRange(this, key, ret, def, min, max);
 	return ret;
 }
 
-double ConfigTag::getFloat(const std::string& key, double def, double min, double max)
+long double ConfigTag::getFloat(const std::string& key, long double def, long double min, long double max) const
 {
 	std::string result;
 	if (!readString(key, result))
 		return def;
 
-	double res = strtod(result.c_str(), NULL);
-	CheckRange(tag, key, res, def, min, max);
+	long double res = strtold(result.c_str(), nullptr);
+	CheckRange(this, key, res, def, min, max);
 	return res;
 }
 
-bool ConfigTag::getBool(const std::string &key, bool def)
+bool ConfigTag::getBool(const std::string& key, bool def) const
 {
 	std::string result;
 	if(!readString(key, result) || result.empty())
 		return def;
 
-	if (stdalgo::string::equalsci(result, "yes") || stdalgo::string::equalsci(result, "true") || stdalgo::string::equalsci(result, "on") || result == "1")
+	if (insp::equalsci(result, "yes") || insp::equalsci(result, "true") || insp::equalsci(result, "on"))
 		return true;
 
-	if (stdalgo::string::equalsci(result, "no") || stdalgo::string::equalsci(result, "false") || stdalgo::string::equalsci(result, "off") || result == "0")
+	if (insp::equalsci(result, "no") || insp::equalsci(result, "false") || insp::equalsci(result, "off"))
 		return false;
 
-	ServerInstance->Logs->Log("CONFIG", LOG_DEFAULT, "Value of <" + tag + ":" + key + "> at " + getTagLocation() +
-		" is not valid, ignoring");
+	LogMalformed(key, result, def ? "yes" : "no", "is not a boolean");
 	return def;
 }
 
-unsigned char ConfigTag::getCharacter(const std::string &key, unsigned char def)
+unsigned char ConfigTag::getCharacter(const std::string& key, unsigned char def, bool emptynul) const
 {
 	std::string result;
-	if (!readString(key, result) || result.size() != 1)
+	if (!readString(key, result))
 		return def;
 
+	if (result.size() != 1)
+		return result.empty() && emptynul ? 0 : def;
+
 	return result[0];
 }
 
-
-std::string ConfigTag::getTagLocation()
-{
-	return src_name + ":" + ConvToStr(src_line);
-}
-
-ConfigTag* ConfigTag::create(const std::string& Tag, const std::string& file, int line, ConfigItems*& Items)
-{
-	ConfigTag* rv = new ConfigTag(Tag, file, line);
-	Items = &rv->items;
-	return rv;
-}
-
-ConfigTag::ConfigTag(const std::string& Tag, const std::string& file, int line)
-	: tag(Tag), src_name(file), src_line(line)
-{
-}
-
-OperInfo::OperInfo(const std::string& Name)
+ConfigTag::ConfigTag(const std::string& Name, const FilePosition& Source)
 	: name(Name)
+	, source(Source)
 {
 }
-
-std::string OperInfo::getConfig(const std::string& key)
-{
-	std::string rv;
-	if (type_block)
-		type_block->readString(key, rv);
-	if (oper_block)
-		oper_block->readString(key, rv);
-	return rv;
-}
