@@ -1,11 +1,11 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2018-2020 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2018, 2020-2022 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2014, 2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
+ *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
  *
@@ -26,23 +26,31 @@
 #include "inspircd.h"
 #include "core_info.h"
 
+enum
+{
+	// From RFC 1459.
+	RPL_ADMINME = 256,
+	RPL_ADMINLOC1 = 257,
+	RPL_ADMINLOC2  = 258,
+	RPL_ADMINEMAIL = 259,
+};
+
 CommandAdmin::CommandAdmin(Module* parent)
 	: ServerTargetCommand(parent, "ADMIN")
 {
-	Penalty = 2;
-	syntax = "[<servername>]";
+	penalty = 2000;
+	syntax = { "[<servername>]" };
 }
 
-/** Handle /ADMIN
- */
 CmdResult CommandAdmin::Handle(User* user, const Params& parameters)
 {
-	if (parameters.size() > 0 && !irc::equals(parameters[0], ServerInstance->Config->ServerName))
-		return CMD_SUCCESS;
+	if (!parameters.empty() && !irc::equals(parameters[0], ServerInstance->Config->ServerName))
+		return CmdResult::SUCCESS;
+
 	user->WriteRemoteNumeric(RPL_ADMINME, ServerInstance->Config->GetServerName(), "Administrative info");
-	if (!AdminName.empty())
-		user->WriteRemoteNumeric(RPL_ADMINLOC1, InspIRCd::Format("Name: %s", AdminName.c_str()));
-	user->WriteRemoteNumeric(RPL_ADMINLOC2, InspIRCd::Format("Nickname: %s", AdminNick.c_str()));
-	user->WriteRemoteNumeric(RPL_ADMINEMAIL, InspIRCd::Format("Email: %s", AdminEmail.c_str()));
-	return CMD_SUCCESS;
+	user->WriteRemoteNumeric(RPL_ADMINLOC1, adminname);
+	if (!admindesc.empty())
+	   user->WriteRemoteNumeric(RPL_ADMINLOC2, admindesc);
+	user->WriteRemoteNumeric(RPL_ADMINEMAIL, adminemail);
+	return CmdResult::SUCCESS;
 }
