@@ -1,10 +1,10 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2018-2020, 2022-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2014 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2013, 2018-2019 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
@@ -26,24 +26,33 @@
 #include "inspircd.h"
 #include "core_info.h"
 
-CommandVersion::CommandVersion(Module* parent)
-	: Command(parent, "VERSION", 0, 0)
+CommandVersion::CommandVersion(Module* parent, ISupportManager& isupportmgr)
+	: Command(parent, "VERSION")
+	, isupport(isupportmgr)
+	, operversion(RPL_VERSION)
+	, version(RPL_VERSION)
 {
-	syntax = "[<servername>]";
+	syntax = { "[<servername>]" };
+}
+
+void CommandVersion::BuildNumerics()
+{
+	version.GetParams().clear();
+	version.push(INSPIRCD_BRANCH ".");
+	version.push(ServerInstance->Config->GetServerName());
+	version.push(ServerInstance->Config->CustomVersion);
+
+	operversion.GetParams().clear();
+	operversion.push(INSPIRCD_VERSION ".");
+	operversion.push(ServerInstance->Config->ServerName);
+	operversion.push("[" + ServerInstance->Config->ServerId + "] " + ServerInstance->Config->CustomVersion);
 }
 
 CmdResult CommandVersion::Handle(User* user, const Params& parameters)
 {
-	Numeric::Numeric numeric(RPL_VERSION);
-	irc::tokenstream tokens(ServerInstance->GetVersionString(user->IsOper()));
-	for (std::string token; tokens.GetTrailing(token); )
-		numeric.push(token);
-	user->WriteNumeric(numeric);
-
-	LocalUser *lu = IS_LOCAL(user);
-	if (lu != NULL)
-	{
-		ServerInstance->ISupport.SendTo(lu);
-	}
-	return CMD_SUCCESS;
+	user->WriteNumeric(user->IsOper() ? operversion : version);
+	LocalUser* luser = IS_LOCAL(user);
+	if (luser)
+		isupport.SendTo(luser);
+	return CmdResult::SUCCESS;
 }
