@@ -1,7 +1,7 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2017-2019, 2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2018-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2014-2016, 2018 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
@@ -29,33 +29,33 @@
 static Events::ModuleEventProvider* reloadevprov;
 static ClientProtocol::Serializer* dummyserializer;
 
-class DummySerializer : public ClientProtocol::Serializer
+class DummySerializer final
+	: public ClientProtocol::Serializer
 {
-	bool Parse(LocalUser* user, const std::string& line, ClientProtocol::ParseOutput& parseoutput) CXX11_OVERRIDE
+	bool Parse(LocalUser* user, const std::string& line, ClientProtocol::ParseOutput& parseoutput) override
 	{
 		return false;
 	}
 
-	ClientProtocol::SerializedMessage Serialize(const ClientProtocol::Message& msg, const ClientProtocol::TagSelection& tagwl) const CXX11_OVERRIDE
+	ClientProtocol::SerializedMessage Serialize(const ClientProtocol::Message& msg, const ClientProtocol::TagSelection& tagwl) const override
 	{
-		return ClientProtocol::SerializedMessage();
+		return {};
 	}
 
- public:
+public:
 	DummySerializer(Module* mod)
 		: ClientProtocol::Serializer(mod, "dummy")
 	{
 	}
 };
 
-class CommandReloadmodule : public Command
+class CommandReloadmodule final
+	: public Command
 {
 	Events::ModuleEventProvider evprov;
 	DummySerializer dummyser;
 
- public:
-	/** Constructor for reloadmodule.
-	 */
+public:
 	CommandReloadmodule(Module* parent)
 		: Command(parent, "RELOADMODULE", 1)
 		, evprov(parent, "event/reloadmodule")
@@ -63,26 +63,21 @@ class CommandReloadmodule : public Command
 	{
 		reloadevprov = &evprov;
 		dummyserializer = &dummyser;
-		flags_needed = 'o';
-		syntax = "<modulename>";
+		access_needed = CmdAccess::OPERATOR;
+		syntax = { "<modulename>" };
 	}
 
-	/** Handle command.
-	 * @param parameters The parameters to the command
-	 * @param user The user issuing the command
-	 * @return A value from CmdResult to indicate command success or failure.
-	 */
-	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE;
+	CmdResult Handle(User* user, const Params& parameters) override;
 };
 
 namespace ReloadModule
 {
 
-class DataKeeper
+class DataKeeper final
 {
 	/** Data we save for each mode and extension provided by the module
 	 */
-	struct ProviderInfo
+	struct ProviderInfo final
 	{
 		std::string itemname;
 		union
@@ -111,7 +106,7 @@ class DataKeeper
 		}
 	};
 
-	struct InstanceData
+	struct InstanceData final
 	{
 		/** Position of the ModeHandler or ExtensionItem that the serialized data belongs to
 		 */
@@ -147,7 +142,8 @@ class DataKeeper
 		}
 	};
 
-	struct OwnedModesExts : public ModesExts
+	struct OwnedModesExts
+		: public ModesExts
 	{
 		/** User uuid or channel name
 		 */
@@ -160,7 +156,8 @@ class DataKeeper
 	};
 
 	// Data saved for each channel
-	struct ChanData : public OwnedModesExts
+	struct ChanData final
+		: public OwnedModesExts
 	{
 		/** Type of data stored for each member who has any affected modes or extensions set
 		 */
@@ -177,9 +174,10 @@ class DataKeeper
 	};
 
 	// Data saved for each user
-	struct UserData : public OwnedModesExts
+	struct UserData final
+		: public OwnedModesExts
 	{
-		static const size_t UNUSED_INDEX = (size_t)-1;
+		static constexpr size_t UNUSED_INDEX = SIZE_MAX;
 		size_t serializerindex;
 
 		UserData(User* user, size_t serializeridx)
@@ -295,7 +293,7 @@ class DataKeeper
 	 */
 	void VerifyServiceProvider(const ProviderInfo& service, const char* type);
 
- public:
+public:
 	/** Save module state
 	 * @param currmod Module whose data to save
 	 */
@@ -315,17 +313,14 @@ void DataKeeper::DoSaveUsers()
 {
 	ModesExts currdata;
 
-	const user_hash& users = ServerInstance->Users->GetUsers();
-	for (user_hash::const_iterator i = users.begin(); i != users.end(); ++i)
+	for (const auto& [_, user] : ServerInstance->Users.GetUsers())
 	{
-		User* const user = i->second;
-
 		// Serialize user modes
 		for (size_t j = 0; j < handledmodes[MODETYPE_USER].size(); j++)
 		{
 			ModeHandler* mh = handledmodes[MODETYPE_USER][j].mh;
 			if (user->IsModeSet(mh))
-				currdata.modelist.push_back(InstanceData(j, mh->GetUserParameter(user)));
+				currdata.modelist.emplace_back(j, mh->GetUserParameter(user));
 		}
 
 		// Serialize all extensions attached to the User
@@ -338,7 +333,7 @@ void DataKeeper::DoSaveUsers()
 		// have to do anything with this user when restoring
 		if ((!currdata.empty()) || (serializerindex != UserData::UNUSED_INDEX))
 		{
-			userdatalist.push_back(UserData(user, serializerindex));
+			userdatalist.emplace_back(user, serializerindex);
 			userdatalist.back().swap(currdata);
 		}
 	}
@@ -352,7 +347,7 @@ size_t DataKeeper::GetSerializerIndex(ClientProtocol::Serializer* serializer)
 			return i;
 	}
 
-	handledserializers.push_back(ProviderInfo(serializer));
+	handledserializers.emplace_back(serializer);
 	return handledserializers.size()-1;
 }
 
@@ -375,9 +370,9 @@ void DataKeeper::SaveExtensions(Extensible* extensible, std::vector<InstanceData
 
 	// Position of the extension saved in the handledexts list
 	size_t index = 0;
-	for (std::vector<ProviderInfo>::const_iterator i = handledexts.begin(); i != handledexts.end(); ++i, index++)
+	for (const auto& prov : handledexts)
 	{
-		ExtensionItem* const item = i->extitem;
+		ExtensionItem* const item = prov.extitem;
 		Extensible::ExtensibleStore::const_iterator it = setexts.find(item);
 		if (it == setexts.end())
 			continue;
@@ -385,7 +380,7 @@ void DataKeeper::SaveExtensions(Extensible* extensible, std::vector<InstanceData
 		std::string value = item->ToInternal(extensible, it->second);
 		// If the serialized value is empty the extension won't be saved and restored
 		if (!value.empty())
-			extdata.push_back(InstanceData(index, value));
+			extdata.emplace_back(index, value);
 	}
 }
 
@@ -395,11 +390,8 @@ void DataKeeper::SaveListModes(Channel* chan, ListModeBase* lm, size_t index, Mo
 	if (!list)
 		return;
 
-	for (ListModeBase::ModeList::const_iterator i = list->begin(); i != list->end(); ++i)
-	{
-		const ListModeBase::ListItem& listitem = *i;
-		currdata.modelist.push_back(InstanceData(index, listitem.mask));
-	}
+	for (const auto& listitem : *list)
+		currdata.modelist.emplace_back(index, listitem.mask);
 }
 
 void DataKeeper::DoSaveChans()
@@ -407,11 +399,8 @@ void DataKeeper::DoSaveChans()
 	ModesExts currdata;
 	std::vector<OwnedModesExts> currmemberdata;
 
-	const chan_hash& chans = ServerInstance->GetChans();
-	for (chan_hash::const_iterator i = chans.begin(); i != chans.end(); ++i)
+	for (const auto& [_, chan] : ServerInstance->Channels.GetChans())
 	{
-		Channel* const chan = i->second;
-
 		// Serialize channel modes
 		for (size_t j = 0; j < handledmodes[MODETYPE_CHANNEL].size(); j++)
 		{
@@ -420,7 +409,7 @@ void DataKeeper::DoSaveChans()
 			if (lm)
 				SaveListModes(chan, lm, j, currdata);
 			else if (chan->IsModeSet(mh))
-				currdata.modelist.push_back(InstanceData(j, chan->GetModeParameter(mh)));
+				currdata.modelist.emplace_back(j, chan->GetModeParameter(mh));
 		}
 
 		// Serialize all extensions attached to the Channel
@@ -432,7 +421,7 @@ void DataKeeper::DoSaveChans()
 		// Same logic as in DoSaveUsers() plus we consider the modes and extensions of all members
 		if ((!currdata.empty()) || (!currmemberdata.empty()))
 		{
-			chandatalist.push_back(ChanData(chan));
+			chandatalist.emplace_back(chan);
 			chandatalist.back().swap(currdata);
 			chandatalist.back().memberdatalist.swap(currmemberdata);
 		}
@@ -442,17 +431,14 @@ void DataKeeper::DoSaveChans()
 void DataKeeper::SaveMemberData(Channel* chan, std::vector<OwnedModesExts>& memberdatalist)
 {
 	ModesExts currdata;
-	const Channel::MemberMap& users = chan->GetUsers();
-	for (Channel::MemberMap::const_iterator i = users.begin(); i != users.end(); ++i)
+	for (const auto& [_, memb] : chan->GetUsers())
 	{
-		Membership* const memb = i->second;
-
 		for (size_t j = 0; j < handledmodes[MODETYPE_CHANNEL].size(); j++)
 		{
 			ModeHandler* mh = handledmodes[MODETYPE_CHANNEL][j].mh;
 			const PrefixMode* const pm = mh->IsPrefixMode();
 			if ((pm) && (memb->HasMode(pm)))
-				currdata.modelist.push_back(InstanceData(j, memb->user->uuid)); // Need to pass the user's uuid to the mode parser to set the mode later
+				currdata.modelist.emplace_back(j, memb->user->uuid); // Need to pass the user's uuid to the mode parser to set the mode later
 		}
 
 		SaveExtensions(memb, currdata.extlist);
@@ -460,7 +446,7 @@ void DataKeeper::SaveMemberData(Channel* chan, std::vector<OwnedModesExts>& memb
 		// Same logic as in DoSaveUsers()
 		if (!currdata.empty())
 		{
-			memberdatalist.push_back(OwnedModesExts(memb->user->uuid));
+			memberdatalist.emplace_back(memb->user->uuid);
 			memberdatalist.back().swap(currdata);
 		}
 	}
@@ -468,20 +454,19 @@ void DataKeeper::SaveMemberData(Channel* chan, std::vector<OwnedModesExts>& memb
 
 void DataKeeper::RestoreMemberData(Channel* chan, const std::vector<ChanData::MemberData>& memberdatalist, Modes::ChangeList& modechange)
 {
-	for (std::vector<ChanData::MemberData>::const_iterator i = memberdatalist.begin(); i != memberdatalist.end(); ++i)
+	for (const auto& md : memberdatalist)
 	{
-		const ChanData::MemberData& md = *i;
-		User* const user = ServerInstance->FindUUID(md.owner);
+		User* const user = ServerInstance->Users.FindUUID(md.owner);
 		if (!user)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "User %s is gone (while processing %s)", md.owner.c_str(), chan->name.c_str());
+			ServerInstance->Logs.Debug(MODNAME, "User {} is gone (while processing {})", md.owner, chan->name);
 			continue;
 		}
 
 		Membership* const memb = chan->GetUser(user);
 		if (!memb)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Member %s is no longer on channel %s", md.owner.c_str(), chan->name.c_str());
+			ServerInstance->Logs.Debug(MODNAME, "Member {} is no longer on channel {}", md.owner, chan->name);
 			continue;
 		}
 
@@ -491,12 +476,10 @@ void DataKeeper::RestoreMemberData(Channel* chan, const std::vector<ChanData::Me
 
 void DataKeeper::CreateModeList(ModeType modetype)
 {
-	const ModeParser::ModeHandlerMap& modes = ServerInstance->Modes->GetModes(modetype);
-	for (ModeParser::ModeHandlerMap::const_iterator i = modes.begin(); i != modes.end(); ++i)
+	for (const auto& [_, mh] : ServerInstance->Modes.GetModes(modetype))
 	{
-		ModeHandler* mh = i->second;
 		if (mh->creator == mod)
-			handledmodes[modetype].push_back(ProviderInfo(mh));
+			handledmodes[modetype].emplace_back(mh);
 	}
 }
 
@@ -504,12 +487,10 @@ void DataKeeper::Save(Module* currmod)
 {
 	this->mod = currmod;
 
-	const ExtensionManager::ExtMap& allexts = ServerInstance->Extensions.GetExts();
-	for (ExtensionManager::ExtMap::const_iterator i = allexts.begin(); i != allexts.end(); ++i)
+	for (const auto& [_, ext] : ServerInstance->Extensions.GetExts())
 	{
-		ExtensionItem* ext = i->second;
 		if (ext->creator == mod)
-			handledexts.push_back(ProviderInfo(ext));
+			handledexts.emplace_back(ext);
 	}
 
 	CreateModeList(MODETYPE_USER);
@@ -518,36 +499,33 @@ void DataKeeper::Save(Module* currmod)
 	CreateModeList(MODETYPE_CHANNEL);
 	DoSaveChans();
 
-	FOREACH_MOD_CUSTOM(*reloadevprov, ReloadModule::EventListener, OnReloadModuleSave, (mod, this->moddata));
+	reloadevprov->Call(&ReloadModule::EventListener::OnReloadModuleSave, mod, this->moddata);
 
-	ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Saved data about %lu users %lu chans %lu modules", (unsigned long)userdatalist.size(), (unsigned long)chandatalist.size(), (unsigned long)moddata.list.size());
+	ServerInstance->Logs.Debug(MODNAME, "Saved data about {} users {} chans {} modules", userdatalist.size(), chandatalist.size(), moddata.list.size());
 }
 
 void DataKeeper::VerifyServiceProvider(const ProviderInfo& service, const char* type)
 {
 	const ServiceProvider* sp = service.extitem;
 	if (!sp)
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "%s \"%s\" is no longer available", type, service.itemname.c_str());
+		ServerInstance->Logs.Debug(MODNAME, "{} \"{}\" is no longer available", type, service.itemname);
 	else if (sp->creator != mod)
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "%s \"%s\" is now handled by %s", type, service.itemname.c_str(), (sp->creator ? sp->creator->ModuleSourceFile.c_str() : "<core>"));
+		ServerInstance->Logs.Debug(MODNAME, "{} \"{}\" is now handled by {}", type, service.itemname, (sp->creator ? sp->creator->ModuleFile : "<core>"));
 }
 
 void DataKeeper::LinkModes(ModeType modetype)
 {
-	std::vector<ProviderInfo>& list = handledmodes[modetype];
-	for (std::vector<ProviderInfo>::iterator i = list.begin(); i != list.end(); ++i)
+	for (auto& item : handledmodes[modetype])
 	{
-		ProviderInfo& item = *i;
-		item.mh = ServerInstance->Modes->FindMode(item.itemname, modetype);
+		item.mh = ServerInstance->Modes.FindMode(item.itemname, modetype);
 		VerifyServiceProvider(item, (modetype == MODETYPE_USER ? "User mode" : "Channel mode"));
 	}
 }
 
 void DataKeeper::LinkExtensions()
 {
-	for (std::vector<ProviderInfo>::iterator i = handledexts.begin(); i != handledexts.end(); ++i)
+	for (auto& item : handledexts)
 	{
-		ProviderInfo& item = *i;
 		item.extitem = ServerInstance->Extensions.GetItem(item.itemname);
 		VerifyServiceProvider(item.extitem, "Extension");
 	}
@@ -555,9 +533,8 @@ void DataKeeper::LinkExtensions()
 
 void DataKeeper::LinkSerializers()
 {
-	for (std::vector<ProviderInfo>::iterator i = handledserializers.begin(); i != handledserializers.end(); ++i)
+	for (auto& item : handledserializers)
 	{
-		ProviderInfo& item = *i;
 		item.serializer = ServerInstance->Modules.FindDataService<ClientProtocol::Serializer>(item.itemname);
 		VerifyServiceProvider(item.serializer, "Serializer");
 	}
@@ -578,14 +555,14 @@ void DataKeeper::Restore(Module* newmod)
 	DoRestoreChans();
 	DoRestoreModules();
 
-	ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Restore finished");
+	ServerInstance->Logs.Debug(MODNAME, "Restore finished");
 }
 
 void DataKeeper::Fail()
 {
-	this->mod = NULL;
+	this->mod = nullptr;
 
-	ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Restore failed, notifying modules");
+	ServerInstance->Logs.Debug(MODNAME, "Restore failed, notifying modules");
 	DoRestoreModules();
 }
 
@@ -597,20 +574,14 @@ void DataKeeper::RestoreObj(const OwnedModesExts& data, Extensible* extensible,
 
 void DataKeeper::RestoreExtensions(const std::vector<InstanceData>& list, Extensible* extensible)
 {
-	for (std::vector<InstanceData>::const_iterator i = list.begin(); i != list.end(); ++i)
-	{
-		const InstanceData& id = *i;
+	for (const auto& id : list)
 		handledexts[id.index].extitem->FromInternal(extensible, id.serialized);
-	}
 }
 
 void DataKeeper::RestoreModes(const std::vector<InstanceData>& list, ModeType modetype, Modes::ChangeList& modechange)
 {
-	for (std::vector<InstanceData>::const_iterator i = list.begin(); i != list.end(); ++i)
-	{
-		const InstanceData& id = *i;
+	for (const auto& id : list)
 		modechange.push_add(handledmodes[modetype][id.index].mh, id.serialized);
-	}
 }
 
 bool DataKeeper::RestoreSerializer(size_t serializerindex, User* user)
@@ -639,16 +610,15 @@ bool DataKeeper::RestoreSerializer(size_t serializerindex, User* user)
 
 void DataKeeper::DoRestoreUsers()
 {
-	ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Restoring user data");
+	ServerInstance->Logs.Debug(MODNAME, "Restoring user data");
 	Modes::ChangeList modechange;
 
-	for (std::vector<UserData>::const_iterator i = userdatalist.begin(); i != userdatalist.end(); ++i)
+	for (const auto& userdata : userdatalist)
 	{
-		const UserData& userdata = *i;
-		User* const user = ServerInstance->FindUUID(userdata.owner);
+		User* const user = ServerInstance->Users.FindUUID(userdata.owner);
 		if (!user)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "User %s is gone", userdata.owner.c_str());
+			ServerInstance->Logs.Debug(MODNAME, "User {} is gone", userdata.owner);
 			continue;
 		}
 
@@ -657,57 +627,56 @@ void DataKeeper::DoRestoreUsers()
 			continue;
 
 		RestoreObj(userdata, user, MODETYPE_USER, modechange);
-		ServerInstance->Modes.Process(ServerInstance->FakeClient, NULL, user, modechange, ModeParser::MODE_LOCALONLY);
+		ServerInstance->Modes.Process(ServerInstance->FakeClient, nullptr, user, modechange, ModeParser::MODE_LOCALONLY);
 		modechange.clear();
 	}
 }
 
 void DataKeeper::DoRestoreChans()
 {
-	ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Restoring channel data");
+	ServerInstance->Logs.Debug(MODNAME, "Restoring channel data");
 	Modes::ChangeList modechange;
 
-	for (std::vector<ChanData>::const_iterator i = chandatalist.begin(); i != chandatalist.end(); ++i)
+	for (const auto& chandata : chandatalist)
 	{
-		const ChanData& chandata = *i;
-		Channel* const chan = ServerInstance->FindChan(chandata.owner);
+		Channel* const chan = ServerInstance->Channels.Find(chandata.owner);
 		if (!chan)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Channel %s not found", chandata.owner.c_str());
+			ServerInstance->Logs.Debug(MODNAME, "Channel {} not found", chandata.owner);
 			continue;
 		}
 
 		RestoreObj(chandata, chan, MODETYPE_CHANNEL, modechange);
 		// Process the mode change before applying any prefix modes
-		ServerInstance->Modes.Process(ServerInstance->FakeClient, chan, NULL, modechange, ModeParser::MODE_LOCALONLY);
+		ServerInstance->Modes.Process(ServerInstance->FakeClient, chan, nullptr, modechange, ModeParser::MODE_LOCALONLY);
 		modechange.clear();
 
 		// Restore all member data
 		RestoreMemberData(chan, chandata.memberdatalist, modechange);
-		ServerInstance->Modes.Process(ServerInstance->FakeClient, chan, NULL, modechange, ModeParser::MODE_LOCALONLY);
+		ServerInstance->Modes.Process(ServerInstance->FakeClient, chan, nullptr, modechange, ModeParser::MODE_LOCALONLY);
 		modechange.clear();
 	}
 }
 
 void DataKeeper::DoRestoreModules()
 {
-	for (ReloadModule::CustomData::List::iterator i = moddata.list.begin(); i != moddata.list.end(); ++i)
+	for (const auto& data : moddata.list)
 	{
-		ReloadModule::CustomData::Data& data = *i;
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Calling module data handler %p", (void*)data.handler);
+		ServerInstance->Logs.Debug(MODNAME, "Calling module data handler {}", fmt::ptr(data.handler));
 		data.handler->OnReloadModuleRestore(mod, data.data);
 	}
 }
 
 } // namespace ReloadModule
 
-class ReloadAction : public ActionBase
+class ReloadAction final
+	: public ActionBase
 {
 	Module* const mod;
 	const std::string uuid;
 	const std::string passedname;
 
- public:
+public:
 	ReloadAction(Module* m, const std::string& uid, const std::string& passedmodname)
 		: mod(m)
 		, uuid(uid)
@@ -715,35 +684,35 @@ class ReloadAction : public ActionBase
 	{
 	}
 
-	void Call() CXX11_OVERRIDE
+	void Call() override
 	{
 		ReloadModule::DataKeeper datakeeper;
 		datakeeper.Save(mod);
 
-		std::string name = mod->ModuleSourceFile;
-		ServerInstance->Modules->DoSafeUnload(mod);
+		std::string name = mod->ModuleFile;
+		ServerInstance->Modules.DoSafeUnload(mod);
 		ServerInstance->GlobalCulls.Apply();
-		bool result = ServerInstance->Modules->Load(name);
+		bool result = ServerInstance->Modules.Load(name);
 
 		if (result)
 		{
-			Module* newmod = ServerInstance->Modules->Find(name);
+			Module* newmod = ServerInstance->Modules.Find(name);
 			datakeeper.Restore(newmod);
-			ServerInstance->SNO->WriteGlobalSno('a', "The %s module was reloaded.", passedname.c_str());
+			ServerInstance->SNO.WriteGlobalSno('a', "The {} module was reloaded.", passedname);
 		}
 		else
 		{
 			datakeeper.Fail();
-			ServerInstance->SNO->WriteGlobalSno('a', "Failed to reload the %s module.", passedname.c_str());
+			ServerInstance->SNO.WriteGlobalSno('a', "Failed to reload the {} module.", passedname);
 		}
 
-		User* user = ServerInstance->FindUUID(uuid);
+		auto* user = ServerInstance->Users.FindUUID(uuid);
 		if (user)
 		{
 			if (result)
-				user->WriteNumeric(RPL_LOADEDMODULE, passedname, InspIRCd::Format("The %s module was reloaded.", passedname.c_str()));
+				user->WriteNumeric(RPL_LOADEDMODULE, passedname, INSP_FORMAT("The {} module was reloaded.", passedname));
 			else
-				user->WriteNumeric(ERR_CANTUNLOADMODULE, passedname, InspIRCd::Format("Failed to reload the %s module.", passedname.c_str()));
+				user->WriteNumeric(ERR_CANTUNLOADMODULE, passedname, INSP_FORMAT("Failed to reload the {} module.", passedname));
 		}
 
 		ServerInstance->GlobalCulls.AddItem(this);
@@ -752,43 +721,40 @@ class ReloadAction : public ActionBase
 
 CmdResult CommandReloadmodule::Handle(User* user, const Params& parameters)
 {
-	Module* m = ServerInstance->Modules->Find(parameters[0]);
+	Module* m = ServerInstance->Modules.Find(parameters[0]);
 	if (m == creator)
 	{
 		user->WriteNumeric(ERR_CANTUNLOADMODULE, parameters[0], "You cannot reload core_reloadmodule (unload and load it)");
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 
 	if (creator->dying)
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 
 	if ((m) && (ServerInstance->Modules.CanUnload(m)))
 	{
 		ServerInstance->AtomicActions.AddAction(new ReloadAction(m, user->uuid, parameters[0]));
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 	}
 	else
 	{
 		user->WriteNumeric(ERR_CANTUNLOADMODULE, parameters[0], "Could not find a loaded module by that name");
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 }
 
-class CoreModReloadmodule : public Module
+class CoreModReloadmodule final
+	: public Module
 {
- private:
+private:
 	CommandReloadmodule cmd;
 
- public:
+public:
 	CoreModReloadmodule()
-		: cmd(this)
+		: Module(VF_CORE | VF_VENDOR, "Provides the RELOADMODULE command")
+		, cmd(this)
 	{
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Provides the RELOADMODULE command", VF_CORE | VF_VENDOR);
-	}
 };
 
 MODULE_INIT(CoreModReloadmodule)
