@@ -1,14 +1,13 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2017 B00mX0r <b00mx0r@aureus.pw>
  *   Copyright (C) 2014, 2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2013, 2016, 2018, 2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2016, 2018, 2020-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2006-2007 Craig Edwards <brain@inspircd.org>
+ *   Copyright (C) 2007 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -30,9 +29,10 @@
 CommandNick::CommandNick(Module* parent)
 	: SplitCommand(parent, "NICK", 1)
 {
+	allow_empty_last_param = true;
+	penalty = 0;
+	syntax = { "<newnick>" };
 	works_before_reg = true;
-	syntax = "<newnick>";
-	Penalty = 0;
 }
 
 /** Handle nick changes from users.
@@ -45,13 +45,13 @@ CmdResult CommandNick::HandleLocal(LocalUser* user, const Params& parameters)
 	std::string newnick = parameters[0];
 
 	// anything except the initial NICK gets a flood penalty
-	if (user->registered == REG_ALL)
+	if (user->IsFullyConnected())
 		user->CommandFloodPenalty += 4000;
 
 	if (newnick.empty())
 	{
 		user->WriteNumeric(ERR_NONICKNAMEGIVEN, "No nickname given");
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 
 	if (newnick == "0")
@@ -61,41 +61,40 @@ CmdResult CommandNick::HandleLocal(LocalUser* user, const Params& parameters)
 	else if (!ServerInstance->IsNick(newnick))
 	{
 		user->WriteNumeric(ERR_ERRONEUSNICKNAME, newnick, "Erroneous Nickname");
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 
-	ModResult MOD_RESULT;
-	FIRST_MOD_RESULT(OnUserPreNick, MOD_RESULT, (user, newnick));
+	ModResult modres;
+	FIRST_MOD_RESULT(OnUserPreNick, modres, (user, newnick));
 
 	// If a module denied the change, abort now
-	if (MOD_RESULT == MOD_RES_DENY)
-		return CMD_FAILURE;
+	if (modres == MOD_RES_DENY)
+		return CmdResult::FAILURE;
 
 	// Disallow the nick change if <security:restrictbannedusers> is on and there is a ban matching this user in
 	// one of the channels they are on
 	if (ServerInstance->Config->RestrictBannedUsers != ServerConfig::BUT_NORMAL)
 	{
-		for (User::ChanList::iterator i = user->chans.begin(); i != user->chans.end(); ++i)
+		for (const auto* memb : user->chans)
 		{
-			Channel* chan = (*i)->chan;
-			if (chan->GetPrefixValue(user) < VOICE_VALUE && chan->IsBanned(user))
+			if (memb->chan->GetPrefixValue(user) < VOICE_VALUE && memb->chan->IsBanned(user))
 			{
 				if (ServerInstance->Config->RestrictBannedUsers == ServerConfig::BUT_RESTRICT_NOTIFY)
-					user->WriteNumeric(ERR_CANTCHANGENICK, InspIRCd::Format("Cannot change nickname while on %s (you're banned)",
-						chan->name.c_str()));
-				return CMD_FAILURE;
+					user->WriteNumeric(ERR_CANTCHANGENICK, INSP_FORMAT("Cannot change nickname while on {} (you're banned)",
+						memb->chan->name));
+				return CmdResult::FAILURE;
 			}
 		}
 	}
 
 	if (!user->ChangeNick(newnick))
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 
-	if (user->registered < REG_NICKUSER)
+	if (user->connected < User::CONN_NICKUSER)
 	{
-		user->registered = (user->registered | REG_NICK);
+		user->connected |= User::CONN_NICK;
 		return CommandUser::CheckRegister(user);
 	}
 
-	return CMD_SUCCESS;
+	return CmdResult::SUCCESS;
 }
