@@ -1,13 +1,13 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2017-2018 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2013-2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2017-2018, 2020, 2022-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2014-2015 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2006-2007 Craig Edwards <brain@inspircd.org>
+ *   Copyright (C) 2007 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -35,34 +35,33 @@ enum
 CommandUser::CommandUser(Module* parent)
 	: SplitCommand(parent, "USER", 4, 4)
 {
-	allow_empty_last_param = false;
+	penalty = 0;
+	syntax = { "<username> <unused> <unused> :<realname>" };
 	works_before_reg = true;
-	Penalty = 0;
-	syntax = "<username> <unused> <unused> :<realname>";
 }
 
 CmdResult CommandUser::HandleLocal(LocalUser* user, const Params& parameters)
 {
 	/* A user may only send the USER command once */
-	if (!(user->registered & REG_USER))
+	if (!(user->connected & User::CONN_USER))
 	{
-		if (!ServerInstance->IsIdent(parameters[0]))
+		if (!ServerInstance->IsUser(parameters[0]))
 		{
 			user->WriteNumeric(ERR_INVALIDUSERNAME, name, "Your username is not valid");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 		else
 		{
-			user->ChangeIdent(parameters[0]);
+			user->ChangeRealUser(parameters[0], true);
 			user->ChangeRealName(parameters[3]);
-			user->registered = (user->registered | REG_USER);
+			user->connected |= User::CONN_USER;
 		}
 	}
 	else
 	{
-		user->WriteNumeric(ERR_ALREADYREGISTERED, "You may not reregister");
+		user->WriteNumeric(ERR_ALREADYREGISTERED, "You may not resend the USER command");
 		user->CommandFloodPenalty += 1000;
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 
 	/* parameters 2 and 3 are local and remote hosts, and are ignored */
@@ -71,15 +70,16 @@ CmdResult CommandUser::HandleLocal(LocalUser* user, const Params& parameters)
 
 CmdResult CommandUser::CheckRegister(LocalUser* user)
 {
-	// If the user is registered, return CMD_SUCCESS/CMD_FAILURE depending on what modules say, otherwise just
-	// return CMD_SUCCESS without doing anything, knowing the other handler will call us again
-	if (user->registered == REG_NICKUSER)
+	// If the user is fully connected, return CmdResult::SUCCESS/CmdResult::FAILURE depending on
+	// what modules say, otherwise just return CmdResult::SUCCESS without doing anything, knowing
+	// the other handler will call us again
+	if (user->connected == User::CONN_NICKUSER)
 	{
-		ModResult MOD_RESULT;
-		FIRST_MOD_RESULT(OnUserRegister, MOD_RESULT, (user));
-		if (MOD_RESULT == MOD_RES_DENY)
-			return CMD_FAILURE;
+		ModResult modres;
+		FIRST_MOD_RESULT(OnUserRegister, modres, (user));
+		if (modres == MOD_RES_DENY)
+			return CmdResult::FAILURE;
 	}
 
-	return CMD_SUCCESS;
+	return CmdResult::SUCCESS;
 }
