@@ -1,16 +1,17 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2024 Glen Miner <shaggie76@gmail.com>
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2018 Adam <Adam@anope.org>
- *   Copyright (C) 2017-2019, 2021-2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2017-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013-2014, 2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2009 John Brooks <special@inspircd.org>
+ *   Copyright (C) 2009 John Brooks <john@jbrooks.io>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
+ *   Copyright (C) 2007-2008 Craig Edwards <brain@inspircd.org>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2006-2008 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -28,7 +29,9 @@
 
 #include "inspircd.h"
 #include "modules/account.h"
+#include "modules/isupport.h"
 #include "modules/who.h"
+#include "timeutils.h"
 
 enum
 {
@@ -40,17 +43,18 @@ enum
 	RPL_WHOSPCRPL = 354
 };
 
-static const char whox_field_order[] = "tcuihsnfdlaor";
-static const char who_field_order[] = "cuhsnf";
+static constexpr char whox_field_order[] = "tcuihsnfdlaor";
+static constexpr char who_field_order[] = "cuhsnf";
 
-struct WhoData : public Who::Request
+struct WhoData final
+	: public Who::Request
 {
-	bool GetFieldIndex(char flag, size_t& out) const CXX11_OVERRIDE
+	bool GetFieldIndex(char flag, size_t& out) const override
 	{
 		if (!whox)
 		{
 			const char* pos = strchr(who_field_order, flag);
-			if (pos == NULL)
+			if (!pos)
 				return false;
 
 			out = pos - who_field_order;
@@ -111,9 +115,11 @@ struct WhoData : public Who::Request
 	}
 };
 
-class CommandWho : public SplitCommand
+class CommandWho final
+	: public SplitCommand
 {
- private:
+private:
+	Account::API accountapi;
 	ChanModeReference secretmode;
 	ChanModeReference privatemode;
 	UserModeReference invisiblemode;
@@ -124,28 +130,22 @@ class CommandWho : public SplitCommand
 	void BuildOpLevels()
 	{
 		// Build a map of prefixes ordered descending by their rank.
-		typedef std::multimap<unsigned int, const PrefixMode*, std::greater<unsigned int> > RankMap;
-		RankMap ranks;
-		const ModeParser::PrefixModeList& modes = ServerInstance->Modes.GetPrefixModes();
-		for (ModeParser::PrefixModeList::const_iterator iter = modes.begin(); iter != modes.end(); ++iter)
-		{
-			const PrefixMode* pm = *iter;
+		std::multimap<ModeHandler::Rank, const PrefixMode*, std::greater<>> ranks;
+		for (const auto* pm : ServerInstance->Modes.GetPrefixModes())
 			ranks.insert(std::make_pair(pm->GetPrefixRank(), pm));
-		}
 
 		// Now we have the ranks ordered we can assign them levels.
-		unsigned int lastrank = 0;
+		ModeHandler::Rank lastrank = 0;
 		unsigned int oplevel = 0;
-		for (RankMap::const_iterator iter = ranks.begin(); iter != ranks.end(); ++iter)
+		for (const auto& [_, pm] : ranks)
 		{
-			const PrefixMode* pm = iter->second;
-			if (iter != ranks.begin() && pm->GetPrefixRank() != lastrank)
+			if (lastrank && pm->GetPrefixRank() != lastrank)
 				oplevel++; // Keep the same op level for modes with the same prefix rank.
 
 			lastrank = pm->GetPrefixRank();
 			oplevels[pm->GetModeChar()] = ConvToStr(oplevel);
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Assigned oplevel %u to the %c (%s) prefix mode.",
-				oplevel, pm->GetModeChar(), pm->name.c_str());
+			ServerInstance->Logs.Debug(MODNAME, "Assigned oplevel {} to the {} ({}) prefix mode.",
+				oplevel, pm->GetModeChar(), pm->name);
 		}
 	}
 
@@ -167,13 +167,10 @@ class CommandWho : public SplitCommand
 	/** Gets the first channel which is visible between the source and the target users. */
 	Membership* GetFirstVisibleChannel(const WhoData& data, LocalUser* source, User* user)
 	{
-		for (User::ChanList::iterator iter = user->chans.begin(); iter != user->chans.end(); ++iter)
+		for (auto* memb : user->chans)
 		{
-			Membership* memb = *iter;
-
 			// Let a module handle this first if it wants to.
-			ModResult res;
-			FIRST_MOD_RESULT_CUSTOM(whovisibleevprov, Who::VisibleEventListener, OnWhoVisible, res, (data, source, memb));
+			ModResult res = whovisibleevprov.FirstResult(&Who::VisibleEventListener::OnWhoVisible, data, source, memb);
 			if (res == MOD_RES_ALLOW)
 				return memb; // Module explicitly picked this chan.
 
@@ -187,17 +184,17 @@ class CommandWho : public SplitCommand
 			if (source == user || (!memb->chan->IsModeSet(secretmode) && !memb->chan->IsModeSet(privatemode)) || memb->chan->HasUser(source))
 				return memb;
 		}
-		return NULL;
+		return nullptr;
 	}
 
 	/** Determines whether WHO flags match a specific channel user. */
-	bool MatchChannel(LocalUser* source, Membership* memb, WhoData& data);
+	static bool MatchChannel(LocalUser* source, Membership* memb, WhoData& data);
 
 	/** Determines whether WHO flags match a specific user. */
 	bool MatchUser(LocalUser* source, User* target, WhoData& data);
 
 	/** Performs a WHO request on a channel. */
-	void WhoChannel(LocalUser* source, const std::vector<std::string>& parameters, Channel* c, WhoData& data);
+	void WhoChannel(LocalUser* source, const std::vector<std::string>& parameters, WhoData& data);
 
 	/** Template for getting a user from various types of collection. */
 	template<typename T>
@@ -207,11 +204,12 @@ class CommandWho : public SplitCommand
 	template<typename T>
 	void WhoUsers(LocalUser* source, const std::vector<std::string>& parameters, const T& users, WhoData& data);
 
- public:
+public:
 	insp::flat_map<char, std::string> oplevels;
 
 	CommandWho(Module* parent)
 		: SplitCommand(parent, "WHO", 1, 3)
+		, accountapi(parent)
 		, secretmode(parent, "secret")
 		, privatemode(parent, "private")
 		, invisiblemode(parent, "invisible")
@@ -219,18 +217,20 @@ class CommandWho : public SplitCommand
 		, whomatchevprov(parent, "event/who-match")
 		, whovisibleevprov(parent, "event/who-visible")
 	{
-		allow_empty_last_param = false;
-		syntax = "<server>|<nick>|<channel>|<realname>|<host>|0 [[Aafhilmnoprstux][%acdfhilnorstu] <server>|<nick>|<channel>|<realname>|<host>|0]";
+		syntax = {
+			"<server>|<nick>|<channel>|<realname>|<host>|0 [Aafhilmnoprstux]%[acdfhilnorstu]",
+			"<unused> [Aafhilmnoprstux]%[acdfhilnorstu] :<server>|<nick>|<channel>|<realname>|<host>|0]"
+		};
 	}
 
 	/** Sends a WHO reply to a user. */
 	void SendWhoLine(LocalUser* user, const std::vector<std::string>& parameters, Membership* memb, User* u, WhoData& data);
 
-	CmdResult HandleLocal(LocalUser* user, const Params& parameters) CXX11_OVERRIDE;
+	CmdResult HandleLocal(LocalUser* user, const Params& parameters) override;
 };
 
 template<> User* CommandWho::GetUser(UserManager::OperList::const_iterator& t) { return *t; }
-template<> User* CommandWho::GetUser(user_hash::const_iterator& t) { return t->second; }
+template<> User* CommandWho::GetUser(UserMap::const_iterator& t) { return t->second; }
 
 bool CommandWho::MatchChannel(LocalUser* source, Membership* memb, WhoData& data)
 {
@@ -260,8 +260,8 @@ bool CommandWho::MatchChannel(LocalUser* source, Membership* memb, WhoData& data
 
 bool CommandWho::MatchUser(LocalUser* source, User* user, WhoData& data)
 {
-	// Users who are not fully registered can never match.
-	if (user->registered != REG_ALL)
+	// Users who are not fully connected can never match.
+	if (!user->IsFullyConnected())
 		return false;
 
 	bool source_has_users_auspex = source->HasPrivPermission("users/auspex");
@@ -282,8 +282,7 @@ bool CommandWho::MatchUser(LocalUser* source, User* user, WhoData& data)
 		return false;
 
 	// Let a module handle this first if it wants to.
-	ModResult res;
-	FIRST_MOD_RESULT_CUSTOM(whomatchevprov, Who::MatchEventListener, OnWhoMatch, res, (data, source, user));
+	ModResult res = whomatchevprov.FirstResult(&Who::MatchEventListener::OnWhoMatch, data, source, user);
 	if (res == MOD_RES_ALLOW)
 		return true; // Module explicitly matched.
 
@@ -293,13 +292,12 @@ bool CommandWho::MatchUser(LocalUser* source, User* user, WhoData& data)
 	// The source wants to match against users' away messages.
 	bool match = false;
 	if (data.flags['A'])
-		match = user->IsAway() && InspIRCd::Match(user->awaymsg, data.matchtext, ascii_case_insensitive_map);
+		match = user->IsAway() && InspIRCd::Match(user->away->message, data.matchtext, ascii_case_insensitive_map);
 
 	// The source wants to match against users' account names.
 	else if (data.flags['a'])
 	{
-		const AccountExtItem* accountext = GetAccountExtItem();
-		const std::string* account = accountext ? accountext->get(user) : NULL;
+		const std::string* account = accountapi ? accountapi->GetAccountName(user) : nullptr;
 		match = account && InspIRCd::Match(*account, data.matchtext);
 	}
 
@@ -312,7 +310,7 @@ bool CommandWho::MatchUser(LocalUser* source, User* user, WhoData& data)
 
 	// The source wants to match against users' IP addresses.
 	else if (data.flags['i'])
-		match = source_can_see_target && InspIRCd::MatchCIDR(user->GetIPString(), data.matchtext, ascii_case_insensitive_map);
+		match = source_can_see_target && InspIRCd::MatchCIDR(user->GetAddress(), data.matchtext, ascii_case_insensitive_map);
 
 	// The source wants to match against users' modes.
 	else if (data.flags['m'])
@@ -320,9 +318,8 @@ bool CommandWho::MatchUser(LocalUser* source, User* user, WhoData& data)
 		if (source_can_see_target)
 		{
 			bool set = true;
-			for (std::string::const_iterator iter = data.matchtext.begin(); iter != data.matchtext.end(); ++iter)
+			for (const auto chr : data.matchtext)
 			{
-				unsigned char chr = static_cast<unsigned char>(*iter);
 				switch (chr)
 				{
 					// The following user modes should be set.
@@ -357,10 +354,11 @@ bool CommandWho::MatchUser(LocalUser* source, User* user, WhoData& data)
 		if (source_can_see_target && lu)
 		{
 			irc::portparser portrange(data.matchtext, false);
-			long port;
-			while ((port = portrange.GetToken()))
+			while (long port = portrange.GetToken())
 			{
-				if (port == lu->server_sa.port())
+				if (port >= std::numeric_limits<in_port_t>::min()
+					&& port <= std::numeric_limits<in_port_t>::max()
+					&& static_cast<in_port_t>(port) == lu->server_sa.port())
 				{
 					match = true;
 					break;
@@ -383,21 +381,24 @@ bool CommandWho::MatchUser(LocalUser* source, User* user, WhoData& data)
 	// The source wants to match against users' connection times.
 	else if (data.flags['t'])
 	{
-		time_t seconds = ServerInstance->Time() - InspIRCd::Duration(data.matchtext);
+		time_t seconds = ServerInstance->Time() - Duration::From(data.matchtext);
 		if (user->signon >= seconds)
 			match = true;
 	}
 
-	// The source wants to match against users' idents.
+	// The source wants to match against users' usernames.
 	else if (data.flags['u'])
-		match = InspIRCd::Match(user->ident, data.matchtext, ascii_case_insensitive_map);
+	{
+		const std::string username = user->GetUser(source_can_see_target && data.flags['x']);
+		match = InspIRCd::Match(username, data.matchtext, ascii_case_insensitive_map);
+	}
 
 	// The <name> passed to WHO is matched against users' host, server,
 	// real name and nickname if the channel <name> cannot be found.
 	else
 	{
-		const std::string host = user->GetHost(source_can_see_target && data.flags['x']);
-		match = InspIRCd::Match(host, data.matchtext, ascii_case_insensitive_map);
+		const std::string hostname = user->GetHost(source_can_see_target && data.flags['x']);
+		match = InspIRCd::Match(hostname, data.matchtext, ascii_case_insensitive_map);
 
 		if (!match)
 		{
@@ -416,18 +417,14 @@ bool CommandWho::MatchUser(LocalUser* source, User* user, WhoData& data)
 	return match;
 }
 
-void CommandWho::WhoChannel(LocalUser* source, const std::vector<std::string>& parameters, Channel* chan, WhoData& data)
+void CommandWho::WhoChannel(LocalUser* source, const std::vector<std::string>& parameters, WhoData& data)
 {
-	if (!CanView(chan, source))
+	if (!CanView(data.matchchan, source))
 		return;
 
-	bool inside = chan->HasUser(source);
-	const Channel::MemberMap& users = chan->GetUsers();
-	for (Channel::MemberMap::const_iterator iter = users.begin(); iter != users.end(); ++iter)
+	bool inside = data.matchchan->HasUser(source);
+	for (const auto& [user, memb] : data.matchchan->GetUsers())
 	{
-		User* user = iter->first;
-		Membership* memb = iter->second;
-
 		// Only show invisible users if the source is in the channel or has the users/auspex priv.
 		if (!inside && user->IsModeSet(invisiblemode) && !source->HasPrivPermission("users/auspex"))
 			continue;
@@ -449,7 +446,7 @@ void CommandWho::WhoUsers(LocalUser* source, const std::vector<std::string>& par
 		User* user = GetUser(iter);
 
 		// Only show users in response to a fuzzy WHO if we can see them normally.
-		bool can_see_normally = user == source || source->SharesChannelWith(user) || !user->IsModeSet(invisiblemode);
+		bool can_see_normally = user == source || !user->IsModeSet(invisiblemode) || source->SharesChannelWith(user);
 		if (data.fuzzy_match && !can_see_normally && !source_has_users_auspex)
 			continue;
 
@@ -457,7 +454,7 @@ void CommandWho::WhoUsers(LocalUser* source, const std::vector<std::string>& par
 		if (!MatchUser(source, user, data))
 			continue;
 
-		SendWhoLine(source, parameters, NULL, user, data);
+		SendWhoLine(source, parameters, nullptr, user, data);
 	}
 }
 
@@ -480,13 +477,13 @@ void CommandWho::SendWhoLine(LocalUser* source, const std::vector<std::string>&
 		if (data.whox_fields['c'])
 			wholine.push(memb ? memb->chan->name : "*");
 
-		// Include the user's ident.
+		// Include the user's username.
 		if (data.whox_fields['u'])
-			wholine.push(user->ident);
+			wholine.push(user->GetUser(source_can_see_target && data.flags['x']));
 
 		// Include the user's IP address.
 		if (data.whox_fields['i'])
-			wholine.push(source_can_see_target ? user->GetIPString() : "255.255.255.255");
+			wholine.push(source_can_see_target ? user->GetAddress() : "255.255.255.255");
 
 		// Include the user's hostname.
 		if (data.whox_fields['h'])
@@ -541,8 +538,7 @@ void CommandWho::SendWhoLine(LocalUser* source, const std::vector<std::string>&
 		// Include the user's account name.
 		if (data.whox_fields['a'])
 		{
-			const AccountExtItem* accountext = GetAccountExtItem();
-			const std::string* account = accountext ? accountext->get(user) : NULL;
+			const std::string* account = accountapi ? accountapi->GetAccountName(user) : nullptr;
 			wholine.push(account ? *account : "0");
 		}
 
@@ -554,7 +550,7 @@ void CommandWho::SendWhoLine(LocalUser* source, const std::vector<std::string>&
 			if (oplevels.empty())
 				BuildOpLevels();
 
-			wholine.push(memb && !memb->modes.empty() ? oplevels[memb->modes[0]] : "n/a");
+			wholine.push(memb && memb->GetModeChar() ? oplevels[memb->GetModeChar()] : "n/a");
 		}
 
 		// Include the user's real name.
@@ -568,8 +564,8 @@ void CommandWho::SendWhoLine(LocalUser* source, const std::vector<std::string>&
 		// Include the channel name.
 		wholine.push(memb ? memb->chan->name : "*");
 
-		// Include the user's ident.
-		wholine.push(user->ident);
+		// Include the user's username.
+		wholine.push(user->GetUser(source_can_see_target && data.flags['x']));
 
 		// Include the user's hostname.
 		wholine.push(user->GetHost(source_can_see_target && data.flags['x']));
@@ -608,8 +604,7 @@ void CommandWho::SendWhoLine(LocalUser* source, const std::vector<std::string>&
 		wholine.GetParams().back().append(user->GetRealName());
 	}
 
-	ModResult res;
-	FIRST_MOD_RESULT_CUSTOM(whoevprov, Who::EventListener, OnWhoLine, res, (data, source, user, memb, wholine));
+	ModResult res = whoevprov.FirstResult(&Who::EventListener::OnWhoLine, data, source, user, memb, wholine);
 	if (res != MOD_RES_DENY)
 		data.results.push_back(wholine);
 }
@@ -619,60 +614,59 @@ CmdResult CommandWho::HandleLocal(LocalUser* user, const Params& parameters)
 	WhoData data(parameters);
 
 	// Is the source running a WHO on a channel?
-	data.matchchan = ServerInstance->FindChan(data.matchtext);
+	data.matchchan = ServerInstance->Channels.Find(data.matchtext);
 	if (data.matchchan)
-		WhoChannel(user, parameters, data.matchchan, data);
+		WhoChannel(user, parameters, data);
 
 	// If we only want to match against opers we only have to iterate the oper list.
 	else if (data.flags['o'])
-		WhoUsers(user, parameters, ServerInstance->Users->all_opers, data);
+		WhoUsers(user, parameters, ServerInstance->Users.all_opers, data);
 
 	// Otherwise we have to use the global user list.
 	else
-		WhoUsers(user, parameters, ServerInstance->Users->GetUsers(), data);
+		WhoUsers(user, parameters, ServerInstance->Users.GetUsers(), data);
 
 	// Send the results to the source.
-	for (std::vector<Numeric::Numeric>::const_iterator n = data.results.begin(); n != data.results.end(); ++n)
-		user->WriteNumeric(*n);
+	for (const auto& numeric : data.results)
+		user->WriteNumeric(numeric);
 	user->WriteNumeric(RPL_ENDOFWHO, (data.matchtext.empty() ? "*" : data.matchtext.c_str()), "End of /WHO list.");
 
 	// Penalize the source a bit for large queries with one unit of penalty per 200 results.
 	user->CommandFloodPenalty += data.results.size() * 5;
-	return CMD_SUCCESS;
+	return CmdResult::SUCCESS;
 }
 
-class CoreModWho : public Module
+class CoreModWho final
+	: public Module
+	, public ISupport::EventListener
 {
- private:
+private:
 	CommandWho cmd;
 
- public:
+public:
 	CoreModWho()
-		: cmd(this)
+		: Module(VF_CORE | VF_VENDOR, "Provides the WHO command")
+		, ISupport::EventListener(this)
+		, cmd(this)
 	{
 	}
 
-	void On005Numeric(std::map<std::string, std::string>& tokens) CXX11_OVERRIDE
+	void OnBuildISupport(ISupport::TokenMap& tokens) override
 	{
 		tokens["WHOX"];
 	}
 
-	void OnServiceAdd(ServiceProvider& provider) CXX11_OVERRIDE
+	void OnServiceAdd(ServiceProvider& provider) override
 	{
 		// If the service is a prefix mode we need to rebuild the oplevel map.
 		if (provider.service == SERVICE_MODE && static_cast<ModeHandler&>(provider).IsPrefixMode())
 			cmd.oplevels.clear();
 	}
 
-	void OnServiceDel(ServiceProvider& provider) CXX11_OVERRIDE
+	void OnServiceDel(ServiceProvider& provider) override
 	{
 		this->OnServiceAdd(provider);
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Provides the WHO command", VF_VENDOR|VF_CORE);
-	}
 };
 
 MODULE_INIT(CoreModWho)
