@@ -2,10 +2,10 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2022 Val Lorentz <progval+git@progval.net>
- *   Copyright (C) 2013, 2017-2018, 2020-2022 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012-2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2017-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2012, 2014-2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2006-2007 Dennis Friis <peavey@inspircd.org>
@@ -27,30 +27,32 @@
 #include "inspircd.h"
 #include "modules/stats.h"
 #include "modules/whois.h"
+#include "timeutils.h"
 
 enum
 {
 	// From RFC 1459.
 	RPL_WHOWASUSER = 314,
 	RPL_ENDOFWHOWAS = 369,
-
-	// InspIRCd-specific.
-	RPL_WHOWASIP = 652
+	ERR_WASNOSUCHNICK = 406,
 };
 
 namespace WhoWas
 {
 	/** One entry for a nick. There may be multiple entries for a nick. */
-	struct Entry
+	struct Entry final
 	{
-		/** Real host */
+		/** Real hostname */
 		const std::string host;
 
-		/** Displayed host */
+		/** Displayed hostname */
 		const std::string dhost;
 
-		/** Ident */
-		const std::string ident;
+		/** Real username */
+		const std::string user;
+
+		/** Displayed username */
+		const std::string duser;
 
 		/** Server name */
 		const std::string server;
@@ -58,15 +60,25 @@ namespace WhoWas
 		/** Real name */
 		const std::string real;
 
+		/** IP address or UNIX socket path. */
+		const std::string address;
+
 		/** Signon time */
 		const time_t signon;
 
 		/** Initialize this Entry with a user */
 		Entry(User* user);
+
+		/** Retrieves the real hostname of the user. */
+		const auto& GetHost() const { return host.empty() ? dhost : host; }
+
+		/** Retrieves the real username of the user. */
+		const auto& GetUser() const { return user.empty() ? duser : user; }
 	};
 
 	/** Everything known about one nick */
-	struct Nick : public insp::intrusive_list_node<Nick>
+	struct Nick final
+		: public insp::intrusive_list_node<Nick>
 	{
 		/** A group of users related by nickname */
 		typedef std::deque<Entry*> List;
@@ -87,10 +99,10 @@ namespace WhoWas
 		~Nick();
 	};
 
-	class Manager
+	class Manager final
 	{
-	 public:
-		struct Stats
+	public:
+		struct Stats final
 		{
 			/** Number of currently existing WhoWas::Entry objects */
 			size_t entrycount;
@@ -99,7 +111,7 @@ namespace WhoWas
 		/** Add a user to the whowas database. Called when a user quits.
 		 * @param user The user to add to the database
 		 */
-		void Add(User* user);
+		void Add(User* user, const std::string& nickname);
 
 		/** Retrieves statistics about the whowas database
 		 * @return Whowas statistics as a WhoWas::Manager::Stats struct
@@ -116,7 +128,7 @@ namespace WhoWas
 		 * @param NewMaxGroups Maximum number of entries per nick
 		 * @param NewMaxKeep Seconds how long each nick should be kept
 		 */
-		void UpdateConfig(unsigned int NewGroupSize, unsigned int NewMaxGroups, unsigned int NewMaxKeep);
+		void UpdateConfig(unsigned int NewGroupSize, unsigned int NewMaxGroups, unsigned long NewMaxKeep);
 
 		/** Retrieves all data known about a given nick
 		 * @param nick Nickname to find, case insensitive (IRC casemapping)
@@ -129,18 +141,15 @@ namespace WhoWas
 		 */
 		bool IsEnabled() const;
 
-		/** Constructor */
-		Manager();
-
 		/** Destructor */
 		~Manager();
 
-	 private:
+	private:
 		/** Order in which the users were added into the map, used to remove oldest nick */
 		typedef insp::intrusive_list_tail<Nick> FIFO;
 
 		/** Sets of users in the whowas system */
-		typedef TR1NS::unordered_map<std::string, WhoWas::Nick*, irc::insensitive, irc::StrHashComp> whowas_users;
+		typedef std::unordered_map<std::string, WhoWas::Nick*, irc::insensitive, irc::StrHashComp> whowas_users;
 
 		/** Primary container, links nicknames tracked by WHOWAS to a list of records */
 		whowas_users whowas;
@@ -149,15 +158,15 @@ namespace WhoWas
 		FIFO whowas_fifo;
 
 		/** Max number of WhoWas entries per user. */
-		unsigned int GroupSize;
+		unsigned int GroupSize = 0;
 
 		/** Max number of cumulative user-entries in WhoWas.
 		 * When max reached and added to, push out oldest entry FIFO style.
 		 */
-		unsigned int MaxGroups;
+		unsigned int MaxGroups = 0;
 
 		/** Max seconds a user is kept in WhoWas before being pruned. */
-		unsigned int MaxKeep;
+		unsigned long MaxKeep = 0;
 
 		/** Shrink all data structures to honor the current settings */
 		void Prune();
@@ -174,21 +183,23 @@ namespace WhoWas
 	};
 }
 
-class CommandWhowas : public Command
+class CommandWhowas final
+	: public Command
 {
-  public:
+public:
 	// Manager handling all whowas database related tasks
 	WhoWas::Manager manager;
 
 	CommandWhowas(Module* parent);
-	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE;
+	CmdResult Handle(User* user, const Params& parameters) override;
 };
 
 CommandWhowas::CommandWhowas( Module* parent)
 	: Command(parent, "WHOWAS", 1)
 {
-	syntax = "<nick> [<count>]";
-	Penalty = 2;
+	allow_empty_last_param = true;
+	penalty = 2000;
+	syntax = { "<nick> [<count>]" };
 }
 
 CmdResult CommandWhowas::Handle(User* user, const Params& parameters)
@@ -197,13 +208,13 @@ CmdResult CommandWhowas::Handle(User* user, const Params& parameters)
 	if (!manager.IsEnabled())
 	{
 		user->WriteNumeric(ERR_UNKNOWNCOMMAND, name, "This command has been disabled.");
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 
 	if (parameters[0].empty())
 	{
 		user->WriteNumeric(ERR_NONICKNAMEGIVEN, "No nickname given");
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 
 	const WhoWas::Nick* const nick = manager.FindNick(parameters[0]);
@@ -213,74 +224,63 @@ CmdResult CommandWhowas::Handle(User* user, const Params& parameters)
 	}
 	else
 	{
-		const WhoWas::Nick::List& list = nick->entries;
-		WhoWas::Nick::List::const_reverse_iterator last = list.rend();
+		WhoWas::Nick::List::const_reverse_iterator last = nick->entries.rend();
 		if (parameters.size() > 1)
 		{
 			size_t count = ConvToNum<size_t>(parameters[1]);
-			if (count > 0 && (size_t) std::distance(list.rbegin(), last) > count)
-				last = list.rbegin() + count;
+			if (count > 0 && (size_t) std::distance(nick->entries.rbegin(), last) > count)
+				last = nick->entries.rbegin() + count;
 		}
 
-		for (WhoWas::Nick::List::const_reverse_iterator i = list.rbegin(); i != last; ++i)
+		for (const auto* u : insp::iterator_range(nick->entries.rbegin(), last))
 		{
-			WhoWas::Entry* u = *i;
-
-			user->WriteNumeric(RPL_WHOWASUSER, parameters[0], u->ident, u->dhost, '*', u->real);
+			user->WriteNumeric(RPL_WHOWASUSER, parameters[0], u->duser, u->dhost, '*', u->real);
 
 			if (user->HasPrivPermission("users/auspex"))
-				user->WriteNumeric(RPL_WHOWASIP, parameters[0], InspIRCd::Format("was connecting from *@%s", u->host.c_str()));
+				user->WriteNumeric(RPL_WHOISACTUALLY, parameters[0], INSP_FORMAT("{}@{}", u->GetUser(), u->GetHost()), u->address, "was connecting from");
 
-			std::string signon = InspIRCd::TimeString(u->signon);
+			const std::string signon = Time::ToString(u->signon);
 			bool hide_server = (!ServerInstance->Config->HideServer.empty() && !user->HasPrivPermission("servers/auspex"));
 			user->WriteNumeric(RPL_WHOISSERVER, parameters[0], (hide_server ? ServerInstance->Config->HideServer : u->server), signon);
 		}
 	}
 
 	user->WriteNumeric(RPL_ENDOFWHOWAS, parameters[0], "End of WHOWAS");
-	return CMD_SUCCESS;
-}
-
-WhoWas::Manager::Manager()
-	: GroupSize(0), MaxGroups(0), MaxKeep(0)
-{
+	return CmdResult::SUCCESS;
 }
 
 const WhoWas::Nick* WhoWas::Manager::FindNick(const std::string& nickname) const
 {
 	whowas_users::const_iterator it = whowas.find(nickname);
 	if (it == whowas.end())
-		return NULL;
+		return nullptr;
 	return it->second;
 }
 
 WhoWas::Manager::Stats WhoWas::Manager::GetStats() const
 {
 	size_t entrycount = 0;
-	for (whowas_users::const_iterator i = whowas.begin(); i != whowas.end(); ++i)
-	{
-		WhoWas::Nick::List& list = i->second->entries;
-		entrycount += list.size();
-	}
+	for (const auto& [_, nick] : whowas)
+		entrycount += nick->entries.size();
 
 	Stats stats;
 	stats.entrycount = entrycount;
 	return stats;
 }
 
-void WhoWas::Manager::Add(User* user)
+void WhoWas::Manager::Add(User* user, const std::string& nickname)
 {
 	if (!IsEnabled())
 		return;
 
 	// Insert nick if it doesn't exist
 	// 'first' will point to the newly inserted element or to the existing element with an equivalent key
-	std::pair<whowas_users::iterator, bool> ret = whowas.insert(std::make_pair(user->nick, static_cast<WhoWas::Nick*>(NULL)));
+	std::pair<whowas_users::iterator, bool> ret = whowas.emplace(nickname, nullptr);
 
 	if (ret.second) // If inserted
 	{
 		// This nick is new, create a list for it and add the first record to it
-		WhoWas::Nick* nick = new WhoWas::Nick(ret.first->first);
+		auto* nick = new WhoWas::Nick(ret.first->first);
 		nick->entries.push_back(new Entry(user));
 		ret.first->second = nick;
 
@@ -362,11 +362,8 @@ void WhoWas::Manager::Maintain()
 
 WhoWas::Manager::~Manager()
 {
-	for (whowas_users::iterator i = whowas.begin(); i != whowas.end(); ++i)
-	{
-		WhoWas::Nick* nick = i->second;
+	for (const auto& [_, nick] : whowas)
 		delete nick;
-	}
 }
 
 bool WhoWas::Manager::IsEnabled() const
@@ -374,7 +371,7 @@ bool WhoWas::Manager::IsEnabled() const
 	return ((GroupSize != 0) && (MaxGroups != 0));
 }
 
-void WhoWas::Manager::UpdateConfig(unsigned int NewGroupSize, unsigned int NewMaxGroups, unsigned int NewMaxKeep)
+void WhoWas::Manager::UpdateConfig(unsigned int NewGroupSize, unsigned int NewMaxGroups, unsigned long NewMaxKeep)
 {
 	if ((NewGroupSize == GroupSize) && (NewMaxGroups == MaxGroups) && (NewMaxKeep == MaxKeep))
 		return;
@@ -398,19 +395,21 @@ void WhoWas::Manager::PurgeNick(WhoWas::Nick* nick)
 	whowas_users::iterator it = whowas.find(nick->nick);
 	if (it == whowas.end())
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "ERROR: Inconsistency detected in whowas database, please report");
+		ServerInstance->Logs.Debug(MODNAME, "BUG: Inconsistency detected in whowas database, please report");
 		return;
 	}
 	PurgeNick(it);
 }
 
-WhoWas::Entry::Entry(User* user)
-	: host(user->GetRealHost())
-	, dhost(user->GetDisplayedHost())
-	, ident(user->ident)
-	, server(user->server->GetPublicName())
-	, real(user->GetRealName())
-	, signon(user->signon)
+WhoWas::Entry::Entry(User* u)
+	: host(u->GetRealHost() == u->GetDisplayedHost() ? "" : u->GetRealHost())
+	, dhost(u->GetDisplayedHost())
+	, user(u->GetRealUser() == u->GetDisplayedUser() ? "" : u->GetRealUser())
+	, duser(u->GetDisplayedUser())
+	, server(u->server->GetName())
+	, real(u->GetRealName())
+	, address(u->GetAddress())
+	, signon(u->signon)
 {
 }
 
@@ -425,29 +424,41 @@ WhoWas::Nick::~Nick()
 	stdalgo::delete_all(entries);
 }
 
-class ModuleWhoWas : public Module, public Stats::EventListener
+class ModuleWhoWas final
+	: public Module
+	, public Stats::EventListener
 {
+public:
 	CommandWhowas cmd;
+	bool nickupdate;
 
- public:
+public:
 	ModuleWhoWas()
-		: Stats::EventListener(this)
+		: Module(VF_CORE | VF_VENDOR, "Provides the WHOWAS command")
+		, Stats::EventListener(this)
 		, cmd(this)
 	{
 	}
 
-	void OnGarbageCollect() CXX11_OVERRIDE
+	void OnGarbageCollect() override
 	{
 		// Remove all entries older than MaxKeep
 		cmd.manager.Maintain();
 	}
 
-	void OnUserQuit(User* user, const std::string& message, const std::string& oper_message) CXX11_OVERRIDE
+	void OnUserQuit(User* user, const std::string& message, const std::string& oper_message) override
 	{
-		cmd.manager.Add(user);
+		if (user->nick != user->uuid)
+			cmd.manager.Add(user, user->nick);
 	}
 
-	ModResult OnStats(Stats::Context& stats) CXX11_OVERRIDE
+	void OnUserPostNick(User* user, const std::string& oldnick) override
+	{
+		if (nickupdate && oldnick != user->uuid)
+			cmd.manager.Add(user, oldnick);
+	}
+
+	ModResult OnStats(Stats::Context& stats) override
 	{
 		if (stats.GetSymbol() == 'z')
 			stats.AddRow(249, "Whowas entries: "+ConvToStr(cmd.manager.GetStats().entrycount));
@@ -455,19 +466,15 @@ class ModuleWhoWas : public Module, public Stats::EventListener
 		return MOD_RES_PASSTHRU;
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("whowas");
-		unsigned int NewGroupSize = tag->getUInt("groupsize", 10, 0, 10000);
-		unsigned int NewMaxGroups = tag->getUInt("maxgroups", 10240, 0, 1000000);
-		unsigned int NewMaxKeep = tag->getDuration("maxkeep", 3600, 3600);
+		const auto& tag = ServerInstance->Config->ConfValue("whowas");
+		const auto groupsize = tag->getNum<unsigned int>("groupsize", 10);
+		const auto maxgroups = tag->getNum<unsigned int>("maxgroups", 10000);
+		const auto maxkeep = tag->getDuration("maxkeep", 7*24*60*60, 60*60);
+		nickupdate = tag->getBool("nickupdate", true);
 
-		cmd.manager.UpdateConfig(NewGroupSize, NewMaxGroups, NewMaxKeep);
-	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Provides the WHOWAS command", VF_CORE | VF_VENDOR);
+		cmd.manager.UpdateConfig(groupsize, maxgroups, maxkeep);
 	}
 };
 
