@@ -2,11 +2,11 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
- *   Copyright (C) 2018, 2021-2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2018, 2020-2023, 2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
+ *   Copyright (C) 2014 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2012, 2014 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
@@ -33,65 +33,64 @@
 CommandGline::CommandGline(Module* parent)
 	: Command(parent, "GLINE", 1, 3)
 {
-	flags_needed = 'o';
-	syntax = "<user@host>[,<user@host>]+ [<duration> :<reason>]";
+	access_needed = CmdAccess::OPERATOR;
+	syntax = { "<user@host>[,<user@host>]+ [[<duration>] :<reason>]" };
 }
 
-/** Handle /GLINE
- */
 CmdResult CommandGline::Handle(User* user, const Params& parameters)
 {
 	if (CommandParser::LoopCall(user, this, parameters, 0))
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 
 	std::string target = parameters[0];
-	if (parameters.size() >= 3)
+	if (parameters.size() > 1)
 	{
-		IdentHostPair ih;
-		User* find = ServerInstance->FindNick(target);
-		if ((find) && (find->registered == REG_ALL))
+		UserHostPair ih;
+		auto* find = ServerInstance->Users.Find(target, true);
+		if (find)
 		{
-			ih.first = find->GetBanIdent();
-			ih.second = find->GetIPString();
+			ih.first = find->GetBanUser(true);
+			ih.second = find->GetAddress();
 			target = ih.first + "@" + ih.second;
 		}
 		else
-			ih = ServerInstance->XLines->IdentSplit(target);
+			ih = XLineManager::SplitUserHost(target);
 
 		if (ih.first.empty())
 		{
 			user->WriteNotice("*** Target not found.");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
 		InsaneBan::IPHostMatcher matcher;
 		if (InsaneBan::MatchesEveryone(ih.first + "@" + ih.second, matcher, user, 'G', "hostmasks"))
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 
 		else if (target.find('!') != std::string::npos)
 		{
 			user->WriteNotice("*** G-line cannot operate on nick!user@host masks.");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
-		unsigned long duration;
-		if (!InspIRCd::Duration(parameters[1], duration))
+		unsigned long duration = 0;
+		if (parameters.size() > 2 && !Duration::TryFrom(parameters[1], duration))
 		{
 			user->WriteNotice("*** Invalid duration for G-line.");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
-		GLine* gl = new GLine(ServerInstance->Time(), duration, user->nick, parameters[2], ih.first, ih.second);
+
+		auto* gl = new GLine(ServerInstance->Time(), duration, user->nick, parameters.back(), ih.first, ih.second);
 		if (ServerInstance->XLines->AddLine(gl, user))
 		{
 			if (!duration)
 			{
-				ServerInstance->SNO->WriteToSnoMask('x', "%s added a permanent G-line on %s: %s", user->nick.c_str(), target.c_str(), parameters[2].c_str());
+				ServerInstance->SNO.WriteToSnoMask('x', "{} added a permanent G-line on {}: {}",
+					user->nick, target, gl->reason);
 			}
 			else
 			{
-				ServerInstance->SNO->WriteToSnoMask('x', "%s added a timed G-line on %s, expires in %s (on %s): %s",
-					user->nick.c_str(), target.c_str(), InspIRCd::DurationString(duration).c_str(),
-					InspIRCd::TimeString(ServerInstance->Time() + duration).c_str(), parameters[2].c_str());
+				ServerInstance->SNO.WriteToSnoMask('x', "{} added a timed G-line on {}, expires in {} (on {}): {}",
+					user->nick, target, Duration::ToString(duration), Time::FromNow(duration), gl->reason);
 			}
 
 			ServerInstance->XLines->ApplyLines();
@@ -107,9 +106,9 @@ CmdResult CommandGline::Handle(User* user, const Params& parameters)
 	{
 		std::string reason;
 
-		if (ServerInstance->XLines->DelLine(target.c_str(), "G", reason, user))
+		if (ServerInstance->XLines->DelLine(target, "G", reason, user))
 		{
-			ServerInstance->SNO->WriteToSnoMask('x', "%s removed G-line on %s: %s", user->nick.c_str(), target.c_str(), reason.c_str());
+			ServerInstance->SNO.WriteToSnoMask('x', "{} removed G-line on {}: {}", user->nick, target, reason);
 		}
 		else
 		{
@@ -117,5 +116,5 @@ CmdResult CommandGline::Handle(User* user, const Params& parameters)
 		}
 	}
 
-	return CMD_SUCCESS;
+	return CmdResult::SUCCESS;
 }
