@@ -2,11 +2,11 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
+ *   Copyright (C) 2018, 2020, 2022-2023, 2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2017-2018, 2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2014 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2012, 2014 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2009 Matt Smith <dz@inspircd.org>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
@@ -34,34 +34,33 @@
 CommandZline::CommandZline(Module* parent)
 	: Command(parent, "ZLINE", 1, 3)
 {
-	flags_needed = 'o';
-	syntax = "<ipmask>[,<ipmask>]+ [<duration> :<reason>]";
+	access_needed = CmdAccess::OPERATOR;
+	syntax = { "<ipmask>[,<ipmask>]+ [[<duration>] :<reason>]" };
 }
 
 CmdResult CommandZline::Handle(User* user, const Params& parameters)
 {
 	if (CommandParser::LoopCall(user, this, parameters, 0))
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 
 	std::string target = parameters[0];
-	if (parameters.size() >= 3)
+	if (parameters.size() > 1)
 	{
 		if (target.find('!') != std::string::npos)
 		{
 			user->WriteNotice("*** You cannot include a nickname in a Z-line, a Z-line must ban only an IP mask.");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
-		User *u = ServerInstance->FindNick(target);
-
-		if ((u) && (u->registered == REG_ALL))
+		auto* u = ServerInstance->Users.Find(target, true);
+		if (u)
 		{
-			target = u->GetIPString();
+			target = u->GetAddress();
 		}
 
 		const char* ipaddr = target.c_str();
 
-		if (strchr(ipaddr,'@'))
+		if (strchr(ipaddr, '@'))
 		{
 			while (*ipaddr != '@')
 				ipaddr++;
@@ -70,26 +69,27 @@ CmdResult CommandZline::Handle(User* user, const Params& parameters)
 
 		IPMatcher matcher;
 		if (InsaneBan::MatchesEveryone(ipaddr, matcher, user, 'Z', "ipmasks"))
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 
-		unsigned long duration;
-		if (!InspIRCd::Duration(parameters[1], duration))
+		unsigned long duration = 0;
+		if (parameters.size() > 2 && !Duration::TryFrom(parameters[1], duration))
 		{
 			user->WriteNotice("*** Invalid duration for Z-line.");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
-		ZLine* zl = new ZLine(ServerInstance->Time(), duration, user->nick, parameters[2], ipaddr);
-		if (ServerInstance->XLines->AddLine(zl,user))
+
+		auto* zl = new ZLine(ServerInstance->Time(), duration, user->nick, parameters.back(), ipaddr);
+		if (ServerInstance->XLines->AddLine(zl, user))
 		{
 			if (!duration)
 			{
-				ServerInstance->SNO->WriteToSnoMask('x', "%s added a permanent Z-line on %s: %s", user->nick.c_str(), ipaddr, parameters[2].c_str());
+				ServerInstance->SNO.WriteToSnoMask('x', "{} added a permanent Z-line on {}: {}",
+					user->nick, ipaddr, zl->reason);
 			}
 			else
 			{
-				ServerInstance->SNO->WriteToSnoMask('x', "%s added a timed Z-line on %s, expires in %s (on %s): %s",
-					user->nick.c_str(), ipaddr, InspIRCd::DurationString(duration).c_str(),
-					InspIRCd::TimeString(ServerInstance->Time() + duration).c_str(), parameters[2].c_str());
+				ServerInstance->SNO.WriteToSnoMask('x', "{} added a timed Z-line on {}, expires in {} (on {}): {}",
+					user->nick, ipaddr, Duration::ToString(duration), Time::FromNow(duration), zl->reason);
 			}
 			ServerInstance->XLines->ApplyLines();
 		}
@@ -103,21 +103,21 @@ CmdResult CommandZline::Handle(User* user, const Params& parameters)
 	{
 		std::string reason;
 
-		if (ServerInstance->XLines->DelLine(target.c_str(), "Z", reason, user))
+		if (ServerInstance->XLines->DelLine(target, "Z", reason, user))
 		{
-			ServerInstance->SNO->WriteToSnoMask('x', "%s removed Z-line on %s: %s", user->nick.c_str(), target.c_str(), reason.c_str());
+			ServerInstance->SNO.WriteToSnoMask('x', "{} removed Z-line on {}: {}", user->nick, target, reason);
 		}
 		else
 		{
 			user->WriteNotice("*** Z-line " + target + " not found on the list.");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 	}
 
-	return CMD_SUCCESS;
+	return CmdResult::SUCCESS;
 }
 
 bool CommandZline::IPMatcher::Check(User* user, const std::string& ip) const
 {
-	return InspIRCd::MatchCIDR(user->GetIPString(), ip, ascii_case_insensitive_map);
+	return InspIRCd::MatchCIDR(user->GetAddress(), ip, ascii_case_insensitive_map);
 }
