@@ -2,12 +2,12 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2020 Matt Schatz <genius3000@g3k.solutions>
- *   Copyright (C) 2017-2020, 2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2017-2020, 2022-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2014 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012 ChrisTX <xpipe@hotmail.de>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2007 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2003, 2006 Craig Edwards <brain@inspircd.org>
@@ -25,20 +25,17 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
-#include "inspircd.h"
 #ifndef _WIN32
 # include <dlfcn.h>
 #endif
 
-/** The extension that dynamic libraries end with. */
-#define DLL_EXTENSION ".so"
+#include "inspircd.h"
+#include "dynamic.h"
 
 DLLManager::DLLManager(const std::string& name)
-	: lib(NULL)
-	, libname(name)
+	: libname(name)
 {
-	static size_t extlen = strlen(DLL_EXTENSION);
+	const static size_t extlen = strlen(DLL_EXTENSION);
 	if (name.length() <= extlen || name.compare(name.length() - extlen, name.length(), DLL_EXTENSION))
 	{
 		err.assign(name + " is not a module (no " DLL_EXTENSION " extension)");
@@ -70,21 +67,21 @@ DLLManager::~DLLManager()
 Module* DLLManager::CallInit()
 {
 	if (!lib)
-		return NULL;
+		return nullptr;
 
 	const unsigned long* abi = GetSymbol<const unsigned long>(MODULE_STR_ABI);
 	if (!abi)
 	{
 		err.assign(libname + " is not a module (no ABI symbol)");
-		return NULL;
+		return nullptr;
 	}
 	else if (*abi != MODULE_ABI)
 	{
 		const char* version = GetVersion();
-		err.assign(InspIRCd::Format("%s was built against %s (%lu) which is too %s to use with %s (%lu).",
-			libname.c_str(), version ? version : "an unknown version", *abi,
+		err.assign(INSP_FORMAT("{} was built against {} ({}) which is too {} to use with {} ({}).",
+			libname, version ? version : "an unknown version", *abi,
 			*abi < MODULE_ABI ? "old" : "new", INSPIRCD_VERSION, MODULE_ABI));
-		return NULL;
+		return nullptr;
 	}
 
 	union
@@ -97,7 +94,7 @@ Module* DLLManager::CallInit()
 	if (!vptr)
 	{
 		err.assign(libname + " is not a module (no init symbol)");
-		return NULL;
+		return nullptr;
 	}
 
 	return (*fptr)();
@@ -106,7 +103,7 @@ Module* DLLManager::CallInit()
 void* DLLManager::GetSymbol(const char* name) const
 {
 	if (!lib)
-		return NULL;
+		return nullptr;
 
 #if defined _WIN32
 	return GetProcAddress(lib, name);
