@@ -1,14 +1,14 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
+ *   Copyright (C) 2021 Dominic Hamon
+ *   Copyright (C) 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2017 B00mX0r <b00mx0r@aureus.pw>
- *   Copyright (C) 2016-2019, 2021-2022, 2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2016, 2018 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012 Shawn Smith <ShawnSmith0828@gmail.com>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2008 Thomas Stagner <aquanight@inspircd.org>
+ *   Copyright (C) 2008 Thomas Stagner <aquanight@gmail.com>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2006-2008 Craig Edwards <brain@inspircd.org>
@@ -28,30 +28,24 @@
 
 
 #include "inspircd.h"
+#include "clientprotocolevent.h"
+#include "numerichelper.h"
 
 ModeHandler::ModeHandler(Module* Creator, const std::string& Name, char modeletter, ParamSpec Params, ModeType type, Class mclass)
 	: ServiceProvider(Creator, Name, SERVICE_MODE)
 	, modeid(ModeParser::MODEID_MAX)
 	, parameters_taken(Params)
-	, mode(modeletter)
-	, oper(false)
-	, list(false)
+	, mode(ServerInstance->Config->ConfValue("modes")->getCharacter(Name, modeletter))
 	, m_type(type)
 	, type_id(mclass)
-	, ranktoset(HALFOP_VALUE)
-	, ranktounset(HALFOP_VALUE)
 {
 }
 
-CullResult ModeHandler::cull()
+Cullable::Result ModeHandler::Cull()
 {
 	if (ServerInstance)
-		ServerInstance->Modes->DelMode(this);
-	return classbase::cull();
-}
-
-ModeHandler::~ModeHandler()
-{
+		ServerInstance->Modes.DelMode(this);
+	return Cullable::Cull();
 }
 
 bool ModeHandler::NeedsParam(bool adding) const
@@ -73,14 +67,14 @@ std::string ModeHandler::GetUserParameter(const User* user) const
 	return "";
 }
 
-ModResult ModeHandler::AccessCheck(User*, Channel*, std::string &, bool)
+ModResult ModeHandler::AccessCheck(User*, Channel*, Modes::Change&)
 {
 	return MOD_RES_PASSTHRU;
 }
 
-ModeAction ModeHandler::OnModeChange(User*, User*, Channel*, std::string&, bool)
+bool ModeHandler::OnModeChange(User*, User*, Channel*, Modes::Change&)
 {
-	return MODEACTION_DENY;
+	return false;
 }
 
 void ModeHandler::DisplayList(User*, Channel*)
@@ -93,9 +87,9 @@ void ModeHandler::DisplayEmptyList(User*, Channel*)
 
 void ModeHandler::OnParameterMissing(User* user, User* dest, Channel* channel)
 {
-	std::string message = InspIRCd::Format("You must specify a parameter for the %s mode.", name.c_str());
+	std::string message = INSP_FORMAT("You must specify a parameter for the {} mode.", name);
 	if (!syntax.empty())
-		message.append(InspIRCd::Format(" Syntax: %s.", syntax.c_str()));
+		message.append(INSP_FORMAT(" Syntax: {}.", syntax));
 
 	if (channel)
 		user->WriteNumeric(Numerics::InvalidModeParameter(channel, this, "*", message));
@@ -111,7 +105,7 @@ void ModeHandler::OnParameterInvalid(User* user, Channel* targetchannel, User* t
 		user->WriteNumeric(Numerics::InvalidModeParameter(targetuser, this, "*"));
 }
 
-bool ModeHandler::ResolveModeConflict(std::string& theirs, const std::string& ours, Channel*)
+bool ModeHandler::ResolveModeConflict(const std::string& theirs, const std::string& ours, Channel*)
 {
 	return (theirs < ours);
 }
@@ -122,12 +116,12 @@ void ModeHandler::RegisterService()
 	ServerInstance->Modules.AddReferent((GetModeType() == MODETYPE_CHANNEL ? "mode/" : "umode/") + name, this);
 }
 
-ModeAction SimpleUserModeHandler::OnModeChange(User* source, User* dest, Channel* channel, std::string &parameter, bool adding)
+bool SimpleUserMode::OnModeChange(User* source, User* dest, Channel* channel, Modes::Change& change)
 {
 	/* We're either trying to add a mode we already have or
 		remove a mode we don't have, deny. */
-	if (dest->IsModeSet(this) == adding)
-		return MODEACTION_DENY;
+	if (dest->IsModeSet(this) == change.adding)
+		return false;
 
 	/* adding will be either true or false, depending on if we
 		are adding or removing the mode, since we already checked
@@ -135,18 +129,17 @@ ModeAction SimpleUserModeHandler::OnModeChange(User* source, User* dest, Channel
 		aren't removing a mode we don't have, we don't have to do any
 		other checks here to see if it's true or false, just add or
 		remove the mode */
-	dest->SetMode(this, adding);
+	dest->SetMode(this, change.adding);
 
-	return MODEACTION_ALLOW;
+	return true;
 }
 
-
-ModeAction SimpleChannelModeHandler::OnModeChange(User* source, User* dest, Channel* channel, std::string &parameter, bool adding)
+bool SimpleChannelMode::OnModeChange(User* source, User* dest, Channel* channel, Modes::Change& change)
 {
 	/* We're either trying to add a mode we already have or
 		remove a mode we don't have, deny. */
-	if (channel->IsModeSet(this) == adding)
-		return MODEACTION_DENY;
+	if (channel->IsModeSet(this) == change.adding)
+		return false;
 
 	/* adding will be either true or false, depending on if we
 		are adding or removing the mode, since we already checked
@@ -154,74 +147,75 @@ ModeAction SimpleChannelModeHandler::OnModeChange(User* source, User* dest, Chan
 		aren't removing a mode we don't have, we don't have to do any
 		other checks here to see if it's true or false, just add or
 		remove the mode */
-	channel->SetMode(this, adding);
+	channel->SetMode(this, change.adding);
 
-	return MODEACTION_ALLOW;
+	return true;
 }
 
 ModeWatcher::ModeWatcher(Module* Creator, const std::string& modename, ModeType type)
-	: mode(modename), m_type(type), creator(Creator)
+	: mode(modename)
+	, m_type(type)
+	, creator(Creator)
 {
-	ServerInstance->Modes->AddModeWatcher(this);
+	ServerInstance->Modes.AddModeWatcher(this);
 }
 
 ModeWatcher::~ModeWatcher()
 {
-	ServerInstance->Modes->DelModeWatcher(this);
+	ServerInstance->Modes.DelModeWatcher(this);
 }
 
-bool ModeWatcher::BeforeMode(User*, User*, Channel*, std::string&, bool)
+bool ModeWatcher::BeforeMode(User*, User*, Channel*, Modes::Change&)
 {
 	return true;
 }
 
-void ModeWatcher::AfterMode(User*, User*, Channel*, const std::string&, bool)
+void ModeWatcher::AfterMode(User*, User*, Channel*, const Modes::Change&)
 {
 }
 
-PrefixMode::PrefixMode(Module* Creator, const std::string& Name, char ModeLetter, unsigned int Rank, char PrefixChar)
+PrefixMode::PrefixMode(Module* Creator, const std::string& Name, char ModeLetter, Rank PrefixRank, char PrefixChar)
 	: ModeHandler(Creator, Name, ModeLetter, PARAM_ALWAYS, MODETYPE_CHANNEL, MC_PREFIX)
 	, prefix(PrefixChar)
-	, prefixrank(Rank)
-	, selfremove(true)
+	, prefixrank(PrefixRank)
 {
 	list = true;
 	syntax = "<nick>";
 }
 
-ModResult PrefixMode::AccessCheck(User* src, Channel*, std::string& value, bool adding)
+ModResult PrefixMode::AccessCheck(User* src, Channel*, Modes::Change& change)
 {
-	if (!adding && src->nick == value && selfremove)
+	if (!change.adding && src->nick == change.param && selfremove)
 		return MOD_RES_ALLOW;
 	return MOD_RES_PASSTHRU;
 }
 
-ModeAction PrefixMode::OnModeChange(User* source, User*, Channel* chan, std::string& parameter, bool adding)
+bool PrefixMode::OnModeChange(User* source, User*, Channel* chan, Modes::Change& change)
 {
 	User* target;
 	if (IS_LOCAL(source))
-		target = ServerInstance->FindNickOnly(parameter);
+		target = ServerInstance->Users.FindNick(change.param);
 	else
-		target = ServerInstance->FindNick(parameter);
+		target = ServerInstance->Users.Find(change.param);
 
 	if (!target)
 	{
-		source->WriteNumeric(Numerics::NoSuchNick(parameter));
-		return MODEACTION_DENY;
+		source->WriteNumeric(Numerics::NoSuchNick(change.param));
+		return false;
 	}
 
 	Membership* memb = chan->GetUser(target);
 	if (!memb)
 	{
 		source->WriteNumeric(ERR_USERNOTINCHANNEL, target->nick, chan->name, "They are not on that channel");
-		return MODEACTION_DENY;
+		return false;
 	}
 
-	parameter = target->nick;
-	return (memb->SetPrefix(this, adding) ? MODEACTION_ALLOW : MODEACTION_DENY);
+	change.param = target->nick;
+	return memb->SetPrefix(this, change.adding);
 }
 
-void PrefixMode::Update(unsigned int rank, unsigned int setrank, unsigned int unsetrank, bool selfrm)
+void PrefixMode::Update(ModeHandler::Rank rank, ModeHandler::Rank setrank, ModeHandler::Rank unsetrank, bool selfrm)
 {
 	prefixrank = rank;
 	ranktoset = setrank;
@@ -229,150 +223,139 @@ void PrefixMode::Update(unsigned int rank, unsigned int setrank, unsigned int un
 	selfremove = selfrm;
 }
 
-ModeAction ParamModeBase::OnModeChange(User* source, User*, Channel* chan, std::string& parameter, bool adding)
+bool ParamModeBase::OnModeChange(User* source, User*, Channel* chan, Modes::Change& change)
 {
-	if (adding)
+	if (change.adding)
 	{
-		if (chan->GetModeParameter(this) == parameter)
-			return MODEACTION_DENY;
+		if (chan->GetModeParameter(this) == change.param)
+			return false;
 
-		if (OnSet(source, chan, parameter) != MODEACTION_ALLOW)
-			return MODEACTION_DENY;
+		if (!OnSet(source, chan, change.param))
+			return false;
 
 		chan->SetMode(this, true);
 
 		// Handler might have changed the parameter internally
-		parameter.clear();
-		this->GetParameter(chan, parameter);
+		change.param.clear();
+		this->GetParameter(chan, change.param);
 	}
 	else
 	{
 		if (!chan->IsModeSet(this))
-			return MODEACTION_DENY;
+			return false;
 		this->OnUnsetInternal(source, chan);
 		chan->SetMode(this, false);
 	}
-	return MODEACTION_ALLOW;
+	return true;
 }
 
-ModeAction ModeParser::TryMode(User* user, User* targetuser, Channel* chan, Modes::Change& mcitem, bool SkipACL)
+bool ModeParser::TryMode(User* user, User* targetuser, Channel* chan, Modes::Change& mcitem, bool SkipACL)
 {
 	ModeType type = chan ? MODETYPE_CHANNEL : MODETYPE_USER;
 
 	ModeHandler* mh = mcitem.mh;
-	bool adding = mcitem.adding;
-	const bool needs_param = mh->NeedsParam(adding);
+	const bool needs_param = mh->NeedsParam(mcitem.adding);
 
-	std::string& parameter = mcitem.param;
 	// crop mode parameter size to MODE_PARAM_MAX characters
-	if (parameter.length() > MODE_PARAM_MAX && adding)
-		parameter.erase(MODE_PARAM_MAX);
+	if (mcitem.param.length() > MODE_PARAM_MAX && mcitem.adding)
+		mcitem.param.erase(MODE_PARAM_MAX);
 
-	ModResult MOD_RESULT;
-	FIRST_MOD_RESULT(OnRawMode, MOD_RESULT, (user, chan, mh, parameter, adding));
+	ModResult modres;
+	FIRST_MOD_RESULT(OnRawMode, modres, (user, chan, mcitem));
 
-	if (IS_LOCAL(user) && (MOD_RESULT == MOD_RES_DENY))
-		return MODEACTION_DENY;
+	if (IS_LOCAL(user) && (modres == MOD_RES_DENY))
+		return false;
 
 	const char modechar = mh->GetModeChar();
 
-	if (chan && !SkipACL && (MOD_RESULT != MOD_RES_ALLOW))
+	if (chan && !SkipACL && (modres != MOD_RES_ALLOW))
 	{
-		MOD_RESULT = mh->AccessCheck(user, chan, parameter, adding);
+		modres = mh->AccessCheck(user, chan, mcitem);
 
-		if (MOD_RESULT == MOD_RES_DENY)
-			return MODEACTION_DENY;
-		if (MOD_RESULT == MOD_RES_PASSTHRU)
+		if (modres == MOD_RES_DENY)
+			return false;
+		if (modres == MOD_RES_PASSTHRU)
 		{
-			unsigned int neededrank = mh->GetLevelRequired(adding);
-			unsigned int ourrank = chan->GetPrefixValue(user);
+			ModeHandler::Rank neededrank = mh->GetLevelRequired(mcitem.adding);
+			ModeHandler::Rank ourrank = chan->GetPrefixValue(user);
 			if (ourrank < neededrank)
 			{
-				user->WriteNumeric(Numerics::ChannelPrivilegesNeeded(chan, neededrank, InspIRCd::Format("%s channel mode %c (%s)",
-					adding ? "set" : "unset", mh->GetModeChar(), mh->name.c_str())));
-				return MODEACTION_DENY;
+				user->WriteNumeric(Numerics::ChannelPrivilegesNeeded(chan, neededrank, INSP_FORMAT("{} channel mode {} ({})",
+					mcitem.adding ? "set" : "unset", mh->GetModeChar(), mh->name)));
+				return false;
 			}
 		}
 	}
 
 	// Ask mode watchers whether this mode change is OK
-	std::pair<ModeWatcherMap::iterator, ModeWatcherMap::iterator> itpair = modewatchermap.equal_range(mh->name);
-	for (ModeWatcherMap::iterator i = itpair.first; i != itpair.second; ++i)
+	for (const auto& [_, mw] : insp::equal_range(modewatchermap, mh->name))
 	{
-		ModeWatcher* mw = i->second;
 		if (mw->GetModeType() == type)
 		{
-			if (!mw->BeforeMode(user, targetuser, chan, parameter, adding))
-				return MODEACTION_DENY;
+			if (!mw->BeforeMode(user, targetuser, chan, mcitem))
+				return false;
 
 			// A module whacked the parameter completely, and there was one. Abort.
-			if ((needs_param) && (parameter.empty()))
-				return MODEACTION_DENY;
+			if ((needs_param) && (mcitem.param.empty()))
+				return false;
 		}
 	}
 
-	if ((chan || (!chan && adding)) && IS_LOCAL(user) && mh->NeedsOper() && !user->HasModePermission(mh))
+	if ((chan || (!chan && mcitem.adding)) && IS_LOCAL(user) && mh->NeedsOper() && !user->HasModePermission(mh))
 	{
 		/* It's an oper only mode, and they don't have access to it. */
 		if (user->IsOper())
 		{
-			user->WriteNumeric(ERR_NOPRIVILEGES, InspIRCd::Format("Permission Denied - Oper type %s does not have access to %sset %s mode %c",
-					user->oper->name.c_str(), adding ? "" : "un", type == MODETYPE_CHANNEL ? "channel" : "user", modechar));
+			user->WriteNumeric(ERR_NOPRIVILEGES, INSP_FORMAT("Permission Denied - Oper type {} does not have access to {} {} mode {}",
+				user->oper->GetType(), mcitem.adding ? "set" : "unset", type == MODETYPE_CHANNEL ? "channel" : "user", modechar));
 		}
 		else
 		{
-			user->WriteNumeric(ERR_NOPRIVILEGES, InspIRCd::Format("Permission Denied - Only operators may %sset %s mode %c",
-					adding ? "" : "un", type == MODETYPE_CHANNEL ? "channel" : "user", modechar));
+			user->WriteNumeric(ERR_NOPRIVILEGES, INSP_FORMAT("Permission Denied - Only operators may {} {} mode {}",
+				mcitem.adding ? "set" : "unset", type == MODETYPE_CHANNEL ? "channel" : "user", modechar));
 		}
-		return MODEACTION_DENY;
+		return false;
 	}
 
 	/* Call the handler for the mode */
-	ModeAction ma = mh->OnModeChange(user, targetuser, chan, parameter, adding);
+	if (!mh->OnModeChange(user, targetuser, chan, mcitem))
+		return false;
 
-	if ((needs_param) && (parameter.empty()))
-		return MODEACTION_DENY;
+	if ((needs_param) && (mcitem.param.empty()))
+		return false;
 
-	if (ma != MODEACTION_ALLOW)
-		return ma;
-
-	itpair = modewatchermap.equal_range(mh->name);
-	for (ModeWatcherMap::iterator i = itpair.first; i != itpair.second; ++i)
+	for (const auto& [_, mw] : insp::equal_range(modewatchermap, mh->name))
 	{
-		ModeWatcher* mw = i->second;
 		if (mw->GetModeType() == type)
-			mw->AfterMode(user, targetuser, chan, parameter, adding);
+			mw->AfterMode(user, targetuser, chan, mcitem);
 	}
 
-	return MODEACTION_ALLOW;
+	return true;
 }
 
-void ModeParser::ModeParamsToChangeList(User* user, ModeType type, const std::vector<std::string>& parameters, Modes::ChangeList& changelist, unsigned int beginindex, unsigned int endindex)
+void ModeParser::ModeParamsToChangeList(User* user, ModeType type, const std::vector<std::string>& parameters, Modes::ChangeList& changelist, size_t beginindex, size_t endindex)
 {
 	if (endindex > parameters.size())
 		endindex = parameters.size();
 
-	const std::string& mode_sequence = parameters[beginindex];
-
 	bool adding = true;
-	unsigned int param_at = beginindex+1;
+	size_t param_at = beginindex+1;
 
-	for (std::string::const_iterator letter = mode_sequence.begin(); letter != mode_sequence.end(); letter++)
+	for (const auto modechar : parameters[beginindex])
 	{
-		unsigned char modechar = *letter;
 		if (modechar == '+' || modechar == '-')
 		{
 			adding = (modechar == '+');
 			continue;
 		}
 
-		ModeHandler *mh = this->FindMode(modechar, type);
+		ModeHandler* mh = this->FindMode(modechar, type);
 		if (!mh)
 		{
 			/* No mode handler? Unknown mode character then. */
 			int numeric = (type == MODETYPE_CHANNEL ? ERR_UNKNOWNMODE : ERR_UNKNOWNSNOMASK);
 			const char* typestr = (type == MODETYPE_CHANNEL ? "channel" : "user");
-			user->WriteNumeric(numeric, modechar, InspIRCd::Format("is not a recognised %s mode.", typestr));
+			user->WriteNumeric(numeric, modechar, INSP_FORMAT("is not a recognised {} mode.", typestr));
 			continue;
 		}
 
@@ -404,7 +387,7 @@ static bool IsModeParamValid(User* user, Channel* targetchannel, User* targetuse
 }
 
 // Returns true if we should apply a merged mode, false if we should skip it
-static bool ShouldApplyMergedMode(Channel* chan, Modes::Change& item)
+static bool ShouldApplyMergedMode(Channel* chan, const Modes::Change& item)
 {
 	ModeHandler* mh = item.mh;
 	if ((!chan) || (!chan->IsModeSet(mh)) || (mh->IsListMode()))
@@ -412,7 +395,7 @@ static bool ShouldApplyMergedMode(Channel* chan, Modes::Change& item)
 		return true;
 
 	// Mode handler decides
-	std::string ours = chan->GetModeParameter(mh);
+	const std::string ours = chan->GetModeParameter(mh);
 	return mh->ResolveModeConflict(item.param, ours, chan);
 }
 
@@ -420,26 +403,24 @@ void ModeParser::Process(User* user, Channel* targetchannel, User* targetuser, M
 {
 	// Call ProcessSingle until the entire list is processed, but at least once to ensure
 	// LastParse and LastChangeList are cleared
-	unsigned int processed = 0;
+	size_t processed = 0;
 	do
 	{
-		unsigned int n = ProcessSingle(user, targetchannel, targetuser, changelist, flags, processed);
-		processed += n;
+		processed += ProcessSingle(user, targetchannel, targetuser, changelist, flags, processed);
 	}
 	while (processed < changelist.size());
 }
 
-unsigned int ModeParser::ProcessSingle(User* user, Channel* targetchannel, User* targetuser, Modes::ChangeList& changelist, ModeProcessFlag flags, unsigned int beginindex)
+size_t ModeParser::ProcessSingle(User* user, Channel* targetchannel, User* targetuser, Modes::ChangeList& changelist, ModeProcessFlag flags, size_t beginindex)
 {
 	LastChangeList.clear();
 
-	unsigned int modes_processed = 0;
+	size_t modes_processed = 0;
 	Modes::ChangeList::List& list = changelist.getlist();
-	for (Modes::ChangeList::List::iterator i = list.begin()+beginindex; i != list.end(); ++i)
+
+	for (auto& item : insp::iterator_range(list.begin() + beginindex, list.end()))
 	{
 		modes_processed++;
-
-		Modes::Change& item = *i;
 		ModeHandler* mh = item.mh;
 
 		// If a mode change has been given for a mode that does not exist then reject
@@ -461,9 +442,7 @@ unsigned int ModeParser::ProcessSingle(User* user, Channel* targetchannel, User*
 				continue;
 		}
 
-		ModeAction ma = TryMode(user, targetuser, targetchannel, item, (!(flags & MODE_CHECKACCESS)));
-
-		if (ma != MODEACTION_ALLOW)
+		if (!TryMode(user, targetuser, targetchannel, item, (!(flags & MODE_CHECKACCESS))))
 			continue;
 
 		LastChangeList.push(mh, item.adding, item.param);
@@ -498,23 +477,20 @@ unsigned int ModeParser::ProcessSingle(User* user, Channel* targetchannel, User*
 void ModeParser::ShowListModeList(User* user, Channel* chan, ModeHandler* mh)
 {
 	{
-		ModResult MOD_RESULT;
-		FIRST_MOD_RESULT(OnRawMode, MOD_RESULT, (user, chan, mh, "", true));
-		if (MOD_RESULT == MOD_RES_DENY)
+		Modes::Change modechange(mh, true, "");
+		ModResult modres;
+		FIRST_MOD_RESULT(OnRawMode, modres, (user, chan, modechange));
+		if (modres == MOD_RES_DENY)
 			return;
 
 		bool display = true;
 
 		// Ask mode watchers whether it's OK to show the list
-		std::pair<ModeWatcherMap::iterator, ModeWatcherMap::iterator> itpair = modewatchermap.equal_range(mh->name);
-		for (ModeWatcherMap::iterator i = itpair.first; i != itpair.second; ++i)
+		for (const auto& [_, mw] : insp::equal_range(modewatchermap, mh->name))
 		{
-			ModeWatcher* mw = i->second;
 			if (mw->GetModeType() == MODETYPE_CHANNEL)
 			{
-				std::string dummyparam;
-
-				if (!mw->BeforeMode(user, NULL, chan, dummyparam, true))
+				if (!mw->BeforeMode(user, nullptr, chan, modechange))
 				{
 					// A mode watcher doesn't want us to show the list
 					display = false;
@@ -530,58 +506,64 @@ void ModeParser::ShowListModeList(User* user, Channel* chan, ModeHandler* mh)
 	}
 }
 
-void ModeParser::CleanMask(std::string &mask)
+void ModeParser::CleanMask(std::string& mask)
 {
-	std::string::size_type pos_of_pling = mask.find_first_of('!');
-	std::string::size_type pos_of_at = mask.find_first_of('@');
-	std::string::size_type pos_of_dot = mask.find_first_of('.');
-	std::string::size_type pos_of_colons = mask.find("::"); /* Because ipv6 addresses are colon delimited -- double so it treats extban as nick */
+	auto pos_of_pling    = mask.find_first_of('!');
+	auto pos_of_at       = mask.find_first_of('@',  pos_of_pling == std::string::npos ? 0 : pos_of_pling);
+	auto pos_of_hostchar = mask.find_first_of(":.", pos_of_at    == std::string::npos ? 0 : pos_of_at);
 
-	if (mask.length() >= 2 && mask[1] == ':')
-		return; // if it's an extban, don't even try guess how it needs to be formed.
-
-	if ((pos_of_pling == std::string::npos) && (pos_of_at == std::string::npos))
+	if (pos_of_pling == mask.length()-1 || pos_of_at == mask.length()-1)
 	{
-		/* Just a nick, or just a host - or clearly ipv6 (starting with :) */
-		if ((pos_of_dot == std::string::npos) && (pos_of_colons == std::string::npos) && mask[0] != ':')
-		{
-			/* It has no '.' in it, it must be a nick. */
-			mask.append("!*@*");
-		}
+		// Malformed mask; needs * after the ! or @.
+		mask.append("*");
+	}
+
+	if (pos_of_pling == 0 || pos_of_at == 0)
+	{
+		// Malformed mask; needs * before the ! or @.
+		mask.insert(0, "*");
+	}
+
+	if (pos_of_pling == std::string::npos && pos_of_at == std::string::npos)
+	{
+		if (pos_of_hostchar == std::string::npos)
+			mask.append("!*@*"); // The mask looks like "nick".
 		else
-		{
-			/* Got a dot in it? Has to be a host */
-			mask = "*!*@" + mask;
-		}
+			mask.insert(0, "*!*@"); // The mask looks like "host".
 	}
-	else if ((pos_of_pling == std::string::npos) && (pos_of_at != std::string::npos))
+	else if (pos_of_pling == std::string::npos && pos_of_at != std::string::npos)
 	{
-		/* Has an @ but no !, its a user@host */
-		mask = "*!" + mask;
+		// The mask looks like "user@host".
+		mask.insert(0, "*!");
+
 	}
-	else if ((pos_of_pling != std::string::npos) && (pos_of_at == std::string::npos))
+	else if (pos_of_pling != std::string::npos && pos_of_at == std::string::npos)
 	{
-		/* Has a ! but no @, it must be a nick!ident */
+		// The mask looks like "nick!user".
 		mask.append("@*");
 	}
+	else if (pos_of_at-pos_of_pling == 1)
+	{
+		// The mask looks like "nick!@host".
+		mask.insert(pos_of_at, "*");
+	}
 }
 
-ModeHandler::Id ModeParser::AllocateModeId(ModeType mt)
+ModeHandler::Id ModeParser::AllocateModeId(ModeHandler* mh)
 {
 	for (ModeHandler::Id i = 0; i != MODEID_MAX; ++i)
 	{
-		if (!modehandlersbyid[mt][i])
+		if (!modehandlersbyid[mh->GetModeType()][i])
 			return i;
 	}
 
-	throw ModuleException("Out of ModeIds");
+	throw ModuleException(mh->creator, "Out of mode ids");
 }
 
 void ModeParser::AddMode(ModeHandler* mh)
 {
 	if (!ModeParser::IsModeChar(mh->GetModeChar()))
-		throw ModuleException(InspIRCd::Format("Mode letter for %s is invalid: %c",
-			mh->name.c_str(), mh->GetModeChar()));
+		throw ModuleException(mh->creator, INSP_FORMAT("Mode letter for {} is invalid: {}", mh->name, mh->GetModeChar()));
 
 	/* A mode prefix of ',' is not acceptable, it would fuck up server to server.
 	 * A mode prefix of ':' will fuck up both server to server, and client to server.
@@ -590,33 +572,36 @@ void ModeParser::AddMode(ModeHandler* mh)
 	PrefixMode* pm = mh->IsPrefixMode();
 	if (pm)
 	{
-		if ((pm->GetPrefix() > 126) || (pm->GetPrefix() == ',') || (pm->GetPrefix() == ':') || (pm->GetPrefix() == '#'))
-			throw ModuleException(InspIRCd::Format("Mode prefix for %s is invalid: %c",
-				mh->name.c_str(), pm->GetPrefix()));
+		if ((pm->GetPrefix() > 126) || (pm->GetPrefix() == ',') || (pm->GetPrefix() == ':') || ServerInstance->Channels.IsPrefix(pm->GetPrefix()))
+			throw ModuleException(mh->creator, INSP_FORMAT("Mode prefix for {} is invalid: {}", mh->name, pm->GetPrefix()));
 
 		PrefixMode* otherpm = FindPrefix(pm->GetPrefix());
 		if (otherpm)
-			throw ModuleException(InspIRCd::Format("Mode prefix for %s already used by %s from %s: %c",
-				mh->name.c_str(), otherpm->name.c_str(), otherpm->creator->ModuleSourceFile.c_str(), pm->GetPrefix()));
+		{
+			throw ModuleException(mh->creator, INSP_FORMAT("Mode prefix for {} already used by {} from {}: {}",
+				mh->name, otherpm->name, otherpm->creator->ModuleFile, pm->GetPrefix()));
+		}
 	}
 
-	ModeHandler*& slot = modehandlers[mh->GetModeType()][mh->GetModeChar()-65];
+	ModeHandler*& slot = modehandlers[mh->GetModeType()][ModeParser::GetModeIndex(mh->GetModeChar())];
 	if (slot)
-		throw ModuleException(InspIRCd::Format("Mode letter for %s already used by %s from %s: %c",
-			mh->name.c_str(), slot->name.c_str(), slot->creator->ModuleSourceFile.c_str(), mh->GetModeChar()));
+	{
+		throw ModuleException(mh->creator, INSP_FORMAT("Mode letter for {} already used by {} from {}: {}",
+			mh->name, slot->name, slot->creator->ModuleFile, mh->GetModeChar()));
+	}
 
 	// The mode needs an id if it is either a user mode, a simple mode (flag) or a parameter mode.
 	// Otherwise (for listmodes and prefix modes) the id remains MODEID_MAX, which is invalid.
 	ModeHandler::Id modeid = MODEID_MAX;
 	if ((mh->GetModeType() == MODETYPE_USER) || (mh->IsParameterMode()) || (!mh->IsListMode()))
-		modeid = AllocateModeId(mh->GetModeType());
+		modeid = AllocateModeId(mh);
 
-	std::pair<ModeHandlerMap::iterator, bool> res = modehandlersbyname[mh->GetModeType()].insert(std::make_pair(mh->name, mh));
+	std::pair<ModeHandlerMap::iterator, bool> res = modehandlersbyname[mh->GetModeType()].emplace(mh->name, mh);
 	if (!res.second)
 	{
 		ModeHandler* othermh = res.first->second;
-		throw ModuleException(InspIRCd::Format("Mode name %s already used by %c from %s",
-			mh->name.c_str(), othermh->GetModeChar(), othermh->creator->ModuleSourceFile.c_str()));
+		throw ModuleException(mh->creator, INSP_FORMAT("Mode name {} already used by {} from {}",
+			mh->name, othermh->GetModeChar(), othermh->creator->ModuleFile));
 	}
 
 	// Everything is fine, add the mode
@@ -645,7 +630,7 @@ bool ModeParser::DelMode(ModeHandler* mh)
 	if ((mhmapit == mhmap.end()) || (mhmapit->second != mh))
 		return false;
 
-	ModeHandler*& slot = modehandlers[mh->GetModeType()][mh->GetModeChar()-65];
+	ModeHandler*& slot = modehandlers[mh->GetModeType()][ModeParser::GetModeIndex(mh->GetModeChar())];
 	if (slot != mh)
 		return false;
 
@@ -656,8 +641,8 @@ bool ModeParser::DelMode(ModeHandler* mh)
 	{
 		case MODETYPE_USER:
 		{
-			const user_hash& users = ServerInstance->Users->GetUsers();
-			for (user_hash::const_iterator i = users.begin(); i != users.end(); )
+			const UserMap& users = ServerInstance->Users.GetUsers();
+			for (UserMap::const_iterator i = users.begin(); i != users.end(); )
 			{
 				User* user = i->second;
 				++i;
@@ -667,8 +652,8 @@ bool ModeParser::DelMode(ModeHandler* mh)
 		break;
 		case MODETYPE_CHANNEL:
 		{
-			const chan_hash& chans = ServerInstance->GetChans();
-			for (chan_hash::const_iterator i = chans.begin(); i != chans.end(); )
+			const ChannelMap& chans = ServerInstance->Channels.GetChans();
+			for (ChannelMap::const_iterator i = chans.begin(); i != chans.end(); )
 			{
 				// The channel may not be in the hash after RemoveMode(), see m_permchannels
 				Channel* chan = i->second;
@@ -676,7 +661,7 @@ bool ModeParser::DelMode(ModeHandler* mh)
 
 				Modes::ChangeList changelist;
 				mh->RemoveMode(chan, changelist);
-				this->Process(ServerInstance->FakeClient, chan, NULL, changelist, MODE_LOCALONLY);
+				this->Process(ServerInstance->FakeClient, chan, nullptr, changelist, MODE_LOCALONLY);
 			}
 		}
 		break;
@@ -684,8 +669,8 @@ bool ModeParser::DelMode(ModeHandler* mh)
 
 	mhmap.erase(mhmapit);
 	if (mh->GetId() != MODEID_MAX)
-		modehandlersbyid[mh->GetModeType()][mh->GetId()] = NULL;
-	slot = NULL;
+		modehandlersbyid[mh->GetModeType()][mh->GetId()] = nullptr;
+	slot = nullptr;
 	if (mh->IsPrefixMode())
 		mhlist.prefix.erase(std::find(mhlist.prefix.begin(), mhlist.prefix.end(), mh->IsPrefixMode()));
 	else if (mh->IsListModeBase())
@@ -700,32 +685,31 @@ ModeHandler* ModeParser::FindMode(const std::string& modename, ModeType mt)
 	if (it != mhmap.end())
 		return it->second;
 
-	return NULL;
+	return nullptr;
 }
 
-ModeHandler* ModeParser::FindMode(unsigned const char modeletter, ModeType mt)
+ModeHandler* ModeParser::FindMode(unsigned char modeletter, ModeType mt)
 {
 	if (!ModeParser::IsModeChar(modeletter))
-		return NULL;
+		return nullptr;
 
-	return modehandlers[mt][modeletter-65];
+	return modehandlers[mt][ModeParser::GetModeIndex(modeletter)];
 }
 
 PrefixMode* ModeParser::FindPrefixMode(unsigned char modeletter)
 {
 	ModeHandler* mh = FindMode(modeletter, MODETYPE_CHANNEL);
 	if (!mh)
-		return NULL;
+		return nullptr;
 	return mh->IsPrefixMode();
 }
 
-PrefixMode* ModeParser::FindNearestPrefixMode(unsigned int rank)
+
+PrefixMode* ModeParser::FindNearestPrefixMode(ModeHandler::Rank rank) const
 {
-	PrefixMode* pm = NULL;
-	const PrefixModeList& prefixmodes = GetPrefixModes();
-	for (PrefixModeList::const_iterator i = prefixmodes.begin(); i != prefixmodes.end(); ++i)
+	PrefixMode* pm = nullptr;
+	for (auto* thispm : GetPrefixModes())
 	{
-		PrefixMode* thispm = *i;
 		if (thispm->GetPrefixRank() < rank)
 			continue; // Not ranked high enough.
 
@@ -737,101 +721,19 @@ PrefixMode* ModeParser::FindNearestPrefixMode(unsigned int rank)
 	return pm;
 }
 
-PrefixMode* ModeParser::FindPrefix(unsigned const char pfxletter)
+PrefixMode* ModeParser::FindPrefix(unsigned char pfxletter) const
 {
-	const PrefixModeList& list = GetPrefixModes();
-	for (PrefixModeList::const_iterator i = list.begin(); i != list.end(); ++i)
+	for (auto* pm : GetPrefixModes())
 	{
-		PrefixMode* pm = *i;
 		if (pm->GetPrefix() == pfxletter)
 			return pm;
 	}
-	return NULL;
-}
-
-std::string ModeParser::GiveModeList(ModeType mt)
-{
-	std::string type1;	/* Listmodes EXCEPT those with a prefix */
-	std::string type2;	/* Modes that take a param when adding or removing */
-	std::string type3;	/* Modes that only take a param when adding */
-	std::string type4;	/* Modes that dont take a param */
-
-	for (unsigned char mode = 'A'; mode <= 'z'; mode++)
-	{
-		ModeHandler* mh = modehandlers[mt][mode-65];
-		if (mh)
-		{
-			/* One parameter when adding */
-			if (mh->NeedsParam(true))
-			{
-				PrefixMode* pm = mh->IsPrefixMode();
-				if ((mh->IsListMode()) && ((!pm) || (pm->GetPrefix() == 0)))
-				{
-					type1 += mh->GetModeChar();
-				}
-				else
-				{
-					/* ... and one parameter when removing */
-					if (mh->NeedsParam(false))
-					{
-						/* But not a list mode */
-						if (!pm)
-						{
-							type2 += mh->GetModeChar();
-						}
-					}
-					else
-					{
-						/* No parameters when removing */
-						type3 += mh->GetModeChar();
-					}
-				}
-			}
-			else
-			{
-				type4 += mh->GetModeChar();
-			}
-		}
-	}
-
-	return type1 + "," + type2 + "," + type3 + "," + type4;
-}
-
-struct PrefixModeSorter
-{
-	bool operator()(PrefixMode* lhs, PrefixMode* rhs)
-	{
-		return lhs->GetPrefixRank() < rhs->GetPrefixRank();
-	}
-};
-
-std::string ModeParser::BuildPrefixes(bool lettersAndModes)
-{
-	std::string mletters;
-	std::string mprefixes;
-	std::vector<PrefixMode*> prefixes;
-
-	const PrefixModeList& list = GetPrefixModes();
-	for (PrefixModeList::const_iterator i = list.begin(); i != list.end(); ++i)
-	{
-		PrefixMode* pm = *i;
-		if (pm->GetPrefix())
-			prefixes.push_back(pm);
-	}
-
-	std::sort(prefixes.begin(), prefixes.end(), PrefixModeSorter());
-	for (std::vector<PrefixMode*>::const_reverse_iterator n = prefixes.rbegin(); n != prefixes.rend(); ++n)
-	{
-		mletters += (*n)->GetPrefix();
-		mprefixes += (*n)->GetModeChar();
-	}
-
-	return lettersAndModes ? "(" + mprefixes + ")" + mletters : mletters;
+	return nullptr;
 }
 
 void ModeParser::AddModeWatcher(ModeWatcher* mw)
 {
-	modewatchermap.insert(std::make_pair(mw->GetModeName(), mw));
+	modewatchermap.emplace(mw->GetModeName(), mw);
 }
 
 bool ModeParser::DelModeWatcher(ModeWatcher* mw)
@@ -856,7 +758,7 @@ void ModeHandler::RemoveMode(User* user)
 	{
 		Modes::ChangeList changelist;
 		changelist.push_remove(this);
-		ServerInstance->Modes->Process(ServerInstance->FakeClient, NULL, user, changelist, ModeParser::MODE_LOCALONLY);
+		ServerInstance->Modes.Process(ServerInstance->FakeClient, nullptr, user, changelist, ModeParser::MODE_LOCALONLY);
 	}
 }
 
@@ -874,17 +776,34 @@ void ModeHandler::RemoveMode(Channel* channel, Modes::ChangeList& changelist)
 
 void PrefixMode::RemoveMode(Channel* chan, Modes::ChangeList& changelist)
 {
-	const Channel::MemberMap& userlist = chan->GetUsers();
-	for (Channel::MemberMap::const_iterator i = userlist.begin(); i != userlist.end(); ++i)
+	for (const auto& [user, memb] : chan->GetUsers())
 	{
-		if (i->second->HasMode(this))
-			changelist.push_remove(this, i->first->nick);
+		if (memb->HasMode(this))
+			changelist.push_remove(this, user->nick);
 	}
 }
 
 bool ModeParser::IsModeChar(char chr)
 {
-	return ((chr >= 'A' && chr <= 'Z') || (chr >= 'a' && chr <= 'z'));
+	return ((chr >= '0' && chr <= '9') || (chr >= 'A' && chr <= 'Z') || (chr >= 'a' && chr <= 'z'));
+}
+
+size_t ModeParser::GetModeIndex(char chr)
+{
+	// Bitset layout:
+	//   0123456789                 = 10 [0-9]
+	//   ABCDEFGHIJKLMNOPQRSTUVWXYZ = 26 [10-35]
+	//   abcdefghijklmnopqrstuvwxyz = 26 [36-61]
+	if (chr >= '0' && chr <= '9')
+		return chr - '0';
+
+	if (chr >= 'A' && chr <= 'Z')
+		return chr - 'A' + 10; // [0-9] = 10
+
+	if (chr >= 'a' && chr <= 'z')
+		return chr - 'a' + 36; // [0-9]+[A-Z] = 10+26 = 36
+
+	return ModeParser::MODEID_MAX;
 }
 
 ModeParser::ModeParser()
@@ -893,7 +812,3 @@ ModeParser::ModeParser()
 	memset(modehandlers, 0, sizeof(modehandlers));
 	memset(modehandlersbyid, 0, sizeof(modehandlersbyid));
 }
-
-ModeParser::~ModeParser()
-{
-}
