@@ -1,10 +1,11 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2020 Joel Sing <joel@sing.id.au>
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2016-2023 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2014, 2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2016-2017, 2019-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2014 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2013-2016 Adam <Adam@anope.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -23,14 +24,17 @@
 /// $CompilerFlags: find_compiler_flags("lber" "") find_compiler_flags("ldap" "")
 /// $LinkerFlags: find_linker_flags("lber" "-llber") find_linker_flags("ldap" "-lldap_r")
 
-/// $PackageInfo: require_system("arch") libldap
-/// $PackageInfo: require_system("centos") openldap-devel
-/// $PackageInfo: require_system("debian") libldap2-dev
-/// $PackageInfo: require_system("rocky") openldap-devel
-/// $PackageInfo: require_system("ubuntu") libldap2-dev
+/// $PackageInfo: require_system("alpine") openldap-dev pkgconf
+/// $PackageInfo: require_system("arch") libldap pkgconf
+/// $PackageInfo: require_system("darwin") openldap pkg-config
+/// $PackageInfo: require_system("debian~") libldap2-dev pkg-config
+/// $PackageInfo: require_system("rhel~") openldap-devel pkg-config
+
 
 #include "inspircd.h"
 #include "modules/ldap.h"
+#include "threadsocket.h"
+#include "utility/string.h"
 
 #if defined LDAP_API_FEATURE_X_OPENLDAP_REENTRANT && !LDAP_API_FEATURE_X_OPENLDAP_REENTRANT
 # error InspIRCd requires OpenLDAP to be built as reentrant.
@@ -43,17 +47,21 @@
 #endif
 
 #ifdef _WIN32
+# include <Wininet.h>
 # include <Winldap.h>
 # include <WinBer.h>
-# include "http_parser.c"
 # define LDAP_OPT_SUCCESS LDAP_SUCCESS
 # define LDAP_OPT_NETWORK_TIMEOUT LDAP_OPT_SEND_TIMEOUT
 # define LDAP_STR(X) const_cast<PSTR>((X).c_str())
 # define LDAP_SASL_SIMPLE static_cast<PSTR>(0)
 # define LDAP_TIME(X) reinterpret_cast<PLDAP_TIMEVAL>(&(X))
+# define LDAP_VENDOR_VERSION_MAJOR (LDAP_VERSION / 100)
+# define LDAP_VENDOR_VERSION_MINOR (LDAP_VERSION / 10 % 10)
+# define LDAP_VENDOR_VERSION_PATCH (LDAP_VERSION / 10)
 # define ldap_first_message ldap_first_entry
 # define ldap_next_message ldap_next_entry
 # define ldap_unbind_ext(LDAP, UNUSED1, UNUSED2) ldap_unbind(LDAP)
+# pragma comment(lib, "Wininet.lib")
 # pragma comment(lib, "Wldap32.lib")
 #else
 # include <ldap.h>
@@ -69,11 +77,11 @@ class LDAPService;
 
 class LDAPRequest
 {
- public:
+public:
 	LDAPService* service;
 	LDAPInterface* inter;
-	LDAPMessage* message; /* message returned by ldap_ */
-	LDAPResult* result; /* final result */
+	LDAPMessage* message = nullptr; /* message returned by ldap_ */
+	LDAPResult* result = nullptr; /* final result */
 	struct timeval tv;
 	QueryType type;
 	int success;
@@ -81,8 +89,6 @@ class LDAPRequest
 	LDAPRequest(LDAPService* s, LDAPInterface* i, int c)
 		: service(s)
 		, inter(i)
-		, message(NULL)
-		, result(NULL)
 		, success(c)
 	{
 		type = QUERY_UNKNOWN;
@@ -93,7 +99,7 @@ class LDAPRequest
 	virtual ~LDAPRequest()
 	{
 		delete result;
-		if (message != NULL)
+		if (message != nullptr)
 			ldap_msgfree(message);
 	}
 
@@ -101,11 +107,12 @@ class LDAPRequest
 	virtual std::string info() = 0;
 };
 
-class LDAPBind : public LDAPRequest
+class LDAPBind final
+	: public LDAPRequest
 {
 	std::string who, pass;
 
- public:
+public:
 	LDAPBind(LDAPService* s, LDAPInterface* i, const std::string& w, const std::string& p)
 		: LDAPRequest(s, i, LDAP_SUCCESS)
 		, who(w)
@@ -114,17 +121,18 @@ class LDAPBind : public LDAPRequest
 		type = QUERY_BIND;
 	}
 
-	int run() CXX11_OVERRIDE;
-	std::string info() CXX11_OVERRIDE;
+	int run() override;
+	std::string info() override;
 };
 
-class LDAPSearchRequest : public LDAPRequest
+class LDAPSearchRequest final
+	: public LDAPRequest
 {
 	std::string base;
 	int searchscope;
 	std::string filter;
 
- public:
+public:
 	LDAPSearchRequest(LDAPService* s, LDAPInterface* i, const std::string& b, int se, const std::string& f)
 		: LDAPRequest(s, i, LDAP_SUCCESS)
 		, base(b)
@@ -134,16 +142,17 @@ class LDAPSearchRequest : public LDAPRequest
 		type = QUERY_SEARCH;
 	}
 
-	int run() CXX11_OVERRIDE;
-	std::string info() CXX11_OVERRIDE;
+	int run() override;
+	std::string info() override;
 };
 
-class LDAPAdd : public LDAPRequest
+class LDAPAdd final
+	: public LDAPRequest
 {
 	std::string dn;
 	LDAPMods attributes;
 
- public:
+public:
 	LDAPAdd(LDAPService* s, LDAPInterface* i, const std::string& d, const LDAPMods& attr)
 		: LDAPRequest(s, i, LDAP_SUCCESS)
 		, dn(d)
@@ -152,15 +161,16 @@ class LDAPAdd : public LDAPRequest
 		type = QUERY_ADD;
 	}
 
-	int run() CXX11_OVERRIDE;
-	std::string info() CXX11_OVERRIDE;
+	int run() override;
+	std::string info() override;
 };
 
-class LDAPDel : public LDAPRequest
+class LDAPDel final
+	: public LDAPRequest
 {
 	std::string dn;
 
- public:
+public:
 	LDAPDel(LDAPService* s, LDAPInterface* i, const std::string& d)
 		: LDAPRequest(s, i, LDAP_SUCCESS)
 		, dn(d)
@@ -168,16 +178,17 @@ class LDAPDel : public LDAPRequest
 		type = QUERY_DELETE;
 	}
 
-	int run() CXX11_OVERRIDE;
-	std::string info() CXX11_OVERRIDE;
+	int run() override;
+	std::string info() override;
 };
 
-class LDAPModify : public LDAPRequest
+class LDAPModify final
+	: public LDAPRequest
 {
 	std::string base;
 	LDAPMods attributes;
 
- public:
+public:
 	LDAPModify(LDAPService* s, LDAPInterface* i, const std::string& b, const LDAPMods& attr)
 		: LDAPRequest(s, i, LDAP_SUCCESS)
 		, base(b)
@@ -186,15 +197,16 @@ class LDAPModify : public LDAPRequest
 		type = QUERY_MODIFY;
 	}
 
-	int run() CXX11_OVERRIDE;
-	std::string info() CXX11_OVERRIDE;
+	int run() override;
+	std::string info() override;
 };
 
-class LDAPCompare : public LDAPRequest
+class LDAPCompare final
+	: public LDAPRequest
 {
 	std::string dn, attr, val;
 
- public:
+public:
 	LDAPCompare(LDAPService* s, LDAPInterface* i, const std::string& d, const std::string& a, const std::string& v)
 		: LDAPRequest(s, i, LDAP_COMPARE_TRUE)
 		, dn(d)
@@ -204,16 +216,18 @@ class LDAPCompare : public LDAPRequest
 		type = QUERY_COMPARE;
 	}
 
-	int run() CXX11_OVERRIDE;
-	std::string info() CXX11_OVERRIDE;
+	int run() override;
+	std::string info() override;
 };
 
-class LDAPService : public LDAPProvider, public SocketThread
+class LDAPService final
+	: public LDAPProvider
+	, public SocketThread
 {
- private:
-	LDAP* con;
-	reference<ConfigTag> config;
-	time_t last_connect;
+private:
+	LDAP* con = nullptr;
+	std::shared_ptr<ConfigTag> config;
+	time_t last_connect = 0;
 	int searchscope;
 	time_t timeout;
 
@@ -221,36 +235,39 @@ class LDAPService : public LDAPProvider, public SocketThread
 	// Windows LDAP does not implement this so we need to do it.
 	int ldap_initialize(LDAP** ldap, const char* url)
 	{
-		http_parser_url up;
-		http_parser_url_init(&up);
-		if (http_parser_parse_url(url, strlen(url), 0, &up) != 0)
-			return LDAP_CONNECT_ERROR; // Malformed url.
+		URL_COMPONENTS urlComponents;
+		memset(&urlComponents, 0, sizeof(urlComponents));
+		urlComponents.dwStructSize = sizeof(urlComponents);
 
-		if (!(up.field_set & (1 << UF_HOST)))
-			return LDAP_CONNECT_ERROR; // Missing the host.
+		std::vector<char> schemebuf(16);
+		urlComponents.lpszScheme = schemebuf.data();
+		urlComponents.dwSchemeLength = schemebuf.size();
+
+		std::vector<char> hostbuf(1024);
+		urlComponents.lpszHostName = hostbuf.data();
+		urlComponents.dwHostNameLength = hostbuf.size();
+
+		if (!InternetCrackUrlA(url, 0, 0, &urlComponents))
+			return LDAP_CONNECT_ERROR; // Malformed url.
 
 		unsigned long port = 389; // Default plaintext port.
 		bool secure = false; // LDAP defaults to plaintext.
-		if (up.field_set & (1 << UF_SCHEMA))
+		if (urlComponents.dwSchemeLength > 0)
 		{
-			const std::string schema(url, up.field_data[UF_SCHEMA].off, up.field_data[UF_SCHEMA].len);
-			if (stdalgo::string::equalsci(schema, "ldaps"))
+			const std::string scheme(urlComponents.lpszScheme);
+			if (insp::equalsci(scheme, "ldaps"))
 			{
 				port = 636; // Default encrypted port.
 				secure = true;
 			}
-			else if (!stdalgo::string::equalsci(schema, "ldap"))
+			else if (!insp::equalsci(scheme, "ldap"))
 				return LDAP_CONNECT_ERROR; // Invalid protocol.
 		}
 
-		if (up.field_set & (1 << UF_PORT))
-		{
-			const std::string portstr(url, up.field_data[UF_PORT].off, up.field_data[UF_PORT].len);
-			port = ConvToNum<unsigned long>(portstr);
-		}
+		if (urlComponents.nPort > 0)
+			port = urlComponents.nPort;
 
-		const std::string host(url, up.field_data[UF_HOST].off, up.field_data[UF_HOST].len);
-		*ldap = ldap_sslinit(LDAP_STR(host), port, secure);
+		*ldap = ldap_sslinit(urlComponents.lpszHostName, port, secure);
 		if (!*ldap)
 			return LdapGetLastError(); // Something went wrong, find out what.
 
@@ -259,7 +276,7 @@ class LDAPService : public LDAPProvider, public SocketThread
 	}
 #endif
 
- public:
+public:
 	static LDAPMod** BuildMods(const LDAPMods& attributes)
 	{
 		LDAPMod** mods = new LDAPMod*[attributes.size() + 1];
@@ -267,7 +284,7 @@ class LDAPService : public LDAPProvider, public SocketThread
 		for (unsigned int x = 0; x < attributes.size(); ++x)
 		{
 			const LDAPModification& l = attributes[x];
-			LDAPMod* mod = new LDAPMod;
+			auto* mod = new LDAPMod();
 			mods[x] = mod;
 
 			if (l.op == LDAPModification::LDAP_ADD)
@@ -293,14 +310,14 @@ class LDAPService : public LDAPProvider, public SocketThread
 
 	static void FreeMods(LDAPMod** mods)
 	{
-		for (unsigned int i = 0; mods[i] != NULL; ++i)
+		for (unsigned int i = 0; mods[i] != nullptr; ++i)
 		{
 			LDAPMod* mod = mods[i];
-			if (mod->mod_type != NULL)
+			if (mod->mod_type != nullptr)
 				free(mod->mod_type);
-			if (mod->mod_values != NULL)
+			if (mod->mod_values != nullptr)
 			{
-				for (unsigned int j = 0; mod->mod_values[j] != NULL; ++j)
+				for (unsigned int j = 0; mod->mod_values[j] != nullptr; ++j)
 					free(mod->mod_values[j]);
 				delete[] mod->mod_values;
 			}
@@ -308,7 +325,7 @@ class LDAPService : public LDAPProvider, public SocketThread
 		delete[] mods;
 	}
 
- private:
+private:
 	void Reconnect()
 	{
 		// Only try one connect a minute. It is an expensive blocking operation
@@ -316,7 +333,7 @@ class LDAPService : public LDAPProvider, public SocketThread
 			throw LDAPException("Unable to connect to LDAP service " + this->name + ": reconnecting too fast");
 		last_connect = ServerInstance->Time();
 
-		ldap_unbind_ext(this->con, NULL, NULL);
+		ldap_unbind_ext(this->con, nullptr, nullptr);
 		Connect();
 	}
 
@@ -325,8 +342,8 @@ class LDAPService : public LDAPProvider, public SocketThread
 		int ret = ldap_set_option(this->con, option, value);
 		if (ret != LDAP_OPT_SUCCESS)
 		{
-			ldap_unbind_ext(this->con, NULL, NULL);
-			this->con = NULL;
+			ldap_unbind_ext(this->con, nullptr, nullptr);
+			this->con = nullptr;
 		}
 		return ret;
 	}
@@ -338,19 +355,19 @@ class LDAPService : public LDAPProvider, public SocketThread
 		this->UnlockQueueWakeup();
 	}
 
- public:
+public:
 	typedef std::vector<LDAPRequest*> query_queue;
 	query_queue queries, results;
-	Mutex process_mutex; /* held when processing requests not in either queue */
+	std::mutex process_mutex; /* held when processing requests not in either queue */
 
-	LDAPService(Module* c, ConfigTag* tag)
+	LDAPService(Module* c, const std::shared_ptr<ConfigTag>& tag)
 		: LDAPProvider(c, "LDAP/" + tag->getString("id"))
-		, con(NULL), config(tag), last_connect(0)
+		, config(tag)
 	{
 		std::string scope = config->getString("searchscope");
-		if (stdalgo::string::equalsci(scope, "base"))
+		if (insp::equalsci(scope, "base"))
 			searchscope = LDAP_SCOPE_BASE;
-		else if (stdalgo::string::equalsci(scope, "onelevel"))
+		else if (insp::equalsci(scope, "onelevel"))
 			searchscope = LDAP_SCOPE_ONELEVEL;
 		else
 			searchscope = LDAP_SCOPE_SUBTREE;
@@ -359,14 +376,12 @@ class LDAPService : public LDAPProvider, public SocketThread
 		Connect();
 	}
 
-	~LDAPService()
+	~LDAPService() override
 	{
 		this->LockQueue();
 
-		for (unsigned int i = 0; i < this->queries.size(); ++i)
+		for (auto* req : this->queries)
 		{
-			LDAPRequest* req = this->queries[i];
-
 			/* queries have no results yet */
 			req->result = new LDAPResult();
 			req->result->type = req->type;
@@ -377,10 +392,8 @@ class LDAPService : public LDAPProvider, public SocketThread
 		}
 		this->queries.clear();
 
-		for (unsigned int i = 0; i < this->results.size(); ++i)
+		for (auto* req : this->results)
 		{
-			LDAPRequest* req = this->results[i];
-
 			/* even though this may have already finished successfully we return that it didn't */
 			req->result->error = "LDAP Interface is going away";
 			req->inter->OnError(*req->result);
@@ -391,7 +404,7 @@ class LDAPService : public LDAPProvider, public SocketThread
 
 		this->UnlockQueue();
 
-		ldap_unbind_ext(this->con, NULL, NULL);
+		ldap_unbind_ext(this->con, nullptr, nullptr);
 	}
 
 	void Connect()
@@ -412,53 +425,47 @@ class LDAPService : public LDAPProvider, public SocketThread
 			throw LDAPException("Unable to set timeout for " + this->name + ": " + ldap_err2string(i));
 	}
 
-	void BindAsManager(LDAPInterface* i) CXX11_OVERRIDE
+	void BindAsManager(LDAPInterface* i) override
 	{
 		std::string binddn = config->getString("binddn");
 		std::string bindauth = config->getString("bindauth");
 		this->Bind(i, binddn, bindauth);
 	}
 
-	void Bind(LDAPInterface* i, const std::string& who, const std::string& pass) CXX11_OVERRIDE
+	void Bind(LDAPInterface* i, const std::string& who, const std::string& pass) override
 	{
-		LDAPBind* b = new LDAPBind(this, i, who, pass);
-		QueueRequest(b);
+		QueueRequest(new LDAPBind(this, i, who, pass));
 	}
 
-	void Search(LDAPInterface* i, const std::string& base, const std::string& filter) CXX11_OVERRIDE
+	void Search(LDAPInterface* i, const std::string& base, const std::string& filter) override
 	{
-		if (i == NULL)
+		if (!i)
 			throw LDAPException("No interface");
 
-		LDAPSearchRequest* s = new LDAPSearchRequest(this, i, base, searchscope, filter);
-		QueueRequest(s);
+		QueueRequest(new LDAPSearchRequest(this, i, base, searchscope, filter));
 	}
 
-	void Add(LDAPInterface* i, const std::string& dn, LDAPMods& attributes) CXX11_OVERRIDE
+	void Add(LDAPInterface* i, const std::string& dn, LDAPMods& attributes) override
 	{
-		LDAPAdd* add = new LDAPAdd(this, i, dn, attributes);
-		QueueRequest(add);
+		QueueRequest(new LDAPAdd(this, i, dn, attributes));
 	}
 
-	void Del(LDAPInterface* i, const std::string& dn) CXX11_OVERRIDE
+	void Del(LDAPInterface* i, const std::string& dn) override
 	{
-		LDAPDel* del = new LDAPDel(this, i, dn);
-		QueueRequest(del);
+		QueueRequest(new LDAPDel(this, i, dn));
 	}
 
-	void Modify(LDAPInterface* i, const std::string& base, LDAPMods& attributes) CXX11_OVERRIDE
+	void Modify(LDAPInterface* i, const std::string& base, LDAPMods& attributes) override
 	{
-		LDAPModify* mod = new LDAPModify(this, i, base, attributes);
-		QueueRequest(mod);
+		QueueRequest(new LDAPModify(this, i, base, attributes));
 	}
 
-	void Compare(LDAPInterface* i, const std::string& dn, const std::string& attr, const std::string& val) CXX11_OVERRIDE
+	void Compare(LDAPInterface* i, const std::string& dn, const std::string& attr, const std::string& val) override
 	{
-		LDAPCompare* comp = new LDAPCompare(this, i, dn, attr, val);
-		QueueRequest(comp);
+		QueueRequest(new LDAPCompare(this, i, dn, attr, val));
 	}
 
- private:
+private:
 	void BuildReply(int res, LDAPRequest* req)
 	{
 		LDAPResult* ldap_result = req->result = new LDAPResult();
@@ -466,11 +473,11 @@ class LDAPService : public LDAPProvider, public SocketThread
 
 		if (res != req->success)
 		{
-			ldap_result->error = InspIRCd::Format("%s (%s)", ldap_err2string(res), req->info().c_str());
+			ldap_result->error = INSP_FORMAT("{} ({})", ldap_err2string(res), req->info());
 			return;
 		}
 
-		if (req->message == NULL)
+		if (!req->message)
 		{
 			return;
 		}
@@ -482,14 +489,14 @@ class LDAPService : public LDAPProvider, public SocketThread
 			LDAPAttributes attributes;
 
 			char* dn = ldap_get_dn(this->con, cur);
-			if (dn != NULL)
+			if (dn != nullptr)
 			{
 				attributes["dn"].push_back(dn);
 				ldap_memfree(dn);
-				dn = NULL;
+				dn = nullptr;
 			}
 
-			BerElement* ber = NULL;
+			BerElement* ber = nullptr;
 
 			for (char* attr = ldap_first_attribute(this->con, cur, &ber); attr; attr = ldap_next_attribute(this->con, cur, ber))
 			{
@@ -498,13 +505,13 @@ class LDAPService : public LDAPProvider, public SocketThread
 
 				std::vector<std::string> attrs;
 				for (int j = 0; j < count; ++j)
-					attrs.push_back(vals[j]->bv_val);
+					attrs.emplace_back(vals[j]->bv_val);
 				attributes[attr] = attrs;
 
 				ldap_value_free_len(vals);
 				ldap_memfree(attr);
 			}
-			if (ber != NULL)
+			if (ber != nullptr)
 				ber_free(ber, 0);
 
 			ldap_result->messages.push_back(attributes);
@@ -513,7 +520,7 @@ class LDAPService : public LDAPProvider, public SocketThread
 
 	void SendRequests()
 	{
-		process_mutex.Lock();
+		process_mutex.lock();
 
 		query_queue q;
 		this->LockQueue();
@@ -522,13 +529,12 @@ class LDAPService : public LDAPProvider, public SocketThread
 
 		if (q.empty())
 		{
-			process_mutex.Unlock();
+			process_mutex.unlock();
 			return;
 		}
 
-		for (unsigned int i = 0; i < q.size(); ++i)
+		for (auto* req : q)
 		{
-			LDAPRequest* req = q[i];
 			int ret = req->run();
 
 			if (ret == LDAP_SERVER_DOWN || ret == LDAP_TIMEOUT)
@@ -538,7 +544,7 @@ class LDAPService : public LDAPProvider, public SocketThread
 				{
 					Reconnect();
 				}
-				catch (const LDAPException &)
+				catch (const LDAPException&)
 				{
 				}
 
@@ -554,13 +560,13 @@ class LDAPService : public LDAPProvider, public SocketThread
 
 		this->NotifyParent();
 
-		process_mutex.Unlock();
+		process_mutex.unlock();
 	}
 
- public:
-	void Run() CXX11_OVERRIDE
+public:
+	void OnStart() override
 	{
-		while (!this->GetExitFlag())
+		while (!this->IsStopping())
 		{
 			this->LockQueue();
 			if (this->queries.empty())
@@ -571,7 +577,7 @@ class LDAPService : public LDAPProvider, public SocketThread
 		}
 	}
 
-	void OnNotify() CXX11_OVERRIDE
+	void OnNotify() override
 	{
 		query_queue r;
 
@@ -579,9 +585,8 @@ class LDAPService : public LDAPProvider, public SocketThread
 		this->results.swap(r);
 		this->UnlockQueue();
 
-		for (unsigned int i = 0; i < r.size(); ++i)
+		for (auto* req : r)
 		{
-			LDAPRequest* req = r[i];
 			LDAPInterface* li = req->inter;
 			LDAPResult* res = req->result;
 
@@ -600,22 +605,26 @@ class LDAPService : public LDAPProvider, public SocketThread
 	}
 };
 
-class ModuleLDAP : public Module
+class ModuleLDAP final
+	: public Module
 {
 	typedef insp::flat_map<std::string, LDAPService*> ServiceMap;
 	ServiceMap LDAPServices;
 
- public:
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+public:
+	void init() override
+	{
+		ServerInstance->Logs.Normal(MODNAME, "Module was compiled against LDAP ({}) version {}.{}.{}",
+			LDAP_VENDOR_NAME, LDAP_VENDOR_VERSION_MAJOR, LDAP_VENDOR_VERSION_MINOR,
+			LDAP_VENDOR_VERSION_PATCH);
+	}
+	void ReadConfig(ConfigStatus& status) override
 	{
 		ServiceMap conns;
 
-		ConfigTagList tags = ServerInstance->Config->ConfTags("database");
-		for (ConfigIter i = tags.first; i != tags.second; i++)
+		for (const auto& [_, tag] : ServerInstance->Config->ConfTags("database"))
 		{
-			const reference<ConfigTag>& tag = i->second;
-
-			if (!stdalgo::string::equalsci(tag->getString("module"), "ldap"))
+			if (!insp::equalsci(tag->getString("module"), "ldap"))
 				continue;
 
 			std::string id = tag->getString("id");
@@ -623,11 +632,11 @@ class ModuleLDAP : public Module
 			ServiceMap::iterator curr = LDAPServices.find(id);
 			if (curr == LDAPServices.end())
 			{
-				LDAPService* conn = new LDAPService(this, tag);
+				auto* conn = new LDAPService(this, tag);
 				conns[id] = conn;
 
-				ServerInstance->Modules->AddService(*conn);
-				ServerInstance->Threads.Start(conn);
+				ServerInstance->Modules.AddService(*conn);
+				conn->Start();
 			}
 			else
 			{
@@ -636,11 +645,10 @@ class ModuleLDAP : public Module
 			}
 		}
 
-		for (ServiceMap::iterator i = LDAPServices.begin(); i != LDAPServices.end(); ++i)
+		for (auto& [_, conn] : LDAPServices)
 		{
-			LDAPService* conn = i->second;
-			ServerInstance->Modules->DelService(*conn);
-			conn->join();
+			ServerInstance->Modules.DelService(*conn);
+			conn->Stop();
 			conn->OnNotify();
 			delete conn;
 		}
@@ -648,16 +656,14 @@ class ModuleLDAP : public Module
 		LDAPServices.swap(conns);
 	}
 
-	void OnUnloadModule(Module* m) CXX11_OVERRIDE
+	void OnUnloadModule(Module* m) override
 	{
-		for (ServiceMap::iterator it = this->LDAPServices.begin(); it != this->LDAPServices.end(); ++it)
+		for (const auto& [_, s] : LDAPServices)
 		{
-			LDAPService* s = it->second;
-
-			s->process_mutex.Lock();
+			s->process_mutex.lock();
 			s->LockQueue();
 
-			for (unsigned int i = s->queries.size(); i > 0; --i)
+			for (size_t i = s->queries.size(); i > 0; --i)
 			{
 				LDAPRequest* req = s->queries[i - 1];
 				LDAPInterface* li = req->inter;
@@ -669,7 +675,7 @@ class ModuleLDAP : public Module
 				}
 			}
 
-			for (unsigned int i = s->results.size(); i > 0; --i)
+			for (size_t i = s->results.size(); i > 0; --i)
 			{
 				LDAPRequest* req = s->results[i - 1];
 				LDAPInterface* li = req->inter;
@@ -682,25 +688,24 @@ class ModuleLDAP : public Module
 			}
 
 			s->UnlockQueue();
-			s->process_mutex.Unlock();
+			s->process_mutex.unlock();
 		}
 	}
 
-	~ModuleLDAP()
+	ModuleLDAP()
+		: Module(VF_VENDOR, "Provides the ability for LDAP modules to query a LDAP directory.")
 	{
-		for (ServiceMap::iterator i = LDAPServices.begin(); i != LDAPServices.end(); ++i)
+	}
+
+	~ModuleLDAP() override
+	{
+		for (auto& [_, conn] : LDAPServices)
 		{
-			LDAPService* conn = i->second;
-			conn->join();
+			conn->Stop();
 			conn->OnNotify();
 			delete conn;
 		}
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Provides the ability for LDAP modules to query a LDAP directory.", VF_VENDOR);
-	}
 };
 
 int LDAPBind::run()
@@ -709,7 +714,7 @@ int LDAPBind::run()
 	cred.bv_val = strdup(pass.c_str());
 	cred.bv_len = pass.length();
 
-	int i = ldap_sasl_bind_s(service->GetConnection(), LDAP_STR(who), LDAP_SASL_SIMPLE, &cred, NULL, NULL, NULL);
+	int i = ldap_sasl_bind_s(service->GetConnection(), LDAP_STR(who), LDAP_SASL_SIMPLE, &cred, nullptr, nullptr, nullptr);
 
 	free(cred.bv_val);
 
@@ -723,7 +728,7 @@ std::string LDAPBind::info()
 
 int LDAPSearchRequest::run()
 {
-	return ldap_search_ext_s(service->GetConnection(), LDAP_STR(base), searchscope, LDAP_STR(filter), NULL, 0, NULL, NULL, LDAP_TIME(tv), 0, &message);
+	return ldap_search_ext_s(service->GetConnection(), LDAP_STR(base), searchscope, LDAP_STR(filter), nullptr, 0, nullptr, nullptr, LDAP_TIME(tv), 0, &message);
 }
 
 std::string LDAPSearchRequest::info()
@@ -734,7 +739,7 @@ std::string LDAPSearchRequest::info()
 int LDAPAdd::run()
 {
 	LDAPMod** mods = LDAPService::BuildMods(attributes);
-	int i = ldap_add_ext_s(service->GetConnection(), LDAP_STR(dn), mods, NULL, NULL);
+	int i = ldap_add_ext_s(service->GetConnection(), LDAP_STR(dn), mods, nullptr, nullptr);
 	LDAPService::FreeMods(mods);
 	return i;
 }
@@ -746,7 +751,7 @@ std::string LDAPAdd::info()
 
 int LDAPDel::run()
 {
-	return ldap_delete_ext_s(service->GetConnection(), LDAP_STR(dn), NULL, NULL);
+	return ldap_delete_ext_s(service->GetConnection(), LDAP_STR(dn), nullptr, nullptr);
 }
 
 std::string LDAPDel::info()
@@ -757,7 +762,7 @@ std::string LDAPDel::info()
 int LDAPModify::run()
 {
 	LDAPMod** mods = LDAPService::BuildMods(attributes);
-	int i = ldap_modify_ext_s(service->GetConnection(), LDAP_STR(base), mods, NULL, NULL);
+	int i = ldap_modify_ext_s(service->GetConnection(), LDAP_STR(base), mods, nullptr, nullptr);
 	LDAPService::FreeMods(mods);
 	return i;
 }
@@ -774,9 +779,9 @@ int LDAPCompare::run()
 	cred.bv_len = val.length();
 
 #ifdef _WIN32
-	int ret = ldap_compare_ext_s(service->GetConnection(), LDAP_STR(dn), LDAP_STR(attr), NULL, &cred, NULL, NULL);
+	int ret = ldap_compare_ext_s(service->GetConnection(), LDAP_STR(dn), LDAP_STR(attr), nullptr, &cred, nullptr, nullptr);
 #else
-	int ret = ldap_compare_ext_s(service->GetConnection(), dn.c_str(), attr.c_str(), &cred, NULL, NULL);
+	int ret = ldap_compare_ext_s(service->GetConnection(), dn.c_str(), attr.c_str(), &cred, nullptr, nullptr);
 #endif
 
 	free(cred.bv_val);
