@@ -1,13 +1,13 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2016 Adam <Adam@anope.org>
+ *   Copyright (C) 2024 Larry Williamson <l422y@l422y.com>
  *   Copyright (C) 2015 Daniel Vassdal <shutter@canternet.org>
- *   Copyright (C) 2013, 2016-2020, 2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2016-2017, 2019-2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2015 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2008 Thomas Stagner <aquanight@inspircd.org>
+ *   Copyright (C) 2008 Thomas Stagner <aquanight@gmail.com>
  *   Copyright (C) 2007, 2009 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2007, 2009 Craig Edwards <brain@inspircd.org>
  *   Copyright (C) 2007 Robin Burchell <robin+git@viroteck.net>
@@ -26,40 +26,22 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/// $CompilerFlags: -Iexecute("pg_config --includedir" "POSTGRESQL_INCLUDE_DIR")
-/// $LinkerFlags: -Lexecute("pg_config --libdir" "POSTGRESQL_LIBRARY_DIR") -lpq
+/// $CompilerFlags: find_compiler_flags("libpq")
+/// $LinkerFlags: find_linker_flags("libpq")
 
-/// $PackageInfo: require_system("arch") postgresql-libs
-/// $PackageInfo: require_system("centos") postgresql-devel
-/// $PackageInfo: require_system("darwin") libpq
-/// $PackageInfo: require_system("debian") libpq-dev
-/// $PackageInfo: require_system("rocky") postgresql-devel
-/// $PackageInfo: require_system("ubuntu") libpq-dev
+/// $PackageInfo: require_system("alpine") pcre2-dev pkgconf
+/// $PackageInfo: require_system("arch") pkgconf postgresql-libs
+/// $PackageInfo: require_system("darwin") libpq pkg-config
+/// $PackageInfo: require_system("debian~") libpq-dev pkg-config
+/// $PackageInfo: require_system("rhel~") pkg-config postgresql-devel
 
 
-#ifdef __GNUC__
-# pragma GCC diagnostic push
-#endif
-
-// Fix warnings about the use of commas at end of enumerator lists on C++03.
-#if defined __clang__
-# pragma clang diagnostic ignored "-Wc++11-extensions"
-#elif defined __GNUC__
-# if (__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 8))
-#  pragma GCC diagnostic ignored "-Wpedantic"
-# else
-#  pragma GCC diagnostic ignored "-pedantic"
-# endif
-#endif
+#include <libpq-fe.h>
+#include <pg_config.h>
 
 #include "inspircd.h"
-#include <cstdlib>
-#include <libpq-fe.h>
 #include "modules/sql.h"
-
-#ifdef __GNUC__
-# pragma GCC diagnostic pop
-#endif
+#include "utility/string.h"
 
 /* SQLConn rewritten by peavey to
  * use EventHandler instead of
@@ -92,22 +74,29 @@ enum SQLstatus
 	WWRITE
 };
 
-class ReconnectTimer : public Timer
+class ReconnectTimer final
+	: public Timer
 {
- private:
+private:
 	ModulePgSQL* mod;
- public:
-	ReconnectTimer(ModulePgSQL* m) : Timer(5, false), mod(m)
+public:
+	ReconnectTimer(ModulePgSQL* m)
+		: Timer(5, false)
+		, mod(m)
 	{
 	}
-	bool Tick(time_t TIME) CXX11_OVERRIDE;
+	bool Tick() override;
 };
 
-struct QueueItem
+struct QueueItem final
 {
 	SQL::Query* c;
 	std::string q;
-	QueueItem(SQL::Query* C, const std::string& Q) : c(C), q(Q) {}
+	QueueItem(SQL::Query* C, const std::string& Q)
+		: c(C)
+		, q(Q)
+	{
+	}
 };
 
 /** PgSQLresult is a subclass of the mostly-pure-virtual class SQLresult.
@@ -117,11 +106,12 @@ struct QueueItem
  * data is passes to the module nearly as directly as if it was using the API directly itself.
  */
 
-class PgSQLresult : public SQL::Result
+class PgSQLresult final
+	: public SQL::Result
 {
 	PGresult* res;
-	int currentrow;
-	int rows;
+	int currentrow = 0;
+	int rows = 0;
 	std::vector<std::string> colnames;
 
 	void getColNames()
@@ -132,32 +122,33 @@ class PgSQLresult : public SQL::Result
 			colnames[i] = PQfname(res, i);
 		}
 	}
- public:
-	PgSQLresult(PGresult* result) : res(result), currentrow(0)
+public:
+	PgSQLresult(PGresult* result)
+		: res(result)
 	{
 		rows = PQntuples(res);
 		if (!rows)
 			rows = ConvToNum<int>(PQcmdTuples(res));
 	}
 
-	~PgSQLresult()
+	~PgSQLresult() override
 	{
 		PQclear(res);
 	}
 
-	int Rows() CXX11_OVERRIDE
+	int Rows() override
 	{
 		return rows;
 	}
 
-	void GetCols(std::vector<std::string>& result) CXX11_OVERRIDE
+	void GetCols(std::vector<std::string>& result) override
 	{
 		if (colnames.empty())
 			getColNames();
 		result = colnames;
 	}
 
-	bool HasColumn(const std::string& column, size_t& index) CXX11_OVERRIDE
+	bool HasColumn(const std::string& column, size_t& index) override
 	{
 		if (colnames.empty())
 			getColNames();
@@ -177,17 +168,18 @@ class PgSQLresult : public SQL::Result
 	{
 		char* v = PQgetvalue(res, row, column);
 		if (!v || PQgetisnull(res, row, column))
-			return SQL::Field();
+			return std::nullopt;
 
-		return SQL::Field(std::string(v, PQgetlength(res, row, column)));
+		return std::string(v, PQgetlength(res, row, column));
 	}
 
-	bool GetRow(SQL::Row& result) CXX11_OVERRIDE
+	bool GetRow(SQL::Row& result) override
 	{
 		if (currentrow >= PQntuples(res))
 			return false;
 		int ncols = PQnfields(res);
-
+		
+		result.clear();
 		for(int i = 0; i < ncols; i++)
 		{
 			result.push_back(GetValue(currentrow, i));
@@ -200,34 +192,34 @@ class PgSQLresult : public SQL::Result
 
 /** SQLConn represents one SQL session.
  */
-class SQLConn : public SQL::Provider, public EventHandler
+class SQLConn final
+	: public SQL::Provider
+	, public EventHandler
 {
- public:
-	reference<ConfigTag> conf;	/* The <database> entry */
+public:
+	std::shared_ptr<ConfigTag> conf; /* The <database> entry */
 	std::deque<QueueItem> queue;
-	PGconn*			sql;		/* PgSQL database connection handle */
-	SQLstatus		status;		/* PgSQL database connection status */
-	QueueItem		qinprog;	/* If there is currently a query in progress */
+	PGconn* sql = nullptr; /* PgSQL database connection handle */
+	SQLstatus status = CWRITE; /* PgSQL database connection status */
+	QueueItem qinprog; /* If there is currently a query in progress */
 
-	SQLConn(Module* Creator, ConfigTag* tag)
+	SQLConn(Module* Creator, const std::shared_ptr<ConfigTag>& tag)
 		: SQL::Provider(Creator, tag->getString("id"))
 		, conf(tag)
-		, sql(NULL)
-		, status(CWRITE)
-		, qinprog(NULL, "")
+		, qinprog(nullptr, "")
 	{
 		if (!DoConnect())
 			DelayReconnect();
 	}
 
-	CullResult cull() CXX11_OVERRIDE
+	Cullable::Result Cull() override
 	{
-		this->SQL::Provider::cull();
-		ServerInstance->Modules->DelService(*this);
-		return this->EventHandler::cull();
+		this->SQL::Provider::Cull();
+		ServerInstance->Modules.DelService(*this);
+		return this->EventHandler::Cull();
 	}
 
-	~SQLConn()
+	~SQLConn() override
 	{
 		SQL::Error err(SQL::BAD_DBID);
 		if (qinprog.c)
@@ -235,26 +227,26 @@ class SQLConn : public SQL::Provider, public EventHandler
 			qinprog.c->OnError(err);
 			delete qinprog.c;
 		}
-		for(std::deque<QueueItem>::iterator i = queue.begin(); i != queue.end(); i++)
+		for (const auto& item : queue)
 		{
-			SQL::Query* q = i->c;
+			SQL::Query* q = item.c;
 			q->OnError(err);
 			delete q;
 		}
 		Close();
 	}
 
-	void OnEventHandlerRead() CXX11_OVERRIDE
+	void OnEventHandlerRead() override
 	{
 		DoEvent();
 	}
 
-	void OnEventHandlerWrite() CXX11_OVERRIDE
+	void OnEventHandlerWrite() override
 	{
 		DoEvent();
 	}
 
-	void OnEventHandlerError(int errornum) CXX11_OVERRIDE
+	void OnEventHandlerError(int errornum) override
 	{
 		DelayReconnect();
 	}
@@ -306,7 +298,7 @@ class SQLConn : public SQL::Provider, public EventHandler
 		if (conf->readString("pass", item))
 			conninfo << " password = '" << EscapeDSN(item) << "'";
 
-		if (conf->getBool("ssl"))
+		if (conf->getBool("tls", conf->getBool("ssl", true)))
 			conninfo << " sslmode = 'require'";
 		else
 			conninfo << " sslmode = 'disable'";
@@ -316,8 +308,8 @@ class SQLConn : public SQL::Provider, public EventHandler
 
 	bool HandleConnectError(const char* reason)
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Could not connect to the \"%s\" database: %s",
-			GetId().c_str(), reason);
+		ServerInstance->Logs.Critical(MODNAME, "Could not connect to the \"{}\" database: {}",
+			GetId(), reason);
 		return false;
 	}
 
@@ -427,7 +419,7 @@ restart:
 				}
 
 				delete qinprog.c;
-				qinprog = QueueItem(NULL, "");
+				qinprog = QueueItem(nullptr, "");
 				goto restart;
 			}
 			else
@@ -460,28 +452,28 @@ restart:
 		}
 	}
 
-	void Submit(SQL::Query *req, const std::string& q) CXX11_OVERRIDE
+	void Submit(SQL::Query* req, const std::string& q) override
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Executing PostgreSQL query: " + q);
+		ServerInstance->Logs.Debug(MODNAME, "Executing PostgreSQL query: {}", q);
 		if (qinprog.q.empty())
 		{
-			DoQuery(QueueItem(req,q));
+			DoQuery(QueueItem(req, q));
 		}
 		else
 		{
 			// wait your turn.
-			queue.push_back(QueueItem(req,q));
+			queue.emplace_back(req, q);
 		}
 	}
 
-	void Submit(SQL::Query *req, const std::string& q, const SQL::ParamList& p) CXX11_OVERRIDE
+	void Submit(SQL::Query* req, const std::string& q, const SQL::ParamList& p) override
 	{
 		std::string res;
 		unsigned int param = 0;
-		for(std::string::size_type i = 0; i < q.length(); i++)
+		for (const auto chr : q)
 		{
-			if (q[i] != '?')
-				res.push_back(q[i]);
+			if (chr != '?')
+				res.push_back(chr);
 			else
 			{
 				if (param < p.size())
@@ -489,17 +481,17 @@ restart:
 					std::string parm = p[param++];
 					std::vector<char> buffer(parm.length() * 2 + 1);
 					int error;
-					size_t escapedsize = PQescapeStringConn(sql, &buffer[0], parm.data(), parm.length(), &error);
+					size_t escapedsize = PQescapeStringConn(sql, buffer.data(), parm.data(), parm.length(), &error);
 					if (error)
-						ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "BUG: Apparently PQescapeStringConn() failed");
-					res.append(&buffer[0], escapedsize);
+						ServerInstance->Logs.Debug(MODNAME, "BUG: Apparently PQescapeStringConn() failed");
+					res.append(buffer.data(), escapedsize);
 				}
 			}
 		}
 		Submit(req, res);
 	}
 
-	void Submit(SQL::Query *req, const std::string& q, const SQL::ParamMap& p) CXX11_OVERRIDE
+	void Submit(SQL::Query* req, const std::string& q, const SQL::ParamMap& p) override
 	{
 		std::string res;
 		for(std::string::size_type i = 0; i < q.length(); i++)
@@ -520,10 +512,10 @@ restart:
 					std::string parm = it->second;
 					std::vector<char> buffer(parm.length() * 2 + 1);
 					int error;
-					size_t escapedsize = PQescapeStringConn(sql, &buffer[0], parm.data(), parm.length(), &error);
+					size_t escapedsize = PQescapeStringConn(sql, buffer.data(), parm.data(), parm.length(), &error);
 					if (error)
-						ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "BUG: Apparently PQescapeStringConn() failed");
-					res.append(&buffer[0], escapedsize);
+						ServerInstance->Logs.Debug(MODNAME, "BUG: Apparently PQescapeStringConn() failed");
+					res.append(buffer.data(), escapedsize);
 				}
 			}
 		}
@@ -563,29 +555,45 @@ restart:
 		if(sql)
 		{
 			PQfinish(sql);
-			sql = NULL;
+			sql = nullptr;
 		}
 	}
 };
 
-class ModulePgSQL : public Module
+class ModulePgSQL final
+	: public Module
 {
- public:
+public:
 	ConnMap connections;
-	ReconnectTimer* retimer;
+	ReconnectTimer* retimer = nullptr;
 
 	ModulePgSQL()
-		: retimer(NULL)
+		: Module(VF_VENDOR, "Provides the ability for SQL modules to query a PostgreSQL database.")
 	{
 	}
 
-	~ModulePgSQL()
+	~ModulePgSQL() override
 	{
 		delete retimer;
 		ClearAllConnections();
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void init() override
+	{
+		int pqversion = PQlibVersion();
+		int minor = pqversion / 100 % 100;
+		int revision = pqversion % 100;
+		if (pqversion >= 10'00'00)
+		{
+			// Ref: https://www.postgresql.org/docs/current/libpq-misc.html#LIBPQ-PQLIBVERSION
+			minor = revision;
+			revision = 0;
+		}
+		ServerInstance->Logs.Normal(MODNAME, "Module was compiled against libpq version {} and is running against version {}.{}.{}",
+			PG_VERSION, pqversion / 10000, minor, revision);
+	}
+
+	void ReadConfig(ConfigStatus& status) override
 	{
 		ReadConf();
 	}
@@ -593,20 +601,21 @@ class ModulePgSQL : public Module
 	void ReadConf()
 	{
 		ConnMap conns;
-		ConfigTagList tags = ServerInstance->Config->ConfTags("database");
-		for(ConfigIter i = tags.first; i != tags.second; i++)
+
+		for (const auto& [_, tag] : ServerInstance->Config->ConfTags("database"))
 		{
-			if (!stdalgo::string::equalsci(i->second->getString("module"), "pgsql"))
+			if (!insp::equalsci(tag->getString("module"), "pgsql"))
 				continue;
-			std::string id = i->second->getString("id");
+
+			std::string id = tag->getString("id");
 			ConnMap::iterator curr = connections.find(id);
 			if (curr == connections.end())
 			{
-				SQLConn* conn = new SQLConn(this, i->second);
+				auto* conn = new SQLConn(this, tag);
 				if (conn->status != DEAD)
 				{
-					conns.insert(std::make_pair(id, conn));
-					ServerInstance->Modules->AddService(*conn);
+					conns.emplace(id, conn);
+					ServerInstance->Modules.AddService(*conn);
 				}
 				// If the connection is dead it has already been queued for culling
 				// at the end of the main loop so we don't need to delete it here.
@@ -623,25 +632,24 @@ class ModulePgSQL : public Module
 
 	void ClearAllConnections()
 	{
-		for(ConnMap::iterator i = connections.begin(); i != connections.end(); i++)
+		for (const auto& [_, conn] : connections)
 		{
-			i->second->cull();
-			delete i->second;
+			conn->Cull();
+			delete conn;
 		}
 		connections.clear();
 	}
 
-	void OnUnloadModule(Module* mod) CXX11_OVERRIDE
+	void OnUnloadModule(Module* mod) override
 	{
 		SQL::Error err(SQL::BAD_DBID);
-		for(ConnMap::iterator i = connections.begin(); i != connections.end(); i++)
+		for (const auto& [_, conn] : connections)
 		{
-			SQLConn* conn = i->second;
 			if (conn->qinprog.c && conn->qinprog.c->creator == mod)
 			{
 				conn->qinprog.c->OnError(err);
 				delete conn->qinprog.c;
-				conn->qinprog.c = NULL;
+				conn->qinprog.c = nullptr;
 			}
 			std::deque<QueueItem>::iterator j = conn->queue.begin();
 			while (j != conn->queue.end())
@@ -658,16 +666,11 @@ class ModulePgSQL : public Module
 			}
 		}
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Provides the ability for SQL modules to query a PostgreSQL database.", VF_VENDOR);
-	}
 };
 
-bool ReconnectTimer::Tick(time_t time)
+bool ReconnectTimer::Tick()
 {
-	mod->retimer = NULL;
+	mod->retimer = nullptr;
 	mod->ReadConf();
 	delete this;
 	return false;
@@ -676,7 +679,7 @@ bool ReconnectTimer::Tick(time_t time)
 void SQLConn::DelayReconnect()
 {
 	status = DEAD;
-	ModulePgSQL* mod = (ModulePgSQL*)(Module*)creator;
+	auto* mod = static_cast<ModulePgSQL*>(creator.ptr());
 
 	ConnMap::iterator it = mod->connections.find(conf->getString("id"));
 	if (it != mod->connections.end())
