@@ -1,12 +1,11 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2020 Matt Schatz <genius3000@g3k.solutions>
- *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2017, 2023 Wade Cline <wadecline@hotmail.com>
- *   Copyright (C) 2016 Adam <Adam@anope.org>
+ *   Copyright (C) 2014, 2016-2017, 2019-2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2014 Julien Vehent <julien@linuxwall.info>
- *   Copyright (C) 2013-2014, 2016-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2017 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012 ChrisTX <xpipe@hotmail.de>
@@ -32,89 +31,46 @@
 /// $CompilerFlags: find_compiler_flags("openssl" "")
 /// $LinkerFlags: find_linker_flags("openssl" "-lssl -lcrypto")
 
+/// $PackageInfo: require_system("alpine") openssl-dev pkgconf
 /// $PackageInfo: require_system("arch") openssl pkgconf
-/// $PackageInfo: require_system("centos") openssl-devel pkgconfig
 /// $PackageInfo: require_system("darwin") openssl pkg-config
-/// $PackageInfo: require_system("debian") libssl-dev openssl pkg-config
-/// $PackageInfo: require_system("rocky") openssl-devel pkgconfig
-/// $PackageInfo: require_system("ubuntu") libssl-dev openssl pkg-config
+/// $PackageInfo: require_system("debian~") libssl-dev openssl pkg-config
+/// $PackageInfo: require_system("rhel~") openssl-devel pkgconfig
 
 
 #include "inspircd.h"
 #include "iohook.h"
 #include "modules/ssl.h"
-
-#ifdef __GNUC__
-# pragma GCC diagnostic push
-#endif
-
-// Ignore OpenSSL deprecation warnings on OS X Lion and newer.
-#if defined __APPLE__
-# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
-
-// LibreSSL lies about the version of OpenSSL it is compatible with and is a general pain
-// to support. Support for it was removed in the master branch at the same time that
-// support for OpenSSL pre-1.1 was.
-#if defined __GNUC__ && defined LIBRESSL_VERSION_NUMBER
-# undef OPENSSL_VERSION_NUMBER
-# define OPENSSL_VERSION_NUMBER 0x10000000L
-# warning LibreSSL support will be removed in v4. Consider using the ssl_gnutls or ssl_mbedtls modules instead if you can not use OpenSSL.
-#endif
-
-// Fix warnings about the use of `long long` on C++03.
-#if defined __clang__
-# pragma clang diagnostic ignored "-Wc++11-long-long"
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wlong-long"
-#endif
+#include "stringutils.h"
+#include "timeutils.h"
+#include "utility/string.h"
 
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/dh.h>
 
-#ifdef __GNUC__
-# pragma GCC diagnostic pop
-#endif
-
 #ifdef _WIN32
 # define timegm _mkgmtime
 # pragma comment(lib, "libcrypto.lib")
 # pragma comment(lib, "libssl.lib")
 #endif
 
-// Compatibility layer to allow OpenSSL 1.0 to use the 1.1 API.
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-
-// BIO is opaque in OpenSSL 1.1 but the access API does not exist in 1.0.
-# define BIO_get_data(BIO) BIO->ptr
-# define BIO_set_data(BIO, VALUE) BIO->ptr = VALUE;
-# define BIO_set_init(BIO, VALUE) BIO->init = VALUE;
-
-// These functions have been renamed in OpenSSL 1.1.
-# define OpenSSL_version SSLeay_version
-# define X509_getm_notAfter X509_get_notAfter
-# define X509_getm_notBefore X509_get_notBefore
-# define OPENSSL_init_ssl(OPTIONS, SETTINGS) \
-	SSL_library_init(); \
-	SSL_load_error_strings();
-
-// These macros have been renamed in OpenSSL 1.1.
-# define OPENSSL_VERSION SSLEAY_VERSION
-
+#if OPENSSL_VERSION_NUMBER < 0x10101000L
+# error OpenSSL 1.1.1 or newer is required by the ssl_openssl module.
+#elif OPENSSL_VERSION_NUMBER < 0x30000000L
+# define OPENSSL_VERSION_STR OPENSSL_VERSION_TEXT
+# define OPENSSL_VERSION_STRING OPENSSL_VERSION
 #else
-# define INSPIRCD_OPENSSL_OPAQUE_BIO
-# if OPENSSL_VERSION_NUMBER >= 0x30000000L
-#  define INSPIRCD_OPENSSL_AUTO_DH
-# endif
+# define INSPIRCD_OPENSSL_AUTO_DH
 #endif
 
 static bool SelfSigned = false;
 static int exdataindex;
+static Module* thismod;
 
 char* get_error()
 {
-	return ERR_error_string(ERR_get_error(), NULL);
+	return ERR_error_string(ERR_get_error(), nullptr);
 }
 
 static int OnVerify(int preverify_ok, X509_STORE_CTX* ctx);
@@ -122,26 +78,29 @@ static void StaticSSLInfoCallback(const SSL* ssl, int where, int rc);
 
 namespace OpenSSL
 {
-	class Exception : public ModuleException
+	class Exception final
+		: public ModuleException
 	{
-	 public:
-		Exception(const std::string& reason)
-			: ModuleException(reason) { }
+	public:
+		Exception(const std::string& msg)
+			: ModuleException(thismod, msg)
+		{
+		}
 	};
 
 #ifndef INSPIRCD_OPENSSL_AUTO_DH
-	class DHParams
+	class DHParams final
 	{
 		DH* dh;
 
-	 public:
+	public:
 		DHParams(const std::string& filename)
 		{
 			BIO* dhpfile = BIO_new_file(filename.c_str(), "r");
-			if (dhpfile == NULL)
+			if (!dhpfile)
 				throw Exception("Couldn't open DH file " + filename);
 
-			dh = PEM_read_bio_DHparams(dhpfile, NULL, NULL, NULL);
+			dh = PEM_read_bio_DHparams(dhpfile, nullptr, nullptr, nullptr);
 			BIO_free(dhpfile);
 
 			if (!dh)
@@ -160,18 +119,18 @@ namespace OpenSSL
 	};
 #endif
 
-	class Context
+	class Context final
 	{
 		SSL_CTX* const ctx;
 		long ctx_options;
 
-	 public:
+	public:
 		Context(SSL_CTX* context)
 			: ctx(context)
 		{
 			// Sane default options for OpenSSL see https://www.openssl.org/docs/ssl/SSL_CTX_set_options.html
 			// and when choosing a cipher, use the server's preferences instead of the client preferences.
-			long opts = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION | SSL_OP_CIPHER_SERVER_PREFERENCE | SSL_OP_SINGLE_DH_USE;
+			long opts = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_TLSv1 | SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION | SSL_OP_CIPHER_SERVER_PREFERENCE | SSL_OP_SINGLE_DH_USE;
 			// Only turn options on if they exist
 #ifdef SSL_OP_SINGLE_ECDH_USE
 			opts |= SSL_OP_SINGLE_ECDH_USE;
@@ -187,7 +146,7 @@ namespace OpenSSL
 			mode |= SSL_MODE_RELEASE_BUFFERS;
 #endif
 			SSL_CTX_set_mode(ctx, mode);
-			SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);
+			SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, nullptr);
 			SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_OFF);
 			SSL_CTX_set_info_callback(ctx, StaticSSLInfoCallback);
 		}
@@ -212,21 +171,9 @@ namespace OpenSSL
 			if (nid == NID_undef)
 				throw Exception("Unknown curve: " + curvename);
 
-# if OPENSSL_VERSION_NUMBER >= 0x10101000L
 			ERR_clear_error();
 			if (!SSL_CTX_set1_groups(ctx, &nid, 1))
 				throw Exception("Couldn't set ECDH curve");
-# else
-			EC_KEY* eckey = EC_KEY_new_by_curve_name(nid);
-			if (!eckey)
-				throw Exception("Unable to create EC key object");
-
-			ERR_clear_error();
-			bool ret = (SSL_CTX_set_tmp_ecdh(ctx, eckey) >= 0);
-			EC_KEY_free(eckey);
-			if (!ret)
-				throw Exception("Couldn't set ECDH parameters");
-# endif
 		}
 #endif
 
@@ -237,14 +184,12 @@ namespace OpenSSL
 			return SSL_CTX_set_cipher_list(ctx, ciphers.c_str());
 		}
 
-#if OPENSSL_VERSION_NUMBER >= 0x10101000L
 		bool SetCiphersuites(const std::string& ciphers)
 		{
 			// TLSv1.3+ ciphers.
 			ERR_clear_error();
 			return SSL_CTX_set_ciphersuites(ctx, ciphers.c_str());
 		}
-#endif
 
 		bool SetCerts(const std::string& filename)
 		{
@@ -261,7 +206,7 @@ namespace OpenSSL
 		bool SetCA(const std::string& filename)
 		{
 			ERR_clear_error();
-			return SSL_CTX_load_verify_locations(ctx, filename.c_str(), 0);
+			return SSL_CTX_load_verify_locations(ctx, filename.c_str(), nullptr);
 		}
 
 		void SetCRL(const std::string& crlfile, const std::string& crlpath, const std::string& crlmode)
@@ -271,38 +216,37 @@ namespace OpenSSL
 
 			/* Set CRL mode */
 			unsigned long crlflags = X509_V_FLAG_CRL_CHECK;
-			if (stdalgo::string::equalsci(crlmode, "chain"))
+			if (insp::equalsci(crlmode, "chain"))
 			{
 				crlflags |= X509_V_FLAG_CRL_CHECK_ALL;
 			}
-			else if (!stdalgo::string::equalsci(crlmode, "leaf"))
+			else if (!insp::equalsci(crlmode, "leaf"))
 			{
-				throw ModuleException("Unknown mode '" + crlmode + "'; expected either 'chain' (default) or 'leaf'");
+				throw ModuleException(thismod, "Unknown mode '" + crlmode + "'; expected either 'chain' (default) or 'leaf'");
 			}
 
 			/* Load CRL files */
 			X509_STORE* store = SSL_CTX_get_cert_store(ctx);
 			if (!store)
 			{
-				throw ModuleException("Unable to get X509_STORE from TLS (SSL) context; this should never happen");
+				throw ModuleException(thismod, "Unable to get X509_STORE from TLS context; this should never happen");
 			}
 			ERR_clear_error();
 			if (!X509_STORE_load_locations(store,
-				crlfile.empty() ? NULL : crlfile.c_str(),
-				crlpath.empty() ? NULL : crlpath.c_str()))
+				crlfile.empty() ? nullptr : crlfile.c_str(),
+				crlpath.empty() ? nullptr : crlpath.c_str()))
 			{
-				int err = ERR_get_error();
-				throw ModuleException("Unable to load CRL file '" + crlfile + "' or CRL path '" + crlpath + "': '" + (err ? ERR_error_string(err, NULL) : "unknown") + "'");
+				unsigned long err = ERR_get_error();
+				throw ModuleException(thismod, "Unable to load CRL file '" + crlfile + "' or CRL path '" + crlpath + "': '" + (err ? ERR_error_string(err, nullptr) : "unknown") + "'");
 			}
 
 			/* Set CRL mode */
 			if (X509_STORE_set_flags(store, crlflags) != 1)
 			{
-				throw ModuleException("Unable to set X509 CRL flags");
+				throw ModuleException(thismod, "Unable to set X509 CRL flags");
 			}
 		}
 
-
 		long GetDefaultContextOptions() const
 		{
 			return ctx_options;
@@ -338,7 +282,7 @@ namespace OpenSSL
 		}
 	};
 
-	class Profile
+	class Profile final
 	{
 		/** Name of this profile
 		 */
@@ -357,7 +301,7 @@ namespace OpenSSL
 
 		/** Digest to use when generating fingerprints
 		 */
-		const EVP_MD* digest;
+		std::vector<const EVP_MD*> digests;
 
 		/** Last error, set by error_callback()
 		 */
@@ -383,10 +327,10 @@ namespace OpenSSL
 		 * @param tag Config tag defining this profile
 		 * @param context Context object to manipulate
 		 */
-		void SetContextOptions(const std::string& ctxname, ConfigTag* tag, Context& context)
+		void SetContextOptions(const std::string& ctxname, const std::shared_ptr<ConfigTag>& tag, Context& context)
 		{
-			long setoptions = tag->getInt(ctxname + "setoptions", 0);
-			long clearoptions = tag->getInt(ctxname + "clearoptions", 0);
+			long setoptions = tag->getNum<long>(ctxname + "setoptions", 0);
+			long clearoptions = tag->getNum<long>(ctxname + "clearoptions", 0);
 
 #ifdef SSL_OP_NO_COMPRESSION
 			// Disable compression by default
@@ -394,13 +338,9 @@ namespace OpenSSL
 				setoptions |= SSL_OP_NO_COMPRESSION;
 #endif
 
-			// Disable TLSv1.0 by default.
-			if (!tag->getBool("tlsv1", false))
-				setoptions |= SSL_OP_NO_TLSv1;
-
 #ifdef SSL_OP_NO_TLSv1_1
-			// Enable TLSv1.1 by default.
-			if (!tag->getBool("tlsv11", true))
+			// Disable TLSv1.1 by default.
+			if (!tag->getBool("tlsv11", false))
 				setoptions |= SSL_OP_NO_TLSv1_1;
 #endif
 
@@ -419,31 +359,36 @@ namespace OpenSSL
 			if (!setoptions && !clearoptions)
 				return; // Nothing to do
 
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Setting %s %s context options, default: %ld set: %ld clear: %ld", name.c_str(), ctxname.c_str(), ctx.GetDefaultContextOptions(), setoptions, clearoptions);
+			ServerInstance->Logs.Debug(MODNAME, "Setting {} {} context options, default: {} set: {} clear: {}", name, ctxname, ctx.GetDefaultContextOptions(), setoptions, clearoptions);
 			long final = context.SetRawContextOptions(setoptions, clearoptions);
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "%s %s context options: %ld", name.c_str(), ctxname.c_str(), final);
+			ServerInstance->Logs.Normal(MODNAME, "{} {} context options: {}", name, ctxname, final);
 		}
 
-	 public:
-		Profile(const std::string& profilename, ConfigTag* tag)
+	public:
+		Profile(const std::string& profilename, const std::shared_ptr<ConfigTag>& tag)
 			: name(profilename)
 #ifndef INSPIRCD_OPENSSL_AUTO_DH
 			, dh(ServerInstance->Config->Paths.PrependConfig(tag->getString("dhfile", "dhparams.pem", 1)))
 #endif
-			, ctx(SSL_CTX_new(SSLv23_server_method()))
-			, clientctx(SSL_CTX_new(SSLv23_client_method()))
+			, ctx(SSL_CTX_new(TLS_server_method()))
+			, clientctx(SSL_CTX_new(TLS_client_method()))
 			, allowrenego(tag->getBool("renegotiation")) // Disallow by default
-			, outrecsize(tag->getUInt("outrecsize", 2048, 512, 16384))
+			, outrecsize(tag->getNum<unsigned int>("outrecsize", 2048, 512, 16384))
 		{
 #ifndef INSPIRCD_OPENSSL_AUTO_DH
 			if ((!ctx.SetDH(dh)) || (!clientctx.SetDH(dh)))
 				throw Exception("Couldn't set DH parameters");
 #endif
 
-			const std::string hash = tag->getString("hash", "md5", 1);
-			digest = EVP_get_digestbyname(hash.c_str());
-			if (digest == NULL)
-				throw Exception("Unknown hash type " + hash);
+			irc::spacesepstream hashstream(tag->getString("hash", "sha256 md5", 1));
+			for (std::string hash; hashstream.GetToken(hash); )
+			{
+				const auto* digest = EVP_get_digestbyname(hash.c_str());
+				if (!digest)
+					throw Exception("Unknown hash type " + hash);
+
+				digests.push_back(digest);
+			}
 
 			const std::string ciphers = tag->getString("ciphers");
 			if (!ciphers.empty())
@@ -458,15 +403,11 @@ namespace OpenSSL
 			const std::string ciphersuites = tag->getString("ciphersuites");
 			if (!ciphersuites.empty())
 			{
-#if OPENSSL_VERSION_NUMBER >= 0x10101000L
 				if ((!ctx.SetCiphersuites(ciphersuites)) || (!clientctx.SetCiphersuites(ciphersuites)))
 				{
 					ERR_print_errors_cb(error_callback, this);
 					throw Exception("Can't set ciphersuite list to \"" + ciphersuites + "\" " + lasterr);
 				}
-#else
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "You have configured <sslprofile:ciphersuites> but your version of OpenSSL does not support TLSv1.3+");
-#endif
 			}
 
 #ifndef OPENSSL_NO_ECDH
@@ -500,7 +441,7 @@ namespace OpenSSL
 			if ((!ctx.SetCA(filename)) || (!clientctx.SetCA(filename)))
 			{
 				ERR_print_errors_cb(error_callback, this);
-				ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Can't read CA list from %s. This is only a problem if you want to verify client certificates, otherwise it's safe to ignore this message. Error: %s", filename.c_str(), lasterr.c_str());
+				ServerInstance->Logs.Normal(MODNAME, "Can't read CA list from {}. This is only a problem if you want to verify client certificates, otherwise it's safe to ignore this message. Error: {}", filename, lasterr);
 			}
 
 			// Load the CRLs.
@@ -517,7 +458,7 @@ namespace OpenSSL
 		const std::string& GetName() const { return name; }
 		SSL* CreateServerSession() { return ctx.CreateServerSession(); }
 		SSL* CreateClientSession() { return clientctx.CreateClientSession(); }
-		const EVP_MD* GetDigest() { return digest; }
+		const std::vector<const EVP_MD*> GetDigests() { return digests; }
 		bool AllowRenegotiation() const { return allowrenego; }
 		unsigned int GetOutgoingRecordSize() const { return outrecsize; }
 	};
@@ -545,10 +486,11 @@ namespace OpenSSL
 			return 0;
 		}
 
+		// These signatures are required by the BIO_meth_set_write|read interface
+		// even though they lead to shortening issues.
 		static int read(BIO* bio, char* buf, int len);
 		static int write(BIO* bio, const char* buf, int len);
 
-#ifdef INSPIRCD_OPENSSL_OPAQUE_BIO
 		static BIO_METHOD* alloc()
 		{
 			BIO_METHOD* meth = BIO_meth_new(100 | BIO_TYPE_SOURCE_SINK, "inspircd");
@@ -559,31 +501,12 @@ namespace OpenSSL
 			BIO_meth_set_destroy(meth, OpenSSL::BIOMethod::destroy);
 			return meth;
 		}
-#endif
 	}
 }
 
-// BIO_METHOD is opaque in OpenSSL 1.1 so we can't do this.
-// See OpenSSL::BIOMethod::alloc for the new method.
-#ifndef INSPIRCD_OPENSSL_OPAQUE_BIO
-static BIO_METHOD biomethods =
-{
-	(100 | BIO_TYPE_SOURCE_SINK),
-	"inspircd",
-	OpenSSL::BIOMethod::write,
-	OpenSSL::BIOMethod::read,
-	NULL, // puts
-	NULL, // gets
-	OpenSSL::BIOMethod::ctrl,
-	OpenSSL::BIOMethod::create,
-	OpenSSL::BIOMethod::destroy, // destroy, does nothing, see function body for more info
-	NULL // callback_ctrl
-};
-#else
 static BIO_METHOD* biomethods;
-#endif
 
-static int OnVerify(int preverify_ok, X509_STORE_CTX *ctx)
+static int OnVerify(int preverify_ok, X509_STORE_CTX* ctx)
 {
 	/* XXX: This will allow self signed certificates.
 	 * In the future if we want an option to not allow this,
@@ -597,11 +520,12 @@ static int OnVerify(int preverify_ok, X509_STORE_CTX *ctx)
 	return 1;
 }
 
-class OpenSSLIOHook : public SSLIOHook
+class OpenSSLIOHook final
+	: public SSLIOHook
 {
- private:
+private:
 	SSL* sess;
-	bool data_to_write;
+	bool data_to_write = false;
 
 	// Returns 1 if handshake succeeded, 0 if it is still in progress, -1 if it failed
 	int Handshake(StreamSocket* user)
@@ -655,15 +579,15 @@ class OpenSSLIOHook : public SSLIOHook
 			SSL_shutdown(sess);
 			SSL_free(sess);
 		}
-		sess = NULL;
-		certificate = NULL;
+		sess = nullptr;
+		certificate = nullptr;
 		status = STATUS_NONE;
 	}
 
 	void VerifyCertificate()
 	{
 		X509* cert;
-		ssl_cert* certinfo = new ssl_cert;
+		auto* certinfo = new ssl_cert();
 		this->certificate = certinfo;
 		unsigned int n;
 		unsigned char md[EVP_MAX_MD_SIZE];
@@ -692,29 +616,36 @@ class OpenSSLIOHook : public SSLIOHook
 		GetDNString(X509_get_subject_name(cert), certinfo->dn);
 		GetDNString(X509_get_issuer_name(cert), certinfo->issuer);
 
-		if (!X509_digest(cert, GetProfile().GetDigest(), md, &n))
+		for (const auto* digest : GetProfile().GetDigests())
 		{
-			certinfo->error = "Out of memory generating fingerprint";
-		}
-		else
-		{
-			certinfo->fingerprint = BinToHex(md, n);
+			if (!X509_digest(cert, digest, md, &n))
+			{
+				certinfo->error = "Out of memory generating fingerprint";
+			}
+			else
+			{
+				certinfo->fingerprints.push_back(Hex::Encode(md, n));
+			}
 		}
 
 		certinfo->activation = GetTime(X509_getm_notBefore(cert));
 		certinfo->expiration = GetTime(X509_getm_notAfter(cert));
 
-#if OPENSSL_VERSION_NUMBER < 0x10101000L
-# define ASN1_TIME_cmp_time_t ASN1_UTCTIME_cmp_time_t
-#endif
-
 		int activated = ASN1_TIME_cmp_time_t(X509_getm_notBefore(cert), ServerInstance->Time());
 		if (activated != -1 && activated != 0)
-			certinfo->error = "Certificate not activated";
+		{
+			certinfo->error = INSP_FORMAT("Certificate not active for {} (on {})",
+				Duration::ToString(certinfo->activation - ServerInstance->Time()),
+				Time::ToString(certinfo->activation));
+		}
 
 		int expired = ASN1_TIME_cmp_time_t(X509_getm_notAfter(cert), ServerInstance->Time());
 		if (expired != 0 && expired != 1)
-			certinfo->error = "Certificate has expired";
+		{
+			certinfo->error = INSP_FORMAT("Certificate expired {} ago (on {})",
+				Duration::ToString(ServerInstance->Time() - certinfo->expiration),
+				Time::ToString(certinfo->expiration));
+		}
 
 		X509_free(cert);
 	}
@@ -731,7 +662,6 @@ class OpenSSLIOHook : public SSLIOHook
 
 	static time_t GetTime(ASN1_TIME* x509time)
 	{
-#if OPENSSL_VERSION_NUMBER >= 0x10101000L
 		if (!x509time)
 			return 0;
 
@@ -740,10 +670,6 @@ class OpenSSLIOHook : public SSLIOHook
 			return 0;
 
 		return timegm(&ts);
-#else
-		// OpenSSL 1.1 is required for ASN_TIME_to_tm.
-		return 0;
-#endif
 	}
 
 	void SSLInfoCallback(int where, int rc)
@@ -767,7 +693,7 @@ class OpenSSLIOHook : public SSLIOHook
 		if (status != STATUS_NONE)
 			return true;
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Session %p killed, attempted to renegotiate", (void*)sess);
+		ServerInstance->Logs.Debug(MODNAME, "Session {} killed, attempted to renegotiate", fmt::ptr(sess));
 		CloseSession();
 		sock->SetError("Renegotiation is not allowed");
 		return false;
@@ -791,18 +717,13 @@ class OpenSSLIOHook : public SSLIOHook
 	// Calls our private SSLInfoCallback()
 	friend void StaticSSLInfoCallback(const SSL* ssl, int where, int rc);
 
- public:
-	OpenSSLIOHook(IOHookProvider* hookprov, StreamSocket* sock, SSL* session)
+public:
+	OpenSSLIOHook(const std::shared_ptr<IOHookProvider>& hookprov, StreamSocket* sock, SSL* session)
 		: SSLIOHook(hookprov)
 		, sess(session)
-		, data_to_write(false)
 	{
 		// Create BIO instance and store a pointer to the socket in it which will be used by the read and write functions
-#ifdef INSPIRCD_OPENSSL_OPAQUE_BIO
 		BIO* bio = BIO_new(biomethods);
-#else
-		BIO* bio = BIO_new(&biomethods);
-#endif
 		BIO_set_data(bio, sock);
 		SSL_set_bio(sess, bio, bio);
 
@@ -811,12 +732,12 @@ class OpenSSLIOHook : public SSLIOHook
 		Handshake(sock);
 	}
 
-	void OnStreamSocketClose(StreamSocket* user) CXX11_OVERRIDE
+	void OnStreamSocketClose(StreamSocket* user) override
 	{
 		CloseSession();
 	}
 
-	int OnStreamSocketRead(StreamSocket* user, std::string& recvq) CXX11_OVERRIDE
+	ssize_t OnStreamSocketRead(StreamSocket* user, std::string& recvq) override
 	{
 		// Finish handshake if needed
 		int prepret = PrepareIO(user);
@@ -827,7 +748,7 @@ class OpenSSLIOHook : public SSLIOHook
 		{
 			ERR_clear_error();
 			char* buffer = ServerInstance->GetReadBuffer();
-			size_t bufsiz = ServerInstance->Config->NetBufferSize;
+			int bufsiz = static_cast<int>(std::min<size_t>(ServerInstance->Config->NetBufferSize, INT_MAX));
 			int ret = SSL_read(sess, buffer, bufsiz);
 
 			if (!CheckRenego(user))
@@ -876,7 +797,7 @@ class OpenSSLIOHook : public SSLIOHook
 		}
 	}
 
-	int OnStreamSocketWrite(StreamSocket* user, StreamSocket::SendQueue& sendq) CXX11_OVERRIDE
+	ssize_t OnStreamSocketWrite(StreamSocket* user, StreamSocket::SendQueue& sendq) override
 	{
 		// Finish handshake if needed
 		int prepret = PrepareIO(user);
@@ -891,7 +812,7 @@ class OpenSSLIOHook : public SSLIOHook
 			ERR_clear_error();
 			FlattenSendQueue(sendq, GetProfile().GetOutgoingRecordSize());
 			const StreamSocket::SendQueue::Element& buffer = sendq.front();
-			int ret = SSL_write(sess, buffer.data(), buffer.size());
+			int ret = SSL_write(sess, buffer.data(), static_cast<int>(buffer.size()));
 
 			if (!CheckRenego(user))
 				return -1;
@@ -939,7 +860,7 @@ class OpenSSLIOHook : public SSLIOHook
 		return 1;
 	}
 
-	void GetCiphersuite(std::string& out) const CXX11_OVERRIDE
+	void GetCiphersuite(std::string& out) const override
 	{
 		if (!IsHookReady())
 			return;
@@ -947,7 +868,7 @@ class OpenSSLIOHook : public SSLIOHook
 		out.append(SSL_get_cipher(sess));
 	}
 
-	bool GetServerName(std::string& out) const CXX11_OVERRIDE
+	bool GetServerName(std::string& out) const override
 	{
 		const char* name = SSL_get_servername(sess, TLSEXT_NAMETYPE_host_name);
 		if (!name)
@@ -978,7 +899,7 @@ static int OpenSSL::BIOMethod::write(BIO* bio, const char* buffer, int size)
 		return -1;
 	}
 
-	int ret = SocketEngine::Send(sock, buffer, size, 0);
+	ssize_t ret = SocketEngine::Send(sock, buffer, size, 0);
 	if ((ret < size) && ((ret > 0) || (SocketEngine::IgnoreError())))
 	{
 		// Blocked, set retry flag for OpenSSL
@@ -986,7 +907,7 @@ static int OpenSSL::BIOMethod::write(BIO* bio, const char* buffer, int size)
 		BIO_set_retry_write(bio);
 	}
 
-	return ret;
+	return static_cast<int>(ret);
 }
 
 static int OpenSSL::BIOMethod::read(BIO* bio, char* buffer, int size)
@@ -1001,7 +922,7 @@ static int OpenSSL::BIOMethod::read(BIO* bio, char* buffer, int size)
 		return -1;
 	}
 
-	int ret = SocketEngine::Recv(sock, buffer, size, 0);
+	ssize_t ret = SocketEngine::Recv(sock, buffer, size, 0);
 	if ((ret < size) && ((ret > 0) || (SocketEngine::IgnoreError())))
 	{
 		// Blocked, set retry flag for OpenSSL
@@ -1009,34 +930,35 @@ static int OpenSSL::BIOMethod::read(BIO* bio, char* buffer, int size)
 		BIO_set_retry_read(bio);
 	}
 
-	return ret;
+	return static_cast<int>(ret);
 }
 
-class OpenSSLIOHookProvider : public SSLIOHookProvider
+class OpenSSLIOHookProvider final
+	: public SSLIOHookProvider
 {
 	OpenSSL::Profile profile;
 
- public:
-	OpenSSLIOHookProvider(Module* mod, const std::string& profilename, ConfigTag* tag)
+public:
+	OpenSSLIOHookProvider(Module* mod, const std::string& profilename, const std::shared_ptr<ConfigTag>& tag)
 		: SSLIOHookProvider(mod, profilename)
 		, profile(profilename, tag)
 	{
-		ServerInstance->Modules->AddService(*this);
+		ServerInstance->Modules.AddService(*this);
 	}
 
-	~OpenSSLIOHookProvider()
+	~OpenSSLIOHookProvider() override
 	{
-		ServerInstance->Modules->DelService(*this);
+		ServerInstance->Modules.DelService(*this);
 	}
 
-	void OnAccept(StreamSocket* sock, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server) CXX11_OVERRIDE
+	void OnAccept(StreamSocket* sock, const irc::sockets::sockaddrs& client, const irc::sockets::sockaddrs& server) override
 	{
-		new OpenSSLIOHook(this, sock, profile.CreateServerSession());
+		new OpenSSLIOHook(shared_from_this(), sock, profile.CreateServerSession());
 	}
 
-	void OnConnect(StreamSocket* sock) CXX11_OVERRIDE
+	void OnConnect(StreamSocket* sock) override
 	{
-		new OpenSSLIOHook(this, sock, profile.CreateClientSession());
+		new OpenSSLIOHook(shared_from_this(), sock, profile.CreateClientSession());
 	}
 
 	OpenSSL::Profile& GetProfile() { return profile; }
@@ -1044,112 +966,101 @@ class OpenSSLIOHookProvider : public SSLIOHookProvider
 
 OpenSSL::Profile& OpenSSLIOHook::GetProfile()
 {
-	IOHookProvider* hookprov = prov;
-	return static_cast<OpenSSLIOHookProvider*>(hookprov)->GetProfile();
+	return std::static_pointer_cast<OpenSSLIOHookProvider>(prov)->GetProfile();
 }
 
-class ModuleSSLOpenSSL : public Module
+class ModuleSSLOpenSSL final
+	: public Module
 {
-	typedef std::vector<reference<OpenSSLIOHookProvider> > ProfileList;
+	typedef std::vector<std::shared_ptr<OpenSSLIOHookProvider>> ProfileList;
 
 	ProfileList profiles;
 
 	void ReadProfiles()
 	{
 		ProfileList newprofiles;
-		ConfigTagList tags = ServerInstance->Config->ConfTags("sslprofile");
-		if (tags.first == tags.second)
-		{
-			// Create a default profile named "openssl"
-			const std::string defname = "openssl";
-			ConfigTag* tag = ServerInstance->Config->ConfValue(defname);
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "No <sslprofile> tags found, using settings from the deprecated <openssl> tag");
+		auto tags = ServerInstance->Config->ConfTags("sslprofile");
+		if (tags.empty())
+			throw ModuleException(this, "You have not specified any <sslprofile> tags that are usable by this module!");
 
+		for (const auto& [_, tag] : tags)
+		{
+			if (!insp::equalsci(tag->getString("provider", "openssl", 1), "openssl"))
+			{
+				ServerInstance->Logs.Debug(MODNAME, "Ignoring non-OpenSSL <sslprofile> tag at {}", tag->source.str());
+				continue;
+			}
+
+			const std::string name = tag->getString("name");
+			if (name.empty())
+			{
+				ServerInstance->Logs.Warning(MODNAME, "Ignoring <sslprofile> tag without name at {}", tag->source.str());
+				continue;
+			}
+
+			std::shared_ptr<OpenSSLIOHookProvider> prov;
 			try
 			{
-				newprofiles.push_back(new OpenSSLIOHookProvider(this, defname, tag));
+				prov = std::make_shared<OpenSSLIOHookProvider>(this, name, tag);
 			}
-			catch (OpenSSL::Exception& ex)
+			catch (const CoreException& ex)
 			{
-				throw ModuleException("Error while initializing the default TLS (SSL) profile - " + ex.GetReason());
+				throw ModuleException(this, "Error while initializing TLS profile \"" + name + "\" at " + tag->source.str() + " - " + ex.GetReason());
 			}
-		}
-		else
-		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "You have defined an <sslprofile> tag; you should use this in place of \"openssl\" when configuring TLS (SSL) connections in <bind:sslprofile> or <link:sslprofile>");
-			for (ConfigIter i = tags.first; i != tags.second; ++i)
-			{
-				ConfigTag* tag = i->second;
-				if (!stdalgo::string::equalsci(tag->getString("provider"), "openssl"))
-				{
-					ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Ignoring non-OpenSSL <sslprofile> tag at " + tag->getTagLocation());
-					continue;
-				}
 
-				std::string name = tag->getString("name");
-				if (name.empty())
-				{
-					ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Ignoring <sslprofile> tag without name at " + tag->getTagLocation());
-					continue;
-				}
-
-				reference<OpenSSLIOHookProvider> prov;
-				try
-				{
-					prov = new OpenSSLIOHookProvider(this, name, tag);
-				}
-				catch (CoreException& ex)
-				{
-					throw ModuleException("Error while initializing TLS (SSL) profile \"" + name + "\" at " + tag->getTagLocation() + " - " + ex.GetReason());
-				}
-
-				newprofiles.push_back(prov);
-			}
+			newprofiles.push_back(prov);
 		}
 
-		for (ProfileList::iterator i = profiles.begin(); i != profiles.end(); ++i)
-		{
-			OpenSSLIOHookProvider& prov = **i;
-			ServerInstance->Modules.DelService(prov);
-		}
+		for (const auto& profile : profiles)
+			ServerInstance->Modules.DelService(*profile);
 
 		profiles.swap(newprofiles);
 	}
 
- public:
+public:
 	ModuleSSLOpenSSL()
+		: Module(VF_VENDOR, "Allows TLS encrypted connections using the OpenSSL library.")
 	{
 		// Initialize OpenSSL
-		OPENSSL_init_ssl(0, NULL);
-#ifdef INSPIRCD_OPENSSL_OPAQUE_BIO
+		OPENSSL_init_ssl(0, nullptr);
 		biomethods = OpenSSL::BIOMethod::alloc();
+
+		thismod = this;
 	}
 
-	~ModuleSSLOpenSSL()
+	~ModuleSSLOpenSSL() override
 	{
 		BIO_meth_free(biomethods);
-#endif
 	}
 
-	void init() CXX11_OVERRIDE
+	void init() override
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "OpenSSL lib version \"%s\" module was compiled for \"" OPENSSL_VERSION_TEXT "\"", OpenSSL_version(OPENSSL_VERSION));
+		ServerInstance->Logs.Normal(MODNAME, "Module was compiled against OpenSSL version {} and is running against version {}",
+			OPENSSL_VERSION_STR, OpenSSL_version(OPENSSL_VERSION_STRING));
 
 		// Register application specific data
 		char exdatastr[] = "inspircd";
-		exdataindex = SSL_get_ex_new_index(0, exdatastr, NULL, NULL, NULL);
+		exdataindex = SSL_get_ex_new_index(0, exdatastr, nullptr, nullptr, nullptr);
 		if (exdataindex < 0)
-			throw ModuleException("Failed to register application specific data");
+			throw ModuleException(this, "Failed to register application specific data");
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("openssl");
-		if (status.initial || tag->getBool("onrehash"))
+		const auto& tag = ServerInstance->Config->ConfValue("openssl");
+		if (status.initial || tag->getBool("onrehash", true))
+		{
+			// Try to help people who have outdated configs.
+			for (const auto& field : {"cafile", "certfile", "ciphers", "clientclearoptions", "clientsetoptions", "compression", "crlfile", "crlmode", "crlpath", "dhfile", "ecdhcurve", "hash", "keyfile", "renegotiation", "requestclientcert", "serverclearoptions", "serversetoptions", "tlsv1", "tlsv11", "tlsv12", "tlsv13"})
+			{
+				if (!tag->getString(field).empty())
+					throw ModuleException(this, "TLS settings have moved from <openssl> to <sslprofile>. See " INSPIRCD_DOCS "modules/ssl_openssl/#sslprofile for more information.");
+			}
 			ReadProfiles();
+		}
 	}
 
-	void OnModuleRehash(User* user, const std::string &param) CXX11_OVERRIDE
+	void OnModuleRehash(User* user, const std::string& param) override
 	{
 		if (!irc::equals(param, "tls") && !irc::equals(param, "ssl"))
 			return;
@@ -1157,41 +1068,36 @@ class ModuleSSLOpenSSL : public Module
 		try
 		{
 			ReadProfiles();
-			ServerInstance->SNO->WriteToSnoMask('a', "OpenSSL TLS (SSL) profiles have been reloaded.");
+			ServerInstance->SNO.WriteToSnoMask('r', "OpenSSL TLS profiles have been reloaded.");
 		}
-		catch (ModuleException& ex)
+		catch (const ModuleException& ex)
 		{
-			ServerInstance->SNO->WriteToSnoMask('a', "Failed to reload the OpenSSL TLS (SSL) profiles. " + ex.GetReason());
+			ServerInstance->SNO.WriteToSnoMask('r', "Failed to reload the OpenSSL TLS profiles. " + ex.GetReason());
 		}
 	}
 
-	void OnCleanup(ExtensionItem::ExtensibleType type, Extensible* item) CXX11_OVERRIDE
+	void OnCleanup(ExtensionType type, Extensible* item) override
 	{
-		if (type == ExtensionItem::EXT_USER)
+		if (type == ExtensionType::USER)
 		{
 			LocalUser* user = IS_LOCAL((User*)item);
 
 			if ((user) && (user->eh.GetModHook(this)))
 			{
-				// User is using TLS (SSL), they're a local user, and they're using one of *our* TLS (SSL) ports.
-				// Potentially there could be multiple TLS (SSL) modules loaded at once on different ports.
-				ServerInstance->Users->QuitUser(user, "OpenSSL module unloading");
+				// User is using TLS, they're a local user, and they're using one of *our* TLS ports.
+				// Potentially there could be multiple TLS modules loaded at once on different ports.
+				ServerInstance->Users.QuitUser(user, "OpenSSL module unloading");
 			}
 		}
 	}
 
-	ModResult OnCheckReady(LocalUser* user) CXX11_OVERRIDE
+	ModResult OnCheckReady(LocalUser* user) override
 	{
 		const OpenSSLIOHook* const iohook = static_cast<OpenSSLIOHook*>(user->eh.GetModHook(this));
 		if ((iohook) && (!iohook->IsHookReady()))
 			return MOD_RES_DENY;
 		return MOD_RES_PASSTHRU;
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows TLS (SSL) encrypted connections using the OpenSSL library.", VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleSSLOpenSSL)
