@@ -1,9 +1,8 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2017-2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012, 2015 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012 Shawn Smith <ShawnSmith0828@gmail.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
@@ -25,85 +24,79 @@
 
 #include "inspircd.h"
 #include "modules/ctctags.h"
+#include "modules/isupport.h"
 #include "modules/who.h"
 #include "modules/whois.h"
 
-class BotTag : public ClientProtocol::MessageTagProvider
+class BotTag final
+	: public CTCTags::TagProvider
 {
- private:
-	SimpleUserModeHandler& botmode;
-	CTCTags::CapReference ctctagcap;
+private:
+	SimpleUserMode& botmode;
 
- public:
-	BotTag(Module* mod, SimpleUserModeHandler& bm)
-		: ClientProtocol::MessageTagProvider(mod)
+public:
+	BotTag(Module* mod, SimpleUserMode& bm)
+		: CTCTags::TagProvider(mod)
 		, botmode(bm)
-		, ctctagcap(mod)
 	{
 	}
 
-	void OnPopulateTags(ClientProtocol::Message& msg) CXX11_OVERRIDE
+	void OnPopulateTags(ClientProtocol::Message& msg) override
 	{
 		User* const user = msg.GetSourceUser();
-		if (!user || !user->IsModeSet(botmode))
-			return;
-
-		// TODO: remove inspircd.org/bot in v4.
-		msg.AddTag("bot", this, "");
-		msg.AddTag("inspircd.org/bot", this, "");
-	}
-
-	bool ShouldSendTag(LocalUser* user, const ClientProtocol::MessageTagData& tagdata) CXX11_OVERRIDE
-	{
-		return ctctagcap.get(user);
+		if (user && user->IsModeSet(botmode))
+			msg.AddTag("bot", this, "");
 	}
 };
 
-class ModuleBotMode
+class ModuleBotMode final
 	: public Module
+	, public ISupport::EventListener
 	, public Who::EventListener
 	, public Whois::EventListener
 {
- private:
-	SimpleUserModeHandler bm;
+private:
+	SimpleUserMode bm;
 	BotTag tag;
 	bool forcenotice;
 
- public:
+public:
 	ModuleBotMode()
-		: Who::EventListener(this)
+		: Module(VF_VENDOR, "Adds user mode B (bot) which marks users with it set as bots in their /WHOIS response.")
+		, ISupport::EventListener(this)
+		, Who::EventListener(this)
 		, Whois::EventListener(this)
 		, bm(this, "bot", 'B')
 		, tag(this, bm)
 	{
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
 		forcenotice = ServerInstance->Config->ConfValue("botmode")->getBool("forcenotice");
 	}
 
-	void On005Numeric(std::map<std::string, std::string>& tokens) CXX11_OVERRIDE
+	void OnBuildISupport(ISupport::TokenMap& tokens) override
 	{
 		tokens["BOT"] = ConvToStr(bm.GetModeChar());
 	}
 
-	ModResult OnUserPreMessage(User* user, const MessageTarget& target, MessageDetails& details) CXX11_OVERRIDE
+	ModResult OnUserPreMessage(User* user, MessageTarget& target, MessageDetails& details) override
 	{
 		// Allow sending if forcenotice is off, the user is not a bot, or if the message is a notice.
-		if (!forcenotice || !user->IsModeSet(bm) || details.type == MSG_NOTICE)
+		if (!forcenotice || !user->IsModeSet(bm) || details.type == MessageType::NOTICE)
 			return MOD_RES_PASSTHRU;
 
 		// Allow sending PRIVMSGs to services pseudoclients.
-		if (target.type == MessageTarget::TYPE_USER && target.Get<User>()->server->IsULine())
+		if (target.type == MessageTarget::TYPE_USER && target.Get<User>()->server->IsService())
 			return MOD_RES_PASSTHRU;
 
 		// Force the message to be broadcast as a NOTICE.
-		details.type = MSG_NOTICE;
+		details.type = MessageType::NOTICE;
 		return MOD_RES_PASSTHRU;
 	}
 
-	ModResult OnWhoLine(const Who::Request& request, LocalUser* source, User* user, Membership* memb, Numeric::Numeric& numeric) CXX11_OVERRIDE
+	ModResult OnWhoLine(const Who::Request& request, LocalUser* source, User* user, Membership* memb, Numeric::Numeric& numeric) override
 	{
 		size_t flag_index;
 		if (!request.GetFieldIndex('f', flag_index))
@@ -115,16 +108,11 @@ class ModuleBotMode
 		return MOD_RES_PASSTHRU;
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds user mode B (bot) which marks users with it set as bots in their /WHOIS response.",VF_VENDOR);
-	}
-
-	void OnWhois(Whois::Context& whois) CXX11_OVERRIDE
+	void OnWhois(Whois::Context& whois) override
 	{
 		if (whois.GetTarget()->IsModeSet(bm))
 		{
-			whois.SendLine(RPL_WHOISBOT, "is a bot on " + ServerInstance->Config->Network);
+			whois.SendLine(RPL_WHOISBOT, "is a bot");
 		}
 	}
 };
