@@ -1,11 +1,11 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2013, 2017-2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2021 Dominic Hamon
+ *   Copyright (C) 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013 Daniel Vassdal <shutter@canternet.org>
- *   Copyright (C) 2012-2015, 2018 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2012, 2014, 2018 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -23,13 +23,15 @@
 
 
 #include "inspircd.h"
-#include "modules/ircv3_servertime.h"
+#include "clientprotocolmsg.h"
 #include "modules/ircv3_batch.h"
+#include "modules/ircv3_servertime.h"
 #include "modules/server.h"
+#include "numerichelper.h"
 
 typedef insp::flat_map<std::string, std::string> HistoryTagMap;
 
-struct HistoryItem
+struct HistoryItem final
 {
 	time_t ts;
 	std::string text;
@@ -41,21 +43,21 @@ struct HistoryItem
 		: ts(ServerInstance->Time())
 		, text(details.text)
 		, type(details.type)
-		, sourcemask(source->GetFullHost())
+		, sourcemask(source->GetMask())
 	{
 		tags.reserve(details.tags_out.size());
-		for (ClientProtocol::TagMap::const_iterator iter = details.tags_out.begin(); iter != details.tags_out.end(); ++iter)
-			tags[iter->first] = iter->second.value;
+		for (const auto& [tagname, tagvalue] : details.tags_out)
+			tags[tagname] = tagvalue.value;
 	}
 };
 
-struct HistoryList
+struct HistoryList final
 {
 	std::deque<HistoryItem> lines;
-	unsigned int maxlen;
-	unsigned int maxtime;
+	unsigned long maxlen;
+	unsigned long maxtime;
 
-	HistoryList(unsigned int len, unsigned int time)
+	HistoryList(unsigned long len, unsigned long time)
 		: maxlen(len)
 		, maxtime(time)
 	{
@@ -74,101 +76,111 @@ struct HistoryList
 	}
 };
 
-class HistoryMode : public ParamMode<HistoryMode, SimpleExtItem<HistoryList> >
+class HistoryMode final
+	: public ParamMode<HistoryMode, SimpleExtItem<HistoryList>>
 {
- public:
-	unsigned int maxlines;
+private:
+	bool ParseDuration(User* user, irc::sepstream& stream, unsigned long& duration)
+	{
+		std::string durationstr;
+		if (!stream.GetToken(durationstr))
+			return false;
+
+		if (!Duration::TryFrom(durationstr, duration) || duration == 0)
+			return false;
+
+		if (IS_LOCAL(user) && maxduration && duration > maxduration)
+			duration = maxduration; // Clamp for local users.
+
+		return true;
+	}
+
+	bool ParseLines(User* user, irc::sepstream& stream, unsigned long& lines)
+	{
+		std::string linesstr;
+		if (!stream.GetToken(linesstr))
+			return false;
+
+		lines = ConvToNum<unsigned long>(linesstr);
+		if (lines == 0)
+			return false;
+
+		if (IS_LOCAL(user) && maxlines && lines > maxlines)
+			lines = maxlines; // Clamp for local users.
+
+		return true;
+	}
+
+public:
+	unsigned long maxduration;
+	unsigned long maxlines;
+
 	HistoryMode(Module* Creator)
-		: ParamMode<HistoryMode, SimpleExtItem<HistoryList> >(Creator, "history", 'H')
+		: ParamMode<HistoryMode, SimpleExtItem<HistoryList>>(Creator, "history", 'H')
 	{
 		syntax = "<max-messages>:<max-duration>";
 	}
 
-	ModeAction OnSet(User* source, Channel* channel, std::string& parameter) CXX11_OVERRIDE
+	bool OnSet(User* source, Channel* channel, std::string& parameter) override
 	{
-		std::string::size_type colon = parameter.find(':');
-		if (colon == std::string::npos)
+		irc::sepstream stream(parameter, ':');
+
+		unsigned long lines;
+		unsigned long duration;
+		if (!ParseLines(source, stream, lines) || !ParseDuration(source, stream, duration))
 		{
 			source->WriteNumeric(Numerics::InvalidModeParameter(channel, this, parameter));
-			return MODEACTION_DENY;
+			return false;
 		}
 
-		std::string duration(parameter, colon+1);
-		if ((IS_LOCAL(source)) && ((duration.length() > 10) || (!InspIRCd::IsValidDuration(duration))))
-		{
-			source->WriteNumeric(Numerics::InvalidModeParameter(channel, this, parameter));
-			return MODEACTION_DENY;
-		}
-
-		unsigned int len = ConvToNum<unsigned int>(parameter.substr(0, colon));
-		unsigned long time;
-		if (!InspIRCd::Duration(duration, time) || len == 0 || (len > maxlines && IS_LOCAL(source)))
-		{
-			source->WriteNumeric(Numerics::InvalidModeParameter(channel, this, parameter));
-			return MODEACTION_DENY;
-		}
-		if (len > maxlines)
-			len = maxlines;
-
-		HistoryList* history = ext.get(channel);
+		HistoryList* history = ext.Get(channel);
 		if (history)
 		{
 			// Shrink the list if the new line number limit is lower than the old one
-			if (len < history->lines.size())
-				history->lines.erase(history->lines.begin(), history->lines.begin() + (history->lines.size() - len));
+			if (lines < history->lines.size())
+				history->lines.erase(history->lines.begin(), history->lines.begin() + (history->lines.size() - lines));
 
-			history->maxlen = len;
-			history->maxtime = time;
+			history->maxlen = lines;
+			history->maxtime = duration;
 			history->Prune();
 		}
 		else
 		{
-			ext.set(channel, new HistoryList(len, time));
+			ext.SetFwd(channel, lines, duration);
 		}
-		return MODEACTION_ALLOW;
+		return true;
 	}
 
 	void SerializeParam(Channel* chan, const HistoryList* history, std::string& out)
 	{
 		out.append(ConvToStr(history->maxlen));
 		out.append(":");
-		out.append(InspIRCd::DurationString(history->maxtime));
+		out.append(Duration::ToString(history->maxtime));
 	}
 };
 
-class NoHistoryMode : public SimpleUserModeHandler
-{
-public:
-	NoHistoryMode(Module* Creator)
-		: SimpleUserModeHandler(Creator, "nohistory", 'N')
-	{
-		if (!ServerInstance->Config->ConfValue("chanhistory")->getBool("enableumode"))
-			DisableAutoRegister();
-	}
-};
-
-class ModuleChanHistory
+class ModuleChanHistory final
 	: public Module
-	, public ServerProtocol::BroadcastEventListener
+	, public ServerProtocol::RouteEventListener
 {
- private:
+private:
 	HistoryMode historymode;
-	NoHistoryMode nohistorymode;
-	bool prefixmsg;
+	SimpleUserMode nohistorymode;
 	UserModeReference botmode;
-	bool dobots;
 	IRCv3::Batch::CapReference batchcap;
 	IRCv3::Batch::API batchmanager;
 	IRCv3::Batch::Batch batch;
 	IRCv3::ServerTime::API servertimemanager;
 	ClientProtocol::MessageTagEvent tagevent;
+	bool prefixmsg;
+	bool savefrombots;
+	bool sendtobots;
 
 	void AddTag(ClientProtocol::Message& msg, const std::string& tagkey, std::string& tagval)
 	{
-		const Events::ModuleEventProvider::SubscriberList& list = tagevent.GetSubscribers();
-		for (Events::ModuleEventProvider::SubscriberList::const_iterator i = list.begin(); i != list.end(); ++i)
+		for (auto* subscriber : tagevent.GetSubscribers())
 		{
-			ClientProtocol::MessageTagProvider* const tagprov = static_cast<ClientProtocol::MessageTagProvider*>(*i);
+			ClientProtocol::MessageTagProvider* const tagprov = static_cast<ClientProtocol::MessageTagProvider*>(subscriber);
 			const ModResult res = tagprov->OnProcessTag(ServerInstance->FakeClient, tagkey, tagval);
 			if (res == MOD_RES_ALLOW)
 				msg.AddTag(tagkey, tagprov, tagval);
@@ -185,12 +197,11 @@ class ModuleChanHistory
 			batch.GetBatchStartMessage().PushParamRef(channel->name);
 		}
 
-		for (std::deque<HistoryItem>::iterator i = list->lines.begin(); i != list->lines.end(); ++i)
+		for (auto& item : list->lines)
 		{
-			HistoryItem& item = *i;
 			ClientProtocol::Messages::Privmsg msg(ClientProtocol::Messages::Privmsg::nocopy, item.sourcemask, channel, item.text, item.type);
-			for (HistoryTagMap::iterator iter = item.tags.begin(); iter != item.tags.end(); ++iter)
-				AddTag(msg, iter->first, iter->second);
+			for (auto& [tagname, tagvalue] : item.tags)
+				AddTag(msg, tagname, tagvalue);
 			if (servertimemanager)
 				servertimemanager->Set(msg, item.ts);
 			batch.AddToBatch(msg);
@@ -201,11 +212,12 @@ class ModuleChanHistory
 			batchmanager->End(batch);
 	}
 
- public:
+public:
 	ModuleChanHistory()
-		: ServerProtocol::BroadcastEventListener(this)
+		: Module(VF_VENDOR, "Adds channel mode H (history) which allows message history to be viewed on joining the channel.")
+		, ServerProtocol::RouteEventListener(this)
 		, historymode(this)
-		, nohistorymode(this)
+		, nohistorymode(this, "nohistory", 'N')
 		, botmode(this, "bot")
 		, batchcap(this)
 		, batchmanager(this)
@@ -215,68 +227,68 @@ class ModuleChanHistory
 	{
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("chanhistory");
-		historymode.maxlines = tag->getUInt("maxlines", 50, 1);
-		prefixmsg = tag->getBool("prefixmsg", tag->getBool("notice", true));
-		dobots = tag->getBool("bots", true);
+		const auto& tag = ServerInstance->Config->ConfValue("chanhistory");
+		historymode.maxduration = tag->getDuration("maxduration", 60*60*24*28);
+		historymode.maxlines = tag->getNum<unsigned long>("maxlines", 50);
+		prefixmsg = tag->getBool("prefixmsg", true);
+		savefrombots = tag->getBool("savefrombots", true);
+		sendtobots = tag->getBool("sendtobots", tag->getBool("bots", true));
 	}
 
-	ModResult OnBroadcastMessage(Channel* channel, const Server* server) CXX11_OVERRIDE
+	ModResult OnRouteMessage(const Channel* channel, const Server* server) override
 	{
-		return channel->IsModeSet(historymode) ? MOD_RES_ALLOW : MOD_RES_PASSTHRU;
+		return channel->IsModeSet(historymode) && !server->IsService() ? MOD_RES_ALLOW : MOD_RES_PASSTHRU;
 	}
 
-	void OnUserPostMessage(User* user, const MessageTarget& target, const MessageDetails& details) CXX11_OVERRIDE
+	void OnUserPostMessage(User* user, const MessageTarget& target, const MessageDetails& details) override
 	{
 		if (target.type != MessageTarget::TYPE_CHANNEL || target.status)
 			return;
 
-		std::string ctcpname;
+		if (user->IsModeSet(botmode) && !savefrombots)
+			return;
+
+		std::string_view ctcpname;
 		if (details.IsCTCP(ctcpname) && !irc::equals(ctcpname, "ACTION"))
 			return;
 
-		HistoryList* list = historymode.ext.get(target.Get<Channel>());
+		HistoryList* list = historymode.ext.Get(target.Get<Channel>());
 		if (!list)
 			return;
 
-		list->lines.push_back(HistoryItem(user, details));
+		list->lines.emplace_back(user, details);
 		if (list->lines.size() > list->maxlen)
 			list->lines.pop_front();
 	}
 
-	void OnPostJoin(Membership* memb) CXX11_OVERRIDE
+	void OnPostJoin(Membership* memb) override
 	{
 		LocalUser* localuser = IS_LOCAL(memb->user);
 		if (!localuser)
 			return;
 
-		if (memb->user->IsModeSet(botmode) && !dobots)
+		if (memb->user->IsModeSet(botmode) && !sendtobots)
 			return;
 
 		if (memb->user->IsModeSet(nohistorymode))
 			return;
 
-		HistoryList* list = historymode.ext.get(memb->chan);
+		HistoryList* list = historymode.ext.Get(memb->chan);
 		if (!list || !list->Prune())
 			return;
 
-		if ((prefixmsg) && (!batchcap.get(localuser)))
+		if ((prefixmsg) && (!batchcap.IsEnabled(localuser)))
 		{
 			std::string message("Replaying up to " + ConvToStr(list->maxlen) + " lines of pre-join history");
 			if (list->maxtime > 0)
-				message.append(" from the last " + InspIRCd::DurationString(list->maxtime));
+				message.append(" from the last " + Duration::ToString(list->maxtime));
 			memb->WriteNotice(message);
 		}
 
 		SendHistory(localuser, memb->chan, list);
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds channel mode H (history) which allows message history to be viewed on joining the channel.", VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleChanHistory)
