@@ -1,9 +1,10 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2021 Dominic Hamon
+ *   Copyright (C) 2019-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2014 Daniel Vassdal <shutter@canternet.org>
  *   Copyright (C) 2013-2015 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2013, 2017, 2019 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2009 Robin Burchell <robin+git@viroteck.net>
@@ -25,6 +26,7 @@
 
 
 #include "inspircd.h"
+#include "extension.h"
 
 static void JoinChannels(LocalUser* u, const std::string& chanlist)
 {
@@ -33,73 +35,74 @@ static void JoinChannels(LocalUser* u, const std::string& chanlist)
 
 	while (chans.GetToken(chan))
 	{
-		if (ServerInstance->IsChannel(chan))
+		if (ServerInstance->Channels.IsChannel(chan))
 			Channel::JoinUser(u, chan);
 	}
 }
 
-class JoinTimer : public Timer
+class JoinTimer final
+	: public Timer
 {
- private:
+private:
 	LocalUser* const user;
 	const std::string channels;
 	SimpleExtItem<JoinTimer>& ext;
 
- public:
+public:
 	JoinTimer(LocalUser* u, SimpleExtItem<JoinTimer>& ex, const std::string& chans, unsigned int delay)
 		: Timer(delay, false)
-		, user(u), channels(chans), ext(ex)
+		, user(u)
+		, channels(chans)
+		, ext(ex)
 	{
 		ServerInstance->Timers.AddTimer(this);
 	}
 
-	bool Tick(time_t time) CXX11_OVERRIDE
+	bool Tick() override
 	{
 		if (user->chans.empty())
 			JoinChannels(user, channels);
 
-		ext.unset(user);
+		ext.Unset(user);
 		return false;
 	}
 };
 
-class ModuleConnJoin : public Module
+class ModuleConnJoin final
+	: public Module
 {
+private:
 	SimpleExtItem<JoinTimer> ext;
 	std::string defchans;
 	unsigned int defdelay;
 
- public:
+public:
 	ModuleConnJoin()
-		: ext("join_timer", ExtensionItem::EXT_USER, this)
+		: Module(VF_VENDOR, "Allows the server administrator to force users to join one or more channels on connect.")
+		, ext(this, "join-timer", ExtensionType::USER)
 	{
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("autojoin");
+		const auto& tag = ServerInstance->Config->ConfValue("autojoin");
 		defchans = tag->getString("channel");
-		defdelay = tag->getDuration("delay", 0, 0, 60*15);
+		defdelay = static_cast<unsigned int>(tag->getDuration("delay", 0, 0, 60*15));
 	}
 
-	void Prioritize() CXX11_OVERRIDE
+	void Prioritize() override
 	{
-		ServerInstance->Modules->SetPriority(this, I_OnPostConnect, PRIORITY_LAST);
+		ServerInstance->Modules.SetPriority(this, I_OnPostConnect, PRIORITY_LAST);
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows the server administrator to force users to join one or more channels on connect.", VF_VENDOR);
-	}
-
-	void OnPostConnect(User* user) CXX11_OVERRIDE
+	void OnPostConnect(User* user) override
 	{
 		LocalUser* localuser = IS_LOCAL(user);
 		if (!localuser)
 			return;
 
 		std::string chanlist = localuser->GetClass()->config->getString("autojoin");
-		unsigned int chandelay = localuser->GetClass()->config->getDuration("autojoindelay", 0, 0, 60*15);
+		unsigned int chandelay = static_cast<unsigned int>(localuser->GetClass()->config->getDuration("autojoindelay", 0, 0, 60*15));
 
 		if (chanlist.empty())
 		{
@@ -112,7 +115,7 @@ class ModuleConnJoin : public Module
 		if (!chandelay)
 			JoinChannels(localuser, chanlist);
 		else
-			ext.set(localuser, new JoinTimer(localuser, ext, chanlist, chandelay));
+			ext.SetFwd(localuser, localuser, ext, chanlist, chandelay);
 	}
 };
 
