@@ -1,13 +1,11 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2018-2019 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012-2013, 2015, 2018 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2018-2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2012, 2018 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2006 Oliver Lupton <om@inspircd.org>
  *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -25,27 +23,31 @@
 
 
 #include "inspircd.h"
+#include "clientprotocolmsg.h"
+#include "extension.h"
 
-class ModuleWaitPong : public Module
+class ModuleWaitPong final
+	: public Module
 {
 	bool sendsnotice;
 	bool killonbadreply;
-	LocalStringExt ext;
+	StringExtItem ext;
 
- public:
+public:
 	ModuleWaitPong()
-		: ext("waitpong_pingstr", ExtensionItem::EXT_USER, this)
+		: Module(VF_VENDOR, "Requires all clients to respond to a PING request before they can fully connect.")
+		, ext(this, "waitpong-cookie", ExtensionType::USER)
 	{
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("waitpong");
+		const auto& tag = ServerInstance->Config->ConfValue("waitpong");
 		sendsnotice = tag->getBool("sendsnotice", false);
 		killonbadreply = tag->getBool("killonbadreply", true);
 	}
 
-	ModResult OnUserRegister(LocalUser* user) CXX11_OVERRIDE
+	ModResult OnUserRegister(LocalUser* user) override
 	{
 		std::string pingrpl = ServerInstance->GenRandomStr(10);
 		{
@@ -54,29 +56,29 @@ class ModuleWaitPong : public Module
 		}
 
 		if(sendsnotice)
-			user->WriteNotice("*** If you are having problems connecting due to registration timeouts type /quote PONG " + pingrpl + " or /raw PONG " + pingrpl + " now.");
+			user->WriteNotice("*** If you are having problems connecting due to connection timeouts type /quote PONG " + pingrpl + " or /raw PONG " + pingrpl + " now.");
 
-		ext.set(user, pingrpl);
+		ext.Set(user, pingrpl);
 		return MOD_RES_PASSTHRU;
 	}
 
-	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) CXX11_OVERRIDE
+	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) override
 	{
 		if (command == "PONG")
 		{
-			std::string* pingrpl = ext.get(user);
+			std::string* pingrpl = ext.Get(user);
 
 			if (pingrpl)
 			{
 				if (!parameters.empty() && *pingrpl == parameters[0])
 				{
-					ext.unset(user);
+					ext.Unset(user);
 					return MOD_RES_DENY;
 				}
 				else
 				{
 					if(killonbadreply)
-						ServerInstance->Users->QuitUser(user, "Incorrect ping reply for registration");
+						ServerInstance->Users.QuitUser(user, "Incorrect ping reply for connection");
 					return MOD_RES_DENY;
 				}
 			}
@@ -84,14 +86,9 @@ class ModuleWaitPong : public Module
 		return MOD_RES_PASSTHRU;
 	}
 
-	ModResult OnCheckReady(LocalUser* user) CXX11_OVERRIDE
+	ModResult OnCheckReady(LocalUser* user) override
 	{
-		return ext.get(user) ? MOD_RES_DENY : MOD_RES_PASSTHRU;
-	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Requires all clients to respond to a PING request before they can fully connect.", VF_VENDOR);
+		return ext.Get(user) ? MOD_RES_DENY : MOD_RES_PASSTHRU;
 	}
 };
 
