@@ -1,9 +1,10 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
  *   Copyright (C) 2014 Googolplexed <googol@googolplexed.net>
- *   Copyright (C) 2013, 2017-2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2019-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2014 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
@@ -25,31 +26,32 @@
 
 
 #include "inspircd.h"
-#include "xline.h"
 #include "modules/server.h"
 #include "modules/webirc.h"
+#include "timeutils.h"
+#include "xline.h"
 
-class ModuleConnectBan CXX11_FINAL
+class ModuleConnectBan final
 	: public Module
 	, public ServerProtocol::LinkEventListener
 	, public WebIRC::EventListener
 {
- private:
+private:
 	typedef std::map<irc::sockets::cidr_mask, unsigned int> ConnectMap;
 
 	ConnectMap connects;
-	unsigned int threshold;
-	unsigned int banduration;
+	unsigned long threshold;
+	unsigned long banduration;
 	unsigned int ipv4_cidr;
 	unsigned int ipv6_cidr;
 	unsigned long bootwait;
 	unsigned long splitwait;
-	time_t ignoreuntil;
+	time_t ignoreuntil = 0;
 	std::string banmessage;
 
-	unsigned char GetRange(LocalUser* user)
+	unsigned char GetRange(LocalUser* user) const
 	{
-		int family = user->client_sa.family();
+		sa_family_t family = user->client_sa.family();
 		switch (family)
 		{
 			case AF_INET:
@@ -64,7 +66,7 @@ class ModuleConnectBan CXX11_FINAL
 		}
 
 		// If we have reached this point then we have encountered a bug.
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "BUG: ModuleConnectBan::GetRange(): socket type %d is unknown!", family);
+		ServerInstance->Logs.Debug(MODNAME, "BUG: ModuleConnectBan::GetRange(): socket type {} is unknown!", family);
 		return 0;
 	}
 
@@ -78,65 +80,60 @@ class ModuleConnectBan CXX11_FINAL
 		return user->GetClass() && !user->GetClass()->config->getBool("useconnectban", true);
 	}
 
- public:
+public:
 	// Stop GCC warnings about the deprecated OnServerSplit event.
 	using ServerProtocol::LinkEventListener::OnServerSplit;
 
 	ModuleConnectBan()
-		: ServerProtocol::LinkEventListener(this)
+		: Module(VF_VENDOR, "Z-lines IP addresses which make excessive connections to the server.")
+		, ServerProtocol::LinkEventListener(this)
 		, WebIRC::EventListener(this)
-		, ignoreuntil(0)
 	{
 	}
 
-	void Prioritize() CXX11_OVERRIDE
+	void Prioritize() override
 	{
-		Module* corexline = ServerInstance->Modules->Find("core_xline");
-		ServerInstance->Modules->SetPriority(this, I_OnSetUserIP, PRIORITY_AFTER, corexline);
+		Module* corexline = ServerInstance->Modules.Find("core_xline");
+		ServerInstance->Modules.SetPriority(this, I_OnChangeRemoteAddress, PRIORITY_AFTER, corexline);
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		return Version("Z-lines IP addresses which make excessive connections to the server.", VF_VENDOR);
-	}
+		const auto& tag = ServerInstance->Config->ConfValue("connectban");
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
-	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("connectban");
-
-		ipv4_cidr = tag->getUInt("ipv4cidr", ServerInstance->Config->c_ipv4_range, 1, 32);
-		ipv6_cidr = tag->getUInt("ipv6cidr", ServerInstance->Config->c_ipv6_range, 1, 128);
-		threshold = tag->getUInt("threshold", 10, 1);
+		ipv4_cidr = tag->getNum<unsigned int>("ipv4cidr", ServerInstance->Config->IPv4Range, 1, 32);
+		ipv6_cidr = tag->getNum<unsigned int>("ipv6cidr", ServerInstance->Config->IPv6Range, 1, 128);
+		threshold = tag->getNum<unsigned long>("threshold", 10, 2);
 		bootwait = tag->getDuration("bootwait", 60*2);
 		splitwait = tag->getDuration("splitwait", 60*2);
-		banduration = tag->getDuration("banduration", tag->getDuration("duration", 6*60*60), 1);
+		banduration = tag->getDuration("banduration", 6*60*60, 1);
 		banmessage = tag->getString("banmessage", "Your IP range has been attempting to connect too many times in too short a duration. Wait a while, and you will be able to connect.");
 
 		if (status.initial)
 			ignoreuntil = ServerInstance->Time() + bootwait;
 	}
 
-	void OnWebIRCAuth(LocalUser* user, const WebIRC::FlagMap* flags) CXX11_OVERRIDE
+	void OnWebIRCAuth(LocalUser* user, const WebIRC::FlagMap* flags) override
 	{
 		if (IsExempt(user))
 			return;
 
 		// HACK: Lower the connection attempts for the gateway IP address. The user
 		// will be rechecked for connect spamming shortly after when their IP address
-		// is changed and OnSetUserIP is called.
+		// is changed and OnChangeRemoteAddress is called.
 		irc::sockets::cidr_mask mask(user->client_sa, GetRange(user));
 		ConnectMap::iterator iter = connects.find(mask);
 		if (iter != connects.end() && iter->second)
 			iter->second--;
 	}
 
-	void OnServerSplit(const Server* server, bool error) CXX11_OVERRIDE
+	void OnServerSplit(const Server* server, bool error) override
 	{
 		if (splitwait)
 			ignoreuntil = std::max<time_t>(ignoreuntil, ServerInstance->Time() + splitwait);
 	}
 
-	void OnSetUserIP(LocalUser* u) CXX11_OVERRIDE
+	void OnChangeRemoteAddress(LocalUser* u) override
 	{
 		if (IsExempt(u) || ignoreuntil > ServerInstance->Time())
 			return;
@@ -157,17 +154,17 @@ class ModuleConnectBan CXX11_FINAL
 					maskstr.insert(maskstr.begin(), 1, '0');
 
 				// Create Z-line for set duration.
-				ZLine* zl = new ZLine(ServerInstance->Time(), banduration, MODNAME "@" + ServerInstance->Config->ServerName, banmessage, maskstr);
-				if (!ServerInstance->XLines->AddLine(zl, NULL))
+				auto* zl = new ZLine(ServerInstance->Time(), banduration, MODNAME "@" + ServerInstance->Config->ServerName, banmessage, maskstr);
+				if (!ServerInstance->XLines->AddLine(zl, nullptr))
 				{
 					delete zl;
 					return;
 				}
 
-				ServerInstance->SNO->WriteToSnoMask('x', "%s added a timed Z-line on %s, expires in %s (on %s): %s",
-					zl->source.c_str(), maskstr.c_str(), InspIRCd::DurationString(zl->duration).c_str(),
-					InspIRCd::TimeString(zl->expiry).c_str(), zl->reason.c_str());
-				ServerInstance->SNO->WriteGlobalSno('a', "Connect flooding from IP range %s (%d)", maskstr.c_str(), threshold);
+				ServerInstance->SNO.WriteToSnoMask('x', "{} added a timed Z-line on {}, expires in {} (on {}): {}",
+					zl->source, maskstr, Duration::ToString(zl->duration),
+					Time::ToString(zl->expiry), zl->reason);
+				ServerInstance->SNO.WriteGlobalSno('a', "Connect flooding from IP range {} ({})", maskstr, threshold);
 				connects.erase(i);
 				ServerInstance->XLines->ApplyLines();
 			}
@@ -178,9 +175,9 @@ class ModuleConnectBan CXX11_FINAL
 		}
 	}
 
-	void OnGarbageCollect() CXX11_OVERRIDE
+	void OnGarbageCollect() override
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Clearing map.");
+		ServerInstance->Logs.Debug(MODNAME, "Clearing map.");
 		connects.clear();
 	}
 };
