@@ -1,15 +1,15 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2016 Adam <Adam@anope.org>
- *   Copyright (C) 2013, 2017-2020, 2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2014, 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
+ *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Matt Smith <dz@inspircd.org>
- *   Copyright (C) 2008 John Brooks <special@inspircd.org>
+ *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
+ *   Copyright (C) 2008 John Brooks <john@jbrooks.io>
  *   Copyright (C) 2008 Craig Edwards <brain@inspircd.org>
  *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007-2008 Dennis Friis <peavey@inspircd.org>
@@ -30,6 +30,9 @@
 
 
 #include "inspircd.h"
+#include "extension.h"
+#include "numerichelper.h"
+#include "timeutils.h"
 
 enum
 {
@@ -72,22 +75,22 @@ static const char* const helptext[] =
 	"  Your DCCALLOW list will be deleted when you leave IRC."
 };
 
-class BannedFileList
+class BannedFileList final
 {
- public:
+public:
 	std::string filemask;
 	std::string action;
 };
 
-class DCCAllow
+class DCCAllow final
 {
- public:
+public:
 	std::string nickname;
 	std::string hostmask;
 	time_t set_on;
 	unsigned long length;
 
-	DCCAllow() { }
+	DCCAllow() = default;
 
 	DCCAllow(const std::string& nick, const std::string& hm, time_t so, unsigned long ln)
 		: nickname(nick)
@@ -105,25 +108,29 @@ dccallowlist* dl;
 typedef std::vector<BannedFileList> bannedfilelist;
 bannedfilelist bfl;
 
-class DCCAllowExt : public SimpleExtItem<dccallowlist>
+class DCCAllowExt final
+	: public SimpleExtItem<dccallowlist>
 {
- public:
-	unsigned int maxentries;
+public:
+	unsigned long maxentries;
 
 	DCCAllowExt(Module* Creator)
-		: SimpleExtItem<dccallowlist>("dccallow", ExtensionItem::EXT_USER, Creator)
+		: SimpleExtItem<dccallowlist>(Creator, "dccallow", ExtensionType::USER)
 	{
 	}
 
-	void FromInternal(Extensible* container, const std::string& value) CXX11_OVERRIDE
+	void FromInternal(Extensible* container, const std::string& value) noexcept override
 	{
+		if (container->extype != this->extype)
+			return;
+
 		LocalUser* user = IS_LOCAL(static_cast<User*>(container));
 		if (!user)
 			return;
 
 		// Remove the old list and create a new one.
-		unset(user);
-		dccallowlist* list = new dccallowlist();
+		Unset(user, false);
+		auto* list = new dccallowlist();
 
 		irc::spacesepstream ts(value);
 		while (!ts.StreamEnd())
@@ -131,8 +138,8 @@ class DCCAllowExt : public SimpleExtItem<dccallowlist>
 			// Check we have space for another entry.
 			if (list->size() >= maxentries)
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Oversized DCC allow list received for %s: %s",
-					user->uuid.c_str(), value.c_str());
+				ServerInstance->Logs.Debug(MODNAME, "Oversized DCC allow list received for {}: {}",
+					user->uuid, value);
 				delete list;
 				return;
 			}
@@ -144,8 +151,8 @@ class DCCAllowExt : public SimpleExtItem<dccallowlist>
 				!ts.GetNumericToken(dccallow.set_on) ||
 				!ts.GetNumericToken(dccallow.length))
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Malformed DCC allow list received for %s: %s",
-					user->uuid.c_str(), value.c_str());
+				ServerInstance->Logs.Debug(MODNAME, "Malformed DCC allow list received for {}: {}",
+					user->uuid, value);
 				delete list;
 				return;
 			}
@@ -154,11 +161,18 @@ class DCCAllowExt : public SimpleExtItem<dccallowlist>
 			list->push_back(dccallow);
 		}
 
+		// If we have an empty list then don't store it.
+		if (list->empty())
+		{
+			delete list;
+			return;
+		}
+
 		// The value was well formed.
-		set(user, list);
+		Set(user, list);
 	}
 
-	std::string ToInternal(const Extensible* container, void* item) const CXX11_OVERRIDE
+	std::string ToInternal(const Extensible* container, void* item) const noexcept override
 	{
 		dccallowlist* list = static_cast<dccallowlist*>(item);
 		std::string buf;
@@ -179,31 +193,32 @@ class DCCAllowExt : public SimpleExtItem<dccallowlist>
 	}
 };
 
-class CommandDccallow : public Command
+class CommandDccallow final
+	: public Command
 {
- public:
+public:
 	DCCAllowExt& ext;
 	unsigned long defaultlength;
 	CommandDccallow(Module* parent, DCCAllowExt& Ext)
 		: Command(parent, "DCCALLOW", 0)
 		, ext(Ext)
 	{
-		syntax = "[(+|-)<nick> [<time>]]|[LIST|HELP]";
+		syntax = { "[(+|-)<nick> [<time>]]", "LIST", "HELP" };
 		/* XXX we need to fix this so it can work with translation stuff (i.e. move +- into a separate param */
 	}
 
-	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
+	CmdResult Handle(User* user, const Params& parameters) override
 	{
 		/* syntax: DCCALLOW [(+|-)<nick> [<time>]]|[LIST|HELP] */
-		if (!parameters.size())
+		if (parameters.empty())
 		{
 			// display current DCCALLOW list
 			DisplayDCCAllowList(user);
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
-		else if (parameters.size() > 0)
+		else
 		{
-			char action = *parameters[0].c_str();
+			char action = parameters[0][0];
 
 			// if they didn't specify an action, this is probably a command
 			if (action != '+' && action != '-')
@@ -212,31 +227,29 @@ class CommandDccallow : public Command
 				{
 					// list current DCCALLOW list
 					DisplayDCCAllowList(user);
-					return CMD_FAILURE;
+					return CmdResult::FAILURE;
 				}
 				else if (irc::equals(parameters[0], "HELP"))
 				{
 					// display help
 					DisplayHelp(user);
-					return CMD_FAILURE;
+					return CmdResult::FAILURE;
 				}
 				else
 				{
 					user->WriteNumeric(ERR_UNKNOWNDCCALLOWCMD, "DCCALLOW command not understood. For help on DCCALLOW, type /DCCALLOW HELP");
-					return CMD_FAILURE;
+					return CmdResult::FAILURE;
 				}
 			}
 
 			std::string nick(parameters[0], 1);
-			User *target = ServerInstance->FindNickOnly(nick);
-
-			if ((target) && (!target->quitting) && (target->registered == REG_ALL))
+			auto* target = ServerInstance->Users.FindNick(nick, true);
+			if (target && !target->quitting)
 			{
-
 				if (action == '-')
 				{
 					// check if it contains any entries
-					dl = ext.get(user);
+					dl = ext.Get(user);
 					if (dl)
 					{
 						for (dccallowlist::iterator i = dl->begin(); i != dl->end(); ++i)
@@ -245,7 +258,7 @@ class CommandDccallow : public Command
 							if (i->nickname == target->nick)
 							{
 								dl->erase(i);
-								user->WriteNumeric(RPL_DCCALLOWREMOVED, user->nick, InspIRCd::Format("Removed %s from your DCCALLOW list", target->nick.c_str()));
+								user->WriteNumeric(RPL_DCCALLOWREMOVED, user->nick, INSP_FORMAT("Removed {} from your DCCALLOW list", target->nick));
 								break;
 							}
 						}
@@ -256,14 +269,14 @@ class CommandDccallow : public Command
 					if (target == user)
 					{
 						user->WriteNumeric(ERR_DCCALLOWINVALID, user->nick, "You cannot add yourself to your own DCCALLOW list!");
-						return CMD_FAILURE;
+						return CmdResult::FAILURE;
 					}
 
-					dl = ext.get(user);
+					dl = ext.Get(user);
 					if (!dl)
 					{
 						dl = new dccallowlist;
-						ext.set(user, dl);
+						ext.Set(user, dl);
 						// add this user to the userlist
 						ul.push_back(user);
 					}
@@ -271,78 +284,78 @@ class CommandDccallow : public Command
 					if (dl->size() >= ext.maxentries)
 					{
 						user->WriteNumeric(ERR_DCCALLOWINVALID, user->nick, "Too many nicks on DCCALLOW list");
-						return CMD_FAILURE;
+						return CmdResult::FAILURE;
 					}
 
-					for (dccallowlist::const_iterator k = dl->begin(); k != dl->end(); ++k)
+					for (const auto& dccallow : *dl)
 					{
-						if (k->nickname == target->nick)
+						if (dccallow.nickname == target->nick)
 						{
-							user->WriteNumeric(ERR_DCCALLOWINVALID, user->nick, InspIRCd::Format("%s is already on your DCCALLOW list", target->nick.c_str()));
-							return CMD_FAILURE;
+							user->WriteNumeric(ERR_DCCALLOWINVALID, user->nick, INSP_FORMAT("{} is already on your DCCALLOW list", target->nick));
+							return CmdResult::FAILURE;
 						}
 					}
 
-					std::string mask = target->nick+"!"+target->ident+"@"+target->GetDisplayedHost();
+					std::string mask = target->GetMask();
 					unsigned long length;
 					if (parameters.size() < 2)
 					{
 						length = defaultlength;
 					}
-					else if (!InspIRCd::IsValidDuration(parameters[1]))
+					else if (!Duration::IsValid(parameters[1]))
 					{
-						user->WriteNumeric(ERR_DCCALLOWINVALID, user->nick, InspIRCd::Format("%s is not a valid DCCALLOW duration", parameters[1].c_str()));
-						return CMD_FAILURE;
+						user->WriteNumeric(ERR_DCCALLOWINVALID, user->nick, INSP_FORMAT("{} is not a valid DCCALLOW duration", parameters[1]));
+						return CmdResult::FAILURE;
 					}
 					else
 					{
-						if (!InspIRCd::Duration(parameters[1], length))
+						if (!Duration::TryFrom(parameters[1], length))
 						{
 							user->WriteNotice("*** Invalid duration for DCC allow");
-							return CMD_FAILURE;
+							return CmdResult::FAILURE;
 						}
 					}
 
 					if (!InspIRCd::IsValidMask(mask))
 					{
-						return CMD_FAILURE;
+						return CmdResult::FAILURE;
 					}
 
-					dl->push_back(DCCAllow(target->nick, mask, ServerInstance->Time(), length));
+					dl->emplace_back(target->nick, mask, ServerInstance->Time(), length);
 
 					if (length > 0)
 					{
-						user->WriteNumeric(RPL_DCCALLOWTIMED, user->nick, InspIRCd::Format("Added %s to DCCALLOW list for %s", target->nick.c_str(), InspIRCd::DurationString(length).c_str()));
+						user->WriteNumeric(RPL_DCCALLOWTIMED, user->nick, INSP_FORMAT("Added {} to DCCALLOW list for {}", target->nick, Duration::ToString(length)));
 					}
 					else
 					{
-						user->WriteNumeric(RPL_DCCALLOWPERMANENT, user->nick, InspIRCd::Format("Added %s to DCCALLOW list for this session", target->nick.c_str()));
+						user->WriteNumeric(RPL_DCCALLOWPERMANENT, user->nick, INSP_FORMAT("Added {} to DCCALLOW list for this session", target->nick));
 					}
 
 					/* route it. */
-					return CMD_SUCCESS;
+					return CmdResult::SUCCESS;
 				}
 			}
 			else
 			{
 				// nick doesn't exist
 				user->WriteNumeric(Numerics::NoSuchNick(nick));
-				return CMD_FAILURE;
+				return CmdResult::FAILURE;
 			}
 		}
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 
-	RouteDescriptor GetRouting(User* user, const Params& parameters) CXX11_OVERRIDE
+	RouteDescriptor GetRouting(User* user, const Params& parameters) override
 	{
 		return ROUTE_BROADCAST;
 	}
 
-	void DisplayHelp(User* user)
+	static void DisplayHelp(User* user)
 	{
 		user->WriteNumeric(RPL_HELPSTART, "*", "DCCALLOW [(+|-)<nick> [<time>]]|[LIST|HELP]");
-		for (size_t i = 0; i < sizeof(helptext)/sizeof(helptext[0]); i++)
-			user->WriteNumeric(RPL_HELPTXT, "*", helptext[i]);
+		for (const auto& helpline : helptext)
+			user->WriteNumeric(RPL_HELPTXT, "*", helpline);
 		user->WriteNumeric(RPL_ENDOFHELP, "*", "End of DCCALLOW HELP");
 
 		LocalUser* localuser = IS_LOCAL(user);
@@ -355,12 +368,13 @@ class CommandDccallow : public Command
 		// display current DCCALLOW list
 		user->WriteNumeric(RPL_DCCALLOWSTART, "Users on your DCCALLOW list:");
 
-		dl = ext.get(user);
+		dl = ext.Get(user);
 		if (dl)
 		{
-			for (dccallowlist::const_iterator c = dl->begin(); c != dl->end(); ++c)
+			for (const auto& dccallow : *dl)
 			{
-				user->WriteNumeric(RPL_DCCALLOWLIST, user->nick, InspIRCd::Format("%s (%s)", c->nickname.c_str(), c->hostmask.c_str()));
+				user->WriteNumeric(RPL_DCCALLOWLIST, user->nick, INSP_FORMAT("{} ({})",
+					dccallow.nickname, dccallow.hostmask));
 			}
 		}
 
@@ -369,24 +383,26 @@ class CommandDccallow : public Command
 
 };
 
-class ModuleDCCAllow : public Module
+class ModuleDCCAllow final
+	: public Module
 {
+private:
 	DCCAllowExt ext;
 	CommandDccallow cmd;
-	bool blockchat;
+	bool blockchat = false;
 	std::string defaultaction;
 
- public:
+public:
 	ModuleDCCAllow()
-		: ext(this)
+		: Module(VF_VENDOR | VF_COMMON, "Allows the server administrator to configure what files are allowed to be sent via DCC SEND and allows users to configure who can send them DCC CHAT and DCC SEND requests.")
+		, ext(this)
 		, cmd(this, ext)
-		, blockchat(false)
 	{
 	}
 
-	void OnUserQuit(User* user, const std::string &reason, const std::string &oper_message) CXX11_OVERRIDE
+	void OnUserQuit(User* user, const std::string& reason, const std::string& oper_message) override
 	{
-		dccallowlist* udl = ext.get(user);
+		dccallowlist* udl = ext.Get(user);
 
 		// remove their DCCALLOW list if they have one
 		if (udl)
@@ -397,26 +413,26 @@ class ModuleDCCAllow : public Module
 		RemoveNick(user);
 	}
 
-	void OnUserPostNick(User* user, const std::string &oldnick) CXX11_OVERRIDE
+	void OnUserPostNick(User* user, const std::string& oldnick) override
 	{
 		RemoveNick(user);
 	}
 
-	ModResult OnUserPreMessage(User* user, const MessageTarget& target, MessageDetails& details) CXX11_OVERRIDE
+	ModResult OnUserPreMessage(User* user, MessageTarget& target, MessageDetails& details) override
 	{
 		if (!IS_LOCAL(user))
 			return MOD_RES_PASSTHRU;
 
 		if (target.type == MessageTarget::TYPE_USER)
 		{
-			User* u = target.Get<User>();
+			auto* u = target.Get<User>();
 
 			/* Always allow a user to dcc themselves (although... why?) */
 			if (user == u)
 				return MOD_RES_PASSTHRU;
 
-			std::string ctcpname;
-			std::string ctcpbody;
+			std::string_view ctcpname;
+			std::string_view ctcpbody;
 			if (details.IsCTCP(ctcpname, ctcpbody))
 			{
 				Expire();
@@ -426,25 +442,27 @@ class ModuleDCCAllow : public Module
 
 				if (irc::equals(ctcpname, "DCC") && !ctcpbody.empty())
 				{
-					dl = ext.get(u);
-					if (dl && dl->size())
+					dl = ext.Get(u);
+					if (dl && !dl->empty())
 					{
-						for (dccallowlist::const_iterator iter = dl->begin(); iter != dl->end(); ++iter)
-							if (InspIRCd::Match(user->GetFullHost(), iter->hostmask))
+						for (const auto& dccallow : *dl)
+						{
+							if (InspIRCd::Match(user->GetMask(), dccallow.hostmask))
 								return MOD_RES_PASSTHRU;
+						}
 					}
 
 					size_t s = ctcpbody.find(' ');
 					if (s == std::string::npos)
 						return MOD_RES_PASSTHRU;
 
-					const std::string type = ctcpbody.substr(0, s);
+					const std::string type = std::string(ctcpbody).substr(0, s);
 
 					if (irc::equals(type, "SEND"))
 					{
 						size_t first;
 
-						std::string buf = ctcpbody.substr(s + 1);
+						std::string buf = std::string(ctcpbody).substr(s + 1);
 
 						if (!buf.empty() && buf[0] == '"')
 						{
@@ -468,12 +486,12 @@ class ModuleDCCAllow : public Module
 						std::string filename = buf.substr(first, s);
 
 						bool found = false;
-						for (unsigned int i = 0; i < bfl.size(); i++)
+						for (const auto& bf : bfl)
 						{
-							if (InspIRCd::Match(filename, bfl[i].filemask, ascii_case_insensitive_map))
+							if (InspIRCd::Match(filename, bf.filemask, ascii_case_insensitive_map))
 							{
 								/* We have a matching badfile entry, override whatever the default action is */
-								if (stdalgo::string::equalsci(bfl[i].action, "allow"))
+								if (insp::equalsci(bf.action, "allow"))
 									return MOD_RES_PASSTHRU;
 								else
 								{
@@ -488,14 +506,14 @@ class ModuleDCCAllow : public Module
 							return MOD_RES_PASSTHRU;
 
 						user->WriteNotice("The user " + u->nick + " is not accepting DCC SENDs from you. Your file " + filename + " was not sent.");
-						u->WriteNotice(user->nick + " (" + user->ident + "@" + user->GetDisplayedHost() + ") attempted to send you a file named " + filename + ", which was blocked.");
+						u->WriteNotice(user->nick + " (" + user->GetUserHost() + ") attempted to send you a file named " + filename + ", which was blocked.");
 						u->WriteNotice("If you trust " + user->nick + " and were expecting this, you can type /DCCALLOW HELP for information on the DCCALLOW system.");
 						return MOD_RES_DENY;
 					}
 					else if (blockchat && irc::equals(type, "CHAT"))
 					{
 						user->WriteNotice("The user " + u->nick + " is not accepting DCC CHAT requests from you.");
-						u->WriteNotice(user->nick + " (" + user->ident + "@" + user->GetDisplayedHost() + ") attempted to initiate a DCC CHAT session, which was blocked.");
+						u->WriteNotice(user->nick + " (" + user->GetUserHost() + ") attempted to initiate a DCC CHAT session, which was blocked.");
 						u->WriteNotice("If you trust " + user->nick + " and were expecting this, you can type /DCCALLOW HELP for information on the DCCALLOW system.");
 						return MOD_RES_DENY;
 					}
@@ -510,10 +528,10 @@ class ModuleDCCAllow : public Module
 		for (userlist::iterator iter = ul.begin(); iter != ul.end();)
 		{
 			User* u = (User*)(*iter);
-			dl = ext.get(u);
+			dl = ext.Get(u);
 			if (dl)
 			{
-				if (dl->size())
+				if (!dl->empty())
 				{
 					dccallowlist::iterator iter2 = dl->begin();
 					while (iter2 != dl->end())
@@ -521,7 +539,7 @@ class ModuleDCCAllow : public Module
 						time_t expires = iter2->set_on + iter2->length;
 						if (iter2->length != 0 && expires <= ServerInstance->Time())
 						{
-							u->WriteNumeric(RPL_DCCALLOWEXPIRED, u->nick, InspIRCd::Format("DCCALLOW entry for %s has expired", iter2->nickname.c_str()));
+							u->WriteNumeric(RPL_DCCALLOWEXPIRED, u->nick, INSP_FORMAT("DCCALLOW entry for {} has expired", iter2->nickname));
 							iter2 = dl->erase(iter2);
 						}
 						else
@@ -544,11 +562,11 @@ class ModuleDCCAllow : public Module
 		/* Iterate through all DCCALLOW lists and remove user */
 		for (userlist::iterator iter = ul.begin(); iter != ul.end();)
 		{
-			User *u = (User*)(*iter);
-			dl = ext.get(u);
+			User* u = (User*)(*iter);
+			dl = ext.Get(u);
 			if (dl)
 			{
-				if (dl->size())
+				if (!dl->empty())
 				{
 					for (dccallowlist::iterator i = dl->begin(); i != dl->end(); ++i)
 					{
@@ -556,7 +574,7 @@ class ModuleDCCAllow : public Module
 						{
 
 							u->WriteNotice(i->nickname + " left the network or changed their nickname and has been removed from your DCCALLOW list");
-							u->WriteNumeric(RPL_DCCALLOWREMOVED, u->nick, InspIRCd::Format("Removed %s from your DCCALLOW list", i->nickname.c_str()));
+							u->WriteNumeric(RPL_DCCALLOWREMOVED, u->nick, INSP_FORMAT("Removed {} from your DCCALLOW list", i->nickname));
 							dl->erase(i);
 							break;
 						}
@@ -571,7 +589,7 @@ class ModuleDCCAllow : public Module
 		}
 	}
 
-	void RemoveFromUserlist(User *user)
+	static void RemoveFromUserlist(User* user)
 	{
 		// remove user from userlist
 		for (userlist::iterator j = ul.begin(); j != ul.end(); ++j)
@@ -585,30 +603,25 @@ class ModuleDCCAllow : public Module
 		}
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
 		bannedfilelist newbfl;
-		ConfigTagList tags = ServerInstance->Config->ConfTags("banfile");
-		for (ConfigIter i = tags.first; i != tags.second; ++i)
+
+		for (const auto& [_, tag] : ServerInstance->Config->ConfTags("banfile"))
 		{
 			BannedFileList bf;
-			bf.filemask = i->second->getString("pattern");
-			bf.action = i->second->getString("action");
+			bf.filemask = tag->getString("pattern");
+			bf.action = tag->getString("action");
 			newbfl.push_back(bf);
 		}
 		bfl.swap(newbfl);
 
-		ConfigTag* tag = ServerInstance->Config->ConfValue("dccallow");
-		cmd.ext.maxentries = tag->getUInt("maxentries", 20, 1);
+		const auto& tag = ServerInstance->Config->ConfValue("dccallow");
+		cmd.ext.maxentries = tag->getNum<unsigned long>("maxentries", 20, 1);
 		cmd.defaultlength = tag->getDuration("length", 0);
 		blockchat = tag->getBool("blockchat");
 		defaultaction = tag->getString("action");
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows the server administrator to configure what files are allowed to be sent via DCC SEND and allows users to configure who can send them DCC CHAT and DCC SEND requests.", VF_COMMON | VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleDCCAllow)
