@@ -2,9 +2,9 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
+ *   Copyright (C) 2015, 2017-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2014 Adam <Adam@anope.org>
- *   Copyright (C) 2013, 2015, 2017-2018, 2021 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012-2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2012-2013, 2015-2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
@@ -30,72 +30,68 @@
 #include "modules/stats.h"
 #include "modules/who.h"
 #include "modules/whois.h"
+#include "timeutils.h"
 
 /** Handles user mode +H
  */
-class HideOper : public SimpleUserModeHandler
+class HideOper final
+	: public SimpleUserMode
 {
- public:
-	size_t opercount;
+public:
+	size_t opercount = 0;
 
-	HideOper(Module* Creator) : SimpleUserModeHandler(Creator, "hideoper", 'H')
-		, opercount(0)
+	HideOper(Module* Creator)
+		: SimpleUserMode(Creator, "hideoper", 'H', true)
 	{
-		oper = true;
 	}
 
-	ModeAction OnModeChange(User* source, User* dest, Channel* channel, std::string& parameter, bool adding) CXX11_OVERRIDE
+	bool OnModeChange(User* source, User* dest, Channel* channel, Modes::Change& change) override
 	{
-		if (SimpleUserModeHandler::OnModeChange(source, dest, channel, parameter, adding) == MODEACTION_DENY)
-			return MODEACTION_DENY;
+		if (!SimpleUserMode::OnModeChange(source, dest, channel, change))
+			return false;
 
-		if (adding)
+		if (change.adding)
 			opercount++;
 		else
 			opercount--;
 
-		return MODEACTION_ALLOW;
+		return true;
 	}
 };
 
-class ModuleHideOper
+class ModuleHideOper final
 	: public Module
 	, public Stats::EventListener
 	, public Who::EventListener
 	, public Whois::LineEventListener
 {
- private:
+private:
 	HideOper hm;
-	bool active;
+	bool active = false;
 
- public:
+public:
 	ModuleHideOper()
-		: Stats::EventListener(this)
+		: Module(VF_VENDOR, "Adds user mode H (hideoper) which hides the server operator status of a user from unprivileged users.")
+		, Stats::EventListener(this)
 		, Who::EventListener(this)
 		, Whois::LineEventListener(this)
 		, hm(this)
-		, active(false)
 	{
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds user mode H (hideoper) which hides the server operator status of a user from unprivileged users.", VF_VENDOR);
-	}
-
-	void OnUserQuit(User* user, const std::string&, const std::string&) CXX11_OVERRIDE
+	void OnUserQuit(User* user, const std::string&, const std::string&) override
 	{
 		if (user->IsModeSet(hm))
 			hm.opercount--;
 	}
 
-	ModResult OnNumeric(User* user, const Numeric::Numeric& numeric) CXX11_OVERRIDE
+	ModResult OnNumeric(User* user, const Numeric::Numeric& numeric) override
 	{
 		if (numeric.GetNumeric() != RPL_LUSEROP || active || user->HasPrivPermission("users/auspex"))
 			return MOD_RES_PASSTHRU;
 
 		// If there are no visible operators then we shouldn't send the numeric.
-		size_t opercount = ServerInstance->Users->all_opers.size() - hm.opercount;
+		size_t opercount = ServerInstance->Users.all_opers.size() - hm.opercount;
 		if (opercount)
 		{
 			active = true;
@@ -105,7 +101,7 @@ class ModuleHideOper
 		return MOD_RES_DENY;
 	}
 
-	ModResult OnWhoisLine(Whois::Context& whois, Numeric::Numeric& numeric) CXX11_OVERRIDE
+	ModResult OnWhoisLine(Whois::Context& whois, Numeric::Numeric& numeric) override
 	{
 		/* Dont display numeric 313 (RPL_WHOISOPER) if they have +H set and the
 		 * person doing the WHOIS is not an oper
@@ -122,7 +118,7 @@ class ModuleHideOper
 		return MOD_RES_PASSTHRU;
 	}
 
-	ModResult OnWhoLine(const Who::Request& request, LocalUser* source, User* user, Membership* memb, Numeric::Numeric& numeric) CXX11_OVERRIDE
+	ModResult OnWhoLine(const Who::Request& request, LocalUser* source, User* user, Membership* memb, Numeric::Numeric& numeric) override
 	{
 		if (user->IsModeSet(hm) && !source->HasPrivPermission("users/auspex"))
 		{
@@ -147,27 +143,44 @@ class ModuleHideOper
 		return MOD_RES_PASSTHRU;
 	}
 
-	ModResult OnStats(Stats::Context& stats) CXX11_OVERRIDE
+	ModResult OnStats(Stats::Context& stats) override
 	{
 		if (stats.GetSymbol() != 'P')
 			return MOD_RES_PASSTHRU;
 
-		unsigned int count = 0;
-		const UserManager::OperList& opers = ServerInstance->Users->all_opers;
-		for (UserManager::OperList::const_iterator i = opers.begin(); i != opers.end(); ++i)
+		bool source_has_priv = stats.GetSource()->HasPrivPermission("users/auspex");
+		for (auto* oper : ServerInstance->Users.all_opers)
 		{
-			User* oper = *i;
-			if (!oper->server->IsULine() && (stats.GetSource()->IsOper() || !oper->IsModeSet(hm)))
-			{
-				LocalUser* lu = IS_LOCAL(oper);
-				const std::string idle = lu ? InspIRCd::DurationString(ServerInstance->Time() - lu->idle_lastmsg) : "unavailable";
-				stats.AddRow(249, InspIRCd::Format("%s (%s@%s) Idle: %s", oper->nick.c_str(),
-					oper->ident.c_str(), oper->GetDisplayedHost().c_str(), idle.c_str()));
-				count++;
-			}
-		}
-		stats.AddRow(249, ConvToStr(count)+" OPER(s)");
+			if (oper->server->IsService() || (oper->IsModeSet(hm) && !source_has_priv))
+				continue;
 
+			std::string extra;
+			if (oper->IsAway())
+			{
+				const std::string awayperiod = Duration::ToString(ServerInstance->Time() - oper->away->time);
+				const std::string awaytime = Time::ToString(oper->away->time);
+
+				extra = INSP_FORMAT(": away for {} [since {}] ({})", awayperiod, awaytime, oper->away->message);
+			}
+
+			auto* loper = IS_LOCAL(oper);
+			if (loper)
+			{
+				const std::string idleperiod = Duration::ToString(ServerInstance->Time() - loper->idle_lastmsg);
+				const std::string idletime = Time::ToString(loper->idle_lastmsg);
+
+				extra += INSP_FORMAT("{} idle for {} [since {}]", extra.empty() ? ':' : ',', idleperiod, idletime);
+			}
+
+			stats.AddGenericRow(INSP_FORMAT("\x02{}\x02 ({}){}", oper->nick, oper->GetUserHost(), extra));
+		}
+
+		// Sort opers alphabetically.
+		std::sort(stats.GetRows().begin(), stats.GetRows().end(), [](const auto& lhs, const auto& rhs) {
+			return lhs.GetParams()[1] < rhs.GetParams()[1];
+		});
+
+		stats.AddGenericRow(INSP_FORMAT("{} server operator{} total", stats.GetRows().size(), stats.GetRows().size() ? "s" : ""));
 		return MOD_RES_DENY;
 	}
 };
