@@ -1,14 +1,15 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2018 edef <edef@edef.eu>
- *   Copyright (C) 2013-2014, 2017-2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013-2014, 2017, 2019-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
- *   Copyright (C) 2007 John Brooks <special@inspircd.org>
+ *   Copyright (C) 2007 John Brooks <john@jbrooks.io>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2006, 2008 Craig Edwards <brain@inspircd.org>
  *
@@ -25,37 +26,22 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/// $CompilerFlags: -Ivendor_directory("http_parser")
-
 
 #include "inspircd.h"
 #include "iohook.h"
 #include "modules/httpd.h"
+#include "utility/string.h"
 
 #ifdef __GNUC__
 # pragma GCC diagnostic push
 #endif
 
-// Fix warnings about the use of commas at end of enumerator lists and long long
-// on C++03.
-#if defined __clang__
-# pragma clang diagnostic ignored "-Wc++11-extensions"
-# pragma clang diagnostic ignored "-Wc++11-long-long"
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wlong-long"
-# if (__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 8))
-#  pragma GCC diagnostic ignored "-Wpedantic"
-# else
-#  pragma GCC diagnostic ignored "-pedantic"
-# endif
-#endif
-
 // Fix warnings about shadowing in http_parser.
 #ifdef __GNUC__
 # pragma GCC diagnostic ignored "-Wshadow"
 #endif
 
-#include <http_parser.c>
+#include <http_parser/http_parser.c>
 
 #ifdef __GNUC__
 # pragma GCC diagnostic pop
@@ -71,9 +57,12 @@ static http_parser_settings parser_settings;
 
 /** A socket used for HTTP transport
  */
-class HttpServerSocket : public BufferedSocket, public Timer, public insp::intrusive_list_node<HttpServerSocket>
+class HttpServerSocket final
+	: public BufferedSocket
+	, public Timer
+	, public insp::intrusive_list_node<HttpServerSocket>
 {
- private:
+private:
 	friend class ModuleHttpServer;
 
 	http_parser parser;
@@ -83,18 +72,18 @@ class HttpServerSocket : public BufferedSocket, public Timer, public insp::intru
 	HTTPHeaders headers;
 	std::string body;
 	size_t total_buffers;
-	int status_code;
+	int status_code = 0;
 
 	/** True if this object is in the cull list
 	 */
-	bool waitingcull;
-	bool messagecomplete;
+	bool waitingcull = false;
+	bool messagecomplete = false;
 
-	bool Tick(time_t currtime) CXX11_OVERRIDE
+	bool Tick() override
 	{
 		if (!messagecomplete)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "HTTP socket %d timed out", GetFd());
+			ServerInstance->Logs.Debug(MODNAME, "HTTP socket {} timed out", GetFd());
 			Close();
 			return false;
 		}
@@ -216,23 +205,20 @@ class HttpServerSocket : public BufferedSocket, public Timer, public insp::intru
 		return 0;
 	}
 
- public:
-	HttpServerSocket(int newfd, const std::string& IP, ListenSocket* via, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server, unsigned int timeoutsec)
+public:
+	HttpServerSocket(int newfd, const std::string& IP, ListenSocket* via, const irc::sockets::sockaddrs& client, const irc::sockets::sockaddrs& server, unsigned long timeoutsec)
 		: BufferedSocket(newfd)
-		, Timer(timeoutsec)
+		, Timer(timeoutsec, false)
 		, ip(IP)
-		, status_code(0)
-		, waitingcull(false)
-		, messagecomplete(false)
 	{
 		if ((!via->iohookprovs.empty()) && (via->iohookprovs.back()))
 		{
 			via->iohookprovs.back()->OnAccept(this, client, server);
 			// IOHook may have errored
-			if (!getError().empty())
+			if (!GetError().empty())
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "HTTP socket %d encountered a hook error: %s",
-					GetFd(), getError().c_str());
+				ServerInstance->Logs.Debug(MODNAME, "HTTP socket {} encountered a hook error: {}",
+					GetFd(), GetError());
 				Close();
 				return;
 			}
@@ -243,12 +229,12 @@ class HttpServerSocket : public BufferedSocket, public Timer, public insp::intru
 		ServerInstance->Timers.AddTimer(this);
 	}
 
-	~HttpServerSocket()
+	~HttpServerSocket() override
 	{
 		sockets.erase(this);
 	}
 
-	void Close() CXX11_OVERRIDE
+	void Close() override
 	{
 		if (waitingcull || !HasFd())
 			return;
@@ -258,34 +244,34 @@ class HttpServerSocket : public BufferedSocket, public Timer, public insp::intru
 		ServerInstance->GlobalCulls.AddItem(this);
 	}
 
-	void OnError(BufferedSocketError err) CXX11_OVERRIDE
+	void OnError(BufferedSocketError err) override
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "HTTP socket %d encountered an error: %d - %s",
-			GetFd(), err, getError().c_str());
+		ServerInstance->Logs.Debug(MODNAME, "HTTP socket {} encountered an error: {} - {}",
+			GetFd(), (int)err, GetError());
 		Close();
 	}
 
-	void SendHTTPError(unsigned int response, const char* errstr = NULL)
+	void SendHTTPError(unsigned int response, const char* errstr = nullptr)
 	{
 		if (!errstr)
 			errstr = http_status_str((http_status)response);
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Sending HTTP error %u: %s", response, errstr);
+		ServerInstance->Logs.Debug(MODNAME, "Sending HTTP error {}: {}", response, errstr);
 		static HTTPHeaders empty;
-		std::string data = InspIRCd::Format(
+		std::string data = INSP_FORMAT(
 			"<html><head></head><body style='font-family: sans-serif; text-align: center'>"
-			"<h1 style='font-size: 48pt'>Error %u</h1><h2 style='font-size: 24pt'>%s</h2><hr>"
+			"<h1 style='font-size: 48pt'>Error {}</h1><h2 style='font-size: 24pt'>{}</h2><hr>"
 			"<small>Powered by <a href='https://www.inspircd.org'>InspIRCd</a></small></body></html>",
 			response, errstr);
 
 		Page(data, response, &empty);
 	}
 
-	void SendHeaders(unsigned long size, unsigned int response, HTTPHeaders &rheaders)
+	void SendHeaders(unsigned long size, unsigned int response, HTTPHeaders& rheaders)
 	{
-		WriteData(InspIRCd::Format("HTTP/%u.%u %u %s\r\n", parser.http_major ? parser.http_major : 1, parser.http_major ? parser.http_minor : 1, response, http_status_str((http_status)response)));
+		WriteData(INSP_FORMAT("HTTP/{}.{} {} {}\r\n", parser.http_major ? parser.http_major : 1, parser.http_major ? parser.http_minor : 1, response, http_status_str((http_status)response)));
 
-		rheaders.CreateHeader("Date", InspIRCd::TimeString(ServerInstance->Time(), "%a, %d %b %Y %H:%M:%S GMT", true));
+		rheaders.CreateHeader("Date", Time::ToString(ServerInstance->Time(), "%a, %d %b %Y %H:%M:%S GMT", true));
 		rheaders.CreateHeader("Server", INSPIRCD_BRANCH);
 		rheaders.SetHeader("Content-Length", ConvToStr(size));
 
@@ -303,7 +289,7 @@ class HttpServerSocket : public BufferedSocket, public Timer, public insp::intru
 		WriteData("\r\n");
 	}
 
-	void OnDataReady() CXX11_OVERRIDE
+	void OnDataReady() override
 	{
 		if (parser.upgrade || HTTP_PARSER_ERRNO(&parser))
 			return;
@@ -316,17 +302,16 @@ class HttpServerSocket : public BufferedSocket, public Timer, public insp::intru
 
 	void ServeData()
 	{
-		ModResult MOD_RESULT;
 		std::string method = http_method_str(static_cast<http_method>(parser.method));
 		HTTPRequestURI parsed;
 		ParseURI(uri, parsed);
 		HTTPRequest acl(method, parsed, &headers, this, ip, body);
-		FIRST_MOD_RESULT_CUSTOM(*aclevprov, HTTPACLEventListener, OnHTTPACLCheck, MOD_RESULT, (acl));
-		if (MOD_RESULT != MOD_RES_DENY)
+		ModResult res = aclevprov->FirstResult(&HTTPACLEventListener::OnHTTPACLCheck, acl);
+		if (res != MOD_RES_DENY)
 		{
 			HTTPRequest request(method, parsed, &headers, this, ip, body);
-			FIRST_MOD_RESULT_CUSTOM(*reqevprov, HTTPRequestEventListener, OnHTTPRequest, MOD_RESULT, (request));
-			if (MOD_RESULT == MOD_RES_PASSTHRU)
+			res = reqevprov->FirstResult(&HTTPRequestEventListener::OnHTTPRequest, request);
+			if (res == MOD_RES_PASSTHRU)
 			{
 				SendHTTPError(404);
 			}
@@ -376,7 +361,7 @@ class HttpServerSocket : public BufferedSocket, public Timer, public insp::intru
 			}
 
 			out.path.reserve(url.field_data[UF_PATH].len);
-			out.path.append("/").append(stdalgo::string::join(pathsegments, '/'));
+			out.path.append("/").append(insp::join(pathsegments, '/'));
 		}
 
 		if (url.field_set & (1 << UF_FRAGMENT))
@@ -394,41 +379,45 @@ class HttpServerSocket : public BufferedSocket, public Timer, public insp::intru
 			eq_pos = token.find('=');
 			if (eq_pos == std::string::npos)
 			{
-				out.query_params.insert(std::make_pair(token, ""));
+				out.query_params.emplace(token, "");
 			}
 			else
 			{
-				out.query_params.insert(std::make_pair(token.substr(0, eq_pos), token.substr(eq_pos + 1)));
+				out.query_params.emplace(token.substr(0, eq_pos), token.substr(eq_pos + 1));
 			}
 		}
 		return true;
 	}
 };
 
-class HTTPdAPIImpl : public HTTPdAPIBase
+class HTTPdAPIImpl final
+	: public HTTPdAPIBase
 {
- public:
+public:
 	HTTPdAPIImpl(Module* parent)
 		: HTTPdAPIBase(parent)
 	{
 	}
 
-	void SendResponse(HTTPDocumentResponse& resp) CXX11_OVERRIDE
+	void SendResponse(HTTPDocumentResponse& resp) override
 	{
 		resp.src.sock->Page(resp.document, resp.responsecode, &resp.headers);
 	}
 };
 
-class ModuleHttpServer : public Module
+class ModuleHttpServer final
+	: public Module
 {
+private:
 	HTTPdAPIImpl APIImpl;
-	unsigned int timeoutsec;
+	unsigned long timeoutsec;
 	Events::ModuleEventProvider acleventprov;
 	Events::ModuleEventProvider reqeventprov;
 
- public:
+public:
 	ModuleHttpServer()
-		: APIImpl(this)
+		: Module(VF_VENDOR, "Allows the server administrator to serve various useful resources over HTTP.")
+		, APIImpl(this)
 		, acleventprov(this, "event/http-acl")
 		, reqeventprov(this, "event/http-request")
 	{
@@ -437,27 +426,27 @@ class ModuleHttpServer : public Module
 		HttpServerSocket::ConfigureParser();
 	}
 
-	void init() CXX11_OVERRIDE
+	void init() override
 	{
 		HttpModule = this;
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("httpd");
+		const auto& tag = ServerInstance->Config->ConfValue("httpd");
 		timeoutsec = tag->getDuration("timeout", 10, 1);
 	}
 
-	ModResult OnAcceptConnection(int nfd, ListenSocket* from, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server) CXX11_OVERRIDE
+	ModResult OnAcceptConnection(int nfd, ListenSocket* from, const irc::sockets::sockaddrs& client, const irc::sockets::sockaddrs& server) override
 	{
-		if (!stdalgo::string::equalsci(from->bind_tag->getString("type"), "httpd"))
+		if (!insp::equalsci(from->bind_tag->getString("type"), "httpd"))
 			return MOD_RES_PASSTHRU;
 
-		sockets.push_front(new HttpServerSocket(nfd, client->addr(), from, client, server, timeoutsec));
+		sockets.push_front(new HttpServerSocket(nfd, client.addr(), from, client, server, timeoutsec));
 		return MOD_RES_ALLOW;
 	}
 
-	void OnUnloadModule(Module* mod) CXX11_OVERRIDE
+	void OnUnloadModule(Module* mod) override
 	{
 		for (insp::intrusive_list<HttpServerSocket>::const_iterator i = sockets.begin(); i != sockets.end(); )
 		{
@@ -465,25 +454,17 @@ class ModuleHttpServer : public Module
 			++i;
 			if (sock->GetModHook(mod))
 			{
-				sock->cull();
+				sock->Cull();
 				delete sock;
 			}
 		}
 	}
 
-	CullResult cull() CXX11_OVERRIDE
+	Cullable::Result Cull() override
 	{
-		for (insp::intrusive_list<HttpServerSocket>::const_iterator i = sockets.begin(); i != sockets.end(); ++i)
-		{
-			HttpServerSocket* sock = *i;
+		for (auto* sock : sockets)
 			sock->Close();
-		}
-		return Module::cull();
-	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows the server administrator to serve various useful resources over HTTP.", VF_VENDOR);
+		return Module::Cull();
 	}
 };
 
