@@ -1,7 +1,7 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2017-2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2017-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2013 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
@@ -24,6 +24,8 @@
 
 #include "inspircd.h"
 #include "listmode.h"
+#include "modules/extban.h"
+#include "modules/isupport.h"
 
 enum
 {
@@ -32,38 +34,68 @@ enum
 	RPL_ENDOFINVEXLIST = 347
 };
 
-class InviteException : public ListModeBase
+class InviteException final
+	: public ListModeBase
 {
- public:
+private:
+	ExtBan::ManagerRef extbanmgr;
+
+public:
 	InviteException(Module* Creator)
-		: ListModeBase(Creator, "invex", 'I', "End of Channel Invite Exception List", RPL_INVEXLIST, RPL_ENDOFINVEXLIST, true)
+		: ListModeBase(Creator, "invex", 'I', RPL_INVEXLIST, RPL_ENDOFINVEXLIST)
+		, extbanmgr(Creator)
 	{
 		syntax = "<mask>";
 	}
+
+	bool CompareEntry(const std::string& entry, const std::string& value) const override
+	{
+		if (extbanmgr)
+		{
+			auto res = extbanmgr->CompareEntry(this, entry, value);
+			if (res != ExtBan::Comparison::NOT_AN_EXTBAN)
+				return res == ExtBan::Comparison::MATCH;
+		}
+		return irc::equals(entry, value);
+	}
+
+	bool ValidateParam(LocalUser* user, Channel* channel, std::string& parameter) override
+	{
+		if (!extbanmgr || !extbanmgr->Canonicalize(parameter))
+			ModeParser::CleanMask(parameter);
+		return true;
+	}
 };
 
-class ModuleInviteException : public Module
+class ModuleInviteException final
+	: public Module
+	, public ISupport::EventListener
 {
+private:
 	bool invite_bypass_key;
 	InviteException ie;
+
 public:
-	ModuleInviteException() : ie(this)
+	ModuleInviteException()
+		: Module(VF_VENDOR, "Adds channel mode I (invex) which allows channel operators to exempt user masks from channel mode i (inviteonly).")
+		, ISupport::EventListener(this)
+		, ie(this)
 	{
 	}
 
-	void On005Numeric(std::map<std::string, std::string>& tokens) CXX11_OVERRIDE
+	void OnBuildISupport(ISupport::TokenMap& tokens) override
 	{
 		tokens["INVEX"] = ConvToStr(ie.GetModeChar());
 	}
 
-	ModResult OnCheckInvite(User* user, Channel* chan) CXX11_OVERRIDE
+	ModResult OnCheckInvite(User* user, Channel* chan) override
 	{
 		ListModeBase::ModeList* list = ie.GetList(chan);
 		if (list)
 		{
-			for (ListModeBase::ModeList::iterator it = list->begin(); it != list->end(); it++)
+			for (const auto& entry : *list)
 			{
-				if (chan->CheckBan(user, it->mask))
+				if (chan->CheckBan(user, entry.mask))
 				{
 					return MOD_RES_ALLOW;
 				}
@@ -73,23 +105,18 @@ public:
 		return MOD_RES_PASSTHRU;
 	}
 
-	ModResult OnCheckKey(User* user, Channel* chan, const std::string& key) CXX11_OVERRIDE
+	ModResult OnCheckKey(User* user, Channel* chan, const std::string& key) override
 	{
 		if (invite_bypass_key)
 			return OnCheckInvite(user, chan);
 		return MOD_RES_PASSTHRU;
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
 		ie.DoRehash();
 		invite_bypass_key = ServerInstance->Config->ConfValue("inviteexception")->getBool("bypasskey", true);
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds channel mode I (invex) which allows channel operators to exempt user masks from channel mode i (inviteonly).", VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleInviteException)
