@@ -1,16 +1,14 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2018-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2018 A_D
- *   Copyright (C) 2014 Daniel Vassdal <shutter@canternet.org>
- *   Copyright (C) 2013, 2017-2020 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2015 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2006, 2010 Craig Edwards <brain@inspircd.org>
- *   Copyright (C) 2006 Oliver Lupton <om@inspircd.org>
+ *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -27,18 +25,13 @@
 
 
 #include "inspircd.h"
+#include "extension.h"
 #include "modules/invite.h"
+#include "numerichelper.h"
 
-enum
+class KickRejoinData final
 {
-	// From RFC 2182.
-	ERR_UNAVAILRESOURCE = 437
-};
-
-
-class KickRejoinData
-{
-	struct KickedUser
+	struct KickedUser final
 	{
 		std::string uuid;
 		time_t expire;
@@ -54,10 +47,13 @@ class KickRejoinData
 
 	mutable KickedList kicked;
 
- public:
+public:
 	const unsigned int delay;
 
-	KickRejoinData(unsigned int Delay) : delay(Delay) { }
+	KickRejoinData(unsigned int Delay)
+		: delay(Delay)
+	{
+	}
 
 	bool canjoin(LocalUser* user) const
 	{
@@ -86,91 +82,90 @@ class KickRejoinData
 		// One user can be in the list multiple times if the user gets kicked, force joins
 		// (skipping OnUserPreJoin) and gets kicked again, but that's okay because canjoin()
 		// works correctly in this case as well
-		kicked.push_back(KickedUser(user, delay));
+		kicked.emplace_back(user, delay);
 	}
 };
 
 /** Handles channel mode +J
  */
-class KickRejoin : public ParamMode<KickRejoin, SimpleExtItem<KickRejoinData> >
+class KickRejoin final
+	: public ParamMode<KickRejoin, SimpleExtItem<KickRejoinData>>
 {
-	const unsigned int max;
- public:
+public:
+	const unsigned int max = 60;
+
 	KickRejoin(Module* Creator)
-		: ParamMode<KickRejoin, SimpleExtItem<KickRejoinData> >(Creator, "kicknorejoin", 'J')
-		, max(60)
+		: ParamMode<KickRejoin, SimpleExtItem<KickRejoinData>>(Creator, "kicknorejoin", 'J')
 	{
 		syntax = "<seconds>";
 	}
 
-	ModeAction OnSet(User* source, Channel* channel, std::string& parameter) CXX11_OVERRIDE
+	bool OnSet(User* source, Channel* channel, std::string& parameter) override
 	{
 		unsigned int v = ConvToNum<unsigned int>(parameter);
 		if (v <= 0)
 		{
 			source->WriteNumeric(Numerics::InvalidModeParameter(channel, this, parameter));
-			return MODEACTION_DENY;
+			return false;
 		}
 
 		if (IS_LOCAL(source) && v > max)
 			v = max;
 
-		ext.set(channel, new KickRejoinData(v));
-		return MODEACTION_ALLOW;
+		ext.Set(channel, v);
+		return true;
 	}
 
 	void SerializeParam(Channel* chan, const KickRejoinData* krd, std::string& out)
 	{
 		out.append(ConvToStr(krd->delay));
 	}
-
-	std::string GetModuleSettings() const
-	{
-		return ConvToStr(max);
-	}
 };
 
-class ModuleKickNoRejoin : public Module
+class ModuleKickNoRejoin final
+	: public Module
 {
 	KickRejoin kr;
 	Invite::API invapi;
 
 public:
 	ModuleKickNoRejoin()
-		: kr(this)
+		: Module(VF_VENDOR | VF_COMMON, "Adds channel mode J (kicknorejoin) which prevents users from rejoining after being kicked from a channel.")
+		, kr(this)
 		, invapi(this)
 	{
 	}
 
-	ModResult OnUserPreJoin(LocalUser* user, Channel* chan, const std::string& cname, std::string& privs, const std::string& keygiven) CXX11_OVERRIDE
+	ModResult OnUserPreJoin(LocalUser* user, Channel* chan, const std::string& cname, std::string& privs, const std::string& keygiven, bool override) override
 	{
-		if (chan)
+		if (!override && chan)
 		{
-			const KickRejoinData* data = kr.ext.get(chan);
+			const KickRejoinData* data = kr.ext.Get(chan);
 			if ((data) && !invapi->IsInvited(user, chan) && (!data->canjoin(user)))
 			{
-				user->WriteNumeric(ERR_UNAVAILRESOURCE, chan->name, InspIRCd::Format("You must wait %u seconds after being kicked to rejoin (+J is set)", data->delay));
+				user->WriteNumeric(ERR_UNAVAILRESOURCE, chan->name, INSP_FORMAT("You must wait {} seconds after being kicked to rejoin (+{} is set)",
+					data->delay, kr.GetModeChar()));
 				return MOD_RES_DENY;
 			}
 		}
 		return MOD_RES_PASSTHRU;
 	}
 
-	void OnUserKick(User* source, Membership* memb, const std::string &reason, CUList& excepts) CXX11_OVERRIDE
+	void OnUserKick(User* source, Membership* memb, const std::string& reason, CUList& excepts) override
 	{
 		if ((!IS_LOCAL(memb->user)) || (source == memb->user))
 			return;
 
-		KickRejoinData* data = kr.ext.get(memb->chan);
+		KickRejoinData* data = kr.ext.Get(memb->chan);
 		if (data)
 		{
 			data->add(memb->user);
 		}
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
+	void GetLinkData(LinkData& data, std::string& compatdata) override
 	{
-		return Version("Adds channel mode J (kicknorejoin) which prevents users from rejoining after being kicked from a channel.", VF_VENDOR | VF_COMMON, kr.GetModuleSettings());
+		data["max-time"] = compatdata = ConvToStr(kr.max);
 	}
 };
 
