@@ -1,9 +1,10 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2016-2019 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2021 Dominic Hamon
+ *   Copyright (C) 2016-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012, 2014 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007-2009 Robin Burchell <robin+git@viroteck.net>
  *
@@ -22,24 +23,27 @@
 
 
 #include "inspircd.h"
+#include "extension.h"
 #include "modules/exemption.h"
+#include "numerichelper.h"
 
 // The number of seconds nickname changing will be blocked for.
 static unsigned int duration;
 
 /** Holds settings and state associated with channel mode +F
  */
-class nickfloodsettings
+class nickfloodsettings final
 {
- public:
+public:
 	unsigned int secs;
 	unsigned int nicks;
 	time_t reset;
-	time_t unlocktime;
-	unsigned int counter;
+	time_t unlocktime = 0;
+	unsigned int counter = 0;
 
 	nickfloodsettings(unsigned int b, unsigned int c)
-		: secs(b), nicks(c), unlocktime(0), counter(0)
+		: secs(b)
+		, nicks(c)
 	{
 		reset = ServerInstance->Time() + secs;
 	}
@@ -55,7 +59,7 @@ class nickfloodsettings
 			counter++;
 	}
 
-	bool shouldlock()
+	bool shouldlock() const
 	{
 		return ((ServerInstance->Time() <= reset) && (counter == this->nicks));
 	}
@@ -81,22 +85,23 @@ class nickfloodsettings
 
 /** Handles channel mode +F
  */
-class NickFlood : public ParamMode<NickFlood, SimpleExtItem<nickfloodsettings> >
+class NickFlood final
+	: public ParamMode<NickFlood, SimpleExtItem<nickfloodsettings>>
 {
- public:
+public:
 	NickFlood(Module* Creator)
-		: ParamMode<NickFlood, SimpleExtItem<nickfloodsettings> >(Creator, "nickflood", 'F')
+		: ParamMode<NickFlood, SimpleExtItem<nickfloodsettings>>(Creator, "nickflood", 'F')
 	{
 		syntax = "<nick-changes>:<seconds>";
 	}
 
-	ModeAction OnSet(User* source, Channel* channel, std::string& parameter) CXX11_OVERRIDE
+	bool OnSet(User* source, Channel* channel, std::string& parameter) override
 	{
 		std::string::size_type colon = parameter.find(':');
 		if ((colon == std::string::npos) || (parameter.find('-') != std::string::npos))
 		{
 			source->WriteNumeric(Numerics::InvalidModeParameter(channel, this, parameter));
-			return MODEACTION_DENY;
+			return false;
 		}
 
 		/* Set up the flood parameters for this channel */
@@ -106,11 +111,11 @@ class NickFlood : public ParamMode<NickFlood, SimpleExtItem<nickfloodsettings> >
 		if ((nnicks<1) || (nsecs<1))
 		{
 			source->WriteNumeric(Numerics::InvalidModeParameter(channel, this, parameter));
-			return MODEACTION_DENY;
+			return false;
 		}
 
-		ext.set(channel, new nickfloodsettings(nsecs, nnicks));
-		return MODEACTION_ALLOW;
+		ext.SetFwd(channel, nsecs, nnicks);
+		return true;
 	}
 
 	void SerializeParam(Channel* chan, const nickfloodsettings* nfs, std::string& out)
@@ -120,41 +125,42 @@ class NickFlood : public ParamMode<NickFlood, SimpleExtItem<nickfloodsettings> >
 	}
 };
 
-class ModuleNickFlood : public Module
+class ModuleNickFlood final
+	: public Module
 {
+private:
 	CheckExemption::EventProvider exemptionprov;
 	NickFlood nf;
 
- public:
+public:
 	ModuleNickFlood()
-		: exemptionprov(this)
+		: Module(VF_VENDOR, "Adds channel mode F (nickflood) which helps protect against spammers which mass-change nicknames.")
+		, exemptionprov(this)
 		, nf(this)
 	{
 	}
 
-	void ReadConfig(ConfigStatus&) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus&) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("nickflood");
-		duration = tag->getDuration("duration", 60, 10, 600);
+		const auto& tag = ServerInstance->Config->ConfValue("nickflood");
+		duration = static_cast<unsigned int>(tag->getDuration("duration", 60, 10, 600));
 	}
 
-	ModResult OnUserPreNick(LocalUser* user, const std::string& newnick) CXX11_OVERRIDE
+	ModResult OnUserPreNick(LocalUser* user, const std::string& newnick) override
 	{
-		for (User::ChanList::iterator i = user->chans.begin(); i != user->chans.end(); i++)
+		for (const auto* memb : user->chans)
 		{
-			Channel* channel = (*i)->chan;
-			ModResult res;
-
-			nickfloodsettings *f = nf.ext.get(channel);
+			nickfloodsettings* f = nf.ext.Get(memb->chan);
 			if (f)
 			{
-				res = CheckExemption::Call(exemptionprov, user, channel, "nickflood");
+				ModResult res = exemptionprov.Check(user, memb->chan, "nickflood");
 				if (res == MOD_RES_ALLOW)
 					continue;
 
 				if (f->islocked())
 				{
-					user->WriteNumeric(ERR_CANTCHANGENICK, InspIRCd::Format("%s has been locked for nickchanges for %u seconds because there have been more than %u nick changes in %u seconds", channel->name.c_str(), duration, f->nicks, f->secs));
+					user->WriteNumeric(ERR_CANTCHANGENICK, INSP_FORMAT("{} has been locked for nickchanges for {} seconds because there have been more than {} nick changes in {} seconds",
+							memb->chan->name, duration, f->nicks, f->secs));
 					return MOD_RES_DENY;
 				}
 
@@ -162,7 +168,8 @@ class ModuleNickFlood : public Module
 				{
 					f->clear();
 					f->lock();
-					channel->WriteNotice(InspIRCd::Format("No nick changes are allowed for %u seconds because there have been more than %u nick changes in %u seconds.", duration, f->nicks, f->secs));
+					memb->chan->WriteNotice(INSP_FORMAT("No nick changes are allowed for {} seconds because there have been more than {} nick changes in {} seconds.",
+						duration, f->nicks, f->secs));
 					return MOD_RES_DENY;
 				}
 			}
@@ -174,20 +181,17 @@ class ModuleNickFlood : public Module
 	/*
 	 * XXX: HACK: We do the increment on the *POST* event here (instead of all together) because we have no way of knowing whether other modules would block a nickchange.
 	 */
-	void OnUserPostNick(User* user, const std::string &oldnick) CXX11_OVERRIDE
+	void OnUserPostNick(User* user, const std::string& oldnick) override
 	{
 		if (isdigit(user->nick[0])) /* allow switches to UID */
 			return;
 
-		for (User::ChanList::iterator i = user->chans.begin(); i != user->chans.end(); ++i)
+		for (const auto* memb : user->chans)
 		{
-			Channel* channel = (*i)->chan;
-			ModResult res;
-
-			nickfloodsettings *f = nf.ext.get(channel);
+			nickfloodsettings* f = nf.ext.Get(memb->chan);
 			if (f)
 			{
-				res = CheckExemption::Call(exemptionprov, user, channel, "nickflood");
+				ModResult res = exemptionprov.Check(user, memb->chan, "nickflood");
 				if (res == MOD_RES_ALLOW)
 					return;
 
@@ -199,11 +203,6 @@ class ModuleNickFlood : public Module
 			}
 		}
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds channel mode F (nickflood) which helps protect against spammers which mass-change nicknames.", VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleNickFlood)
