@@ -3,13 +3,13 @@
  *
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
- *   Copyright (C) 2013, 2017-2021, 2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2004, 2006 Craig Edwards <brain@inspircd.org>
+ *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -26,44 +26,38 @@
 
 
 #include "inspircd.h"
+#include "modules/callerid.h"
 #include "modules/exemption.h"
+#include "modules/extban.h"
+#include "numerichelper.h"
 
-class NoCTCPUser : public SimpleUserModeHandler
-{
-public:
-	NoCTCPUser(Module* Creator)
-		: SimpleUserModeHandler(Creator, "u_noctcp", 'T')
-	{
-		if (!ServerInstance->Config->ConfValue("noctcp")->getBool("enableumode"))
-			DisableAutoRegister();
-	}
-};
-
-class ModuleNoCTCP : public Module
+class ModuleNoCTCP final
+	: public Module
 {
+private:
+	CallerID::API calleridapi;
 	CheckExemption::EventProvider exemptionprov;
-	SimpleChannelModeHandler nc;
-	NoCTCPUser ncu;
+	ExtBan::Acting extban;
+	SimpleChannelMode nc;
+	SimpleUserMode ncu;
 
- public:
+public:
 	ModuleNoCTCP()
-		: exemptionprov(this)
+		: Module(VF_VENDOR, "Adds channel mode C (noctcp) which allows channels to block messages which contain CTCPs and user mode T (u_noctcp) which allows users to block private messages that contain CTCPs.")
+		, calleridapi(this)
+		, exemptionprov(this)
+		, extban(this, "noctcp", 'C')
 		, nc(this, "noctcp", 'C')
-		, ncu(this)
+		, ncu(this, "u_noctcp", 'T')
 	{
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds channel mode C (noctcp) which allows channels to block messages which contain CTCPs and user mode T (u_noctcp) which allows users to block private messages that contain CTCPs.", VF_VENDOR);
-	}
-
-	ModResult OnUserPreMessage(User* user, const MessageTarget& target, MessageDetails& details) CXX11_OVERRIDE
+	ModResult OnUserPreMessage(User* user, MessageTarget& target, MessageDetails& details) override
 	{
 		if (!IS_LOCAL(user))
 			return MOD_RES_PASSTHRU;
 
-		std::string ctcpname;
+		std::string_view ctcpname;
 		if (!details.IsCTCP(ctcpname) || irc::equals(ctcpname, "ACTION"))
 			return MOD_RES_PASSTHRU;
 
@@ -74,16 +68,14 @@ class ModuleNoCTCP : public Module
 				if (user->HasPrivPermission("channels/ignore-noctcp"))
 					return MOD_RES_PASSTHRU;
 
-				Channel* c = target.Get<Channel>();
-				const Channel::MemberMap& members = c->GetUsers();
-				for (Channel::MemberMap::const_iterator member = members.begin(); member != members.end(); ++member)
+				auto* c = target.Get<Channel>();
+				for (const auto& [u, _] : c->GetUsers())
 				{
-					User* u = member->first;
 					if (u->IsModeSet(ncu))
 						details.exemptions.insert(u);
 				}
 
-				ModResult res = CheckExemption::Call(exemptionprov, user, c, "noctcp");
+				ModResult res = exemptionprov.Check(user, c, "noctcp");
 				if (res == MOD_RES_ALLOW)
 					return MOD_RES_PASSTHRU;
 
@@ -93,24 +85,27 @@ class ModuleNoCTCP : public Module
 					return MOD_RES_DENY;
 				}
 
-				if (c->GetExtBanStatus(user, 'C') == MOD_RES_DENY)
+				if (extban.GetStatus(user, c) == MOD_RES_DENY)
 				{
-					user->WriteNumeric(Numerics::CannotSendTo(c, "CTCPs", 'C', "noctcp"));
+					user->WriteNumeric(Numerics::CannotSendTo(c, "CTCPs", extban));
 					return MOD_RES_DENY;
 				}
 				break;
 			}
 			case MessageTarget::TYPE_USER:
 			{
+				auto* targetuser = target.Get<User>();
 				if (user->HasPrivPermission("users/ignore-noctcp"))
 					return MOD_RES_PASSTHRU;
 
-				User* u = target.Get<User>();
-				if (u->IsModeSet(ncu))
+				if (calleridapi && calleridapi->IsOnAcceptList(user, targetuser))
+					return MOD_RES_PASSTHRU;
+
+				if (targetuser->IsModeSet(ncu))
 				{
 					// Don't send an error message if we're blocking an automatic CTCP reply.
-					if (details.type == MSG_NOTICE)
-						user->WriteNumeric(Numerics::CannotSendTo(u, "CTCPs", &ncu));
+					if (details.type == MessageType::NOTICE)
+						user->WriteNumeric(Numerics::CannotSendTo(targetuser, "CTCPs", &ncu));
 					return MOD_RES_DENY;
 				}
 				break;
@@ -120,10 +115,8 @@ class ModuleNoCTCP : public Module
 				if (user->HasPrivPermission("users/ignore-noctcp"))
 					return MOD_RES_PASSTHRU;
 
-				const UserManager::LocalList& list = ServerInstance->Users.GetLocalUsers();
-				for (UserManager::LocalList::const_iterator iter = list.begin(); iter != list.end(); ++iter)
+				for (auto* u : ServerInstance->Users.GetLocalUsers())
 				{
-					LocalUser* u = *iter;
 					if (u->IsModeSet(ncu))
 						details.exemptions.insert(u);
 				}
@@ -132,11 +125,6 @@ class ModuleNoCTCP : public Module
 		}
 		return MOD_RES_PASSTHRU;
 	}
-
-	void On005Numeric(std::map<std::string, std::string>& tokens) CXX11_OVERRIDE
-	{
-		tokens["EXTBAN"].push_back('C');
-	}
 };
 
 MODULE_INIT(ModuleNoCTCP)
