@@ -1,11 +1,10 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2018-2020 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012-2014 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2019-2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2012, 2014 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2007 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
  *
@@ -24,28 +23,27 @@
 
 
 #include "inspircd.h"
+#include "modules/isupport.h"
+#include "utility/string.h"
 
-class ModuleOperLog : public Module
+class ModuleOperLog final
+	: public Module
 {
+private:
 	bool tosnomask;
 
- public:
-	void init() CXX11_OVERRIDE
+public:
+	ModuleOperLog()
+		: Module(VF_VENDOR, "Allows the server administrator to make the server log when a server operator-only command is executed.")
 	{
-		ServerInstance->SNO->EnableSnomask('r', "OPERLOG");
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows the server administrator to make the server log when a server operator-only command is executed.", VF_VENDOR);
-	}
-
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
 		tosnomask = ServerInstance->Config->ConfValue("operlog")->getBool("tosnomask", false);
 	}
 
-	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) CXX11_OVERRIDE
+	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) override
 	{
 		/* If the command doesnt appear to be valid, we dont want to mess with it. */
 		if (!validated)
@@ -53,14 +51,14 @@ class ModuleOperLog : public Module
 
 		if ((user->IsOper()) && (user->HasCommandPermission(command)))
 		{
-			Command* thiscmd = ServerInstance->Parser.GetHandler(command);
-			if ((thiscmd) && (thiscmd->flags_needed == 'o'))
+			Command* thiscommand = ServerInstance->Parser.GetHandler(command);
+			if ((thiscommand) && (thiscommand->access_needed == CmdAccess::OPERATOR))
 			{
-				std::string msg = "[" + user->GetFullRealHost() + "] " + command + " " + stdalgo::string::join(parameters);
+				std::string msg = "[" + user->GetRealMask() + "] " + command + " " + insp::join(parameters);
 				if (tosnomask)
-					ServerInstance->SNO->WriteGlobalSno('r', msg);
+					ServerInstance->SNO.WriteGlobalSno('o', msg);
 				else
-					ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, msg);
+					ServerInstance->Logs.Normal(MODNAME, msg);
 			}
 		}
 
