@@ -1,15 +1,15 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2019-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2017 B00mX0r <b00mx0r@aureus.pw>
- *   Copyright (C) 2013, 2017, 2019 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2014, 2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012, 2014 Shawn Smith <ShawnSmith0828@gmail.com>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2004, 2006 Craig Edwards <brain@inspircd.org>
+ *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -26,41 +26,44 @@
 
 
 #include "inspircd.h"
+#include "extension.h"
+#include "numerichelper.h"
 
 /** Handle channel mode +L
  */
-class Redirect : public ParamMode<Redirect, LocalStringExt>
+class Redirect final
+	: public ParamMode<Redirect, StringExtItem>
 {
- public:
+public:
 	Redirect(Module* Creator)
-		: ParamMode<Redirect, LocalStringExt>(Creator, "redirect", 'L')
+		: ParamMode<Redirect, StringExtItem>(Creator, "redirect", 'L')
 	{
 		syntax = "<target>";
 	}
 
-	ModeAction OnSet(User* source, Channel* channel, std::string& parameter) CXX11_OVERRIDE
+	bool OnSet(User* source, Channel* channel, std::string& parameter) override
 	{
 		if (IS_LOCAL(source))
 		{
-			if (!ServerInstance->IsChannel(parameter))
+			if (!ServerInstance->Channels.IsChannel(parameter))
 			{
 				source->WriteNumeric(Numerics::NoSuchChannel(parameter));
-				return MODEACTION_DENY;
+				return false;
 			}
 		}
 
 		if (IS_LOCAL(source) && !source->IsOper())
 		{
-			Channel* c = ServerInstance->FindChan(parameter);
+			auto* c = ServerInstance->Channels.Find(parameter);
 			if (!c)
 			{
-				source->WriteNumeric(690, InspIRCd::Format("Target channel %s must exist to be set as a redirect.", parameter.c_str()));
-				return MODEACTION_DENY;
+				source->WriteNumeric(690, INSP_FORMAT("Target channel {} must exist to be set as a redirect.", parameter));
+				return false;
 			}
 			else if (c->GetPrefixValue(source) < OP_VALUE)
 			{
-				source->WriteNumeric(690, InspIRCd::Format("You must be opped on %s to set it as a redirect.", parameter.c_str()));
-				return MODEACTION_DENY;
+				source->WriteNumeric(690, INSP_FORMAT("You must be opped on {} to set it as a redirect.", parameter));
+				return false;
 			}
 		}
 
@@ -68,8 +71,8 @@ class Redirect : public ParamMode<Redirect, LocalStringExt>
 		 * We used to do some checking for circular +L here, but there is no real need for this any more especially as we
 		 * now catch +L looping in PreJoin. Remove it, since O(n) logic makes me sad, and we catch it anyway. :) -- w00t
 		 */
-		ext.set(channel, parameter);
-		return MODEACTION_ALLOW;
+		ext.Set(channel, parameter);
+		return true;
 	}
 
 	void SerializeParam(Channel* chan, const std::string* str, std::string& out)
@@ -78,32 +81,35 @@ class Redirect : public ParamMode<Redirect, LocalStringExt>
 	}
 };
 
-class ModuleRedirect : public Module
+class ModuleRedirect final
+	: public Module
 {
+private:
 	Redirect re;
-	SimpleUserModeHandler antiredirectmode;
+	SimpleUserMode antiredirectmode;
 	ChanModeReference limitmode;
 
- public:
+public:
 	ModuleRedirect()
-		: re(this)
+		: Module(VF_VENDOR, "Allows users to be redirected to another channel when the user limit is reached.")
+		, re(this)
 		, antiredirectmode(this, "antiredirect", 'L')
 		, limitmode(this, "limit")
 	{
 	}
 
-	ModResult OnUserPreJoin(LocalUser* user, Channel* chan, const std::string& cname, std::string& privs, const std::string& keygiven) CXX11_OVERRIDE
+	ModResult OnUserPreJoin(LocalUser* user, Channel* chan, const std::string& cname, std::string& privs, const std::string& keygiven, bool override) override
 	{
-		if (chan)
+		if (!override && chan)
 		{
 			if (chan->IsModeSet(re) && chan->IsModeSet(limitmode))
 			{
-				if (chan->GetUserCounter() >= ConvToNum<size_t>(chan->GetModeParameter(limitmode)))
+				if (chan->GetUsers().size() >= ConvToNum<size_t>(chan->GetModeParameter(limitmode)))
 				{
-					const std::string& channel = *re.ext.get(chan);
+					const std::string& channel = *re.ext.Get(chan);
 
-					/* sometimes broken ulines can make circular or chained +L, avoid this */
-					Channel* destchan = ServerInstance->FindChan(channel);
+					/* sometimes broken services can make circular or chained +L, avoid this */
+					auto* destchan = ServerInstance->Channels.Find(channel);
 					if (destchan && destchan->IsModeSet(re))
 					{
 						user->WriteNumeric(470, cname, '*', "You may not join this channel. A redirect is set, but you may not be redirected as it is a circular loop.");
@@ -126,11 +132,6 @@ class ModuleRedirect : public Module
 		}
 		return MOD_RES_PASSTHRU;
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows users to be redirected to another channel when the user limit is reached.", VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleRedirect)
