@@ -1,14 +1,13 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2017-2020 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2018-2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013-2014, 2016, 2018 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2013 Daniel Vassdal <shutter@canternet.org>
- *   Copyright (C) 2012-2016, 2018 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2007 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2004, 2006 Craig Edwards <brain@inspircd.org>
+ *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -25,45 +24,47 @@
 
 
 #include "inspircd.h"
+#include "clientprotocolmsg.h"
+#include "numerichelper.h"
+#include "utility/string.h"
 
-/** Handle /SAMODE
- */
-class CommandSamode : public Command
+class CommandSamode final
+	: public Command
 {
 	bool logged;
 
- public:
+public:
 	bool active;
-	CommandSamode(Module* Creator) : Command(Creator,"SAMODE", 2)
+	CommandSamode(Module* Creator)
+		: Command(Creator, "SAMODE", 2)
 	{
-		allow_empty_last_param = false;
-		flags_needed = 'o';
-		syntax = "<target> (+|-)<modes> [<mode-parameters>]";
+		access_needed = CmdAccess::OPERATOR;
+		syntax = { "<target> (+|-)<modes> [<mode-parameters>]" };
 		active = false;
 	}
 
-	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
+	CmdResult Handle(User* user, const Params& parameters) override
 	{
-		if (parameters[0].c_str()[0] != '#')
+		if (!ServerInstance->Channels.IsPrefix(parameters[0][0]))
 		{
-			User* target = ServerInstance->FindNickOnly(parameters[0]);
-			if ((!target) || (target->registered != REG_ALL))
+			auto* target = ServerInstance->Users.FindNick(parameters[0], true);
+			if (!target)
 			{
 				user->WriteNumeric(Numerics::NoSuchNick(parameters[0]));
-				return CMD_FAILURE;
+				return CmdResult::FAILURE;
 			}
 
 			// Changing the modes of another user requires a special permission
 			if ((target != user) && (!user->HasPrivPermission("users/samode-usermodes")))
 			{
 				user->WriteNotice("*** You are not allowed to /SAMODE other users (the privilege users/samode-usermodes is needed to /SAMODE others).");
-				return CMD_FAILURE;
+				return CmdResult::FAILURE;
 			}
 		}
 
 		// XXX: Make ModeParser clear LastParse
 		Modes::ChangeList emptychangelist;
-		ServerInstance->Modes->ProcessSingle(ServerInstance->FakeClient, NULL, ServerInstance->FakeClient, emptychangelist);
+		ServerInstance->Modes.ProcessSingle(ServerInstance->FakeClient, nullptr, ServerInstance->FakeClient, emptychangelist);
 
 		logged = false;
 		this->active = true;
@@ -77,41 +78,40 @@ class CommandSamode : public Command
 			//
 			// Viewing the modes of a user or a channel could also result in this, but
 			// that is not possible with /SAMODE because we require at least 2 parameters.
-			LogUsage(user, stdalgo::string::join(parameters));
+			LogUsage(user, insp::join(parameters));
 		}
 
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 	}
 
 	void LogUsage(const User* user, const std::string& text)
 	{
 		logged = true;
-		ServerInstance->SNO->WriteGlobalSno('a', user->nick + " used SAMODE: " + text);
+		ServerInstance->SNO.WriteGlobalSno('a', user->nick + " used SAMODE: " + text);
 	}
 };
 
-class ModuleSaMode : public Module
+class ModuleSaMode final
+	: public Module
 {
+private:
 	CommandSamode cmd;
- public:
+
+public:
 	ModuleSaMode()
-		: cmd(this)
+		: Module(VF_VENDOR, "Adds the /SAMODE command which allows server operators to change the modes of a target (channel, user) that they would not otherwise have the privileges to change.")
+		, cmd(this)
 	{
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds the /SAMODE command which allows server operators to change the modes of a target (channel, user) that they would not otherwise have the privileges to change.", VF_VENDOR);
-	}
-
-	ModResult OnPreMode(User* source, User* dest, Channel* channel, Modes::ChangeList& modes) CXX11_OVERRIDE
+	ModResult OnPreMode(User* source, User* dest, Channel* channel, Modes::ChangeList& modes) override
 	{
 		if (cmd.active)
 			return MOD_RES_ALLOW;
 		return MOD_RES_PASSTHRU;
 	}
 
-	void OnMode(User* user, User* destuser, Channel* destchan, const Modes::ChangeList& modes, ModeParser::ModeProcessFlag processflags) CXX11_OVERRIDE
+	void OnMode(User* user, User* destuser, Channel* destchan, const Modes::ChangeList& modes, ModeParser::ModeProcessFlag processflags) override
 	{
 		if (!cmd.active)
 			return;
@@ -120,9 +120,8 @@ class ModuleSaMode : public Module
 		logtext.push_back(' ');
 		logtext += ClientProtocol::Messages::Mode::ToModeLetters(modes);
 
-		for (Modes::ChangeList::List::const_iterator i = modes.getlist().begin(); i != modes.getlist().end(); ++i)
+		for (const auto& item : modes.getlist())
 		{
-			const Modes::Change& item = *i;
 			if (!item.param.empty())
 				logtext.append(1, ' ').append(item.param);
 		}
@@ -130,13 +129,13 @@ class ModuleSaMode : public Module
 		cmd.LogUsage(user, logtext);
 	}
 
-	void Prioritize() CXX11_OVERRIDE
+	void Prioritize() override
 	{
-		Module* disable = ServerInstance->Modules->Find("m_disable.so");
-		ServerInstance->Modules->SetPriority(this, I_OnRawMode, PRIORITY_BEFORE, disable);
+		Module* disable = ServerInstance->Modules.Find("disable");
+		ServerInstance->Modules.SetPriority(this, I_OnRawMode, PRIORITY_BEFORE, disable);
 
-		Module *override = ServerInstance->Modules->Find("m_override.so");
-		ServerInstance->Modules->SetPriority(this, I_OnPreMode, PRIORITY_BEFORE, override);
+		Module* override = ServerInstance->Modules.Find("override");
+		ServerInstance->Modules.SetPriority(this, I_OnPreMode, PRIORITY_BEFORE, override);
 	}
 };
 
