@@ -3,16 +3,16 @@
  *
  *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
  *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
+ *   Copyright (C) 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2017 B00mX0r <b00mx0r@aureus.pw>
- *   Copyright (C) 2013, 2017-2018, 2020-2022 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012-2013, 2015-2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2012-2013, 2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012 Jens Voss <DukePyrolator@anope.org>
- *   Copyright (C) 2009 John Brooks <special@inspircd.org>
+ *   Copyright (C) 2009 John Brooks <john@jbrooks.io>
  *   Copyright (C) 2009 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008-2009 Craig Edwards <brain@inspircd.org>
- *   Copyright (C) 2008 Thomas Stagner <aquanight@inspircd.org>
+ *   Copyright (C) 2008 Thomas Stagner <aquanight@gmail.com>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -30,68 +30,71 @@
 
 
 #include "inspircd.h"
-#include "xline.h"
 #include "modules/shun.h"
 #include "modules/stats.h"
-
+#include "timeutils.h"
+#include "xline.h"
 
 /** An XLineFactory specialized to generate shun pointers
  */
-class ShunFactory : public XLineFactory
+class ShunFactory final
+	: public XLineFactory
 {
- public:
-	ShunFactory() : XLineFactory("SHUN") { }
+public:
+	ShunFactory()
+		: XLineFactory("SHUN") { }
 
-	/** Generate a shun
-	*/
-	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) override
 	{
 		return new Shun(set_time, duration, source, reason, xline_specific_mask);
 	}
 
-	bool AutoApplyToUserList(XLine* x) CXX11_OVERRIDE
+	bool AutoApplyToUserList(XLine* x) override
 	{
 		return false;
 	}
 };
 
-class CommandShun : public Command
+class CommandShun final
+	: public Command
 {
- public:
-	CommandShun(Module* Creator) : Command(Creator, "SHUN", 1, 3)
+public:
+	CommandShun(Module* Creator)
+		: Command(Creator, "SHUN", 1, 3)
 	{
-		flags_needed = 'o';
-		syntax = "<nick!user@host>[,<nick!user@host>]+ [<duration> :<reason>]";
+		access_needed = CmdAccess::OPERATOR;
+		syntax = { "<nick!user@host>[,<nick!user@host>]+ [<duration> :<reason>]" };
 	}
 
-	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
+	CmdResult Handle(User* user, const Params& parameters) override
 	{
 		/* syntax: SHUN nick!user@host time :reason goes here */
 		/* 'time' is a human-readable timestring, like 2d3h2s. */
 		if (CommandParser::LoopCall(user, this, parameters, 0))
-			return CMD_SUCCESS;
+			return CmdResult::SUCCESS;
 
 		std::string target = parameters[0];
-		User *find = ServerInstance->FindNick(target);
-		if ((find) && (find->registered == REG_ALL))
-			target = "*!" + find->GetBanIdent() + "@" + find->GetIPString();
+
+		auto* find = ServerInstance->Users.Find(target, true);
+		if (find)
+			target = "*!" + find->GetBanUser(true) + "@" + find->GetAddress();
 
 		if (parameters.size() == 1)
 		{
 			std::string reason;
 
-			if (ServerInstance->XLines->DelLine(parameters[0].c_str(), "SHUN", reason, user))
+			if (ServerInstance->XLines->DelLine(parameters[0], "SHUN", reason, user))
 			{
-				ServerInstance->SNO->WriteToSnoMask('x', "%s removed SHUN on %s: %s", user->nick.c_str(), parameters[0].c_str(), reason.c_str());
+				ServerInstance->SNO.WriteToSnoMask('x', "{} removed SHUN on {}: {}", user->nick, parameters[0], reason);
 			}
-			else if (ServerInstance->XLines->DelLine(target.c_str(), "SHUN", reason, user))
+			else if (ServerInstance->XLines->DelLine(target, "SHUN", reason, user))
 			{
-				ServerInstance->SNO->WriteToSnoMask('x', "%s removed SHUN on %s: %s", user->nick.c_str(), target.c_str(), reason.c_str());
+				ServerInstance->SNO.WriteToSnoMask('x', "{} removed SHUN on {}: {}", user->nick, target, reason);
 			}
 			else
 			{
 				user->WriteNotice("*** Shun " + parameters[0] + " not found on the list.");
-				return CMD_FAILURE;
+				return CmdResult::FAILURE;
 			}
 		}
 		else
@@ -101,10 +104,10 @@ class CommandShun : public Command
 			std::string expr;
 			if (parameters.size() > 2)
 			{
-				if (!InspIRCd::Duration(parameters[1], duration))
+				if (!Duration::TryFrom(parameters[1], duration))
 				{
 					user->WriteNotice("*** Invalid duration for SHUN.");
-					return CMD_FAILURE;
+					return CmdResult::FAILURE;
 				}
 				expr = parameters[2];
 			}
@@ -114,60 +117,49 @@ class CommandShun : public Command
 				expr = parameters[1];
 			}
 
-			Shun* r = new Shun(ServerInstance->Time(), duration, user->nick, expr, target);
+			auto* r = new Shun(ServerInstance->Time(), duration, user->nick, expr, target);
 			if (ServerInstance->XLines->AddLine(r, user))
 			{
 				if (!duration)
 				{
-					ServerInstance->SNO->WriteToSnoMask('x', "%s added a permanent SHUN on %s: %s",
-						user->nick.c_str(), target.c_str(), expr.c_str());
+					ServerInstance->SNO.WriteToSnoMask('x', "{} added permanent SHUN for {}: {}",
+						user->nick, target, expr);
 				}
 				else
 				{
-					ServerInstance->SNO->WriteToSnoMask('x', "%s added a timed SHUN on %s, expires in %s (on %s): %s",
-						user->nick.c_str(), target.c_str(), InspIRCd::DurationString(duration).c_str(),
-						InspIRCd::TimeString(ServerInstance->Time() + duration).c_str(), expr.c_str());
+					ServerInstance->SNO.WriteToSnoMask('x', "{} added a timed SHUN on {}, expires in {} (on {}): {}",
+						user->nick, target, Duration::ToString(duration),
+						Time::FromNow(duration), expr);
 				}
 			}
 			else
 			{
 				delete r;
 				user->WriteNotice("*** Shun for " + target + " already exists.");
-				return CMD_FAILURE;
+				return CmdResult::FAILURE;
 			}
 		}
-		return CMD_SUCCESS;
-	}
-
-	RouteDescriptor GetRouting(User* user, const Params& parameters) CXX11_OVERRIDE
-	{
-		if (IS_LOCAL(user))
-			return ROUTE_LOCALONLY; // spanningtree will send ADDLINE
-
-		return ROUTE_BROADCAST;
+		return CmdResult::SUCCESS;
 	}
 };
 
-class ModuleShun : public Module, public Stats::EventListener
+class ModuleShun final
+	: public Module
+	, public Stats::EventListener
 {
- private:
+private:
 	CommandShun cmd;
 	ShunFactory shun;
-	insp::flat_set<std::string, irc::insensitive_swo> cleanedcommands;
-	insp::flat_set<std::string, irc::insensitive_swo> enabledcommands;
-	bool affectopers;
 	bool allowconnect;
 	bool allowtags;
+	TokenList cleanedcommands;
+	TokenList enabledcommands;
 	bool notifyuser;
 
-	bool IsShunned(LocalUser* user)
+	bool IsShunned(LocalUser* user) const
 	{
 		// Exempt the user if they are not fully connected and allowconnect is enabled.
-		if (allowconnect && user->registered != REG_ALL)
-			return false;
-
-		// Exempt the user from shuns if they are an oper and affectopers is disabled.
-		if (!affectopers && user->IsOper())
+		if (allowconnect && !user->IsFullyConnected())
 			return false;
 
 		// Exempt the user from shuns if they are an oper with the servers/ignore-shun privilege.
@@ -178,31 +170,32 @@ class ModuleShun : public Module, public Stats::EventListener
 		return ServerInstance->XLines->MatchesLine("SHUN", user);
 	}
 
- public:
+public:
 	ModuleShun()
-		: Stats::EventListener(this)
+		: Module(VF_VENDOR | VF_COMMON, "Adds the /SHUN command which allows server operators to prevent users from executing commands.")
+		, Stats::EventListener(this)
 		, cmd(this)
 	{
 	}
 
-	void init() CXX11_OVERRIDE
+	void init() override
 	{
 		ServerInstance->XLines->RegisterFactory(&shun);
 	}
 
-	~ModuleShun()
+	~ModuleShun() override
 	{
 		ServerInstance->XLines->DelAll("SHUN");
 		ServerInstance->XLines->UnregisterFactory(&shun);
 	}
 
-	void Prioritize() CXX11_OVERRIDE
+	void Prioritize() override
 	{
-		Module* alias = ServerInstance->Modules->Find("m_alias.so");
-		ServerInstance->Modules->SetPriority(this, I_OnPreCommand, PRIORITY_BEFORE, alias);
+		Module* alias = ServerInstance->Modules.Find("alias");
+		ServerInstance->Modules.SetPriority(this, I_OnPreCommand, PRIORITY_BEFORE, alias);
 	}
 
-	ModResult OnStats(Stats::Context& stats) CXX11_OVERRIDE
+	ModResult OnStats(Stats::Context& stats) override
 	{
 		if (stats.GetSymbol() != 'H')
 			return MOD_RES_PASSTHRU;
@@ -211,32 +204,25 @@ class ModuleShun : public Module, public Stats::EventListener
 		return MOD_RES_DENY;
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("shun");
+		cleanedcommands.Clear();
+		enabledcommands.Clear();
 
-		cleanedcommands.clear();
-		irc::spacesepstream cleanedcmds(tag->getString("cleanedcommands", "AWAY PART QUIT"));
-		for (std::string cleanedcmd; cleanedcmds.GetToken(cleanedcmd); )
-			cleanedcommands.insert(cleanedcmd);
-
-		enabledcommands.clear();
-		irc::spacesepstream enabledcmds(tag->getString("enabledcommands", "ADMIN OPER PING PONG QUIT", 1));
-		for (std::string enabledcmd; enabledcmds.GetToken(enabledcmd); )
-			enabledcommands.insert(enabledcmd);
-
-		affectopers = tag->getBool("affectopers", false);
-		allowtags = tag->getBool("allowtags");
+		const auto& tag = ServerInstance->Config->ConfValue("shun");
 		allowconnect = tag->getBool("allowconnect");
+		allowtags = tag->getBool("allowtags");
+		cleanedcommands.AddList(tag->getString("cleanedcommands", "AWAY PART QUIT"));
+		enabledcommands.AddList(tag->getString("enabledcommands", "ADMIN OPER PING PONG QUIT"));
 		notifyuser = tag->getBool("notifyuser", true);
 	}
 
-	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) CXX11_OVERRIDE
+	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) override
 	{
 		if (validated || !IsShunned(user))
 			return MOD_RES_PASSTHRU;
 
-		if (!enabledcommands.count(command))
+		if (!enabledcommands.Contains(command))
 		{
 			if (notifyuser)
 				user->WriteNotice("*** " + command + " command not processed as you have been blocked from issuing commands.");
@@ -256,7 +242,7 @@ class ModuleShun : public Module, public Stats::EventListener
 			}
 		}
 
-		if (!cleanedcommands.count(command))
+		if (!cleanedcommands.Contains(command))
 			return MOD_RES_PASSTHRU;
 
 		switch (parameters.size())
@@ -277,11 +263,6 @@ class ModuleShun : public Module, public Stats::EventListener
 
 		return MOD_RES_PASSTHRU;
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds the /SHUN command which allows server operators to prevent users from executing commands.", VF_VENDOR|VF_COMMON);
-	}
 };
 
 MODULE_INIT(ModuleShun)
