@@ -1,16 +1,15 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2021 Herman <GermanAizek@yandex.ru>
  *   Copyright (C) 2020 Matt Schatz <genius3000@g3k.solutions>
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2013, 2017-2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013, 2016 Adam <Adam@anope.org>
  *   Copyright (C) 2012-2016, 2018 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007-2009 Robin Burchell <robin+git@viroteck.net>
- *   Copyright (C) 2007-2009 Dennis Friis <peavey@inspircd.org>
+ *   Copyright (C) 2007-2008 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2005, 2007-2009 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -28,48 +27,69 @@
 
 
 #include "inspircd.h"
+#include "clientprotocolmsg.h"
+#include "iohook.h"
 #include "socket.h"
 #include "xline.h"
-#include "iohook.h"
 
-#include "resolvers.h"
-#include "main.h"
-#include "utils.h"
-#include "treeserver.h"
-#include "link.h"
-#include "treesocket.h"
 #include "commands.h"
+#include "link.h"
+#include "main.h"
+#include "resolvers.h"
 #include "translate.h"
+#include "treeserver.h"
+#include "treesocket.h"
+#include "utils.h"
 
 ModuleSpanningTree::ModuleSpanningTree()
-	: Away::EventListener(this)
+	: Module(VF_VENDOR, "Allows linking multiple servers together as part of one network.")
+	, Away::EventListener(this)
 	, Stats::EventListener(this)
 	, CTCTags::EventListener(this)
 	, rconnect(this)
 	, rsquit(this)
 	, map(this)
 	, commands(this)
-	, currmembid(0)
-	, broadcasteventprov(this, "event/server-broadcast")
+	, routeeventprov(this, "event/server-route")
 	, linkeventprov(this, "event/server-link")
 	, messageeventprov(this, "event/server-message")
 	, synceventprov(this, "event/server-sync")
 	, sslapi(this)
-	, servicetag(this)
-	, DNS(this, "DNS")
+	, servertags(this)
+	, DNS(this)
 	, tagevprov(this)
-	, loopCall(false)
 {
 }
 
 SpanningTreeCommands::SpanningTreeCommands(ModuleSpanningTree* module)
-	: svsjoin(module), svspart(module), svsnick(module), metadata(module),
-	uid(module), opertype(module), fjoin(module), ijoin(module), resync(module),
-	fmode(module), ftopic(module), fhost(module), fident(module), fname(module),
-	away(module), addline(module), delline(module), encap(module), idle(module),
-	nick(module), ping(module), pong(module), save(module),
-	server(module), squit(module), snonotice(module),
-	endburst(module), sinfo(module), num(module)
+	: metadata(module)
+	, uid(module)
+	, opertype(module)
+	, fjoin(module)
+	, ijoin(module)
+	, resync(module)
+	, fmode(module)
+	, ftopic(module)
+	, fhost(module)
+	, frhost(module)
+	, fident(module)
+	, fname(module)
+	, away(module)
+	, addline(module)
+	, delline(module)
+	, encap(module)
+	, idle(module)
+	, nick(module)
+	, ping(module)
+	, pong(module)
+	, save(module)
+	, server(module)
+	, squit(module)
+	, snonotice(module)
+	, endburst(module)
+	, sinfo(module)
+	, num(module)
+	, lmode(module)
 {
 }
 
@@ -80,27 +100,24 @@ namespace
 		// Does not change the server of quitting users because those are not in the list
 
 		ServerInstance->FakeClient->server = newserver;
-		const UserManager::LocalList& list = ServerInstance->Users.GetLocalUsers();
-		for (UserManager::LocalList::const_iterator i = list.begin(); i != list.end(); ++i)
-			(*i)->server = newserver;
+		for (auto* user : ServerInstance->Users.GetLocalUsers())
+			user->server = newserver;
 	}
 
 	void ResetMembershipIds()
 	{
 		// Set all membership ids to 0
-		const UserManager::LocalList& list = ServerInstance->Users.GetLocalUsers();
-		for (UserManager::LocalList::iterator i = list.begin(); i != list.end(); ++i)
+		for (const auto* user : ServerInstance->Users.GetLocalUsers())
 		{
-			LocalUser* user = *i;
-			for (User::ChanList::iterator j = user->chans.begin(); j != user->chans.end(); ++j)
-				(*j)->id = 0;
+			for (auto* memb : user->chans)
+				memb->id = 0;
 		}
 	}
 }
 
 void ModuleSpanningTree::init()
 {
-	ServerInstance->SNO->EnableSnomask('l', "LINK");
+	ServerInstance->SNO.EnableSnomask('l', "LINK");
 
 	ResetMembershipIds();
 
@@ -121,11 +138,9 @@ void ModuleSpanningTree::ShowLinks(TreeServer* Current, User* user, int hops)
 		Parent = Current->GetParent()->GetName();
 	}
 
-	const TreeServer::ChildServers& children = Current->GetChildren();
-	for (TreeServer::ChildServers::const_iterator i = children.begin(); i != children.end(); ++i)
+	for (auto* server : Current->GetChildren())
 	{
-		TreeServer* server = *i;
-		if ((server->Hidden) || ((Utils->HideULines) && (server->IsULine())))
+		if ((server->Hidden) || ((Utils->HideServices) && (server->IsService())))
 		{
 			if (user->IsOper())
 			{
@@ -137,8 +152,8 @@ void ModuleSpanningTree::ShowLinks(TreeServer* Current, User* user, int hops)
 			ShowLinks(server, user, hops+1);
 		}
 	}
-	/* Don't display the line if its a uline, hide ulines is on, and the user isn't an oper */
-	if ((Utils->HideULines) && (Current->IsULine()) && (!user->IsOper()))
+	/* Don't display the line if its a service, hide services is on, and the user isn't an oper */
+	if ((Utils->HideServices) && (Current->IsService()) && (!user->IsOper()))
 		return;
 	/* Or if the server is hidden and they're not an oper */
 	else if ((Current->Hidden) && (!user->IsOper()))
@@ -146,16 +161,16 @@ void ModuleSpanningTree::ShowLinks(TreeServer* Current, User* user, int hops)
 
 	user->WriteNumeric(RPL_LINKS, Current->GetName(),
 			(((Utils->FlatLinks) && (!user->IsOper())) ? ServerInstance->Config->GetServerName() : Parent),
-			InspIRCd::Format("%d %s", (((Utils->FlatLinks) && (!user->IsOper())) ? 0 : hops), Current->GetDesc().c_str()));
+			INSP_FORMAT("{} {}", (((Utils->FlatLinks) && (!user->IsOper())) ? 0 : hops), Current->GetDesc()));
 }
 
 void ModuleSpanningTree::HandleLinks(const CommandBase::Params& parameters, User* user)
 {
-	ShowLinks(Utils->TreeRoot,user,0);
+	ShowLinks(Utils->TreeRoot, user, 0);
 	user->WriteNumeric(RPL_ENDOFLINKS, '*', "End of /LINKS list.");
 }
 
-void ModuleSpanningTree::ConnectServer(Autoconnect* a, bool on_timer)
+void ModuleSpanningTree::ConnectServer(const std::shared_ptr<Autoconnect>& a, bool on_timer)
 {
 	if (!a)
 		return;
@@ -178,10 +193,10 @@ void ModuleSpanningTree::ConnectServer(Autoconnect* a, bool on_timer)
 	a->position++;
 	while (a->position < (int)a->servers.size())
 	{
-		Link* x = Utils->FindLink(a->servers[a->position]);
+		std::shared_ptr<Link> x = Utils->FindLink(a->servers[a->position]);
 		if (x)
 		{
-			ServerInstance->SNO->WriteToSnoMask('l', "AUTOCONNECT: Auto-connecting server \002%s\002", x->Name.c_str());
+			ServerInstance->SNO.WriteToSnoMask('l', "AUTOCONNECT: Auto-connecting server \002{}\002", x->Name);
 			ConnectServer(x, a);
 			return;
 		}
@@ -192,68 +207,68 @@ void ModuleSpanningTree::ConnectServer(Autoconnect* a, bool on_timer)
 	a->position = -1;
 }
 
-void ModuleSpanningTree::ConnectServer(Link* x, Autoconnect* y)
+void ModuleSpanningTree::ConnectServer(const std::shared_ptr<Link>& x, const std::shared_ptr<Autoconnect>& y)
 {
 	if (InspIRCd::Match(ServerInstance->Config->ServerName, x->Name, ascii_case_insensitive_map))
 	{
-		ServerInstance->SNO->WriteToSnoMask('l', "CONNECT: Not connecting to myself.");
+		ServerInstance->SNO.WriteToSnoMask('l', "CONNECT: Not connecting to myself.");
 		return;
 	}
 
 	irc::sockets::sockaddrs sa;
 	if (x->IPAddr.find('/') != std::string::npos)
 	{
-		if (!irc::sockets::isunix(x->IPAddr) || !irc::sockets::untosa(x->IPAddr, sa))
+		if (!irc::sockets::isunix(x->IPAddr) || !sa.from_unix(x->IPAddr))
 		{
 			// We don't use the family() != AF_UNSPEC check below for UNIX sockets as
 			// that results in a DNS lookup.
-			ServerInstance->SNO->WriteToSnoMask('l', "CONNECT: Error connecting \002%s\002: %s is not a UNIX socket!",
-				x->Name.c_str(), x->IPAddr.c_str());
+			ServerInstance->SNO.WriteToSnoMask('l', "CONNECT: Error connecting \002{}\002: {} is not a UNIX socket!",
+				x->Name, x->IPAddr);
 			return;
 		}
 	}
 	else
 	{
 		// If this fails then the IP sa will be AF_UNSPEC.
-		irc::sockets::aptosa(x->IPAddr, x->Port, sa);
+		sa.from_ip_port(x->IPAddr, x->Port);
 	}
 
 	/* Do we already have an IP? If so, no need to resolve it. */
 	if (sa.family() != AF_UNSPEC)
 	{
 		// Create a TreeServer object that will start connecting immediately in the background
-		TreeSocket* newsocket = new TreeSocket(x, y, sa);
+		auto* newsocket = new TreeSocket(x, y, sa);
 		if (!newsocket->HasFd())
 		{
-			ServerInstance->SNO->WriteToSnoMask('l', "CONNECT: Error connecting \002%s\002: %s.",
-				x->Name.c_str(), newsocket->getError().c_str());
+			ServerInstance->SNO.WriteToSnoMask('l', "CONNECT: Error connecting \002{}\002: {}.",
+				x->Name, newsocket->GetError());
 			ServerInstance->GlobalCulls.AddItem(newsocket);
 		}
 	}
 	else if (!DNS)
 	{
-		ServerInstance->SNO->WriteToSnoMask('l', "CONNECT: Error connecting \002%s\002: Hostname given and core_dns is not loaded, unable to resolve.", x->Name.c_str());
+		ServerInstance->SNO.WriteToSnoMask('l', "CONNECT: Error connecting \002{}\002: Hostname given and core_dns is not loaded, unable to resolve.", x->Name);
 	}
 	else
 	{
 		// Guess start_type from bindip aftype
 		DNS::QueryType start_type = DNS::QUERY_AAAA;
 		irc::sockets::sockaddrs bind;
-		if ((!x->Bind.empty()) && (irc::sockets::aptosa(x->Bind, 0, bind)))
+		if (!x->Bind.empty() && bind.from_ip(x->Bind))
 		{
 			if (bind.family() == AF_INET)
 				start_type = DNS::QUERY_A;
 		}
 
-		ServernameResolver* snr = new ServernameResolver(*DNS, x->IPAddr, x, start_type, y);
+		auto* snr = new ServerNameResolver(*DNS, x->IPAddr, x, start_type, y);
 		try
 		{
 			DNS->Process(snr);
 		}
-		catch (DNS::Exception& e)
+		catch (const DNS::Exception& e)
 		{
 			delete snr;
-			ServerInstance->SNO->WriteToSnoMask('l', "CONNECT: Error connecting \002%s\002: %s.",x->Name.c_str(), e.GetReason().c_str());
+			ServerInstance->SNO.WriteToSnoMask('l', "CONNECT: Error connecting \002{}\002: {}.", x->Name, e.GetReason());
 			ConnectServer(y, false);
 		}
 	}
@@ -261,9 +276,8 @@ void ModuleSpanningTree::ConnectServer(Link* x, Autoconnect* y)
 
 void ModuleSpanningTree::AutoConnectServers(time_t curtime)
 {
-	for (std::vector<reference<Autoconnect> >::iterator i = Utils->AutoconnectBlocks.begin(); i < Utils->AutoconnectBlocks.end(); ++i)
+	for (const auto& x : Utils->AutoconnectBlocks)
 	{
-		Autoconnect* x = *i;
 		if (curtime >= x->NextConnectTime)
 		{
 			x->NextConnectTime = curtime + x->Period;
@@ -288,7 +302,7 @@ void ModuleSpanningTree::DoConnectTimeout(time_t curtime)
 		}
 		else if (curtime > s->age + (time_t)p.second)
 		{
-			ServerInstance->SNO->WriteToSnoMask('l',"CONNECT: Error connecting \002%s\002 (timeout of %u seconds)",p.first.c_str(),p.second);
+			ServerInstance->SNO.WriteToSnoMask('l', "CONNECT: Error connecting \002{}\002 (timeout of {} seconds)", p.first, p.second);
 			Utils->timeoutlist.erase(me);
 			s->Close();
 		}
@@ -307,16 +321,19 @@ ModResult ModuleSpanningTree::HandleVersion(const CommandBase::Params& parameter
 			return MOD_RES_PASSTHRU;
 		}
 
-		// If an oper wants to see the version then show the full version string instead of the normal,
-		// but only if it is non-empty.
-		// If it's empty it might be that the server is still syncing (full version hasn't arrived yet)
-		// or the server is a 2.0 server and does not send a full version.
-		bool showfull = ((user->IsOper()) && (!found->GetFullVersion().empty()));
-
 		Numeric::Numeric numeric(RPL_VERSION);
-		irc::tokenstream tokens(showfull ? found->GetFullVersion() : found->GetVersion());
-		for (std::string token; tokens.GetTrailing(token); )
-			numeric.push(token);
+		if (user->IsOper())
+		{
+			numeric.push(found->rawversion + ".");
+			numeric.push(found->GetName());
+			numeric.push("[" + found->GetId() + "] " + found->customversion);
+		}
+		else
+		{
+			numeric.push(found->rawbranch + ".");
+			numeric.push(found->GetPublicName());
+			numeric.push(found->customversion);
+		}
 		user->WriteNumeric(numeric);
 	}
 	else
@@ -328,36 +345,35 @@ ModResult ModuleSpanningTree::HandleVersion(const CommandBase::Params& parameter
 
 ModResult ModuleSpanningTree::HandleConnect(const CommandBase::Params& parameters, User* user)
 {
-	for (std::vector<reference<Link> >::iterator i = Utils->LinkBlocks.begin(); i < Utils->LinkBlocks.end(); i++)
+	for (const auto& x : Utils->LinkBlocks)
 	{
-		Link* x = *i;
 		if (InspIRCd::Match(x->Name, parameters[0], ascii_case_insensitive_map))
 		{
 			if (InspIRCd::Match(ServerInstance->Config->ServerName, x->Name, ascii_case_insensitive_map))
 			{
-				user->WriteRemoteNotice(InspIRCd::Format("*** CONNECT: Server \002%s\002 is ME, not connecting.", x->Name.c_str()));
+				user->WriteRemoteNotice("*** CONNECT: Server \002{}\002 is ME, not connecting.", x->Name);
 				return MOD_RES_DENY;
 			}
 
 			TreeServer* CheckDupe = Utils->FindServer(x->Name);
 			if (!CheckDupe)
 			{
-				user->WriteRemoteNotice(InspIRCd::Format("*** CONNECT: Connecting to server: \002%s\002 (%s:%d)", x->Name.c_str(), (x->HiddenFromStats ? "<hidden>" : x->IPAddr.c_str()), x->Port));
+				user->WriteRemoteNotice("*** CONNECT: Connecting to server: \002{}\002 ({}:{})", x->Name, (x->HiddenFromStats ? "<hidden>" : x->IPAddr), x->Port);
 				ConnectServer(x);
 				return MOD_RES_DENY;
 			}
 			else
 			{
-				user->WriteRemoteNotice(InspIRCd::Format("*** CONNECT: Server \002%s\002 already exists on the network and is connected via \002%s\002", x->Name.c_str(), CheckDupe->GetParent()->GetName().c_str()));
+				user->WriteRemoteNotice("*** CONNECT: Server \002{}\002 already exists on the network and is connected via \002{}\002", x->Name, CheckDupe->GetParent()->GetName());
 				return MOD_RES_DENY;
 			}
 		}
 	}
-	user->WriteRemoteNotice(InspIRCd::Format("*** CONNECT: No server matching \002%s\002 could be found in the config file.", parameters[0].c_str()));
+	user->WriteRemoteNotice("*** CONNECT: No server matching \002{}\002 could be found in the config file.", parameters[0]);
 	return MOD_RES_DENY;
 }
 
-void ModuleSpanningTree::OnUserInvite(User* source, User* dest, Channel* channel, time_t expiry, unsigned int notifyrank, CUList& notifyexcepts)
+void ModuleSpanningTree::OnUserInvite(User* source, User* dest, Channel* channel, time_t expiry, ModeHandler::Rank notifyrank, CUList& notifyexcepts)
 {
 	if (IS_LOCAL(source))
 	{
@@ -382,7 +398,7 @@ ModResult ModuleSpanningTree::OnPreTopicChange(User* user, Channel* chan, const
 	return MOD_RES_PASSTHRU;
 }
 
-void ModuleSpanningTree::OnPostTopicChange(User* user, Channel* chan, const std::string &topic)
+void ModuleSpanningTree::OnPostTopicChange(User* user, Channel* chan, const std::string& topic)
 {
 	// Drop remote events on the floor.
 	if (!IS_LOCAL(user))
@@ -396,12 +412,12 @@ void ModuleSpanningTree::OnUserPostMessage(User* user, const MessageTarget& targ
 	if (!IS_LOCAL(user))
 		return;
 
-	const char* message_type = (details.type == MSG_PRIVMSG ? "PRIVMSG" : "NOTICE");
+	const char* message_type = (details.type == MessageType::PRIVMSG ? "PRIVMSG" : "NOTICE");
 	switch (target.type)
 	{
 		case MessageTarget::TYPE_USER:
 		{
-			User* d = target.Get<User>();
+			auto* d = target.Get<User>();
 			if (!IS_LOCAL(d))
 			{
 				CmdBuilder params(user, message_type);
@@ -419,7 +435,7 @@ void ModuleSpanningTree::OnUserPostMessage(User* user, const MessageTarget& targ
 		}
 		case MessageTarget::TYPE_SERVER:
 		{
-			const std::string* serverglob = target.Get<std::string>();
+			const auto* serverglob = target.Get<std::string>();
 			CmdBuilder par(user, message_type);
 			par.push_tags(details.tags_out);
 			par.push(std::string("$") + *serverglob);
@@ -439,7 +455,7 @@ void ModuleSpanningTree::OnUserPostTagMessage(User* user, const MessageTarget& t
 	{
 		case MessageTarget::TYPE_USER:
 		{
-			User* d = target.Get<User>();
+			auto* d = target.Get<User>();
 			if (!IS_LOCAL(d))
 			{
 				CmdBuilder params(user, "TAGMSG");
@@ -456,7 +472,7 @@ void ModuleSpanningTree::OnUserPostTagMessage(User* user, const MessageTarget& t
 		}
 		case MessageTarget::TYPE_SERVER:
 		{
-			const std::string* serverglob = target.Get<std::string>();
+			const auto* serverglob = target.Get<std::string>();
 			CmdBuilder par(user, "TAGMSG");
 			par.push_tags(details.tags_out);
 			par.push(std::string("$") + *serverglob);
@@ -484,20 +500,22 @@ void ModuleSpanningTree::OnUserConnect(LocalUser* user)
 	CommandUID::Builder(user).Broadcast();
 
 	if (user->IsOper())
-		CommandOpertype::Builder(user).Broadcast();
+		CommandOpertype::Builder(user, user->oper).Broadcast();
 
 	if (user->IsAway())
 		CommandAway::Builder(user).Broadcast();
 
-	if (user->uniqueusername) // TODO: convert this to BooleanExtItem in v4.
+	if (user->uniqueusername) // TODO: convert this to BooleanExtItem.
 		CommandMetadata::Builder(user, "uniqueusername", "1").Broadcast();
 
-	for(Extensible::ExtensibleStore::const_iterator i = user->GetExtList().begin(); i != user->GetExtList().end(); i++)
+	for (const auto& [item, obj] : user->GetExtList())
 	{
-		ExtensionItem* item = i->first;
-		std::string value = item->ToNetwork(user, i->second);
+		const std::string value = item->ToNetwork(user, obj);
 		if (!value.empty())
-			ServerInstance->PI->SendMetaData(user, item->name, value);
+		{
+			ServerInstance->PI->SendMetadata(user, item->name, value);
+			item->OnSync(user, obj, nullptr);
+		}
 	}
 
 	Utils->TreeRoot->UserCount++;
@@ -518,7 +536,7 @@ void ModuleSpanningTree::OnUserJoin(Membership* memb, bool sync, bool created_by
 		params.add(memb);
 		params.finalize();
 		params.Broadcast();
-		Utils->SendListLimits(memb->chan, NULL);
+		SpanningTreeUtilities::SendListLimits(memb->chan, nullptr);
 	}
 	else
 	{
@@ -528,37 +546,53 @@ void ModuleSpanningTree::OnUserJoin(Membership* memb, bool sync, bool created_by
 		if (!memb->modes.empty())
 		{
 			params.push(ConvToStr(memb->chan->age));
-			params.push(memb->modes);
+			params.push(memb->GetAllPrefixModes());
 		}
 		params.Broadcast();
 	}
 }
 
-void ModuleSpanningTree::OnChangeHost(User* user, const std::string &newhost)
+void ModuleSpanningTree::OnChangeHost(User* user, const std::string& newhost)
 {
-	if (user->registered != REG_ALL || !IS_LOCAL(user))
+	if (!user->IsFullyConnected() || !IS_LOCAL(user))
 		return;
 
-	CmdBuilder(user, "FHOST").push(newhost).Broadcast();
+	CmdBuilder(user, "FHOST").push(newhost).push('*').Broadcast();
+}
+
+void ModuleSpanningTree::OnChangeRealHost(User* user, const std::string& newhost)
+{
+	if (!user->IsFullyConnected() || !IS_LOCAL(user))
+		return;
+
+	CmdBuilder(user, "FHOST").push('*').push(newhost).Broadcast();
 }
 
 void ModuleSpanningTree::OnChangeRealName(User* user, const std::string& real)
 {
-	if (user->registered != REG_ALL || !IS_LOCAL(user))
+	if (!user->IsFullyConnected() || !IS_LOCAL(user))
 		return;
 
 	CmdBuilder(user, "FNAME").push_last(real).Broadcast();
 }
 
-void ModuleSpanningTree::OnChangeIdent(User* user, const std::string &ident)
+void ModuleSpanningTree::OnChangeUser(User* user, const std::string& newuser)
 {
-	if ((user->registered != REG_ALL) || (!IS_LOCAL(user)))
+	if (!user->IsFullyConnected() || !IS_LOCAL(user))
 		return;
 
-	CmdBuilder(user, "FIDENT").push(ident).Broadcast();
+	CmdBuilder(user, "FIDENT").push(newuser).push("*").Broadcast();
 }
 
-void ModuleSpanningTree::OnUserPart(Membership* memb, std::string &partmessage, CUList& excepts)
+void ModuleSpanningTree::OnChangeRealUser(User* user, const std::string& newuser)
+{
+	if (!user->IsFullyConnected() || !IS_LOCAL(user))
+		return;
+
+	CmdBuilder(user, "FIDENT").push("*").push(newuser).Broadcast();
+}
+
+void ModuleSpanningTree::OnUserPart(Membership* memb, std::string& partmessage, CUList& excepts)
 {
 	if (IS_LOCAL(memb->user))
 	{
@@ -570,12 +604,12 @@ void ModuleSpanningTree::OnUserPart(Membership* memb, std::string &partmessage,
 	}
 }
 
-void ModuleSpanningTree::OnUserQuit(User* user, const std::string &reason, const std::string &oper_message)
+void ModuleSpanningTree::OnUserQuit(User* user, const std::string& reason, const std::string& oper_message)
 {
 	if (IS_LOCAL(user))
 	{
 		if (oper_message != reason)
-			ServerInstance->PI->SendMetaData(user, "operquit", oper_message);
+			ServerInstance->PI->SendMetadata(user, "operquit", oper_message);
 
 		CmdBuilder(user, "QUIT").push_last(reason).Broadcast();
 	}
@@ -583,13 +617,13 @@ void ModuleSpanningTree::OnUserQuit(User* user, const std::string &reason, const
 	{
 		// Hide the message if one of the following is true:
 		// - User is being quit due to a netsplit and quietbursts is on
-		// - Server is a silent uline
+		// - User is on a silent services server
 		TreeServer* server = TreeServer::Get(user);
-		bool hide = (((server->IsDead()) && (Utils->quiet_bursts)) || (server->IsSilentULine()));
+		bool hide = (((server->IsDead()) && (Utils->quiet_bursts)) || (server->IsSilentService()));
 		if (!hide)
 		{
-			ServerInstance->SNO->WriteToSnoMask('Q', "Client exiting on server %s: %s (%s) [%s]",
-				user->server->GetName().c_str(), user->GetFullRealHost().c_str(), user->GetIPString().c_str(), oper_message.c_str());
+			ServerInstance->SNO.WriteToSnoMask('Q', "Client exiting on server {}: {} ({}) [{}]", user->server->GetName(),
+				user->GetRealMask(), user->GetAddress(), oper_message);
 		}
 	}
 
@@ -597,23 +631,23 @@ void ModuleSpanningTree::OnUserQuit(User* user, const std::string &reason, const
 	TreeServer::Get(user)->UserCount--;
 }
 
-void ModuleSpanningTree::OnUserPostNick(User* user, const std::string &oldnick)
+void ModuleSpanningTree::OnUserPostNick(User* user, const std::string& oldnick)
 {
 	if (IS_LOCAL(user))
 	{
 		// The nick TS is updated by the core, we don't do it
 		CmdBuilder params(user, "NICK");
 		params.push(user->nick);
-		params.push(ConvToStr(user->age));
+		params.push(ConvToStr(user->nickchanged));
 		params.Broadcast();
 	}
 	else if (!loopCall)
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "WARNING: Changed nick of remote user %s from %s to %s TS %lu by ourselves!", user->uuid.c_str(), oldnick.c_str(), user->nick.c_str(), (unsigned long) user->age);
+		ServerInstance->Logs.Normal(MODNAME, "WARNING: Changed nick of remote user {} from {} to {} TS {} by ourselves!", user->uuid, oldnick, user->nick, user->nickchanged);
 	}
 }
 
-void ModuleSpanningTree::OnUserKick(User* source, Membership* memb, const std::string &reason, CUList& excepts)
+void ModuleSpanningTree::OnUserKick(User* source, Membership* memb, const std::string& reason, CUList& excepts)
 {
 	if ((!IS_LOCAL(source)) && (source != ServerInstance->FakeClient))
 		return;
@@ -628,16 +662,16 @@ void ModuleSpanningTree::OnUserKick(User* source, Membership* memb, const std::s
 	params.Broadcast();
 }
 
-void ModuleSpanningTree::OnPreRehash(User* user, const std::string &parameter)
+void ModuleSpanningTree::OnPreRehash(User* user, const std::string& parameter)
 {
-	ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "OnPreRehash called with param %s", parameter.c_str());
+	ServerInstance->Logs.Debug(MODNAME, "OnPreRehash called with param {}", parameter);
 
 	// Send out to other servers
 	if (!parameter.empty() && parameter[0] != '-')
 	{
 		CmdBuilder params(user ? user : ServerInstance->FakeClient, "REHASH");
 		params.push(parameter);
-		params.Forward(user ? TreeServer::Get(user)->GetRoute() : NULL);
+		params.Forward(user ? TreeServer::Get(user)->GetRoute() : nullptr);
 	}
 }
 
@@ -653,53 +687,83 @@ void ModuleSpanningTree::ReadConfig(ConfigStatus& status)
 		CommandSInfo::Builder(Utils->TreeRoot, "desc", newdesc).Broadcast();
 	}
 
+	const auto& newcustomversion = ServerInstance->Config->CustomVersion;
+	if (newcustomversion != Utils->TreeRoot->customversion)
+	{
+		Utils->TreeRoot->customversion = newcustomversion;
+		CommandSInfo::Builder(Utils->TreeRoot, "customversion", newcustomversion).Broadcast();
+	}
+
 	// Re-read config stuff
 	try
 	{
 		Utils->ReadConfiguration();
 	}
-	catch (ModuleException& e)
+	catch (const ModuleException& e)
 	{
 		// Refresh the IP cache anyway, so servers read before the error will be allowed to connect
 		Utils->RefreshIPCache();
 		// Always warn local opers with snomask +l, also warn globally (snomask +L) if the rehash was issued by a remote user
 		std::string msg = "Error in configuration: ";
 		msg.append(e.GetReason());
-		ServerInstance->SNO->WriteToSnoMask('l', msg);
+		ServerInstance->SNO.WriteToSnoMask('l', msg);
 		if (status.srcuser && !IS_LOCAL(status.srcuser))
 			ServerInstance->PI->SendSNONotice('L', msg);
 	}
 }
 
+namespace
+{
+	void BroadcastModuleState(Module* mod, bool loading)
+	{
+		std::stringstream buffer;
+		buffer << (loading ? '+' : '-') << ModuleManager::ShrinkModName(mod->ModuleFile);
+
+		std::stringstream compatbuffer;
+		compatbuffer << (loading ? '+' : '-') << mod->ModuleFile;
+
+		if (loading)
+		{
+			const std::string linkstring = SpanningTreeUtilities::BuildLinkString(PROTO_INSPIRCD_4, mod);
+			if (!linkstring.empty())
+				buffer << '=' << linkstring;
+
+			const std::string compatlinkstring = SpanningTreeUtilities::BuildLinkString(PROTO_INSPIRCD_3, mod);
+			if (!compatlinkstring.empty())
+				compatbuffer << '=' << compatlinkstring;
+		}
+
+		for (const auto* child : Utils->TreeRoot->GetChildren())
+		{
+			if (!child->GetSocket())
+				continue; // Should never happen?
+
+			if (child->GetSocket()->proto_version <= PROTO_INSPIRCD_3)
+				CommandMetadata::Builder("modules", buffer.str()).Forward(child);
+			else
+				CommandMetadata::Builder("modules", compatbuffer.str()).Forward(child);
+		}
+	}
+}
+
 void ModuleSpanningTree::OnLoadModule(Module* mod)
 {
-	std::string data;
-	data.push_back('+');
-	data.append(mod->ModuleSourceFile);
-	Version v = mod->GetVersion();
-	if (!v.link_data.empty())
-	{
-		data.push_back('=');
-		data.append(v.link_data);
-	}
-	ServerInstance->PI->SendMetaData("modules", data);
+	BroadcastModuleState(mod, true);
 }
 
 void ModuleSpanningTree::OnUnloadModule(Module* mod)
 {
 	if (!Utils)
 		return;
-	ServerInstance->PI->SendMetaData("modules", "-" + mod->ModuleSourceFile);
 
+	BroadcastModuleState(mod, false);
 	if (mod == this)
 	{
 		// We are being unloaded, inform modules about all servers splitting which cannot be done later when the servers are actually disconnected
-		const server_hash& servers = Utils->serverlist;
-		for (server_hash::const_iterator i = servers.begin(); i != servers.end(); ++i)
+		for (const auto& [_, server] : Utils->serverlist)
 		{
-			TreeServer* server = i->second;
 			if (!server->IsRoot())
-				FOREACH_MOD_CUSTOM(GetLinkEventProvider(), ServerProtocol::LinkEventListener, OnServerSplit, (server, false));
+				linkeventprov.Call(&ServerProtocol::LinkEventListener::OnServerSplit, server, false);
 		}
 		return;
 	}
@@ -708,38 +772,36 @@ void ModuleSpanningTree::OnUnloadModule(Module* mod)
 
 restart:
 	// Close all connections which use an IO hook provided by this module
-	const TreeServer::ChildServers& list = Utils->TreeRoot->GetChildren();
-	for (TreeServer::ChildServers::const_iterator i = list.begin(); i != list.end(); ++i)
+	for (const auto* child : Utils->TreeRoot->GetChildren())
 	{
-		TreeSocket* sock = (*i)->GetSocket();
+		TreeSocket* sock = child->GetSocket();
 		if (sock->GetModHook(mod))
 		{
-			sock->SendError("SSL module unloaded");
+			sock->SendError("TLS module unloaded");
 			sock->Close();
 			// XXX: The list we're iterating is modified by TreeServer::SQuit() which is called by Close()
 			goto restart;
 		}
 	}
 
-	for (SpanningTreeUtilities::TimeoutList::const_iterator i = Utils->timeoutlist.begin(); i != Utils->timeoutlist.end(); ++i)
+	for (const auto& [sock, _] : Utils->timeoutlist)
 	{
-		TreeSocket* sock = i->first;
 		if (sock->GetModHook(mod))
 			sock->Close();
 	}
 }
 
-void ModuleSpanningTree::OnOper(User* user, const std::string &opertype)
+void ModuleSpanningTree::OnOperLogin(User* user, const std::shared_ptr<OperAccount>& oper, bool automatic)
 {
-	if (user->registered != REG_ALL || !IS_LOCAL(user))
+	if (!user->IsFullyConnected() || !IS_LOCAL(user))
 		return;
 
 	// Note: The protocol does not allow direct umode +o;
 	// sending OPERTYPE infers +o modechange locally.
-	CommandOpertype::Builder(user).Broadcast();
+	CommandOpertype::Builder(user, oper, automatic).Broadcast();
 }
 
-void ModuleSpanningTree::OnAddLine(User* user, XLine *x)
+void ModuleSpanningTree::OnAddLine(User* user, XLine* x)
 {
 	if (!x->IsBurstable() || loopCall || (user && !IS_LOCAL(user)))
 		return;
@@ -750,7 +812,7 @@ void ModuleSpanningTree::OnAddLine(User* user, XLine *x)
 	CommandAddLine::Builder(x, user).Broadcast();
 }
 
-void ModuleSpanningTree::OnDelLine(User* user, XLine *x)
+void ModuleSpanningTree::OnDelLine(User* user, XLine* x)
 {
 	if (!x->IsBurstable() || loopCall || (user && !IS_LOCAL(user)))
 		return;
@@ -764,15 +826,15 @@ void ModuleSpanningTree::OnDelLine(User* user, XLine *x)
 	params.Broadcast();
 }
 
-void ModuleSpanningTree::OnUserAway(User* user)
+void ModuleSpanningTree::OnUserAway(User* user, const std::optional<AwayState>& prevstate)
 {
-	if (IS_LOCAL(user))
+	if (IS_LOCAL(user) && user->IsFullyConnected())
 		CommandAway::Builder(user).Broadcast();
 }
 
-void ModuleSpanningTree::OnUserBack(User* user)
+void ModuleSpanningTree::OnUserBack(User* user, const std::optional<AwayState>& prevstate)
 {
-	OnUserAway(user);
+	OnUserAway(user, prevstate);
 }
 
 void ModuleSpanningTree::OnMode(User* source, User* u, Channel* c, const Modes::ChangeList& modes, ModeParser::ModeProcessFlag processflags)
@@ -782,7 +844,7 @@ void ModuleSpanningTree::OnMode(User* source, User* u, Channel* c, const Modes::
 
 	if (u)
 	{
-		if (u->registered != REG_ALL)
+		if (!u->IsFullyConnected())
 			return;
 
 		CmdBuilder params(source, "MODE");
@@ -809,46 +871,40 @@ void ModuleSpanningTree::OnShutdown(const std::string& reason)
 		children.front()->SQuit(reason, true);
 }
 
-void ModuleSpanningTree::OnDecodeMetaData(Extensible* target, const std::string& extname, const std::string& extdata)
+void ModuleSpanningTree::OnDecodeMetadata(Extensible* target, const std::string& extname, const std::string& extdata)
 {
-	// HACK: this should use automatically synced user metadata in v4.
-	User* dest = static_cast<User*>(target);
-	if (dest && (extname == "uniqueusername"))
-		dest->uniqueusername = (extdata != "0");
+	// HACK: this should use automatically synced user metadata.
+	if (target && target->extype == ExtensionType::USER && irc::equals(extname, "uniqueusername"))
+		static_cast<User*>(target)->uniqueusername = (extdata != "0");
 }
 
-CullResult ModuleSpanningTree::cull()
+Cullable::Result ModuleSpanningTree::Cull()
 {
 	if (Utils)
-		Utils->cull();
-	return this->Module::cull();
+		Utils->Cull();
+	return Module::Cull();
 }
 
 ModuleSpanningTree::~ModuleSpanningTree()
 {
 	ServerInstance->PI = &ServerInstance->DefaultProtocolInterface;
 
-	Server* newsrv = new Server(ServerInstance->Config->GetSID(), ServerInstance->Config->ServerName, ServerInstance->Config->ServerDesc);
+	auto* newsrv = new Server(ServerInstance->Config->ServerId, ServerInstance->Config->ServerName, ServerInstance->Config->ServerDesc);
 	SetLocalUsersServer(newsrv);
 
 	delete Utils;
 }
 
-Version ModuleSpanningTree::GetVersion()
-{
-	return Version("Allows linking multiple servers together as part of one network.", VF_VENDOR);
-}
-
 /* It is IMPORTANT that m_spanningtree is the last module in the chain
  * so that any activity it sees is FINAL, e.g. we arent going to send out
- * a NICK message before m_cloaking has finished putting the +x on the user,
+ * a NICK message before the cloak module has finished putting the +x on the user,
  * etc etc.
  * Therefore, we set our priority to PRIORITY_LAST to make sure we end up at the END of
  * the module call queue.
  */
 void ModuleSpanningTree::Prioritize()
 {
-	ServerInstance->Modules->SetPriority(this, PRIORITY_LAST);
+	ServerInstance->Modules.SetPriority(this, PRIORITY_LAST);
 	ServerInstance->Modules.SetPriority(this, I_OnPreTopicChange, PRIORITY_FIRST);
 }
 
