@@ -1,7 +1,7 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2019, 2021-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2015 Attila Molnar <attilamolnar@hush.com>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -25,7 +25,7 @@
 #include "commandbuilder.h"
 
 PingTimer::PingTimer(TreeServer* ts)
-	: Timer(Utils->PingFreq)
+	: Timer(Utils->PingFreq, false)
 	, server(ts)
 	, state(PS_SENDPING)
 {
@@ -48,7 +48,7 @@ PingTimer::State PingTimer::TickInternal()
 	else if (state == PS_WARN)
 	{
 		// No pong arrived in PingWarnTime seconds, send a warning to opers
-		ServerInstance->SNO->WriteToSnoMask('l', "Server \002%s\002 has not responded to PING for %d seconds, high latency.", server->GetName().c_str(), GetInterval());
+		ServerInstance->SNO.WriteToSnoMask('l', "Server \002{}\002 has not responded to PING for {} seconds, high latency.", server->GetName(), GetInterval());
 		return PS_TIMEOUT;
 	}
 	else // PS_TIMEOUT
@@ -83,7 +83,7 @@ void PingTimer::SetState(State newstate)
 	// If state == PS_IDLE, do not set the timer, see above why
 }
 
-bool PingTimer::Tick(time_t currtime)
+bool PingTimer::Tick()
 {
 	if (server->IsDead())
 		return false;
