@@ -2,10 +2,9 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2013, 2018-2020, 2022-2023 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2013 Adam <Adam@anope.org>
+ *   Copyright (C) 2018-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2016, 2018 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008, 2012 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007-2008 Dennis Friis <peavey@inspircd.org>
@@ -26,6 +25,7 @@
 
 
 #include "inspircd.h"
+#include "timeutils.h"
 
 #include "main.h"
 #include "utils.h"
@@ -37,7 +37,7 @@
 /* Handle ERROR command */
 void TreeSocket::Error(CommandBase::Params& params)
 {
-	std::string msg = params.size() ? params[0] : "";
+	const std::string msg = params.empty() ? "" : params[0];
 	SetError("received ERROR " + msg);
 }
 
@@ -86,14 +86,14 @@ void TreeSocket::Split(const std::string& line, std::string& tags, std::string&
 		params.push_back(token);
 }
 
-void TreeSocket::ProcessLine(std::string &line)
+void TreeSocket::ProcessLine(std::string& line)
 {
 	std::string tags;
 	std::string prefix;
 	std::string command;
 	CommandBase::Params params;
 
-	ServerInstance->Logs->Log(MODNAME, LOG_RAWIO, "S[%d] I %s", this->GetFd(), line.c_str());
+	ServerInstance->Logs.RawIO(MODNAME, "S[{}] I {}", GetFd(), line);
 
 	Split(line, tags, prefix, command, params);
 
@@ -160,17 +160,17 @@ void TreeSocket::ProcessLine(std::string &line)
 			{
 				if (!params.empty())
 				{
-					time_t them = ConvToNum<time_t>(params[0]);
+					time_t them = ServerCommand::ExtractTS(params[0]);
 					time_t delta = them - ServerInstance->Time();
-					if ((delta < -60) || (delta > 60))
+					if ((delta < -15) || (delta > 15))
 					{
-						ServerInstance->SNO->WriteGlobalSno('l', "\002ERROR\002: Your clocks are off by %ld seconds (this is more than one minute). Link aborted, \002PLEASE SYNC YOUR CLOCKS!\002", labs((long)delta));
-						SendError("Your clocks are out by "+ConvToStr(labs((long)delta))+" seconds (this is more than one minute). Link aborted, PLEASE SYNC YOUR CLOCKS!");
+						ServerInstance->SNO.WriteGlobalSno('l', "\002ERROR\002: Your clocks are off by {} seconds (this is more than fifteen seconds). Link aborted, \002PLEASE SYNC YOUR CLOCKS!\002", labs((long)delta));
+						SendError("Your clocks are out by "+ConvToStr(labs((long)delta))+" seconds (this is more than fifteen seconds). Link aborted, PLEASE SYNC YOUR CLOCKS!");
 						return;
 					}
-					else if ((delta < -15) || (delta > 15))
+					else if ((delta < -5) || (delta > 5))
 					{
-						ServerInstance->SNO->WriteGlobalSno('l', "\002WARNING\002: Your clocks are off by %ld seconds. Please consider syncing your clocks.", labs((long)delta));
+						ServerInstance->SNO.WriteGlobalSno('l', "\002WARNING\002: Your clocks are off by {} seconds. Please consider syncing your clocks.", labs((long)delta));
 					}
 				}
 
@@ -192,8 +192,8 @@ void TreeSocket::ProcessLine(std::string &line)
 			}
 			else
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Unknown command from fd %d in the WAIT_AUTH_2 phase: %s",
-					GetFd(), command.c_str());
+				ServerInstance->Logs.Debug(MODNAME, "Unknown command from fd {} in the WAIT_AUTH_2 phase: {}",
+					GetFd(), command);
 			}
 
 		break;
@@ -219,8 +219,8 @@ void TreeSocket::ProcessLine(std::string &line)
 			}
 			else
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Unknown command from fd %d in the CONNECTING phase: %s",
-					GetFd(), command.c_str());
+				ServerInstance->Logs.Debug(MODNAME, "Unknown command from fd {} in the CONNECTING phase: {}",
+					GetFd(), command);
 			}
 		break;
 
@@ -254,7 +254,7 @@ User* TreeSocket::FindSource(const std::string& prefix, const std::string& comma
 	else
 	{
 		// If the prefix string is a uuid FindUUID() returns the appropriate User object
-		User* user = ServerInstance->FindUUID(prefix);
+		auto* user = ServerInstance->Users.FindUUID(prefix);
 		if (user)
 			return user;
 	}
@@ -284,7 +284,7 @@ User* TreeSocket::FindSource(const std::string& prefix, const std::string& comma
 	}
 
 	// Unknown prefix
-	return NULL;
+	return nullptr;
 }
 
 void TreeSocket::ProcessTag(User* source, const std::string& tag, ClientProtocol::TagMap& tags)
@@ -303,13 +303,12 @@ void TreeSocket::ProcessTag(User* source, const std::string& tag, ClientProtocol
 		tagkey.assign(tag);
 	}
 
-	const Events::ModuleEventProvider::SubscriberList& list = Utils->Creator->tagevprov.GetSubscribers();
-	for (Events::ModuleEventProvider::SubscriberList::const_iterator i = list.begin(); i != list.end(); ++i)
+	for (auto* subscriber : Utils->Creator->tagevprov.GetSubscribers())
 	{
-		ClientProtocol::MessageTagProvider* const tagprov = static_cast<ClientProtocol::MessageTagProvider*>(*i);
+		ClientProtocol::MessageTagProvider* const tagprov = static_cast<ClientProtocol::MessageTagProvider*>(subscriber);
 		const ModResult res = tagprov->OnProcessTag(source, tagkey, tagval);
 		if (res == MOD_RES_ALLOW)
-			tags.insert(std::make_pair(tagkey, ClientProtocol::MessageTagData(tagprov, tagval)));
+			tags.emplace(tagkey, ClientProtocol::MessageTagData(tagprov, tagval));
 		else if (res == MOD_RES_DENY)
 			break;
 	}
@@ -320,7 +319,7 @@ void TreeSocket::ProcessConnectedLine(std::string& taglist, std::string& prefix,
 	User* who = FindSource(prefix, command);
 	if (!who)
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Command '%s' from unknown prefix '%s'! Dropping entire command.", command.c_str(), prefix.c_str());
+		ServerInstance->Logs.Debug(MODNAME, "Command '{}' from unknown prefix '{}'! Dropping entire command.", command, prefix);
 		return;
 	}
 
@@ -343,7 +342,7 @@ void TreeSocket::ProcessConnectedLine(std::string& taglist, std::string& prefix,
 	TreeServer* const server = TreeServer::Get(who);
 	if (server->GetSocket() != this)
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Protocol violation: Fake direction '%s' from connection '%s'", prefix.c_str(), linkID.c_str());
+		ServerInstance->Logs.Debug(MODNAME, "Protocol violation: Fake direction '{}' from connection '{}'", prefix, linkID);
 		return;
 	}
 
@@ -356,7 +355,7 @@ void TreeSocket::ProcessConnectedLine(std::string& taglist, std::string& prefix,
 
 	ServerCommand* scmd = Utils->Creator->CmdManager.GetHandler(command);
 	CommandBase* cmdbase = scmd;
-	Command* cmd = NULL;
+	Command* cmd = nullptr;
 	if (!scmd)
 	{
 		// Not a special server-to-server command
@@ -404,17 +403,17 @@ void TreeSocket::ProcessConnectedLine(std::string& taglist, std::string& prefix,
 	else
 	{
 		res = cmd->Handle(who, newparams);
-		if (res == CMD_INVALID)
+		if (res == CmdResult::INVALID)
 			throw ProtocolException("Error in command handler");
 	}
 
-	if (res == CMD_SUCCESS)
+	if (res == CmdResult::SUCCESS)
 		Utils->RouteCommand(server->GetRoute(), cmdbase, newparams, who);
 }
 
 void TreeSocket::OnTimeout()
 {
-	ServerInstance->SNO->WriteGlobalSno('l', "CONNECT: Connection to \002%s\002 timed out.", linkID.c_str());
+	ServerInstance->SNO.WriteGlobalSno('l', "CONNECT: Connection to \002{}\002 timed out.", linkID);
 }
 
 void TreeSocket::Close()
@@ -430,15 +429,15 @@ void TreeSocket::Close()
 	// If the connection is fully up (state CONNECTED)
 	// then propagate a netsplit to all peers.
 	if (MyRoot && !MyRoot->IsDead())
-		MyRoot->SQuit(getError(), true);
+		MyRoot->SQuit(GetError(), true);
 	else
-		ServerInstance->SNO->WriteGlobalSno('l', "Connection to '\002%s\002' failed.", linkID.c_str());
+		ServerInstance->SNO.WriteGlobalSno('l', "Connection to '\002{}\002' failed.", linkID);
 
 	time_t server_uptime = ServerInstance->Time() - this->age;
 	if (server_uptime)
 	{
-		std::string timestr = InspIRCd::DurationString(server_uptime);
-		ServerInstance->SNO->WriteGlobalSno('l', "Connection to '\002%s\002' was established for %s", linkID.c_str(), timestr.c_str());
+		std::string timestr = Duration::ToString(server_uptime);
+		ServerInstance->SNO.WriteGlobalSno('l', "Connection to '\002{}\002' was established for {}", linkID, timestr);
 	}
 }
 
