@@ -1,7 +1,7 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013-2014, 2017-2019, 2021-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013-2014, 2018-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013 Adam <Adam@anope.org>
  *   Copyright (C) 2012-2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
@@ -34,29 +34,28 @@
 #include "resolvers.h"
 #include "commandbuilder.h"
 
-SpanningTreeUtilities* Utils = NULL;
+SpanningTreeUtilities* Utils = nullptr;
 
-ModResult ModuleSpanningTree::OnAcceptConnection(int newsock, ListenSocket* from, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server)
+ModResult ModuleSpanningTree::OnAcceptConnection(int newsock, ListenSocket* from, const irc::sockets::sockaddrs& client, const irc::sockets::sockaddrs& server)
 {
-	if (!stdalgo::string::equalsci(from->bind_tag->getString("type"), "servers"))
+	if (!insp::equalsci(from->bind_tag->getString("type"), "servers"))
 		return MOD_RES_PASSTHRU;
 
-	std::string incomingip = client->addr();
-
-	for (std::vector<std::string>::iterator i = Utils->ValidIPs.begin(); i != Utils->ValidIPs.end(); i++)
+	const std::string incomingip = client.addr();
+	for (const auto& validip : Utils->ValidIPs)
 	{
-		if (*i == "*" || *i == incomingip || irc::sockets::cidr_mask(*i).match(*client))
+		if (validip == "*" || validip == incomingip || irc::sockets::cidr_mask(validip).match(client))
 		{
 			/* we don't need to do anything with the pointer, creating it stores it in the necessary places */
 			new TreeSocket(newsock, from, client, server);
 			return MOD_RES_ALLOW;
 		}
 	}
-	ServerInstance->SNO->WriteToSnoMask('l', "Server connection from %s denied (no link blocks with that IP address)", incomingip.c_str());
+	ServerInstance->SNO.WriteToSnoMask('l', "Server connection from {} denied (no link blocks with that IP address)", incomingip);
 	return MOD_RES_DENY;
 }
 
-TreeServer* SpanningTreeUtilities::FindServer(const std::string &ServerName)
+TreeServer* SpanningTreeUtilities::FindServer(const std::string& ServerName)
 {
 	if (InspIRCd::IsSID(ServerName))
 		return this->FindServerID(ServerName);
@@ -68,30 +67,30 @@ TreeServer* SpanningTreeUtilities::FindServer(const std::string &ServerName)
 	}
 	else
 	{
-		return NULL;
+		return nullptr;
 	}
 }
 
 /** Find the first server matching a given glob mask.
  * We iterate over the list and match each one until we get a hit.
  */
-TreeServer* SpanningTreeUtilities::FindServerMask(const std::string &ServerName)
+TreeServer* SpanningTreeUtilities::FindServerMask(const std::string& ServerName)
 {
-	for (server_hash::iterator i = serverlist.begin(); i != serverlist.end(); i++)
+	for (const auto& [name, server] : serverlist)
 	{
-		if (InspIRCd::Match(i->first,ServerName))
-			return i->second;
+		if (InspIRCd::Match(name, ServerName))
+			return server;
 	}
-	return NULL;
+	return nullptr;
 }
 
-TreeServer* SpanningTreeUtilities::FindServerID(const std::string &id)
+TreeServer* SpanningTreeUtilities::FindServerID(const std::string& id)
 {
 	server_hash::iterator iter = sidlist.find(id);
 	if (iter != sidlist.end())
 		return iter->second;
 	else
-		return NULL;
+		return nullptr;
 }
 
 TreeServer* SpanningTreeUtilities::FindRouteTarget(const std::string& target)
@@ -100,21 +99,20 @@ TreeServer* SpanningTreeUtilities::FindRouteTarget(const std::string& target)
 	if (server)
 		return server;
 
-	User* const user = ServerInstance->FindNick(target);
+	User* const user = ServerInstance->Users.Find(target);
 	if (user)
 		return TreeServer::Get(user);
 
-	return NULL;
+	return nullptr;
 }
 
 SpanningTreeUtilities::SpanningTreeUtilities(ModuleSpanningTree* C)
-	: Creator(C), TreeRoot(NULL)
-	, PingFreq(60) // XXX: TreeServer constructor reads this and TreeRoot is created before the config is read, so init it to something (value doesn't matter) to avoid a valgrind warning in TimerManager on unload
+	: Creator(C)
 {
 	ServerInstance->Timers.AddTimer(&RefreshTimer);
 }
 
-CullResult SpanningTreeUtilities::cull()
+Cullable::Result SpanningTreeUtilities::Cull()
 {
 	const TreeServer::ChildServers& children = TreeRoot->GetChildren();
 	while (!children.empty())
@@ -123,14 +121,12 @@ CullResult SpanningTreeUtilities::cull()
 		sock->Close();
 	}
 
-	for(TimeoutList::iterator i = timeoutlist.begin(); i != timeoutlist.end(); ++i)
-	{
-		TreeSocket* s = i->first;
+	for (const auto& [s, _] : timeoutlist)
 		s->Close();
-	}
-	TreeRoot->cull();
 
-	return classbase::cull();
+	TreeRoot->Cull();
+
+	return Cullable::Cull();
 }
 
 SpanningTreeUtilities::~SpanningTreeUtilities()
@@ -139,29 +135,28 @@ SpanningTreeUtilities::~SpanningTreeUtilities()
 }
 
 // Returns a list of DIRECT servers for a specific channel
-void SpanningTreeUtilities::GetListOfServersForChannel(Channel* c, TreeSocketSet& list, char status, const CUList& exempt_list)
+void SpanningTreeUtilities::GetListOfServersForChannel(const Channel* c, TreeSocketSet& list, char status, const CUList& exempt_list) const
 {
-	unsigned int minrank = 0;
+	ModeHandler::Rank minrank = 0;
 	if (status)
 	{
-		PrefixMode* mh = ServerInstance->Modes->FindPrefix(status);
+		PrefixMode* mh = ServerInstance->Modes.FindPrefix(status);
 		if (mh)
 			minrank = mh->GetPrefixRank();
 	}
 
 	TreeServer::ChildServers children = TreeRoot->GetChildren();
-	const Channel::MemberMap& ulist = c->GetUsers();
-	for (Channel::MemberMap::const_iterator i = ulist.begin(); i != ulist.end(); ++i)
+	for (const auto& [user, memb] : c->GetUsers())
 	{
-		if (IS_LOCAL(i->first))
+		if (IS_LOCAL(user))
 			continue;
 
-		if (minrank && i->second->getRank() < minrank)
+		if (minrank && memb->GetRank() < minrank)
 			continue;
 
-		if (exempt_list.find(i->first) == exempt_list.end())
+		if (exempt_list.find(user) == exempt_list.end())
 		{
-			TreeServer* best = TreeServer::Get(i->first);
+			TreeServer* best = TreeServer::Get(user);
 			list.insert(best->GetSocket());
 
 			TreeServer::ChildServers::iterator citer = std::find(children.begin(), children.end(), best);
@@ -172,23 +167,23 @@ void SpanningTreeUtilities::GetListOfServersForChannel(Channel* c, TreeSocketSet
 
 	// Check whether the servers which do not have users in the channel might need this message. This
 	// is used to keep the chanhistory module synchronised between servers.
-	for (TreeServer::ChildServers::const_iterator i = children.begin(); i != children.end(); ++i)
+	for (const auto &[_, server] : Utils->serverlist)
 	{
-		ModResult result;
-		FIRST_MOD_RESULT_CUSTOM(Creator->GetBroadcastEventProvider(), ServerProtocol::BroadcastEventListener, OnBroadcastMessage, result, (c, *i));
+		if (!server->GetRoute())
+			continue; // Local server
+
+		ModResult result = Creator->routeeventprov.FirstResult(&ServerProtocol::RouteEventListener::OnRouteMessage, c, server);
 		if (result == MOD_RES_ALLOW)
-			list.insert((*i)->GetSocket());
+			list.insert(server->GetRoute()->GetSocket());
 	}
 }
 
-void SpanningTreeUtilities::DoOneToAllButSender(const CmdBuilder& params, TreeServer* omitroute)
+void SpanningTreeUtilities::DoOneToAllButSender(const CmdBuilder& params, const TreeServer* omitroute) const
 {
 	const std::string& FullLine = params.str();
 
-	const TreeServer::ChildServers& children = TreeRoot->GetChildren();
-	for (TreeServer::ChildServers::const_iterator i = children.begin(); i != children.end(); ++i)
+	for (const auto* Route : TreeRoot->GetChildren())
 	{
-		TreeServer* Route = *i;
 		// Send the line if the route isn't the path to the one to be omitted
 		if (Route != omitroute)
 		{
@@ -197,9 +192,9 @@ void SpanningTreeUtilities::DoOneToAllButSender(const CmdBuilder& params, TreeSe
 	}
 }
 
-void SpanningTreeUtilities::DoOneToOne(const CmdBuilder& params, Server* server)
+void SpanningTreeUtilities::DoOneToOne(const CmdBuilder& params, const Server* server)
 {
-	TreeServer* ts = static_cast<TreeServer*>(server);
+	const TreeServer* ts = static_cast<const TreeServer*>(server);
 	TreeSocket* sock = ts->GetSocket();
 	if (sock)
 		sock->WriteLine(params);
@@ -208,31 +203,30 @@ void SpanningTreeUtilities::DoOneToOne(const CmdBuilder& params, Server* server)
 void SpanningTreeUtilities::RefreshIPCache()
 {
 	ValidIPs.clear();
-	for (std::vector<reference<Link> >::iterator i = LinkBlocks.begin(); i != LinkBlocks.end(); ++i)
+	for (const auto& L : LinkBlocks)
 	{
-		Link* L = *i;
 		bool isunix = L->IPAddr.find('/') != std::string::npos;
 		if (!L->Port && !isunix)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Ignoring an IP link block without a port.");
+			ServerInstance->Logs.Warning(MODNAME, "Ignoring an IP link block without a port.");
 			/* Invalid link block */
 			continue;
 		}
 
 		ValidIPs.insert(ValidIPs.end(), L->AllowMasks.begin(), L->AllowMasks.end());
 
-		irc::sockets::sockaddrs dummy;
-		bool ipvalid = irc::sockets::aptosa(L->IPAddr, L->Port, dummy);
+		irc::sockets::sockaddrs dummy(false);
+		bool ipvalid = dummy.from_ip_port(L->IPAddr, L->Port);
 		if ((L->IPAddr == "*") || (isunix) || (ipvalid))
 			ValidIPs.push_back(L->IPAddr);
 		else if (this->Creator->DNS)
 		{
-			SecurityIPResolver* sr = new SecurityIPResolver(Creator, *this->Creator->DNS, L->IPAddr, L, DNS::QUERY_AAAA);
+			auto* sr = new SecurityIPResolver(Creator, *this->Creator->DNS, L->IPAddr, L, DNS::QUERY_AAAA);
 			try
 			{
 				this->Creator->DNS->Process(sr);
 			}
-			catch (DNS::Exception &)
+			catch (const DNS::Exception&)
 			{
 				delete sr;
 			}
@@ -242,27 +236,29 @@ void SpanningTreeUtilities::RefreshIPCache()
 
 void SpanningTreeUtilities::ReadConfiguration()
 {
-	ConfigTag* security = ServerInstance->Config->ConfValue("security");
-	ConfigTag* options = ServerInstance->Config->ConfValue("options");
-	FlatLinks = security->getBool("flatlinks");
-	HideULines = security->getBool("hideulines");
-	HideSplits = security->getBool("hidesplits");
-	AnnounceTSChange = options->getBool("announcets");
+	const auto& options = ServerInstance->Config->ConfValue("options");
 	AllowOptCommon = options->getBool("allowmismatch");
-	quiet_bursts = ServerInstance->Config->ConfValue("performance")->getBool("quietbursts");
+	AnnounceTSChange = options->getBool("announcets");
 	PingWarnTime = options->getDuration("pingwarning", 15);
 	PingFreq = options->getDuration("serverpingfreq", 60, 1);
 
+	const auto& security = ServerInstance->Config->ConfValue("security");
+	FlatLinks = security->getBool("flatlinks");
+	HideServices = security->getBool("hideservices", security->getBool("hideulines"));
+	HideSplits = security->getBool("hidesplits");
+
+	const auto& performance = ServerInstance->Config->ConfValue("performance");
+	quiet_bursts = performance->getBool("quietbursts");
+
 	if (PingWarnTime >= PingFreq)
 		PingWarnTime = 0;
 
 	AutoconnectBlocks.clear();
 	LinkBlocks.clear();
-	ConfigTagList tags = ServerInstance->Config->ConfTags("link");
-	for(ConfigIter i = tags.first; i != tags.second; ++i)
+
+	for (const auto& [_, tag] : ServerInstance->Config->ConfTags("link"))
 	{
-		ConfigTag* tag = i->second;
-		reference<Link> L = new Link(tag);
+		auto L = std::make_shared<Link>(tag);
 
 		irc::spacesepstream sep = tag->getString("allowmask");
 		for (std::string s; sep.GetToken(s);)
@@ -272,7 +268,15 @@ void SpanningTreeUtilities::ReadConfiguration()
 		if (path.empty())
 		{
 			L->IPAddr = tag->getString("ipaddr");
-			L->Port = tag->getUInt("port", 0, 1, 65535);
+			L->Port = tag->getNum<in_port_t>("port", 0);
+			if (tag->getBool("sctp"))
+			{
+#ifdef IPPROTO_SCTP
+				L->Protocol = IPPROTO_SCTP;
+#else
+				throw ModuleException((Module*)Creator, "Unable to use SCTP for outgoing connections as this platform does not support SCTP!");
+#endif
+			}
 		}
 		else
 		{
@@ -286,39 +290,39 @@ void SpanningTreeUtilities::ReadConfiguration()
 		L->Fingerprint = tag->getString("fingerprint");
 		L->HiddenFromStats = tag->getBool("statshidden");
 		L->Timeout = tag->getDuration("timeout", 30);
-		L->Hook = tag->getString("sslprofile", tag->getString("ssl"));
+		L->Hook = tag->getString("sslprofile");
 		L->Bind = tag->getString("bind");
 		L->Hidden = tag->getBool("hidden");
 
 		if (L->Name.empty())
-			throw ModuleException("Invalid configuration, found a link tag without a name!" + (!L->IPAddr.empty() ? " IP address: "+L->IPAddr : ""));
+			throw ModuleException((Module*)Creator, "Invalid configuration, found a link tag without a name!" + (!L->IPAddr.empty() ? " IP address: "+L->IPAddr : ""));
 
 		if (L->Name.find('.') == std::string::npos)
-			throw ModuleException("The link name '"+L->Name+"' is invalid as it must contain at least one '.' character");
+			throw ModuleException((Module*)Creator, "The link name '"+L->Name+"' is invalid as it must contain at least one '.' character");
 
 		if (L->Name.length() > ServerInstance->Config->Limits.MaxHost)
-			throw ModuleException("The link name '"+L->Name+"' is invalid as it is longer than " + ConvToStr(ServerInstance->Config->Limits.MaxHost) + " characters");
+			throw ModuleException((Module*)Creator, "The link name '"+L->Name+"' is invalid as it is longer than " + ConvToStr(ServerInstance->Config->Limits.MaxHost) + " characters");
 
 		if (L->RecvPass.empty())
-			throw ModuleException("Invalid configuration for server '"+L->Name+"', recvpass not defined");
+			throw ModuleException((Module*)Creator, "Invalid configuration for server '"+L->Name+"', recvpass not defined");
 
 		if (L->SendPass.empty())
-			throw ModuleException("Invalid configuration for server '"+L->Name+"', sendpass not defined");
+			throw ModuleException((Module*)Creator, "Invalid configuration for server '"+L->Name+"', sendpass not defined");
 
 		if ((L->SendPass.find(' ') != std::string::npos) || (L->RecvPass.find(' ') != std::string::npos))
-			throw ModuleException("Link block '" + L->Name + "' has a password set that contains a space character which is invalid");
+			throw ModuleException((Module*)Creator, "Link block '" + L->Name + "' has a password set that contains a space character which is invalid");
 
 		if ((L->SendPass[0] == ':') || (L->RecvPass[0] == ':'))
-			throw ModuleException("Link block '" + L->Name + "' has a password set that begins with a colon (:) which is invalid");
+			throw ModuleException((Module*)Creator, "Link block '" + L->Name + "' has a password set that begins with a colon (:) which is invalid");
 
 		if (L->IPAddr.empty())
 		{
 			L->IPAddr = "*";
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Configuration warning: Link block '" + L->Name + "' has no IP defined! This will allow any IP to connect as this server, and MAY not be what you want.");
+			ServerInstance->Logs.Warning(MODNAME, "Configuration warning: Link block '" + L->Name + "' has no IP defined! This will allow any IP to connect as this server, and MAY not be what you want.");
 		}
 
 		if (!L->Port && L->IPAddr.find('/') == std::string::npos)
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Configuration warning: Link block '" + L->Name + "' has no port defined, you will not be able to /connect it.");
+			ServerInstance->Logs.Warning(MODNAME, "Configuration warning: Link block '" + L->Name + "' has no port defined, you will not be able to /connect it.");
 
 		std::transform(L->Fingerprint.begin(), L->Fingerprint.end(), L->Fingerprint.begin(), ::tolower);
 		L->Fingerprint.erase(std::remove(L->Fingerprint.begin(), L->Fingerprint.end(), ':'), L->Fingerprint.end());
@@ -326,11 +330,9 @@ void SpanningTreeUtilities::ReadConfiguration()
 		LinkBlocks.push_back(L);
 	}
 
-	tags = ServerInstance->Config->ConfTags("autoconnect");
-	for(ConfigIter i = tags.first; i != tags.second; ++i)
+	for (const auto& [_, tag] : ServerInstance->Config->ConfTags("autoconnect"))
 	{
-		ConfigTag* tag = i->second;
-		reference<Autoconnect> A = new Autoconnect(tag);
+		auto A = std::make_shared<Autoconnect>(tag);
 		A->Period = tag->getDuration("period", 60, 1);
 		A->NextConnectTime = ServerInstance->Time() + A->Period;
 		A->position = -1;
@@ -343,32 +345,31 @@ void SpanningTreeUtilities::ReadConfiguration()
 
 		if (A->servers.empty())
 		{
-			throw ModuleException("Invalid configuration for autoconnect, server cannot be empty!");
+			throw ModuleException((Module*)Creator, "Invalid configuration for autoconnect, server cannot be empty!");
 		}
 
 		AutoconnectBlocks.push_back(A);
 	}
 
-	for (server_hash::const_iterator i = serverlist.begin(); i != serverlist.end(); ++i)
-		i->second->CheckULine();
+	for (const auto& [_, server] : serverlist)
+		server->CheckService();
 
 	RefreshIPCache();
 }
 
-Link* SpanningTreeUtilities::FindLink(const std::string& name)
+std::shared_ptr<Link> SpanningTreeUtilities::FindLink(const std::string& name)
 {
-	for (std::vector<reference<Link> >::iterator i = LinkBlocks.begin(); i != LinkBlocks.end(); ++i)
+	for (const auto& x : LinkBlocks)
 	{
-		Link* x = *i;
 		if (InspIRCd::Match(x->Name, name, ascii_case_insensitive_map))
 		{
 			return x;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
-void SpanningTreeUtilities::SendChannelMessage(User* source, Channel* target, const std::string& text, char status, const ClientProtocol::TagMap& tags, const CUList& exempt_list, const char* message_type, TreeSocket* omit)
+void SpanningTreeUtilities::SendChannelMessage(const User* source, const Channel* target, const std::string& text, char status, const ClientProtocol::TagMap& tags, const CUList& exempt_list, const char* message_type, const TreeSocket* omit) const
 {
 	CmdBuilder msg(source, message_type);
 	msg.push_tags(tags);
@@ -381,29 +382,45 @@ void SpanningTreeUtilities::SendChannelMessage(User* source, Channel* target, co
 
 	TreeSocketSet list;
 	this->GetListOfServersForChannel(target, list, status, exempt_list);
-	for (TreeSocketSet::iterator i = list.begin(); i != list.end(); ++i)
+
+	for (auto* Sock : list)
 	{
-		TreeSocket* Sock = *i;
 		if (Sock != omit)
 			Sock->WriteLine(msg);
 	}
 }
 
+std::string SpanningTreeUtilities::BuildLinkString(uint16_t proto, Module* mod)
+{
+	Module::LinkData data;
+	std::string compatdata;
+	mod->GetLinkData(data, compatdata);
+
+	if (proto <= PROTO_INSPIRCD_3)
+		return compatdata;
+
+	std::stringstream buffer;
+	for (Module::LinkData::const_iterator iter = data.begin(); iter != data.end(); ++iter)
+	{
+		if (iter != data.begin())
+			buffer << '&';
+
+		buffer << iter->first << '=' << Percent::Encode(iter->second);
+	}
+	return buffer.str();
+}
+
 void SpanningTreeUtilities::SendListLimits(Channel* chan, TreeSocket* sock)
 {
 	std::stringstream buffer;
-	const ModeParser::ListModeList& listmodes = ServerInstance->Modes->GetListModes();
-	for (ModeParser::ListModeList::const_iterator i = listmodes.begin(); i != listmodes.end(); ++i)
-	{
-		ListModeBase* lm = *i;
+	for (auto* lm : ServerInstance->Modes.GetListModes())
 		buffer << lm->GetModeChar() << " " << lm->GetLimit(chan) << " ";
-	}
 
 	std::string bufferstr = buffer.str();
 	if (bufferstr.empty())
 		return; // Should never happen.
 
-	bufferstr.erase(bufferstr.end() - 1);
+	bufferstr.pop_back();
 	if (sock)
 		sock->WriteLine(CommandMetadata::Builder(chan, "maxlist", bufferstr));
 	else
