@@ -5,15 +5,15 @@
  *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2017 B00mX0r <b00mx0r@aureus.pw>
  *   Copyright (C) 2016, 2019 Matt Schatz <genius3000@g3k.solutions>
- *   Copyright (C) 2013, 2017-2018, 2020-2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2017, 2019-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013 Daniel Vassdal <shutter@canternet.org>
  *   Copyright (C) 2012-2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2009 John Brooks <special@inspircd.org>
- *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
+ *   Copyright (C) 2009 John Brooks <john@jbrooks.io>
+ *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2005-2008 Craig Edwards <brain@inspircd.org>
+ *   Copyright (C) 2005, 2007-2008 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -31,11 +31,14 @@
 
 #include "inspircd.h"
 #include "listmode.h"
+#include "modules/extban.h"
+#include "numerichelper.h"
+#include "timeutils.h"
 
 // Holds a timed ban
-class TimedBan
+class TimedBan final
 {
- public:
+public:
 	std::string mask;
 	std::string setter;
 	time_t expire;
@@ -45,10 +48,12 @@ class TimedBan
 typedef std::vector<TimedBan> timedbans;
 timedbans TimedBanList;
 
-// Handle /TBAN
-class CommandTban : public Command
+class CommandTban final
+	: public Command
 {
+private:
 	ChanModeReference banmode;
+	ExtBan::ManagerRef extbanmgr;
 
 	bool IsBanSet(Channel* chan, const std::string& mask)
 	{
@@ -59,10 +64,9 @@ class CommandTban : public Command
 		const ListModeBase::ModeList* bans = banlm->GetList(chan);
 		if (bans)
 		{
-			for (ListModeBase::ModeList::const_iterator i = bans->begin(); i != bans->end(); ++i)
+			for (const auto& ban : *bans)
 			{
-				const ListModeBase::ListItem& ban = *i;
-				if (irc::equals(ban.mask, mask))
+				if (banlm->CompareEntry(ban.mask, mask))
 					return true;
 			}
 		}
@@ -70,68 +74,67 @@ class CommandTban : public Command
 		return false;
 	}
 
- public:
+public:
 	bool sendnotice;
 
 	CommandTban(Module* Creator)
-		: Command(Creator,"TBAN", 3)
+		: Command(Creator, "TBAN", 3)
 		, banmode(Creator, "ban")
+		, extbanmgr(Creator)
 	{
-		syntax = "<channel> <duration> <banmask>";
+		syntax = { "<channel> <duration> <banmask>" };
 	}
 
-	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
+	CmdResult Handle(User* user, const Params& parameters) override
 	{
-		Channel* channel = ServerInstance->FindChan(parameters[0]);
+		auto* channel = ServerInstance->Channels.Find(parameters[0]);
 		if (!channel)
 		{
 			user->WriteNumeric(Numerics::NoSuchChannel(parameters[0]));
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
-		unsigned int cm = channel->GetPrefixValue(user);
+		ModeHandler::Rank cm = channel->GetPrefixValue(user);
 		if (cm < HALFOP_VALUE)
 		{
 			user->WriteNumeric(Numerics::ChannelPrivilegesNeeded(channel, HALFOP_VALUE, "set timed bans"));
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
 		TimedBan T;
 		unsigned long duration;
-		if (!InspIRCd::Duration(parameters[1], duration))
+		if (!Duration::TryFrom(parameters[1], duration))
 		{
 			user->WriteNotice("Invalid ban time");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 		unsigned long expire = duration + ServerInstance->Time();
 
 		std::string mask = parameters[2];
-		bool isextban = ((mask.size() > 2) && (mask[1] == ':'));
-		if (!isextban && !InspIRCd::IsValidMask(mask))
-			mask.append("!*@*");
+		if (!extbanmgr || !extbanmgr->Canonicalize(mask))
+			ModeParser::CleanMask(mask);
 
 		if (IsBanSet(channel, mask))
 		{
 			user->WriteNotice("Ban already set");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
 		Modes::ChangeList setban;
 		setban.push_add(*banmode, mask);
+
 		// Pass the user (instead of ServerInstance->FakeClient) to ModeHandler::Process() to
 		// make it so that the user sets the mode themselves
-		ServerInstance->Modes->Process(user, channel, NULL, setban);
-		if (ServerInstance->Modes->GetLastChangeList().empty())
+		ServerInstance->Modes.Process(user, channel, nullptr, setban);
+		if (ServerInstance->Modes.GetLastChangeList().empty())
 		{
 			user->WriteNotice("Invalid ban mask");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
 		// Attempt to find the actual set ban mask.
-		const Modes::ChangeList::List& list = ServerInstance->Modes->GetLastChangeList().getlist();
-		for (Modes::ChangeList::List::const_iterator iter = list.begin(); iter != list.end(); ++iter)
+		for (const auto& mc : ServerInstance->Modes.GetLastChangeList().getlist())
 		{
-			const Modes::Change& mc = *iter;
 			if (mc.mh == *banmode)
 			{
 				// We found the actual mask.
@@ -148,8 +151,9 @@ class CommandTban : public Command
 
 		if (sendnotice)
 		{
-			const std::string message = InspIRCd::Format("Timed ban %s added by %s on %s lasting for %s.",
-			mask.c_str(), user->nick.c_str(), channel->name.c_str(), InspIRCd::DurationString(duration).c_str());
+			const std::string message = INSP_FORMAT("Timed ban {} added by {} on {} lasting for {}.",
+				mask, user->nick, channel->name, Duration::ToString(duration));
+
 			// If halfop is loaded, send notice to halfops and above, otherwise send to ops and above
 			PrefixMode* mh = ServerInstance->Modes.FindNearestPrefixMode(HALFOP_VALUE);
 			char pfxchar = mh ? mh->GetPrefix() : '@';
@@ -157,26 +161,27 @@ class CommandTban : public Command
 			channel->WriteRemoteNotice(message, pfxchar);
 		}
 
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 	}
 
-	RouteDescriptor GetRouting(User* user, const Params& parameters) CXX11_OVERRIDE
+	RouteDescriptor GetRouting(User* user, const Params& parameters) override
 	{
 		return ROUTE_BROADCAST;
 	}
 };
 
-class BanWatcher : public ModeWatcher
+class BanWatcher final
+	: public ModeWatcher
 {
- public:
+public:
 	BanWatcher(Module* parent)
 		: ModeWatcher(parent, "ban", MODETYPE_CHANNEL)
 	{
 	}
 
-	void AfterMode(User* source, User* dest, Channel* chan, const std::string& banmask, bool adding) CXX11_OVERRIDE
+	void AfterMode(User* source, User* dest, Channel* chan, const Modes::Change& change) override
 	{
-		if (adding)
+		if (change.adding)
 			return;
 
 		for (timedbans::iterator i = TimedBanList.begin(); i != TimedBanList.end(); ++i)
@@ -185,7 +190,7 @@ class BanWatcher : public ModeWatcher
 				continue;
 
 			const std::string& target = i->mask;
-			if (irc::equals(banmask, target))
+			if (irc::equals(change.param, target))
 			{
 				TimedBanList.erase(i);
 				break;
@@ -194,11 +199,11 @@ class BanWatcher : public ModeWatcher
 	}
 };
 
-class ChannelMatcher
+class ChannelMatcher final
 {
 	Channel* const chan;
 
- public:
+public:
 	ChannelMatcher(Channel* ch)
 		: chan(ch)
 	{
@@ -210,28 +215,30 @@ class ChannelMatcher
 	}
 };
 
-class ModuleTimedBans : public Module
+class ModuleTimedBans final
+	: public Module
 {
- private:
+private:
 	ChanModeReference banmode;
 	CommandTban cmd;
 	BanWatcher banwatcher;
 
- public:
+public:
 	ModuleTimedBans()
-		: banmode(this, "ban")
+		: Module(VF_VENDOR | VF_COMMON, "Adds the /TBAN command which allows channel operators to add bans which will be expired after the specified period.")
+		, banmode(this, "ban")
 		, cmd(this)
 		, banwatcher(this)
 	{
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("timedbans");
+		const auto& tag = ServerInstance->Config->ConfValue("timedbans");
 		cmd.sendnotice = tag->getBool("sendnotice", true);
 	}
 
-	void OnBackgroundTimer(time_t curtime) CXX11_OVERRIDE
+	void OnBackgroundTimer(time_t curtime) override
 	{
 		timedbans expired;
 		for (timedbans::iterator i = TimedBanList.begin(); i != TimedBanList.end();)
@@ -245,38 +252,31 @@ class ModuleTimedBans : public Module
 				++i;
 		}
 
-		for (timedbans::iterator i = expired.begin(); i != expired.end(); i++)
+		for (const auto& timedban : expired)
 		{
-			const std::string mask = i->mask;
-			Channel* cr = i->chan;
-
 			if (cmd.sendnotice)
 			{
-				const std::string message = InspIRCd::Format("Timed ban %s set by %s on %s has expired.",
-				mask.c_str(), i->setter.c_str(), cr->name.c_str());
+				const std::string message = INSP_FORMAT("Timed ban {} set by {} on {} has expired.",
+					timedban.mask, timedban.setter, timedban.chan->name);
+
 				// If halfop is loaded, send notice to halfops and above, otherwise send to ops and above
 				PrefixMode* mh = ServerInstance->Modes.FindNearestPrefixMode(HALFOP_VALUE);
 				char pfxchar = mh ? mh->GetPrefix() : '@';
 
-				cr->WriteRemoteNotice(message, pfxchar);
+				timedban.chan->WriteRemoteNotice(message, pfxchar);
 			}
 
 			Modes::ChangeList setban;
-			setban.push_remove(*banmode, mask);
-			ServerInstance->Modes->Process(ServerInstance->FakeClient, cr, NULL, setban);
+			setban.push_remove(*banmode, timedban.mask);
+			ServerInstance->Modes.Process(ServerInstance->FakeClient, timedban.chan, nullptr, setban);
 		}
 	}
 
-	void OnChannelDelete(Channel* chan) CXX11_OVERRIDE
+	void OnChannelDelete(Channel* chan) override
 	{
 		// Remove all timed bans affecting the channel from internal bookkeeping
 		TimedBanList.erase(std::remove_if(TimedBanList.begin(), TimedBanList.end(), ChannelMatcher(chan)), TimedBanList.end());
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds the /TBAN command which allows channel operators to add bans which will be expired after the specified period.", VF_COMMON | VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleTimedBans)
