@@ -3,11 +3,10 @@
  *
  *   Copyright (C) 2021 Herman <GermanAizek@yandex.ru>
  *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
- *   Copyright (C) 2014 Justin Crawford <Justasic@Gmail.com>
- *   Copyright (C) 2013, 2015, 2018-2020, 2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2015, 2018-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2013 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012, 2014 Adam <Adam@anope.org>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012 ChrisTX <xpipe@hotmail.de>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
@@ -26,25 +25,32 @@
  */
 
 
-#include "inspircd.h"
-#include "xline.h"
+#include <filesystem>
 #include <fstream>
 
-class ModuleXLineDB
+#include "inspircd.h"
+#include "timeutils.h"
+#include "xline.h"
+
+class ModuleXLineDB final
 	: public Module
 	, public Timer
 {
- private:
+private:
 	bool dirty;
 	std::string xlinedbpath;
+	unsigned long saveperiod;
+	unsigned long maxbackoff;
+	unsigned char backoff;
 
- public:
+public:
 	ModuleXLineDB()
-		: Timer(0, true)
+		: Module(VF_VENDOR, "Allows X-lines to be saved and reloaded on restart.")
+		, Timer(0, true)
 	{
 	}
 
-	void init() CXX11_OVERRIDE
+	void init() override
 	{
 		/* Load the configuration
 		 * Note:
@@ -52,9 +58,12 @@ class ModuleXLineDB
 		 *		Imagine a scenario where the new file already exists. Merging the current XLines with the existing database is likely a bad idea
 		 *		...and so is discarding all current in-memory XLines for the ones in the database.
 		 */
-		ConfigTag* Conf = ServerInstance->Config->ConfValue("xlinedb");
+		const auto& Conf = ServerInstance->Config->ConfValue("xlinedb");
 		xlinedbpath = ServerInstance->Config->Paths.PrependData(Conf->getString("filename", "xline.db", 1));
-		SetInterval(Conf->getDuration("saveperiod", 5));
+		saveperiod = Conf->getDuration("saveperiod", 5);
+		backoff = Conf->getNum<uint8_t>("backoff", 0);
+		maxbackoff = Conf->getDuration("maxbackoff", saveperiod * 120, saveperiod);
+		SetInterval(saveperiod);
 
 		// Read xlines before attaching to events
 		ReadDatabase();
@@ -67,7 +76,7 @@ class ModuleXLineDB
 	 * @param source The sender of the line or NULL for local server
 	 * @param line The xline being added
 	 */
-	void OnAddLine(User* source, XLine* line) CXX11_OVERRIDE
+	void OnAddLine(User* source, XLine* line) override
 	{
 		if (!line->from_config)
 			dirty = true;
@@ -78,17 +87,30 @@ class ModuleXLineDB
 	 * @param source The user removing the line or NULL for local server
 	 * @param line the line being deleted
 	 */
-	void OnDelLine(User* source, XLine* line) CXX11_OVERRIDE
+	void OnDelLine(User* source, XLine* line) override
 	{
 		OnAddLine(source, line);
 	}
 
-	bool Tick(time_t) CXX11_OVERRIDE
+	bool Tick() override
 	{
 		if (dirty)
 		{
 			if (WriteDatabase())
+			{
+				// If we were previously unable to write but now can then reset the time interval.
+				if (GetInterval() != saveperiod)
+					SetInterval(saveperiod, false);
+
 				dirty = false;
+			}
+			else
+			{
+				// Back off a bit to avoid spamming opers.
+				if (backoff > 1)
+					SetInterval(std::min(GetInterval() * backoff, maxbackoff), false);
+				ServerInstance->Logs.Debug(MODNAME, "Trying again in {} seconds", GetInterval());
+			}
 		}
 		return true;
 	}
@@ -101,17 +123,17 @@ class ModuleXLineDB
 		 * Technically, that means that this can block, but I have *never* seen that.
 		 *     -- w00t
 		 */
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Opening temporary database");
+		ServerInstance->Logs.Debug(MODNAME, "Opening temporary database");
 		const std::string xlinenewdbpath = xlinedbpath + ".new." + ConvToStr(ServerInstance->Time());
-		std::ofstream stream(xlinenewdbpath.c_str());
+		std::ofstream stream(xlinenewdbpath);
 		if (!stream.is_open())
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Cannot create database \"%s\"! %s (%d)", xlinenewdbpath.c_str(), strerror(errno), errno);
-			ServerInstance->SNO->WriteToSnoMask('x', "database: cannot create new xline db \"%s\": %s (%d)", xlinenewdbpath.c_str(), strerror(errno), errno);
+			ServerInstance->Logs.Critical(MODNAME, "Cannot create database \"{}\"! {} ({})", xlinenewdbpath, strerror(errno), errno);
+			ServerInstance->SNO.WriteToSnoMask('x', "database: cannot create new xline db \"{}\": {} ({})", xlinenewdbpath, strerror(errno), errno);
 			return false;
 		}
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Opened. Writing..");
+		ServerInstance->Logs.Debug(MODNAME, "Opened. Writing..");
 
 		/*
 		 * Now, much as I hate writing semi-unportable formats, additional
@@ -123,16 +145,14 @@ class ModuleXLineDB
 		stream << "VERSION 1" << std::endl;
 
 		// Now, let's write.
-		std::vector<std::string> types = ServerInstance->XLines->GetAllTypes();
-		for (std::vector<std::string>::const_iterator it = types.begin(); it != types.end(); ++it)
+		for (const auto& xltype : ServerInstance->XLines->GetAllTypes())
 		{
-			XLineLookup* lookup = ServerInstance->XLines->GetAll(*it);
+			XLineLookup* lookup = ServerInstance->XLines->GetAll(xltype);
 			if (!lookup)
 				continue; // Not possible as we just obtained the list from XLineManager
 
-			for (LookupIter i = lookup->begin(); i != lookup->end(); ++i)
+			for (const auto& [_, line] : *lookup)
 			{
-				XLine* line = i->second;
 				if (line->from_config)
 					continue;
 
@@ -142,12 +162,12 @@ class ModuleXLineDB
 			}
 		}
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Finished writing XLines. Checking for error..");
+		ServerInstance->Logs.Debug(MODNAME, "Finished writing XLines. Checking for error..");
 
 		if (stream.fail())
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Cannot write to new database \"%s\"! %s (%d)", xlinenewdbpath.c_str(), strerror(errno), errno);
-			ServerInstance->SNO->WriteToSnoMask('x', "database: cannot write to new xline db \"%s\": %s (%d)", xlinenewdbpath.c_str(), strerror(errno), errno);
+			ServerInstance->Logs.Critical(MODNAME, "Cannot write to new database \"{}\"! {} ({})", xlinenewdbpath, strerror(errno), errno);
+			ServerInstance->SNO.WriteToSnoMask('x', "database: cannot write to new xline db \"{}\": {} ({})", xlinenewdbpath, strerror(errno), errno);
 			return false;
 		}
 		stream.close();
@@ -158,8 +178,8 @@ class ModuleXLineDB
 		// Use rename to move temporary to new db - this is guaranteed not to fuck up, even in case of a crash.
 		if (rename(xlinenewdbpath.c_str(), xlinedbpath.c_str()) < 0)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Cannot replace old database \"%s\" with new database \"%s\"! %s (%d)", xlinedbpath.c_str(), xlinenewdbpath.c_str(), strerror(errno), errno);
-			ServerInstance->SNO->WriteToSnoMask('x', "database: cannot replace old xline db \"%s\" with new db \"%s\": %s (%d)", xlinedbpath.c_str(), xlinenewdbpath.c_str(), strerror(errno), errno);
+			ServerInstance->Logs.Critical(MODNAME, "Cannot replace old database \"{}\" with new database \"{}\"! {} ({})", xlinedbpath, xlinenewdbpath, strerror(errno), errno);
+			ServerInstance->SNO.WriteToSnoMask('x', "database: cannot replace old xline db \"{}\" with new db \"{}\": {} ({})", xlinedbpath, xlinenewdbpath, strerror(errno), errno);
 			return false;
 		}
 
@@ -169,14 +189,15 @@ class ModuleXLineDB
 	bool ReadDatabase()
 	{
 		// If the xline database doesn't exist then we don't need to load it.
-		if (!FileSystem::FileExists(xlinedbpath))
+		std::error_code ec;
+		if (!std::filesystem::is_regular_file(xlinedbpath, ec))
 			return true;
 
-		std::ifstream stream(xlinedbpath.c_str());
+		std::ifstream stream(xlinedbpath);
 		if (!stream.is_open())
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Cannot read database \"%s\"! %s (%d)", xlinedbpath.c_str(), strerror(errno), errno);
-			ServerInstance->SNO->WriteToSnoMask('x', "database: cannot read xline db \"%s\": %s (%d)", xlinedbpath.c_str(), strerror(errno), errno);
+			ServerInstance->Logs.Critical(MODNAME, "Cannot read database \"{}\"! {} ({})", xlinedbpath, strerror(errno), errno);
+			ServerInstance->SNO.WriteToSnoMask('x', "database: cannot read xline db \"{}\": {} ({})", xlinedbpath, strerror(errno), errno);
 			return false;
 		}
 
@@ -195,15 +216,15 @@ class ModuleXLineDB
 				items++;
 			}
 
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Processing %s", line.c_str());
+			ServerInstance->Logs.Debug(MODNAME, "Processing {}", line);
 
 			if (command_p[0] == "VERSION")
 			{
 				if (command_p[1] != "1")
 				{
 					stream.close();
-					ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "I got database version %s - I don't understand it", command_p[1].c_str());
-					ServerInstance->SNO->WriteToSnoMask('x', "database: I got a database version (%s) I don't understand", command_p[1].c_str());
+					ServerInstance->Logs.Critical(MODNAME, "I got database version {} - I don't understand it", command_p[1]);
+					ServerInstance->SNO.WriteToSnoMask('x', "database: I got a database version ({}) I don't understand", command_p[1]);
 					return false;
 				}
 			}
@@ -214,29 +235,35 @@ class ModuleXLineDB
 
 				if (!xlf)
 				{
-					ServerInstance->SNO->WriteToSnoMask('x', "database: Unknown line type (%s).", command_p[1].c_str());
+					ServerInstance->SNO.WriteToSnoMask('x', "database: Unknown line type ({}).", command_p[1]);
 					continue;
 				}
 
 				XLine* xl = xlf->Generate(ServerInstance->Time(), ConvToNum<unsigned long>(command_p[5]), command_p[3], command_p[6], command_p[2]);
 				xl->SetCreateTime(ConvToNum<time_t>(command_p[4]));
 
-				if (ServerInstance->XLines->AddLine(xl, NULL))
+				if (!ServerInstance->XLines->AddLine(xl, nullptr))
 				{
-					ServerInstance->SNO->WriteToSnoMask('x', "database: Added a line of type %s", command_p[1].c_str());
+					continue;
+					delete xl;
+				}
+
+				if (xl->duration)
+				{
+					ServerInstance->SNO.WriteToSnoMask('x', "database: added a timed {}{} on {}, expires in {} (on {}): {}",
+						xl->type, xl->type.length() <= 2 ? "-line" : "", xl->Displayable(),
+						Duration::ToString(xl->duration), Time::FromNow(xl->duration), xl->reason);
 				}
 				else
-					delete xl;
+				{
+					ServerInstance->SNO.WriteToSnoMask('x', "database: added a permanent {}{} on {}: {}", xl->type,
+						xl->type.length() <= 2 ? "-line" : "", xl->Displayable(), xl->reason);
+				}
 			}
 		}
 		stream.close();
 		return true;
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows X-lines to be saved and reloaded on restart.", VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleXLineDB)
