@@ -2,13 +2,13 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2013-2014 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2013, 2017-2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2014 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2013, 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013 Daniel Vassdal <shutter@canternet.org>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2011 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2008 Thomas Stagner <aquanight@inspircd.org>
- *   Copyright (C) 2007 John Brooks <special@inspircd.org>
+ *   Copyright (C) 2008 Thomas Stagner <aquanight@gmail.com>
+ *   Copyright (C) 2007 John Brooks <john@jbrooks.io>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2006 Craig Edwards <brain@inspircd.org>
  *
@@ -26,35 +26,59 @@
  */
 
 
-#include "inspircd.h"
+#ifndef _WIN32
+# include <arpa/inet.h>
+# include <netinet/in.h>
+#endif
 
-bool InspIRCd::BindPort(ConfigTag* tag, const irc::sockets::sockaddrs& sa, std::vector<ListenSocket*>& old_ports)
+#include "inspircd.h"
+#include "utility/string.h"
+
+namespace
+{
+#ifdef IPPROTO_SCTP
+	// Checks whether the system can create SCTP sockets.
+	bool CanCreateSCTPSocket()
+	{
+		int fd = socket(AF_INET, SOCK_STREAM, IPPROTO_SCTP);
+		if (fd >= 0)
+		{
+			SocketEngine::Close(fd);
+			return true;
+		}
+		return false;
+	}
+#endif
+}
+
+bool InspIRCd::BindPort(const std::shared_ptr<ConfigTag>& tag, const irc::sockets::sockaddrs& sa, std::vector<ListenSocket*>& old_ports, sa_family_t protocol)
 {
 	for (std::vector<ListenSocket*>::iterator n = old_ports.begin(); n != old_ports.end(); ++n)
 	{
-		if ((**n).bind_sa == sa)
+		ListenSocket* ls = *n;
+		if (ls->bind_sa == sa && protocol == ls->bind_protocol)
 		{
 			// Replace tag, we know addr and port match, but other info (type, ssl) may not.
-			ServerInstance->Logs->Log("SOCKET", LOG_DEFAULT, "Replacing listener on %s from old tag at %s with new tag from %s",
-				sa.str().c_str(), (*n)->bind_tag->getTagLocation().c_str(), tag->getTagLocation().c_str());
-			(*n)->bind_tag = tag;
-			(*n)->ResetIOHookProvider();
+			ServerInstance->Logs.Debug("SOCKET", "Replacing listener on {} from old tag at {} with new tag from {}",
+				sa.str(), ls->bind_tag->source.str(), tag->source.str());
+			ls->bind_tag = tag;
+			ls->ResetIOHookProvider();
 
 			old_ports.erase(n);
 			return true;
 		}
 	}
 
-	ListenSocket* ll = new ListenSocket(tag, sa);
+	auto* ll = new ListenSocket(tag, sa, protocol);
 	if (!ll->HasFd())
 	{
-		ServerInstance->Logs->Log("SOCKET", LOG_DEFAULT, "Failed to listen on %s from tag at %s: %s",
-			sa.str().c_str(), tag->getTagLocation().c_str(), strerror(errno));
+		ServerInstance->Logs.Normal("SOCKET", "Failed to listen on {} from tag at {}: {}",
+			sa.str(), tag->source.str(), SocketEngine::LastError());
 		delete ll;
 		return false;
 	}
 
-	ServerInstance->Logs->Log("SOCKET", LOG_DEFAULT, "Added a listener on %s from tag at %s", sa.str().c_str(), tag->getTagLocation().c_str());
+	ServerInstance->Logs.Debug("SOCKET", "Added a listener on {} from tag at {}", sa.str(), tag->source.str());
 	ports.push_back(ll);
 	return true;
 }
@@ -64,41 +88,97 @@ size_t InspIRCd::BindPorts(FailedPortList& failed_ports)
 	size_t bound = 0;
 	std::vector<ListenSocket*> old_ports(ports.begin(), ports.end());
 
-	ConfigTagList tags = ServerInstance->Config->ConfTags("bind");
-	for (ConfigIter i = tags.first; i != tags.second; ++i)
+	for (const auto& [_, tag] : ServerInstance->Config->ConfTags("bind"))
 	{
-		ConfigTag* tag = i->second;
-
-		// Are we creating a TCP/IP listener?
+		// Are we creating a TCP/SCTP listener?
 		const std::string address = tag->getString("address");
 		const std::string portlist = tag->getString("port");
 		if (!address.empty() || !portlist.empty())
 		{
 			// InspIRCd supports IPv4 and IPv6 natively; no 4in6 required.
 			if (strncasecmp(address.c_str(), "::ffff:", 7) == 0)
-				this->Logs->Log("SOCKET", LOG_DEFAULT, "Using 4in6 (::ffff:) isn't recommended. You should bind IPv4 addresses directly instead.");
+				this->Logs.Warning("SOCKET", "Using 4in6 (::ffff:) isn't recommended. You should bind IPv4 addresses directly instead.");
+
+			// Try to parse the bind address.
+			irc::sockets::sockaddrs bindspec(true);
+			if (!bindspec.from_ip(address))
+			{
+				failed_ports.emplace_back("Address is not valid: " + address, tag);
+				continue;
+			}
 
 			// A TCP listener with no ports is not very useful.
 			if (portlist.empty())
-				this->Logs->Log("SOCKET", LOG_DEFAULT, "TCP listener on %s at %s has no ports specified!",
-					address.empty() ? "*" : address.c_str(), tag->getTagLocation().c_str());
+			{
+				failed_ports.emplace_back("No ports specified", bindspec, tag);
+				continue;
+			}
 
 			irc::portparser portrange(portlist, false);
-			for (int port; (port = portrange.GetToken()); )
+			while (long port = portrange.GetToken())
 			{
-				irc::sockets::sockaddrs bindspec;
-				if (!irc::sockets::aptosa(address, port, bindspec))
+				// Check if the port is out of range.
+				if (port <= std::numeric_limits<in_port_t>::min() || port > std::numeric_limits<in_port_t>::max())
+				{
+					failed_ports.emplace_back("Port is not valid: " + ConvToStr(port), bindspec, tag);
 					continue;
+				}
 
-				if (!BindPort(tag, bindspec, old_ports))
-					failed_ports.push_back(FailedPort(errno, bindspec, tag));
-				else
-					bound++;
+				switch (bindspec.family())
+				{
+					case AF_INET:
+						bindspec.in4.sin_port = htons(static_cast<in_port_t>(port));
+						break;
+
+					case AF_INET6:
+						bindspec.in6.sin6_port = htons(static_cast<in_port_t>(port));
+						break;
+
+					default:
+						continue; // Should never happen.
+				}
+
+				std::vector<int> protocols;
+				irc::spacesepstream protostream(tag->getString("protocols", "all", 1));
+				for (std::string protocol; protostream.GetToken(protocol); )
+				{
+					if (insp::equalsci(protocol, "all"))
+					{
+						protocols.push_back(0); // IPPROTO_TCP
+#ifdef IPPROTO_SCTP
+						if (CanCreateSCTPSocket())
+							protocols.push_back(IPPROTO_SCTP);
+#endif
+					}
+					else if (insp::equalsci(protocol, "sctp"))
+					{
+#ifdef IPPROTO_SCTP
+						protocols.push_back(IPPROTO_SCTP);
+#else
+						failed_ports.emplace_back("Platform does not support SCTP", tag);
+#endif
+					}
+					else if (insp::equalsci(protocol, "tcp"))
+					{
+						protocols.push_back(0); // IPPROTO_TCP
+					}
+					else
+					{
+						failed_ports.emplace_back("Protocol is not valid: " + protocol, tag);
+					}
+				}
+
+				for (const auto protocol : protocols)
+				{
+					if (!BindPort(tag, bindspec, old_ports, protocol))
+						failed_ports.emplace_back(SocketEngine::LastError(), bindspec, tag);
+					else
+						bound++;
+				}
 			}
 			continue;
 		}
 
-#ifndef _WIN32
 		// Are we creating a UNIX listener?
 		const std::string path = tag->getString("path");
 		if (!path.empty())
@@ -110,41 +190,38 @@ size_t InspIRCd::BindPorts(FailedPortList& failed_ports)
 			irc::sockets::sockaddrs bindspec;
 			if (fullpath.length() > std::min(ServerInstance->Config->Limits.MaxHost, sizeof(bindspec.un.sun_path) - 1))
 			{
-				this->Logs->Log("SOCKET", LOG_DEFAULT, "UNIX listener on %s at %s specified a path that is too long!",
-					fullpath.c_str(), tag->getTagLocation().c_str());
+				failed_ports.emplace_back("Path is too long: " + fullpath, tag);
 				continue;
 			}
 
 			// Check for characters which are problematic in the IRC message format.
 			if (fullpath.find_first_of("\n\r\t!@: ") != std::string::npos)
 			{
-				this->Logs->Log("SOCKET", LOG_DEFAULT, "UNIX listener on %s at %s specified a path containing invalid characters!",
-					fullpath.c_str(), tag->getTagLocation().c_str());
+				failed_ports.emplace_back("Path contains invalid characters: " + fullpath, tag);
 				continue;
 			}
 
-			irc::sockets::untosa(fullpath, bindspec);
-			if (!BindPort(tag, bindspec, old_ports))
-				failed_ports.push_back(FailedPort(errno, bindspec, tag));
+			bindspec.from_unix(fullpath);
+			if (!BindPort(tag, bindspec, old_ports, 0))
+				failed_ports.emplace_back(SocketEngine::LastError(), bindspec, tag);
 			else
 				bound++;
 		}
-#endif
 	}
 
 	std::vector<ListenSocket*>::iterator n = ports.begin();
-	for (std::vector<ListenSocket*>::iterator o = old_ports.begin(); o != old_ports.end(); ++o)
+	for (auto* old_port : old_ports)
 	{
-		while (n != ports.end() && *n != *o)
+		while (n != ports.end() && *n != old_port)
 			n++;
 		if (n == ports.end())
 		{
-			this->Logs->Log("SOCKET", LOG_DEFAULT, "Port bindings slipped out of vector, aborting close!");
+			this->Logs.Warning("SOCKET", "Port bindings slipped out of vector, aborting close!");
 			break;
 		}
 
-		this->Logs->Log("SOCKET", LOG_DEFAULT, "Port binding %s was removed from the config file, closing.",
-			(**n).bind_sa.str().c_str());
+		this->Logs.Debug("SOCKET", "Port binding {} was removed from the config file, closing.",
+			(**n).bind_sa.str());
 		delete *n;
 
 		// this keeps the iterator valid, pointing to the next element
@@ -154,46 +231,52 @@ size_t InspIRCd::BindPorts(FailedPortList& failed_ports)
 	return bound;
 }
 
-bool irc::sockets::aptosa(const std::string& addr, int port, irc::sockets::sockaddrs& sa)
+irc::sockets::sockaddrs::sockaddrs(bool initialize)
 {
-	memset(&sa, 0, sizeof(sa));
-	if (addr.empty() || addr.c_str()[0] == '*')
+	if (initialize)
+		memset(this, 0, sizeof(*this));
+}
+
+bool irc::sockets::sockaddrs::from_ip_port(const std::string& addr, in_port_t port)
+{
+	if (addr.empty() || addr == "*")
 	{
 		if (ServerInstance->Config->WildcardIPv6)
 		{
-			sa.in6.sin6_family = AF_INET6;
-			sa.in6.sin6_port = htons(port);
+			memset(&in6.sin6_addr, 0, sizeof(in6.sin6_addr));
+			in6.sin6_family = AF_INET6;
+			in6.sin6_port = htons(port);
 		}
 		else
 		{
-			sa.in4.sin_family = AF_INET;
-			sa.in4.sin_port = htons(port);
+			memset(&in4.sin_addr, 0, sizeof(in4.sin_addr));
+			in4.sin_family = AF_INET;
+			in4.sin_port = htons(port);
 		}
 		return true;
 	}
-	else if (inet_pton(AF_INET, addr.c_str(), &sa.in4.sin_addr) > 0)
+	else if (inet_pton(AF_INET, addr.c_str(), &in4.sin_addr) > 0)
 	{
-		sa.in4.sin_family = AF_INET;
-		sa.in4.sin_port = htons(port);
+		in4.sin_family = AF_INET;
+		in4.sin_port = htons(port);
 		return true;
 	}
-	else if (inet_pton(AF_INET6, addr.c_str(), &sa.in6.sin6_addr) > 0)
+	else if (inet_pton(AF_INET6, addr.c_str(), &in6.sin6_addr) > 0)
 	{
-		sa.in6.sin6_family = AF_INET6;
-		sa.in6.sin6_port = htons(port);
+		in6.sin6_family = AF_INET6;
+		in6.sin6_port = htons(port);
 		return true;
 	}
 	return false;
 }
 
-bool irc::sockets::untosa(const std::string& path, irc::sockets::sockaddrs& sa)
+bool irc::sockets::sockaddrs::from_unix(const std::string& path)
 {
-	memset(&sa, 0, sizeof(sa));
-	if (path.length() >= sizeof(sa.un.sun_path))
+	if (path.length() >= sizeof(un.sun_path))
 		return false;
 
-	sa.un.sun_family = AF_UNIX;
-	memcpy(&sa.un.sun_path, path.c_str(), path.length() + 1);
+	un.sun_family = AF_UNIX;
+	memcpy(&un.sun_path, path.c_str(), path.length() + 1);
 	return true;
 }
 
@@ -207,13 +290,31 @@ bool irc::sockets::isunix(const std::string& file)
 	return false;
 }
 
-
-int irc::sockets::sockaddrs::family() const
+sa_family_t irc::sockets::sockaddrs::family() const
 {
 	return sa.sa_family;
 }
 
-int irc::sockets::sockaddrs::port() const
+bool irc::sockets::sockaddrs::is_local() const
+{
+	switch (family())
+	{
+		case AF_INET:
+			return irc::sockets::cidr_mask("127.0.0.0/8").match(*this);
+
+		case AF_INET6:
+			return irc::sockets::cidr_mask("::1/128").match(*this);
+
+		case AF_UNIX:
+			return true;
+	}
+
+	// If we have reached this point then we have encountered a bug.
+	ServerInstance->Logs.Debug("SOCKET", "BUG: irc::sockets::sockaddrs::is_local(): socket type {} is unknown!", family());
+	return false;
+}
+
+in_port_t irc::sockets::sockaddrs::port() const
 {
 	switch (family())
 	{
@@ -228,7 +329,7 @@ int irc::sockets::sockaddrs::port() const
 	}
 
 	// If we have reached this point then we have encountered a bug.
-	ServerInstance->Logs->Log("SOCKET", LOG_DEBUG, "BUG: irc::sockets::sockaddrs::port(): socket type %d is unknown!", family());
+	ServerInstance->Logs.Debug("SOCKET", "BUG: irc::sockets::sockaddrs::port(): socket type {} is unknown!", family());
 	return 0;
 }
 
@@ -238,13 +339,13 @@ std::string irc::sockets::sockaddrs::addr() const
 	{
 		case AF_INET:
 			char ip4addr[INET_ADDRSTRLEN];
-			if (!inet_ntop(AF_INET, (void*)&in4.sin_addr, ip4addr, sizeof(ip4addr)))
+			if (!inet_ntop(AF_INET, static_cast<const void*>(&in4.sin_addr), ip4addr, sizeof(ip4addr)))
 				return "0.0.0.0";
 			return ip4addr;
 
 		case AF_INET6:
 			char ip6addr[INET6_ADDRSTRLEN];
-			if (!inet_ntop(AF_INET6, (void*)&in6.sin6_addr, ip6addr, sizeof(ip6addr)))
+			if (!inet_ntop(AF_INET6, static_cast<const void*>(&in6.sin6_addr), ip6addr, sizeof(ip6addr)))
 				return "0:0:0:0:0:0:0:0";
 			return ip6addr;
 
@@ -253,7 +354,7 @@ std::string irc::sockets::sockaddrs::addr() const
 	}
 
 	// If we have reached this point then we have encountered a bug.
-	ServerInstance->Logs->Log("SOCKET", LOG_DEBUG, "BUG: irc::sockets::sockaddrs::addr(): socket type %d is unknown!", family());
+	ServerInstance->Logs.Debug("SOCKET", "BUG: irc::sockets::sockaddrs::addr(): socket type {} is unknown!", family());
 	return "<unknown>";
 }
 
@@ -263,22 +364,22 @@ std::string irc::sockets::sockaddrs::str() const
 	{
 		case AF_INET:
 			char ip4addr[INET_ADDRSTRLEN];
-			if (!inet_ntop(AF_INET, (void*)&in4.sin_addr, ip4addr, sizeof(ip4addr)))
+			if (!inet_ntop(AF_INET, static_cast<const void*>(&in4.sin_addr), ip4addr, sizeof(ip4addr)))
 				strcpy(ip4addr, "0.0.0.0");
-			return InspIRCd::Format("%s:%u", ip4addr, ntohs(in4.sin_port));
+			return INSP_FORMAT("{}:{}", ip4addr, ntohs(in4.sin_port));
 
 		case AF_INET6:
 			char ip6addr[INET6_ADDRSTRLEN];
-			if (!inet_ntop(AF_INET6, (void*)&in6.sin6_addr, ip6addr, sizeof(ip6addr)))
+			if (!inet_ntop(AF_INET6, static_cast<const void*>(&in6.sin6_addr), ip6addr, sizeof(ip6addr)))
 				strcpy(ip6addr, "0:0:0:0:0:0:0:0");
-			return InspIRCd::Format("[%s]:%u", ip6addr, ntohs(in6.sin6_port));
+			return INSP_FORMAT("[{}]:{}", ip6addr, ntohs(in6.sin6_port));
 
 		case AF_UNIX:
 			return un.sun_path;
 	}
 
 	// If we have reached this point then we have encountered a bug.
-	ServerInstance->Logs->Log("SOCKET", LOG_DEBUG, "BUG: irc::sockets::sockaddrs::str(): socket type %d is unknown!", family());
+	ServerInstance->Logs.Debug("SOCKET", "BUG: irc::sockets::sockaddrs::str(): socket type {} is unknown!", family());
 	return "<unknown>";
 }
 
@@ -297,7 +398,7 @@ socklen_t irc::sockets::sockaddrs::sa_size() const
 	}
 
 	// If we have reached this point then we have encountered a bug.
-	ServerInstance->Logs->Log("SOCKET", LOG_DEBUG, "BUG: irc::sockets::sockaddrs::sa_size(): socket type %d is unknown!", family());
+	ServerInstance->Logs.Debug("SOCKET", "BUG: irc::sockets::sockaddrs::sa_size(): socket type {} is unknown!", family());
 	return 0;
 }
 
@@ -319,7 +420,7 @@ bool irc::sockets::sockaddrs::operator==(const irc::sockets::sockaddrs& other) c
 	}
 
 	// If we have reached this point then we have encountered a bug.
-	ServerInstance->Logs->Log("SOCKET", LOG_DEBUG, "BUG: irc::sockets::sockaddrs::operator==(): socket type %d is unknown!", family());
+	ServerInstance->Logs.Debug("SOCKET", "BUG: irc::sockets::sockaddrs::operator==(): socket type {} is unknown!", family());
 	return !memcmp(this, &other, sizeof(*this));
 }
 
@@ -342,18 +443,18 @@ static void sa2cidr(irc::sockets::cidr_mask& cidr, const irc::sockets::sockaddrs
 		case AF_INET:
 			cidr.length = range > 32 ? 32 : range;
 			target_byte = sizeof(sa.in4.sin_addr);
-			base = (unsigned char*)&sa.in4.sin_addr;
+			base = reinterpret_cast<const unsigned char*>(&sa.in4.sin_addr);
 			break;
 
 		case AF_INET6:
 			cidr.length = range > 128 ? 128 : range;
 			target_byte = sizeof(sa.in6.sin6_addr);
-			base = (unsigned char*)&sa.in6.sin6_addr;
+			base = reinterpret_cast<const unsigned char*>(&sa.in6.sin6_addr);
 			break;
 
 		default:
 			// If we have reached this point then we have encountered a bug.
-			ServerInstance->Logs->Log("SOCKET", LOG_DEBUG, "BUG: sa2cidr(): socket type %d is unknown!", cidr.type);
+			ServerInstance->Logs.Debug("SOCKET", "BUG: sa2cidr(): socket type {} is unknown!", cidr.type);
 			cidr.length = 0;
 			return;
 	}
@@ -383,13 +484,21 @@ irc::sockets::cidr_mask::cidr_mask(const std::string& mask)
 
 	if (bits_chars == std::string::npos)
 	{
-		irc::sockets::aptosa(mask, 0, sa);
+		if (!sa.from_ip(mask))
+		{
+			memset(this, 0, sizeof(*this));
+			return;
+		}
 		sa2cidr(*this, sa, 128);
 	}
 	else
 	{
 		unsigned char range = ConvToNum<unsigned char>(mask.substr(bits_chars + 1));
-		irc::sockets::aptosa(mask.substr(0, bits_chars), 0, sa);
+		if (!sa.from_ip(mask.substr(0, bits_chars)))
+		{
+			memset(this, 0, sizeof(*this));
+			return;
+		}
 		sa2cidr(*this, sa, range);
 	}
 }
@@ -405,13 +514,13 @@ std::string irc::sockets::cidr_mask::str() const
 	switch (type)
 	{
 		case AF_INET:
-			base = (unsigned char*)&sa.in4.sin_addr;
+			base = reinterpret_cast<unsigned char*>(&sa.in4.sin_addr);
 			len = 4;
 			maxlen = 32;
 			break;
 
 		case AF_INET6:
-			base = (unsigned char*)&sa.in6.sin6_addr;
+			base = reinterpret_cast<unsigned char*>(&sa.in6.sin6_addr);
 			len = 16;
 			maxlen = 128;
 			break;
@@ -422,7 +531,7 @@ std::string irc::sockets::cidr_mask::str() const
 
 		default:
 			// If we have reached this point then we have encountered a bug.
-			ServerInstance->Logs->Log("SOCKET", LOG_DEBUG, "BUG: irc::sockets::cidr_mask::str(): socket type %d is unknown!", type);
+			ServerInstance->Logs.Debug("SOCKET", "BUG: irc::sockets::cidr_mask::str(): socket type {} is unknown!", type);
 			return "<unknown>";
 	}
 
