@@ -1,9 +1,8 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2021 Herman <GermanAizek@yandex.ru>
  *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
- *   Copyright (C) 2013, 2017-2020, 2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2017-2023, 2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013 Adam <Adam@anope.org>
  *   Copyright (C) 2012-2014, 2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
@@ -27,67 +26,85 @@
 
 
 #include "inspircd.h"
-#include "xline.h"
 #include "modules/stats.h"
+#include "stringutils.h"
+#include "timeutils.h"
+#include "xline.h"
 
 /** An XLineFactory specialized to generate GLine* pointers
  */
-class GLineFactory : public XLineFactory
+class GLineFactory final
+	: public XLineFactory
 {
- public:
-	GLineFactory() : XLineFactory("G") { }
+public:
+	GLineFactory()
+		: XLineFactory("G")
+	{
+	}
 
 	/** Generate a GLine
 	 */
-	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) override
 	{
-		IdentHostPair ih = ServerInstance->XLines->IdentSplit(xline_specific_mask);
+		UserHostPair ih = XLineManager::SplitUserHost(xline_specific_mask);
 		return new GLine(set_time, duration, source, reason, ih.first, ih.second);
 	}
 };
 
 /** An XLineFactory specialized to generate ELine* pointers
  */
-class ELineFactory : public XLineFactory
+class ELineFactory final
+	: public XLineFactory
 {
- public:
-	ELineFactory() : XLineFactory("E") { }
+public:
+	ELineFactory()
+		: XLineFactory("E")
+	{
+	}
 
 	/** Generate an ELine
 	 */
-	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) override
 	{
-		IdentHostPair ih = ServerInstance->XLines->IdentSplit(xline_specific_mask);
+		UserHostPair ih = XLineManager::SplitUserHost(xline_specific_mask);
 		return new ELine(set_time, duration, source, reason, ih.first, ih.second);
 	}
 };
 
 /** An XLineFactory specialized to generate KLine* pointers
  */
-class KLineFactory : public XLineFactory
+class KLineFactory final
+	: public XLineFactory
 {
- public:
-	KLineFactory() : XLineFactory("K") { }
+public:
+	KLineFactory()
+		: XLineFactory("K")
+	{
+	}
 
 	/** Generate a KLine
 	 */
-	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) override
 	{
-		IdentHostPair ih = ServerInstance->XLines->IdentSplit(xline_specific_mask);
+		UserHostPair ih = XLineManager::SplitUserHost(xline_specific_mask);
 		return new KLine(set_time, duration, source, reason, ih.first, ih.second);
 	}
 };
 
 /** An XLineFactory specialized to generate QLine* pointers
  */
-class QLineFactory : public XLineFactory
+class QLineFactory final
+	: public XLineFactory
 {
- public:
-	QLineFactory() : XLineFactory("Q") { }
+public:
+	QLineFactory()
+		: XLineFactory("Q")
+	{
+	}
 
 	/** Generate a QLine
 	 */
-	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) override
 	{
 		return new QLine(set_time, duration, source, reason, xline_specific_mask);
 	}
@@ -95,20 +112,23 @@ class QLineFactory : public XLineFactory
 
 /** An XLineFactory specialized to generate ZLine* pointers
  */
-class ZLineFactory : public XLineFactory
+class ZLineFactory final
+	: public XLineFactory
 {
- public:
-	ZLineFactory() : XLineFactory("Z") { }
+public:
+	ZLineFactory()
+		: XLineFactory("Z")
+	{
+	}
 
 	/** Generate a ZLine
 	 */
-	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) CXX11_OVERRIDE
+	XLine* Generate(time_t set_time, unsigned long duration, const std::string& source, const std::string& reason, const std::string& xline_specific_mask) override
 	{
 		return new ZLine(set_time, duration, source, reason, xline_specific_mask);
 	}
 };
 
-
 /*
  * This is now version 3 of the XLine subsystem, let's see if we can get it as nice and
  * efficient as we can this time so we can close this file and never ever touch it again ..
@@ -141,7 +161,7 @@ class ZLineFactory : public XLineFactory
  *  bans. :)
  */
 
-bool XLine::Matches(User *u)
+bool XLine::Matches(User* u) const
 {
 	return false;
 }
@@ -161,10 +181,8 @@ void XLineManager::CheckELines()
 	if (ELines.empty())
 		return;
 
-	const UserManager::LocalList& list = ServerInstance->Users.GetLocalUsers();
-	for (UserManager::LocalList::const_iterator u2 = list.begin(); u2 != list.end(); u2++)
+	for (auto* u :  ServerInstance->Users.GetLocalUsers())
 	{
-		LocalUser* u = *u2;
 		u->exempt = false;
 
 		/* This uses safe iteration to ensure that if a line expires here, it doesn't trash the iterator */
@@ -175,7 +193,7 @@ void XLineManager::CheckELines()
 			safei = i;
 			safei++;
 
-			XLine *e = i->second;
+			XLine* e = i->second;
 			if ((!e->duration || ServerInstance->Time() < e->expiry) && e->Matches(u))
 				u->exempt = true;
 
@@ -184,13 +202,12 @@ void XLineManager::CheckELines()
 	}
 }
 
-
-XLineLookup* XLineManager::GetAll(const std::string &type)
+XLineLookup* XLineManager::GetAll(const std::string& type)
 {
 	ContainerIter n = lookup_lines.find(type);
 
 	if (n == lookup_lines.end())
-		return NULL;
+		return nullptr;
 
 	LookupIter safei;
 	const time_t current = ServerInstance->Time();
@@ -210,7 +227,7 @@ XLineLookup* XLineManager::GetAll(const std::string &type)
 	return &(n->second);
 }
 
-void XLineManager::DelAll(const std::string &type)
+void XLineManager::DelAll(const std::string& type)
 {
 	ContainerIter n = lookup_lines.find(type);
 
@@ -229,19 +246,19 @@ void XLineManager::DelAll(const std::string &type)
 std::vector<std::string> XLineManager::GetAllTypes()
 {
 	std::vector<std::string> items;
-	for (ContainerIter x = lookup_lines.begin(); x != lookup_lines.end(); ++x)
-		items.push_back(x->first);
+	for (const auto& [type, _] : lookup_lines)
+		items.push_back(type);
 	return items;
 }
 
-IdentHostPair XLineManager::IdentSplit(const std::string &ident_and_host)
+UserHostPair XLineManager::SplitUserHost(const std::string& user_and_host)
 {
-	IdentHostPair n = std::make_pair<std::string,std::string>("*","*");
-	std::string::size_type x = ident_and_host.find('@');
+	UserHostPair n = std::make_pair("*", "*");
+	std::string::size_type x = user_and_host.find('@');
 	if (x != std::string::npos)
 	{
-		n.second = ident_and_host.substr(x + 1,ident_and_host.length());
-		n.first = ident_and_host.substr(0, x);
+		n.second = user_and_host.substr(x + 1, user_and_host.length());
+		n.first = user_and_host.substr(0, x);
 		if (!n.first.length())
 			n.first.assign("*");
 		if (!n.second.length())
@@ -250,7 +267,7 @@ IdentHostPair XLineManager::IdentSplit(const std::string &ident_and_host)
 	else
 	{
 		n.first.clear();
-		n.second = ident_and_host;
+		n.second = user_and_host;
 	}
 
 	return n;
@@ -298,7 +315,6 @@ bool XLineManager::AddLine(XLine* line, User* user)
 		}
 	}
 
-	/*ELine* item = new ELine(ServerInstance->Time(), duration, source, reason, ih.first.c_str(), ih.second.c_str());*/
 	XLineFactory* xlf = GetFactory(line->type);
 	if (!xlf)
 		return false;
@@ -318,7 +334,7 @@ bool XLineManager::AddLine(XLine* line, User* user)
 
 // deletes a line, returns true if the line existed and was removed
 
-bool XLineManager::DelLine(const char* hostmask, const std::string& type, std::string& reason, User* user, bool simulate)
+bool XLineManager::DelLine(const std::string& hostmask, const std::string& type, std::string& reason, User* user, bool simulate)
 {
 	ContainerIter x = lookup_lines.find(type);
 
@@ -349,7 +365,6 @@ bool XLineManager::DelLine(const char* hostmask, const std::string& type, std::s
 	return true;
 }
 
-
 void ELine::Unset()
 {
 	ServerInstance->XLines->CheckELines();
@@ -357,12 +372,12 @@ void ELine::Unset()
 
 // returns a pointer to the reason if a nickname matches a Q-line, NULL if it didn't match
 
-XLine* XLineManager::MatchesLine(const std::string &type, User* user)
+XLine* XLineManager::MatchesLine(const std::string& type, User* user)
 {
 	ContainerIter x = lookup_lines.find(type);
 
 	if (x == lookup_lines.end())
-		return NULL;
+		return nullptr;
 
 	const time_t current = ServerInstance->Time();
 
@@ -388,15 +403,15 @@ XLine* XLineManager::MatchesLine(const std::string &type, User* user)
 
 		i = safei;
 	}
-	return NULL;
+	return nullptr;
 }
 
-XLine* XLineManager::MatchesLine(const std::string &type, const std::string &pattern)
+XLine* XLineManager::MatchesLine(const std::string& type, const std::string& pattern)
 {
 	ContainerIter x = lookup_lines.find(type);
 
 	if (x == lookup_lines.end())
-		return NULL;
+		return nullptr;
 
 	const time_t current = ServerInstance->Time();
 
@@ -423,7 +438,7 @@ XLine* XLineManager::MatchesLine(const std::string &type, const std::string &pat
 
 		i = safei;
 	}
-	return NULL;
+	return nullptr;
 }
 
 // removes lines that have expired
@@ -447,7 +462,6 @@ void XLineManager::ExpireLine(ContainerIter container, LookupIter item, bool sil
 	container->second.erase(item);
 }
 
-
 // applies lines, removing clients and changing nicks etc as applicable
 void XLineManager::ApplyLines()
 {
@@ -460,9 +474,8 @@ void XLineManager::ApplyLines()
 		if (u->exempt)
 			continue;
 
-		for (std::vector<XLine *>::iterator i = pending_lines.begin(); i != pending_lines.end(); i++)
+		for (auto* x : pending_lines)
 		{
-			XLine *x = *i;
 			if (x->Matches(u))
 			{
 				x->Apply(u);
@@ -478,34 +491,6 @@ void XLineManager::ApplyLines()
 	pending_lines.clear();
 }
 
-void XLineManager::InvokeStats(const std::string& type, unsigned int numeric, Stats::Context& stats)
-{
-	ContainerIter n = lookup_lines.find(type);
-
-	time_t current = ServerInstance->Time();
-
-	LookupIter safei;
-
-	if (n != lookup_lines.end())
-	{
-		XLineLookup& list = n->second;
-		for (LookupIter i = list.begin(); i != list.end(); )
-		{
-			safei = i;
-			safei++;
-
-			XLine* e = i->second;
-			if (e->duration && current > e->expiry)
-			{
-				ExpireLine(n, i);
-			}
-			else
-				stats.AddRow(numeric, e->Displayable(), e->set_time, e->duration, e->source, e->reason);
-			i = safei;
-		}
-	}
-}
-
 bool XLineManager::InvokeStats(const std::string& type, Stats::Context& context)
 {
 	ContainerIter citer = lookup_lines.find(type);
@@ -526,49 +511,33 @@ bool XLineManager::InvokeStats(const std::string& type, Stats::Context& context)
 			continue;
 		}
 
-		context.AddRow(RPL_STATS, context.GetSymbol(), xline->Displayable(), xline->set_time, xline->duration, xline->source, xline->reason);
+		context.AddGenericRow(xline->Displayable(), xline->set_time, xline->duration, xline->source, xline->reason);
 	}
 	return true;
 }
 
 XLineManager::XLineManager()
 {
-	GLineFactory* GFact;
-	ELineFactory* EFact;
-	KLineFactory* KFact;
-	QLineFactory* QFact;
-	ZLineFactory* ZFact;
-
-
-	GFact = new GLineFactory;
-	EFact = new ELineFactory;
-	KFact = new KLineFactory;
-	QFact = new QLineFactory;
-	ZFact = new ZLineFactory;
-
-	RegisterFactory(GFact);
-	RegisterFactory(EFact);
-	RegisterFactory(KFact);
-	RegisterFactory(QFact);
-	RegisterFactory(ZFact);
+	RegisterFactory(new ELineFactory());
+	RegisterFactory(new GLineFactory());
+	RegisterFactory(new KLineFactory());
+	RegisterFactory(new QLineFactory());
+	RegisterFactory(new ZLineFactory());
 }
 
 XLineManager::~XLineManager()
 {
-	const char gekqz[] = "GEKQZ";
-	for(unsigned int i=0; i < sizeof(gekqz); i++)
+	for(const char line : "GEKQZ")
 	{
-		XLineFactory* xlf = GetFactory(std::string(1, gekqz[i]));
+		XLineFactory* xlf = GetFactory(std::string(1, line));
 		delete xlf;
 	}
 
 	// Delete all existing XLines
-	for (XLineContainer::iterator i = lookup_lines.begin(); i != lookup_lines.end(); i++)
+	for (const auto& [_, lines] : lookup_lines)
 	{
-		for (XLineLookup::iterator j = i->second.begin(); j != i->second.end(); j++)
-		{
-			delete j->second;
-		}
+		for (const auto& [__, line] : lines)
+			delete line;
 	}
 }
 
@@ -581,36 +550,48 @@ bool XLine::IsBurstable()
 	return !from_config;
 }
 
-void XLine::DefaultApply(User* u, const std::string &line, bool bancache)
+void XLine::DefaultApply(User* u, bool bancache)
 {
-	const std::string banReason = line + "-lined: " + reason;
-
 	if (!ServerInstance->Config->XLineMessage.empty())
 		u->WriteNumeric(ERR_YOUREBANNEDCREEP, ServerInstance->Config->XLineMessage);
 
-	if (ServerInstance->Config->HideBans)
-		ServerInstance->Users->QuitUser(u, line + "-lined", &banReason);
-	else
-		ServerInstance->Users->QuitUser(u, banReason);
+	Template::VariableMap vars = {
+		{ "created",   Time::ToString(set_time)                            },
+		{ "duration",  Duration::ToString(duration)                        },
+		{ "expiry",    Time::ToString(expiry)                              },
+		{ "fulltype",  type.length() <= 2 ? type + "-lined" : type         },
+		{ "reason",    reason                                              },
+		{ "remaining", Duration::ToString(ServerInstance->Time() - expiry) },
+		{ "setter",    source                                              },
+		{ "type",      type                                                },
+	};
 
+	const std::string banreason = Template::Replace(ServerInstance->Config->XLineQuit, vars);
+	if (ServerInstance->Config->XLineQuitPublic.empty())
+		ServerInstance->Users.QuitUser(u, banreason);
+	else
+	{
+		const std::string publicreason = Template::Replace(ServerInstance->Config->XLineQuitPublic, vars);
+		ServerInstance->Users.QuitUser(u, publicreason, &banreason);
+	}
 
 	if (bancache)
 	{
-		ServerInstance->Logs->Log("BANCACHE", LOG_DEBUG, "BanCache: Adding positive hit (" + line + ") for " + u->GetIPString());
-		ServerInstance->BanCache.AddHit(u->GetIPString(), this->type, banReason, (this->duration > 0 ? (this->expiry - ServerInstance->Time()) : 0));
+		ServerInstance->Logs.Debug("BANCACHE", "Adding positive hit ({}) for {}", type, u->GetAddress());
+		ServerInstance->BanCache.AddHit(u->GetAddress(), this->type, banreason, (this->duration > 0 ? (this->expiry - ServerInstance->Time()) : 0));
 	}
 }
 
-bool KLine::Matches(User *u)
+bool KLine::Matches(User* u) const
 {
 	LocalUser* lu = IS_LOCAL(u);
 	if (lu && lu->exempt)
 		return false;
 
-	if (InspIRCd::Match(u->ident, this->identmask, ascii_case_insensitive_map))
+	if (InspIRCd::Match(u->GetRealUser(), this->usermask, ascii_case_insensitive_map))
 	{
 		if (InspIRCd::MatchCIDR(u->GetRealHost(), this->hostmask, ascii_case_insensitive_map) ||
-			InspIRCd::MatchCIDR(u->GetIPString(), this->hostmask, ascii_case_insensitive_map))
+			InspIRCd::MatchCIDR(u->GetAddress(), this->hostmask, ascii_case_insensitive_map))
 		{
 			return true;
 		}
@@ -621,19 +602,19 @@ bool KLine::Matches(User *u)
 
 void KLine::Apply(User* u)
 {
-	DefaultApply(u, "K", (this->identmask ==  "*") ? true : false);
+	DefaultApply(u, this->usermask ==  "*");
 }
 
-bool GLine::Matches(User *u)
+bool GLine::Matches(User* u) const
 {
 	LocalUser* lu = IS_LOCAL(u);
 	if (lu && lu->exempt)
 		return false;
 
-	if (InspIRCd::Match(u->ident, this->identmask, ascii_case_insensitive_map))
+	if (InspIRCd::Match(u->GetRealUser(), this->usermask, ascii_case_insensitive_map))
 	{
 		if (InspIRCd::MatchCIDR(u->GetRealHost(), this->hostmask, ascii_case_insensitive_map) ||
-			InspIRCd::MatchCIDR(u->GetIPString(), this->hostmask, ascii_case_insensitive_map))
+			InspIRCd::MatchCIDR(u->GetAddress(), this->hostmask, ascii_case_insensitive_map))
 		{
 			return true;
 		}
@@ -644,15 +625,15 @@ bool GLine::Matches(User *u)
 
 void GLine::Apply(User* u)
 {
-	DefaultApply(u, "G", (this->identmask == "*") ? true : false);
+	DefaultApply(u, this->usermask == "*");
 }
 
-bool ELine::Matches(User *u)
+bool ELine::Matches(User* u) const
 {
-	if (InspIRCd::Match(u->ident, this->identmask, ascii_case_insensitive_map))
+	if (InspIRCd::Match(u->GetRealUser(), this->usermask, ascii_case_insensitive_map))
 	{
 		if (InspIRCd::MatchCIDR(u->GetRealHost(), this->hostmask, ascii_case_insensitive_map) ||
-			InspIRCd::MatchCIDR(u->GetIPString(), this->hostmask, ascii_case_insensitive_map))
+			InspIRCd::MatchCIDR(u->GetAddress(), this->hostmask, ascii_case_insensitive_map))
 		{
 			return true;
 		}
@@ -661,30 +642,23 @@ bool ELine::Matches(User *u)
 	return false;
 }
 
-bool ZLine::Matches(User *u)
+bool ZLine::Matches(User* u) const
 {
 	LocalUser* lu = IS_LOCAL(u);
 	if (lu && lu->exempt)
 		return false;
 
-	if (InspIRCd::MatchCIDR(u->GetIPString(), this->ipaddr))
-		return true;
-	else
-		return false;
+	return InspIRCd::MatchCIDR(u->GetAddress(), this->ipaddr);
 }
 
 void ZLine::Apply(User* u)
 {
-	DefaultApply(u, "Z", true);
+	DefaultApply(u, true);
 }
 
-
-bool QLine::Matches(User *u)
+bool QLine::Matches(User* u) const
 {
-	if (InspIRCd::Match(u->nick, this->nick))
-		return true;
-
-	return false;
+	return InspIRCd::Match(u->nick, this->nick);
 }
 
 void QLine::Apply(User* u)
@@ -694,45 +668,36 @@ void QLine::Apply(User* u)
 	u->ChangeNick(u->uuid);
 }
 
-
-bool ZLine::Matches(const std::string &str)
+bool ZLine::Matches(const std::string& str) const
 {
-	if (InspIRCd::MatchCIDR(str, this->ipaddr))
-		return true;
-	else
-		return false;
+	return InspIRCd::MatchCIDR(str, this->ipaddr);
 }
 
-bool QLine::Matches(const std::string &str)
+bool QLine::Matches(const std::string& str) const
 {
-	if (InspIRCd::Match(str, this->nick))
-		return true;
-
-	return false;
+	return InspIRCd::Match(str, this->nick);
 }
 
-bool ELine::Matches(const std::string &str)
+bool ELine::Matches(const std::string& str) const
 {
-	return (InspIRCd::MatchCIDR(str, matchtext));
+	return InspIRCd::MatchCIDR(str, matchtext);
 }
 
-bool KLine::Matches(const std::string &str)
+bool KLine::Matches(const std::string& str) const
 {
-	return (InspIRCd::MatchCIDR(str, matchtext));
+	return InspIRCd::MatchCIDR(str, matchtext);
 }
 
-bool GLine::Matches(const std::string &str)
+bool GLine::Matches(const std::string& str) const
 {
-	return (InspIRCd::MatchCIDR(str, matchtext));
+	return InspIRCd::MatchCIDR(str, matchtext);
 }
 
 void ELine::OnAdd()
 {
 	/* When adding one E-line, only check the one E-line */
-	const UserManager::LocalList& list = ServerInstance->Users.GetLocalUsers();
-	for (UserManager::LocalList::const_iterator u2 = list.begin(); u2 != list.end(); u2++)
+	for (auto* u : ServerInstance->Users.GetLocalUsers())
 	{
-		LocalUser* u = *u2;
 		if (this->Matches(u))
 			u->exempt = true;
 	}
@@ -740,32 +705,32 @@ void ELine::OnAdd()
 
 void XLine::DisplayExpiry()
 {
-	bool onechar = (type.length() == 1);
-	ServerInstance->SNO->WriteToSnoMask('x', "Removing an expired %s%s on %s (set by %s %s ago): %s",
-		type.c_str(), (onechar ? "-line" : ""), Displayable().c_str(), source.c_str(), InspIRCd::DurationString(ServerInstance->Time() - set_time).c_str(), reason.c_str());
+	ServerInstance->SNO.WriteToSnoMask('x', "Removing an expired {}{} on {} (set by {} {} ago): {}",
+		type, (type.length() <= 2 ? "-line" : ""), Displayable(), source,
+		Duration::ToString(ServerInstance->Time() - set_time), reason);
 }
 
-const std::string& ELine::Displayable()
+const std::string& ELine::Displayable() const
 {
 	return matchtext;
 }
 
-const std::string& KLine::Displayable()
+const std::string& KLine::Displayable() const
 {
 	return matchtext;
 }
 
-const std::string& GLine::Displayable()
+const std::string& GLine::Displayable() const
 {
 	return matchtext;
 }
 
-const std::string& ZLine::Displayable()
+const std::string& ZLine::Displayable() const
 {
 	return ipaddr;
 }
 
-const std::string& QLine::Displayable()
+const std::string& QLine::Displayable() const
 {
 	return nick;
 }
@@ -799,12 +764,12 @@ bool XLineManager::UnregisterFactory(XLineFactory* xlf)
 	return true;
 }
 
-XLineFactory* XLineManager::GetFactory(const std::string &type)
+XLineFactory* XLineManager::GetFactory(const std::string& type)
 {
 	XLineFactMap::iterator n = line_factory.find(type);
 
 	if (n == line_factory.end())
-		return NULL;
+		return nullptr;
 
 	return n->second;
 }
